@@ -1,3 +1,25 @@
+// ----------------------------------------------------------------------------
+//
+// Digital Modem Program for the Fast Light Toolkit
+//
+// Copyright 2007-2008, Dave Freese, W1HKJ
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
+
 #include <config.h>
 
 #include <cstring>
@@ -208,7 +230,7 @@ void Fl_ComboBox::value( const char *s )
 		}
     }
 	if ( i < listsize)
-		Output->value(datalist[i]->s);
+		Output->value(datalist[idx = i]->s);
 }
 
 void Fl_ComboBox::put_value(const char *s)
@@ -322,5 +344,7 @@ void Fl_ComboBox::textsize (uchar n)
   Output->textsize (n);
 }
 
-
-
+int Fl_ComboBox::size()
+{
+  return listsize;
+}
