@@ -1,27 +1,26 @@
 // ----------------------------------------------------------------------------
 // rtty.cxx  --  RTTY modem
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
 //
-// This file is part of fldigi.  Adapted from code contained in gmfsk source code 
+// This file is part of fldigi.  Adapted from code contained in gmfsk source code
 // distribution.
 //  gmfsk Copyright (C) 2001, 2002, 2003
 //  Tomi Manninen (oh2bns@sral.fi)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -87,7 +86,6 @@ int dspcnt = 0;
 
 
 static char msg1[20];
-static char msg2[20];
 
 double _SHIFT[] = {23, 85, 160, 170, 182, 200, 240, 350, 425, 850};
 double _BAUD[] = {45, 45.45, 50, 56, 75, 100, 110, 150, 200, 300};
@@ -117,13 +115,24 @@ void rtty::rx_init()
 
 void rtty::init()
 {
-	modem::init();
+	bool wfrev = wf->Reverse();
+	bool wfsb = wf->USB();
+	reverse = wfrev ^ !wfsb;
+
+	if (progdefaults.StartAtSweetSpot)
+		set_freq(progdefaults.RTTYsweetspot);
+	else if (progStatus.carrier != 0) {
+		set_freq(progStatus.carrier);
+#if !BENCHMARK_MODE
+		progStatus.carrier = 0;
+#endif
+	} else
+		set_freq(wf->Carrier());
+
 	rx_init();
 	put_MODEstatus(mode);
-	snprintf(msg1, sizeof(msg1), "Shft %-4.0f", rtty_shift); 
+	snprintf(msg1, sizeof(msg1), "%-4.1f / %-4.0f", rtty_baud, rtty_shift);
 	put_Status1(msg1);
-	snprintf(msg2, sizeof(msg2), "Baud %-4.1f", rtty_baud); 
-	put_Status2(msg2);
 	if (progdefaults.PreferXhairScope)
 		set_scope_mode(Digiscope::XHAIRS);
 	else
@@ -142,8 +151,9 @@ rtty::~rtty()
 void rtty::restart()
 {
 	double stl;
-	
-	rtty_shift = shift = _SHIFT[progdefaults.rtty_shift];
+
+	rtty_shift = shift = (progdefaults.rtty_shift >= 0 ?
+			      _SHIFT[progdefaults.rtty_shift] : progdefaults.rtty_custom_shift);
 	rtty_baud = _BAUD[progdefaults.rtty_baud];
 	nbits = rtty_bits = _BITS[progdefaults.rtty_bits];
 	if (rtty_bits == 5)
@@ -163,22 +173,27 @@ void rtty::restart()
 	rxmode = LETTERS;
 	symbollen = (int) (samplerate / rtty_baud + 0.5);
 	set_bandwidth(shift);
-	
-    rtty_BW = shift + 3*rtty_baud;
+
+	rtty_BW = 1.5 * rtty_baud;
 	progdefaults.RTTY_BW = rtty_BW;
 	sldrRTTYbandwidth->value(rtty_BW);
-	
+
 	wf->redraw_marker();
 
-	if (bpfilt) 
-		bpfilt->create_filter(0, rtty_BW / samplerate / 2.0);
-	else
-		bpfilt = new fftfilt(0, rtty_BW / samplerate / 2.0, 1024);
+	bp_filt_lo = (shift/2.0 - rtty_BW/2.0) / samplerate;
+	if (bp_filt_lo < 0) bp_filt_lo = 0;
+	bp_filt_hi = (shift/2.0 + rtty_BW/2.0) / samplerate;
 
-	if (bitfilt)
-		bitfilt->setLength(symbollen / 8);//4);
+	if (bpfilt)
+		bpfilt->create_filter(bp_filt_lo, bp_filt_hi);
 	else
-		bitfilt = new Cmovavg(symbollen / 8);//4);
+		bpfilt = new fftfilt(bp_filt_lo, bp_filt_hi, 1024);
+
+	bflen = symbollen/3;
+	if (bitfilt)
+		bitfilt->setLength(bflen);
+	else
+		bitfilt = new Cmovavg(bflen);
 
 // stop length = 1, 1.5 or 2 bits
 	rtty_stop = progdefaults.rtty_stop;
@@ -191,43 +206,42 @@ void rtty::restart()
 	pipeptr = 0;
 	poscnt = negcnt = 0;
 	posfreq = negfreq = 0.0;
-	
+
 	metric = 0.0;
 
-	snprintf(msg1, sizeof(msg1), "Shft %-4.0f", rtty_shift); 
+	snprintf(msg1, sizeof(msg1), "%-4.1f / %-4.0f", rtty_baud, rtty_shift);
 	put_Status1(msg1);
-	snprintf(msg2, sizeof(msg2), "Baud %-4.1f", rtty_baud); 
-	put_Status2(msg2);
 	put_MODEstatus(mode);
 	for (int i = 0; i < 1024; i++) QI[i].re = QI[i].im = 0.0;
 	sigpwr = 0.0;
 	noisepwr = 0.0;
 	freqerrlo = freqerrhi = 0.0;
 	sigsearch = 0;
-	dspcnt = 2*(nbits + 2);//nbits + 2; //2 * symbollen * (nbits + 2);
-	
+	dspcnt = 2*(nbits + 2);
+
 	clear_zdata = true;
 }
 
 rtty::rtty(trx_mode tty_mode)
 {
-	cap = CAP_AFC | CAP_REV;
+	cap |= CAP_AFC | CAP_REV;
 
 	mode = tty_mode;
 
 	samplerate = RTTY_SampleRate;
-	fragmentsize = 256;
-	
+
 	bpfilt = (fftfilt *)0;
-	
+
 	bitfilt = (Cmovavg *)0;
-	
+
 	hilbert = new C_FIR_filter();
 	hilbert->init_hilbert(37, 1);
 
 	pipe = new double[MAXPIPE];
 	dsppipe = new double [MAXPIPE];
-		
+
+	samples = new complex[8];
+
 	restart();
 }
 
@@ -255,9 +269,9 @@ complex rtty::mixer(complex in)
 	z = z * in;
 
 	phaseacc -= TWOPI * frequency / samplerate;
-	if (phaseacc > M_PI) 
+	if (phaseacc > M_PI)
 		phaseacc -= TWOPI;
-	else if (phaseacc < M_PI) 
+	else if (phaseacc < M_PI)
 		phaseacc += TWOPI;
 
 	return z;
@@ -325,9 +339,9 @@ int rtty::decode_char()
 	return data;
 }
 
-int rtty::rx(bool bit)
+bool rtty::rx(bool bit)
 {
-	int flag = 0;
+	bool flag = false;
 	unsigned char c;
 
 	switch (rxstate) {
@@ -345,18 +359,17 @@ int rtty::rx(bool bit)
 				counter = symbollen;
 				bitcntr = 0;
 				rxdata = 0;
-				flag = 1;
 			} else {
 				rxstate = RTTY_RX_STATE_IDLE;
 			}
-		}
+		} else
+			if (bit) rxstate = RTTY_RX_STATE_IDLE;
 		break;
 
 	case RTTY_RX_STATE_DATA:
 		if (--counter == 0) {
 			rxdata |= bit << bitcntr++;
 			counter = symbollen;
-			flag = 1;
 		}
 
 		if (bitcntr == nbits) {
@@ -374,19 +387,18 @@ int rtty::rx(bool bit)
 			rxstate = RTTY_RX_STATE_STOP;
 			rxdata |= bit << bitcntr++;
 			counter = symbollen;
-			flag = 1;
 		}
 		break;
 
 	case RTTY_RX_STATE_STOP:
 		if (--counter == 0) {
 			if (bit) {
-				if ((metric >= progStatus.sldrSquelchValue && progStatus.sqlonoff)|| !progStatus.sqlonoff) {				
+				if ((metric >= progStatus.sldrSquelchValue && progStatus.sqlonoff)|| !progStatus.sqlonoff) {
 					c = decode_char();
 					if ( c != 0 )
-						put_rx_char(c);
+						put_rx_char(progdefaults.rx_lowercase ? tolower(c) : c);
 				}
-				flag = 2;
+				flag = true;
 			}
 			rxstate = RTTY_RX_STATE_STOP2;
 			counter = symbollen / 2;
@@ -403,16 +415,25 @@ int rtty::rx(bool bit)
 	return flag;
 }
 
+char snrmsg[80];
 void rtty::Metric()
 {
-	double delta = rtty_baud/4.0;
-	noisepwr = wf->powerDensity(frequency - shift * 1.5, delta) +
-	 		   wf->powerDensity(frequency + shift * 1.5, delta) + 1e-10;
-	sigpwr = wf->powerDensity(frequency - shift/2, delta) +
-			 wf->powerDensity(frequency + shift/2, delta) + 1e-10;
-    double snr = sigpwr / noisepwr;
-    metric = decayavg( metric, snr, 8);//16);
-    metric = CLAMP(metric, 0.0, 100.0);
+	double delta = rtty_baud/2.0;
+	double np = 
+		wf->powerDensity(frequency - shift * 1.5, delta) +
+	 	wf->powerDensity(frequency + shift * 1.5, delta) + 1e-10;
+	double sp =
+		wf->powerDensity(frequency - shift/2, delta) +
+		wf->powerDensity(frequency + shift/2, delta) + 1e-10;
+	double snr = 0;
+
+	sigpwr = decayavg( sigpwr, sp, sp - sigpwr > 0 ? 2 : 8);
+	noisepwr = decayavg( noisepwr, np, 32 );
+	snr = 10*log10(sigpwr / ( noisepwr * (1500 / delta))); // 3000 Hz noise bw
+
+	snprintf(snrmsg, sizeof(snrmsg), "s/n %3.0f dB", snr);
+	put_Status2(snrmsg);
+	metric = CLAMP(sigpwr/noisepwr, 0.0, 100.0);
 	display_metric(metric);
 }
 
@@ -459,24 +480,27 @@ int rtty::rx_process(const double *buf, int len)
 	complex z, *zp;
 	double f = 0.0;
 	double fin;
-	static bool bit = true;//false;
-	int n = 0, rxflag;
-	double deadzone = shift/8;
+	static bool bit = true;
+	int n = 0;
+	double deadzone = shift/4;
 	double rotate;
 	double ferr = 0;
 
 	if (progdefaults.RTTY_BW != rtty_BW) {
 		rtty_BW = progdefaults.RTTY_BW;
-		bpfilt->create_filter(0, rtty_BW / samplerate / 2.0);
+		bp_filt_lo = (shift/2.0 - rtty_BW/2.0) / samplerate;
+		if (bp_filt_lo < 0) bp_filt_lo = 0;
+		bp_filt_hi = (shift/2.0 + rtty_BW/2.0) / samplerate;
+		bpfilt->create_filter(bp_filt_lo, bp_filt_hi);
 		wf->redraw_marker();
 	}
 
 	Metric();
 
 	while (len-- > 0) {
-		
+
 // create analytic signal from sound card input samples
-		
+
 		z.re = z.im = *buf++;
 		hilbert->run(z, z);
 
@@ -487,20 +511,18 @@ int rtty::rx_process(const double *buf, int len)
 // bandpass filter using Windowed Sinc - Overlap-Add convolution filter
 
 		n = bpfilt->run(z, &zp);
+
 		if (n) {
 			for (int i = 0; i < n; i++) {
-			
+
 // measure phase difference between successive samples to determine
 // the frequency of the baseband signal (+rtty_baud or -rtty_baud)
 // see class complex definiton for operator %
-			
+
 				fin = (prevsmpl % zp[i]).arg() * samplerate / TWOPI;
 				prevsmpl = zp[i];
 
-// filter the result with a moving average filter
-			
-				f = bitfilt->run(fin);
-			
+
 // track the + and - frequency excursions separately to derive an afc signal
 
                 rotate = -4.0 * M_PI * freqerr / rtty_shift;
@@ -512,7 +534,7 @@ int rtty::rx_process(const double *buf, int len)
 					posfreq += fin;
 					QI[i].re = zp[i].re;
 					QI[i].im = xmix * zp[i].im;
-				} 
+				}
 				if (fin < 0.0) {
 					negcnt++;
 					negfreq += fin;
@@ -520,34 +542,32 @@ int rtty::rx_process(const double *buf, int len)
 					QI[i].im = zp[i].re;
 				}
 				QI[i] = QI[i] * complex(cos(rotate), sin(rotate));
-                avgsig = decayavg(avgsig, 1.25 * zp[i].mag(), 64);
-				
-				if (avgsig > 0) 
+                avgsig = decayavg(avgsig, 1.25 * zp[i].mag(), 128);//64);
+
+				if (avgsig > 0)
 				    QI[i] = QI[i] / avgsig;
 
+				fin = CLAMP(fin, - rtty_shift, rtty_shift);
+// filter the result with a moving average filter
+				f = bitfilt->run(fin);
 //	hysterisis dead zone in frequency discriminator bit detector
-
-				if (dspcnt && (--dspcnt % (nbits + 2) == 0)) {
-					pipe[pipeptr] = f / shift; // display filtered signal		
-					pipeptr = (pipeptr + 1) % symbollen;
-				}
-
 				if (f > deadzone )
 					bit = true;
 				if (f < -deadzone)
 					bit = false;
-//				if (!progdefaults.RTTY_USB)
-//					bit = !bit;
-	
-				rxflag = rx (reverse ? !bit : bit);
 
-				if (rxflag == 2) {
+				if (dspcnt && (--dspcnt % (nbits + 2) == 0)) {
+					pipe[pipeptr] = f / shift;
+					pipeptr = (pipeptr + 1) % symbollen;
+				}
+
+				if ( rx( reverse ? !bit : bit ) ) {
 					dspcnt = symbollen * (nbits + 2);
 
 					if (poscnt && negcnt) {
 						poserr = posfreq/poscnt;
 						negerr = negfreq/negcnt;
-						
+
 // compute the frequency error as the median of + and - relative freq's
 						if (sigsearch) sigsearch--;
 
@@ -605,7 +625,6 @@ double rtty::nco(double freq)
 double rtty::FSKnco()
 {
 	FSKphaseacc += TWOPI * 1000 / samplerate;
-//	FSKphaseacc += TWOPI * 440 / samplerate;
 
 	if (FSKphaseacc > M_PI)
 
@@ -618,7 +637,7 @@ double rtty::FSKnco()
 void rtty::send_symbol(int symbol)
 {
 	double freq;
-	
+
 	if (reverse)
 		symbol = !symbol;
 
@@ -626,7 +645,7 @@ void rtty::send_symbol(int symbol)
 		freq = get_txfreq_woffset() + shift / 2.0;
 	else
 		freq = get_txfreq_woffset() - shift / 2.0;
-    
+
 	for (int i = 0; i < symbollen; i++) {
 		outbuf[i] = nco(freq);
 		if (symbol)
@@ -645,7 +664,7 @@ void rtty::send_stop()
 {
 	double freq;
 	bool invert = reverse;
-	
+
 	if (invert)
 		freq = get_txfreq_woffset() - shift / 2.0;
 	else
@@ -693,8 +712,8 @@ void rtty::send_char(int c)
 			c = letters[c];
 		else
 			c = figures[c];
-		if (c) 
-			put_echo_char(c);
+		if (c)
+			put_echo_char(progdefaults.rx_lowercase ? tolower(c) : c);
 	}
 }
 
@@ -800,7 +819,7 @@ int rtty::tx_process()
 		return 0;
 
 // switch case if necessary
-	
+
 	if ((c & 0x300) != txmode) {// == 0) {
 		if (txmode == FIGURES) {
 			send_char(LETTERS);
