@@ -1,5 +1,4 @@
-//////////////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////////////
+// ----------------------------------------------------------------------------
 //  Font_Browser.cpp      v 0.0.3                        2007-04-21 
 //
 //         for the Fast Light Tool Kit (FLTK) 1.1.x .
@@ -7,25 +6,21 @@
 //    David Freese, w1hkj@w1hkj.com
 //    based on similar widget by Mariwan Jalal
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This file is part of fldigi.
 //
-// This library is distributed in the hope that it will be useful,
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-//
-// Please report all bugs and problems to "fltk_kurdi@yahoo.com".
-//
-//////////////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////////////
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -40,16 +35,20 @@
 #include <FL/fl_draw.H>
 
 #include "font_browser.h"
+#include "flslider2.h"
+#include "gettext.h"
 
 using namespace std;
 
+Font_Browser* font_browser;
+
 // Font Color selected
 
 void Font_Browser::ColorSelect()
 {
     unsigned char r, g, b;
     Fl::get_color(fontcolor, r, g, b);
-    if (fl_color_chooser("Font color", r, g, b) == 0)
+    if (fl_color_chooser(_("Font color"), r, g, b) == 0)
 	    return;
     fontcolor = fl_rgb_color(r, g, b);
     btn_Color->color(fontcolor);
@@ -72,7 +71,7 @@ void Font_Browser::fb_callback(Fl_Widget* w, void* arg)
     else {
 	if (w == fb->lst_Size)
 	    fb->txt_Size->value(strtol(fb->lst_Size->text(fb->lst_Size->value()), NULL, 10));
-	fb->fontsize = fb->txt_Size->value();
+	fb->fontsize = static_cast<int>(fb->txt_Size->value());
     }
     fb->box_Example->SetFont(fb->fontnbr, fb->fontsize, fb->fontcolor);
 }
@@ -119,12 +118,12 @@ void Font_Browser::FontNameSelect()
 Font_Browser::Font_Browser(int x, int y, int w, int h, const char *lbl )
      : Fl_Window(x, y, w, h, lbl)
 {
-    lst_Font = new Fl_Browser(5, 15, 280, 125, "Font:");
+    lst_Font = new Fl_Browser(5, 15, 280, 125, _("Font:"));
     lst_Font->align(FL_ALIGN_TOP_LEFT);
     lst_Font->type(FL_HOLD_BROWSER);
     lst_Font->callback(fb_callback, this);
 
-    txt_Size = new Fl_Value_Input(290, 15, 50, 22, "Size:");
+    txt_Size = new Fl_Value_Input2(290, 15, 50, 22, _("Size:"));
     txt_Size->align(FL_ALIGN_TOP_LEFT);
     txt_Size->range(1.0, 48.0);
     txt_Size->step(1.0);
@@ -134,22 +133,22 @@ Font_Browser::Font_Browser(int x, int y, int w, int h, const char *lbl )
     lst_Size->type(FL_HOLD_BROWSER);
     lst_Size->callback(fb_callback, this);
 
-    btn_OK = new Fl_Return_Button(345, 40, 50, 25, "&OK");
+    btn_OK = new Fl_Return_Button(345, 40, 50, 25, _("&OK"));
     btn_OK->shortcut(0x8006f);
     btn_OK->callback(fb_callback, this);
 
-    btn_Cancel = new Fl_Button(345, 70, 50, 25, "Cancel");
+    btn_Cancel = new Fl_Button(345, 70, 50, 25, _("Cancel"));
     btn_Cancel->labelsize(12);
     btn_Cancel->callback(fb_callback, this);
 
-    btn_Color = new Fl_Button(345, 100, 50, 25, "Color");
+    btn_Color = new Fl_Button(345, 100, 50, 25, _("Color"));
     btn_Color->down_box(FL_BORDER_BOX);
     btn_Color->color(FL_FOREGROUND_COLOR);
     btn_Color->callback(fb_callback, this);
 
-    box_Example = new Preview_Box(5, 145, 390, 75, "That crazy fox jumped over the dog again!\n"
+    box_Example = new Preview_Box(5, 145, 390, 75, _("That crazy fox jumped over the dog again!\n"
 				  "ABCDEFGHIJKLMNOPQRSTUVWXYZ 0123456789\n"
-				  "!\"#$%&'()*+,-./:;<=>?@@[\\]^_`{|}~");
+				  "!\"#$%&'()*+,-./:;<=>?@@[\\]^_`{|}~"));
     box_Example->box(FL_DOWN_BOX);
     box_Example->align(FL_ALIGN_WRAP|FL_ALIGN_CLIP|FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
     resizable(box_Example);
@@ -175,11 +174,12 @@ Font_Browser::Font_Browser(int x, int y, int w, int h, const char *lbl )
     fontnbr = FL_HELVETICA;;
     fontsize = FL_NORMAL_SIZE; // Font Size to be used
     fontcolor = FL_FOREGROUND_COLOR;
+    filter = ALL_TYPES;
 
     lst_Font->value(1);
     FontNameSelect();
 
-    Fl::focus(lst_Font);
+//!    Fl::focus(lst_Font);
 
     xclass(PACKAGE_NAME);
 }
@@ -235,6 +235,86 @@ bool Font_Browser::fixed_width(Fl_Font f)
 	return fl_width('X') == fl_width('i');
 }
 
+#include <vector>
+#include <FL/Fl_Double_Window.H>
+#include <FL/Fl_Progress.H>
+
+class Progress_Window : public Fl_Double_Window
+{
+public:
+	Progress_Window(float min = 0.0f, float max = 100.0f, const char* l = 0)
+		: Fl_Double_Window(200, 34), ps(5, 5, 190, 24, l)
+	{
+		end();
+
+		range(min, max);
+		ps.align(FL_ALIGN_CENTER | FL_ALIGN_INSIDE);
+		ps.selection_color(FL_SELECTION_COLOR);
+		set_modal();
+		callback(nop);
+
+		if (l && *l) {
+			fl_font(FL_HELVETICA, FL_NORMAL_SIZE);
+			int s = (int)(fl_width(l) + fl_width('W'));
+			if (s > ps.w()) {
+				ps.size(s, ps.h());
+				size(ps.w() + 10, h());
+			}
+		}
+		position(Fl::event_x_root() - w() / 2, Fl::event_y_root() - h());
+
+		xclass(PACKAGE_TARNAME);
+		show();
+	}
+	void range(float min, float max) { ps.minimum(min); ps.maximum(max); }
+	void value(float val) { ps.value(val); }
+	static void nop(Fl_Widget*, void*) { }
+private:
+	Fl_Progress ps;
+};
+
+void Font_Browser::fontFilter(filter_t filter)
+{
+	if (this->filter == filter)
+		return;
+
+	int s = lst_Font->size();
+
+	static vector<bool> fixed;
+	if (fixed.empty()) {
+		Progress_Window pw(1, s, _("Reading fonts..."));
+		fixed.resize(s);
+		for (int i = 1; i < s; i++) {
+			fixed[i] = fixed_width((Fl_Font)(intptr_t)(lst_Font->data(i)));
+			pw.value(i);
+			Fl::check();
+		}
+	}
+
+	switch (this->filter = filter) {
+	case FIXED_WIDTH:
+		for (int i = 1; i < s; i++) {
+			if (fixed[i])
+				lst_Font->show(i);
+			else
+				lst_Font->hide(i);
+		}
+		break;
+	case VARIABLE_WIDTH:
+		for (int i = 1; i < s; i++) {
+			if (!fixed[i])
+				lst_Font->show(i);
+			else
+				lst_Font->hide(i);
+		}
+		break;
+	case ALL_TYPES:
+		for (int i = 1; i < s; i++)
+			lst_Font->show(i);
+		break;
+	}
+	lst_Font->topline(lst_Font->value());
+}
 
 //////////////////////////////////////////////////////////////////////
 
