@@ -1,24 +1,28 @@
+// ----------------------------------------------------------------------------
 //
 // dominoex.cxx  --  DominoEX modem
 //
-// Copyright (C) 2008
+// Copyright (C) 2008-2009
 //		David Freese (w1hkj@w1hkj.com)
+// Copyright (C) 2006
+//		Hamish Moffatt (hamish@debian.org)
+//
 // based on code in gmfsk
-//	Copyright (C) 2006
-//	Hamish Moffatt (hamish@debian.org)
-// fldigi is free software; you can redistribute it and/or modify
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -200,7 +204,7 @@ dominoex::~dominoex()
 
 dominoex::dominoex(trx_mode md)
 {
-	cap = CAP_REV;
+	cap |= CAP_REV;
 
 	mode = md;
 
@@ -298,7 +302,7 @@ dominoex::dominoex(trx_mode md)
 	Mu_bitstate = 0;
 	Mu_symbolpair[0] = Mu_symbolpair[1] = 0;
 	Mu_datashreg = 1;
-	init();
+//	init();
 }
 
 //=====================================================================
@@ -510,7 +514,7 @@ void dominoex::synchronize()
 void dominoex::eval_s2n()
 {
 	double s = pipe[pipeptr].vector[currsymbol].mag();
-	double n = pipe[(pipeptr + symlen) % twosym].vector[currsymbol].mag();
+	double n = (NUMTONES - 1 ) * pipe[(pipeptr + symlen) % twosym].vector[currsymbol].mag();
 
 	sig = decayavg( sig, s, abs( s - sig) > 4 ? 4 : 32);
 	noise = decayavg( noise, n, 64);
@@ -520,12 +524,17 @@ void dominoex::eval_s2n()
 	else
 		s2n = 0;
 
-	metric = 4 * s2n;
+//	metric = 4 * s2n;
+	// To partially offset the increase of noise by (THORNUMTONES -1)
+	// in the noise calculation above, 
+	// add 15*log10(THORNUMTONES -1) = 18.4, and multiply by 6
+	metric = 6 * (s2n + 18.4);
+
 	metric = metric < 0 ? 0 : metric > 100 ? 100 : metric;
 
 	display_metric(metric);
 
-	snprintf(dommsg, sizeof(dommsg), "s/n %3.0f dB", s2n > 0 ? s2n : 0);
+	snprintf(dommsg, sizeof(dommsg), "s/n %3.0f dB", s2n );
 	put_Status1(dommsg);
 }
 
