@@ -1,10 +1,10 @@
 //
 //	feld.cxx  --  FELDHELL modem
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
 //
-// This modem code owes much to the efforts of Joe Veldhius, N8FQ
+// This modem code owes much to the efforts of Joe Veldhuis, N8FQ
 //
 // This file is part of fldigi.  Adapted from code contained in gmfsk source code
 // distribution.
@@ -13,19 +13,18 @@
 //  Copyright (C) 2004
 //  Lawrence Glaister (ve7it@shaw.ca)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 
@@ -104,7 +103,7 @@ feld::feld(trx_mode m)
 	mode = m;
 	samplerate = FeldSampleRate;
 
-	cap = CAP_BW;
+	cap |= CAP_BW;
 
  	switch (mode) {
 // Amplitude modulation modes
@@ -154,7 +153,7 @@ feld::feld(trx_mode m)
 			hell_bandwidth = 122.5;
 			phi2freq = samplerate / M_PI / (hell_bandwidth / 2.0);
 			filter_bandwidth = progdefaults.HELL_BW_FSK;
-			cap = CAP_REV;
+			cap |= CAP_REV;
 			break;
 		case MODE_FSKH105:
 			feldcolumnrate = 17.5;
@@ -165,7 +164,7 @@ feld::feld(trx_mode m)
 			hell_bandwidth = 55;
 			phi2freq = samplerate / M_PI / (hell_bandwidth / 2.0);
 			filter_bandwidth = progdefaults.HELL_BW_FSK105;
-			cap = CAP_REV;
+			cap |= CAP_REV;
 			break;
 		case MODE_HELL80:
 			feldcolumnrate = 35;
@@ -176,7 +175,7 @@ feld::feld(trx_mode m)
 			hell_bandwidth = 300;
 			phi2freq = samplerate / M_PI / (hell_bandwidth / 2.0);
 			filter_bandwidth = progdefaults.HELL_BW_HELL80;
-			cap = CAP_REV;
+			cap |= CAP_REV;
 			break;
 		default :
 			feldcolumnrate = 17.5;
@@ -195,6 +194,7 @@ feld::feld(trx_mode m)
 	bpfilt = new fftfilt(0, lp, 1024);
 
 	bbfilt = new Cmovavg(8);
+	average = new Cmovavg( static_cast<int>(500 / downsampleinc));
 
 	minmaxfilt = new Cmovavg(120);
 
@@ -232,16 +232,25 @@ void feld::FSKHELL_rx(complex z)
 {
 	double f;
 	double vid;
+	double avg;
 
 	f = (prev % z).arg() * phi2freq;
 	prev = z;
 	f = bbfilt->run(f);
-	rxcounter += downsampleinc;
+	avg = average->run(z.mag());
 
+	rxcounter += downsampleinc;
 	if (rxcounter < 1.0)
 		return;
 	rxcounter -= 1.0;
 
+	if (avg > agc)
+		agc = avg;
+	else
+		agc *= (1.0 - 0.01 / RxColumnLen);
+	metric = CLAMP(1000*agc, 0.0, 100.0);
+	display_metric(metric);
+
 	vid = f + 0.5;
 	vid = CLAMP(vid, 0.0, 1.0);
 	if (mode == MODE_HELL80)
@@ -254,42 +263,44 @@ void feld::FSKHELL_rx(complex z)
 	col_data[col_pointer + RxColumnLen] = (int)(vid * 255.0);
 	col_pointer++;
 	if (col_pointer == RxColumnLen) {
-		REQ(put_rx_data, col_data, col_data.size());
-		if (!halfwidth)
+		if (metric > progStatus.sldrSquelchValue || progStatus.sqlonoff == false) {
 			REQ(put_rx_data, col_data, col_data.size());
+			if (!halfwidth)
+				REQ(put_rx_data, col_data, col_data.size());
+		}
 		col_pointer = 0;
 		for (int i = 0; i < RxColumnLen; i++)
 			col_data[i] = col_data[i + RxColumnLen];
 	}
-
 }
 
 void feld::rx(complex z)
 {
-	double x;
+	double x, avg;
 
-	x = bbfilt->run(z.mag());
+	x = z.mag();
+	if (x > peakval) peakval = x;
+	avg = average->run(x);
 
 	rxcounter += downsampleinc;
 	if (rxcounter < 1.0)
 		return;
-
 	rxcounter -= 1.0;
 
+	x = peakval;
+	peakval = 0;
 	if (x > peakhold)
 		peakhold = x;
 	else
-	peakhold *= (1 - 0.02 / RxColumnLen);
-	if (x < minhold)
-		minhold = x;
+		peakhold *= (1.0 - 0.02 / RxColumnLen);
+	x = x / peakhold;
+	x = CLAMP (x, 0.0, 1.0);
+
+	if (avg > agc)
+		agc = avg;
 	else
-		minhold *= (1 - 0.02 / RxColumnLen);
-
-	x = CLAMP (x / peakhold, 0.0, 1.0);
-
-	agc = minmaxfilt->run(peakhold - minhold);
-
-	metric = CLAMP(200*agc, 0.0, 100.0);
+		agc *= (1.0 - 0.01 / RxColumnLen);
+	metric = CLAMP(1000*agc, 0.0, 100.0);
 	display_metric(metric);
 
 	if (blackboard)
@@ -446,15 +457,21 @@ double feld::nco(double freq)
 void feld::send_symbol(int currsymb, int nextsymb)
 {
 	double tone = get_txfreq_woffset();
-	double Amp;
+	double tone2 = 0;
+	double Amp = 1.0;
+	double ncoval = 0;
 	int outlen = 0;
 
-	Amp = 1.0;
-	if (mode >= MODE_FSKHELL && mode <= MODE_HELL80)
-		tone += (reverse ? -1 : 1) * (currsymb ? -1 : 1) * bandwidth / 2.0;
+	if (mode >= MODE_FSKHELL && mode <= MODE_HELL80) {
+		tone += (reverse ? -1 : 1) * (prevsymb ? -1 : 1) * bandwidth / 2.0;
+		tone2 += (reverse ? -1 : 1) * (currsymb ? -1 : 1) * bandwidth / 2.0;
+	}
 	for (;;) {
 		switch (mode) {
+			ncoval = nco(tone);
 			case MODE_FSKHELL : case MODE_FSKH105 : case MODE_HELL80 :
+				if ((tone2 != tone) && ((1.0 - fabs(ncoval)) < .001)) 
+					tone = tone2;
 				break;
 			case MODE_HELLX5 : case MODE_HELLX9 :
 				Amp = currsymb;
