@@ -1,20 +1,22 @@
-// arq module arq.cxx
+// ----------------------------------------------------------------------------
+// arq.cxx -- arq module
 // Copyright (c) 2007, Dave Freese, W1HKJ
 //
+// This file is part of fldigi.
 //
-// arq.cxx / arq.h is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// arq.cxx is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with your program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -268,7 +270,7 @@ void arq::connectFrame()
 	Payload += ' ';
 	Payload += MyBlockLengthChar;
 
-	snprintf(szGlobals, 23, " T%ldR%ldW%ld", Timeout/1000, Retries, RetryTime/1000);
+	snprintf(szGlobals, 23, " T%dR%dW%d", Timeout/1000, Retries, RetryTime/1000);
 	Payload.append(szGlobals);
 
 	Frame = Header + Payload;
@@ -637,18 +639,18 @@ void arq::parseCONREQ()
 			int n = 0;
 			while (rcvPayload[++p1] != 'R' && n < 6) num[n++] = rcvPayload[p1];
 			num[n] = 0;
-			sscanf(num, "%ld", &Timeout);
+			sscanf(num, "%d", &Timeout);
 			Timeout *= 1000;
 			if (p1 < rcvPayload.length() && rcvPayload[p1] == 'R') {
 				int n = 0;
 				while (rcvPayload[++p1] != 'W' && n < 6) num[n++] = rcvPayload[p1];
 				num[n] = 0;
-				sscanf(num, "%ld", &Retries);
+				sscanf(num, "%d", &Retries);
 				if (p1 < rcvPayload.length() && rcvPayload[p1] == 'W') {
 					int n = 0;
 					while (++p1 < rcvPayload.length() && n < 6) num[n++] = rcvPayload[p1];
 					num[n] = 0;
-					sscanf(num, "%ld", &RetryTime);
+					sscanf(num, "%d", &RetryTime);
 					RetryTime *= 1000;
 					Timeout += Retries * RetryTime;
 				}
@@ -656,11 +658,11 @@ void arq::parseCONREQ()
 /*			
 			char line[80];
 			string NewValues = "Temporary control parameters set to\n";
-			snprintf(line, 79, "  Retries   = %ld\n", Retries);
+			snprintf(line, 79, "  Retries   = %d\n", Retries);
 			NewValues.append(line);
-			snprintf(line, 79, "  Wait time = %ld secs\n", RetryTime / 1000);
+			snprintf(line, 79, "  Wait time = %d secs\n", RetryTime / 1000);
 			NewValues.append(line);
-			snprintf(line, 79, "  Timeout   = %ld secs\n", Timeout / 1000);
+			snprintf(line, 79, "  Timeout   = %d secs\n", Timeout / 1000);
 			NewValues.append(line);
 			printRX(NewValues);
 */
@@ -1234,11 +1236,10 @@ void arq::abort()
 	LinkState = ABORT;
 }
 
-
 void arq::sendBeacon (string txt)
 {
 	string deText = "<<< FLARQ Beacon >>> de ";
-	deText += MyCall;
+	deText.append(MyCall);
 	TxTextQueue.clear();//erase();
 	addToTxQue(deText);
 	beaconFrame(txt);
