@@ -1,3 +1,25 @@
+// ----------------------------------------------------------------------------
+//      FTextView.h
+//
+// Copyright (C) 2007-2009
+//              Stelios Bounanos, M0GLD
+//
+// This file is part of fldigi.
+//
+// fldigi is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
+//
+// fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
+
 #ifndef FTextRXTX_H_
 #define FTextRXTX_H_
 
@@ -14,8 +36,19 @@ public:
 
 	virtual int	handle(int event);
 
+	virtual void	add(unsigned char c, int attr = RECV);
+	virtual	void	add(const char *s, int attr = RECV)
+        {
+                while (*s)
+                        add(*s++, attr);
+        }
+
 	void		set_quick_entry(bool b);
-	bool		get_quick_entry(void) { return quick_entry; }
+	bool		get_quick_entry(void) { return menu[RX_MENU_QUICK_ENTRY].value(); }
+	void		set_scroll_hints(bool b);
+	bool		get_scroll_hints(void) { return menu[RX_MENU_SCROLL_HINTS].value(); }
+	void		mark(FTextBase::TEXT_ATTR attr = CLICK_START);
+	void		clear(void);
 
 	void		setFont(Fl_Font f, int attr = NATTR);
 
@@ -23,9 +56,9 @@ protected:
 	enum {
 		RX_MENU_QRZ_THIS, RX_MENU_CALL, RX_MENU_NAME, RX_MENU_QTH,
 		RX_MENU_STATE, RX_MENU_PROVINCE,RX_MENU_COUNTRY, RX_MENU_LOC,
-		RX_MENU_RST_IN, RX_MENU_SERIAL, RX_MENU_XCHG,
+		RX_MENU_RST_IN, RX_MENU_XCHG, RX_MENU_SERIAL,
 		RX_MENU_DIV, RX_MENU_COPY, RX_MENU_CLEAR, RX_MENU_SELECT_ALL,
-		RX_MENU_SAVE, RX_MENU_WRAP, RX_MENU_QUICK_ENTRY
+		RX_MENU_SAVE, RX_MENU_WRAP, RX_MENU_QUICK_ENTRY, RX_MENU_SCROLL_HINTS
 	};
 
 	void		handle_clickable(int x, int y);
@@ -40,7 +73,6 @@ protected:
 private:
 	FTextRX();
 	FTextRX(const FTextRX &t);
-	bool	quick_entry;
 
 protected:
 	static Fl_Menu_Item menu[];
@@ -63,7 +95,8 @@ public:
 
 	void		clear(void);
 	void		clear_sent(void);
-	int		nextChar(void);
+	int			nextChar(void);
+	bool		eot(void);
 
 	void		setFont(Fl_Font f, int attr = NATTR);
 
