@@ -16,14 +16,11 @@ public:
 };
 
 extern Fl_Double_Window *dlgViewer;
-extern Fl_Button *btnCloseViewer;
-extern Fl_Button *btnClearViewer;
-extern pskBrowser *brwsViewer;
 
 extern Fl_Double_Window* createViewer();
 
 extern void openViewer();
-extern void viewaddchr(int ch, int freq, char c);
+extern void viewaddchr(int ch, int freq, char c, int md);
 extern void initViewer();
 extern void viewclearchannel(int ch);
 extern void viewer_paste_freq(int freq);
