@@ -2,24 +2,23 @@
 //
 //	fl_digi.h
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
 //
 // This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #ifndef FL_DIGI_H
@@ -27,8 +26,6 @@
 
 #include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Browser.H>
-#include <FL/Fl_Value_Slider.H>
-#include <FL/Fl_Input.H>
 
 #include "combo.h"
 #include "Fl_Tile_Check.h"
@@ -41,7 +38,7 @@
 #include "progress.h"
 #include "FreqControl.h"
 #include "flinput2.h"
-
+#include "flslider2.h"
 
 extern Fl_Double_Window *fl_digi_main;
 extern Fl_Double_Window *scopeview;
@@ -50,13 +47,14 @@ extern Fl_Double_Window *scopeview;
 extern FTextRX			*ReceiveText;
 extern FTextTX			*TransmitText;
 extern Raster			*FHdisp;
-extern Fl_Tile_Check		*TiledGroup;
+extern Fl_Tile_Check	*TiledGroup;
 extern Fl_Box			*StatusBar;
 extern Fl_Box			*Status2;
 extern Fl_Box			*Status1;
+extern Fl_Counter2		*cntCW_WPM;
 extern Fl_Box			*WARNstatus;
 extern Fl_Button		*MODEstatus;
-extern Fl_Slider		*sldrSquelch;
+extern Fl_Slider2		*sldrSquelch;
 extern Progress			*pgrsSquelch;
 extern Fl_Button 		*btnMacro[];
 extern Fl_Input2		*inpFreq;
@@ -72,7 +70,6 @@ extern Fl_Input2		*inpCountry;
 extern Fl_Input2		*inpSerNo;
 extern Fl_Input2		*outSerNo;
 extern Fl_Input2		*inpXchgIn;
-extern Fl_Box			*lblDup;
 extern Fl_Input2		*inpVEprov;
 extern Fl_Input2		*inpLoc;
 extern Fl_Input2		*inpNotes;
@@ -81,6 +78,24 @@ extern Fl_Button		*qsoClear;
 extern Fl_Button		*qsoSave;
 extern Fl_Box			*txtRigName;
 
+extern cFreqControl		*qsoFreqDisp1;
+extern cFreqControl 	*qsoFreqDisp2;
+extern cFreqControl		*qsoFreqDisp3;
+extern Fl_Input2		*inpFreq2;
+extern Fl_Input2		*inpTimeOff2;
+extern Fl_Input2		*inpTimeOn2;
+extern Fl_Button		*btnTimeOn2;
+extern Fl_Input2		*inpName2;
+extern Fl_Input2		*inpRstIn2;
+extern Fl_Input2		*inpRstOut2;
+extern Fl_Button		*qsoClear2;
+extern Fl_Button		*qsoSave2;
+
+extern Fl_Input2		*inpCall1;
+extern Fl_Input2		*inpCall2;
+extern Fl_Input2		*inpCall3;
+extern Fl_Input2		*inpCall4;
+
 extern Fl_Group			*qsoFrameView;
 extern Fl_Group			*QsoButtonFrame;
 extern Fl_Group			*QsoInfoFrame;
@@ -94,31 +109,35 @@ extern Fl_Button		*qso_btnSelFreq;
 extern Fl_Button		*qso_btnDelFreq;
 extern Fl_Button		*qso_btnClearList;
 
-
-extern Fl_Value_Slider		*valRcvMixer;
-extern Fl_Value_Slider		*valXmtMixer;
-extern Fl_Button		*btnAFC;
-extern Fl_Button		*btnSQL;
-extern Fl_Light_Button  *btnRSID;
-extern Fl_Light_Button  *btnTune;
+extern Fl_Value_Slider2		*valRcvMixer;
+extern Fl_Value_Slider2		*valXmtMixer;
+extern Fl_Button			*btnAFC;
+extern Fl_Button			*btnSQL;
+extern Fl_Light_Button		*btnRSID;
+extern Fl_Light_Button		*btnTxRSID;
+extern Fl_Light_Button		*btnTune;
 
 extern Fl_Button		*btnAltMacros;
 extern Fl_Button		*btnMacroTimer;
 
-extern bool				useCheckButtons;
-extern int				altMacros;
+extern bool			bWF_only;
+extern bool			withnoise;
+extern int			altMacros;
 
 extern waterfall		*wf;
 extern Digiscope		*digiscope;
 
+extern std::string		main_window_title;
+
 extern void toggleRSID();
 
 extern void set_menus();
-extern void create_fl_digi_main();
+extern void create_fl_digi_main(int argc, char** argv);
 extern void update_main_title();
 extern void activate_rig_menu_item(bool b);
 extern void activate_test_menu_item(bool b);
 extern void activate_mfsk_image_item(bool b);
+extern void WF_UI();
 
 extern void cb_mnuVisitURL(Fl_Widget*, void* arg);
 
@@ -135,7 +154,7 @@ extern void set_video(double *, int, bool = true);
 extern void set_zdata(complex *, int);
 
 extern void set_CWwpm();
-extern void put_rx_char(unsigned int data);
+extern void put_rx_char(unsigned int data, int style = FTextBase::RECV);
 extern void put_sec_char( char chr );
 
 enum status_timeout {
@@ -155,15 +174,29 @@ extern void show_mode(const std::string& mode);
 extern void show_bw(const std::string& sWidth);
 extern void show_spot(bool v);
 extern void showMacroSet();
+extern void setwfrange();
+
+extern void xmtrcv_selection_color();
+extern void rev_selection_color();
+extern void xmtlock_selection_color();
+extern void sql_selection_color();
+extern void afc_selection_color();
+extern void rxid_selection_color();
+extern void txid_selection_color();
+extern void tune_selection_color();
+extern void spot_selection_color();
 
 extern void put_WARNstatus(double);
 
+extern void qsoSave_cb(Fl_Widget *b, void *);
+
 extern void put_rx_data(int *data, int len);
 extern int get_tx_char();
 extern int  get_secondary_char();
-extern void put_echo_char(unsigned int data);
+extern void put_echo_char(unsigned int data, int style = FTextBase::XMIT);
 extern void resetRTTY();
 extern void resetOLIVIA();
+extern void resetCONTESTIA();
 extern void resetTHOR();
 extern void resetDOMEX();
 extern void resetSoundCard();
@@ -178,8 +211,9 @@ extern void setSqlOnOff(bool b);
 extern bool QueryAfcOnOff();
 extern bool QuerySqlOnOff();
 
-extern void init_modem(trx_mode mode);
-extern void init_modem_sync(trx_mode mode);
+extern void init_modem(trx_mode mode, int freq = 0);
+extern void init_modem_sync(trx_mode mode, int freq = 0);
+extern void init_modem_squelch(trx_mode mode, int freq = 0);
 
 extern void start_tx();
 extern void abort_tx();
@@ -189,6 +223,11 @@ extern void colorize_macros();
 
 extern void set_rtty_tab_widgets();
 extern void set_olivia_tab_widgets();
+extern void set_olivia_default_integ();
+extern void set_dominoex_tab_widgets();
+
+extern void set_contestia_tab_widgets();
+extern void set_contestia_default_integ();
 
 extern void startMacroTimer();
 extern void stopMacroTimer();
@@ -200,9 +239,22 @@ const char* ztime(void);
 
 extern void setTabColors();
 
-void qsy(long long rfc, long long fmid = -1LL);
+extern void toggle_visible_modes(Fl_Widget*, void*);
+
+void qsy(long long rfc, int fmid = -1);
 
 void note_qrg(bool no_dup = true, const char* prefix = " ", const char* suffix = " ",
 	      trx_mode mode = NUM_MODES, long long rfc = 0LL, int afreq = 0);
 
+void set_olivia_bw(int bw);
+void set_olivia_tones(int tones);
+
+void set_contestia_bw(int bw);
+void set_contestia_tones(int tones);
+
+void set_rtty_shift(int shift);
+void set_rtty_baud(float baud);
+void set_rtty_bits(int bits);
+
+void sync_cw_parameters();
 #endif
