@@ -1,25 +1,23 @@
 // ----------------------------------------------------------------------------
 // fldigi-config.h  --  user configuration items for fldigi
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2009
 //		Dave Freese, W1HKJ
 //
-// This file is part of fldigi.  Adapted in part from code contained in gmfsk 
-// source code distribution.
+// This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #ifndef FLDIGI_CONFIG_H
@@ -70,16 +68,14 @@ extern int Hwfall;
 extern int HNOM;
 extern int WNOM;
 extern int Wwfall;
+extern int Haqsoframe;
+extern int Hmenu;
+extern int Hstatus;
+extern int Hmacros;
 
-#define Hmenu		22
-#define Hqsoframe	48
-#define Hnotes		24
-#define Hstatus		22
-#define Hmacros		22
-
-#define Htext		(DEFAULT_HNOM - 4 - Hwfall - Hmenu - Hstatus - Hmacros - Hqsoframe - Hnotes)
-#define Hrcvtxt		(Htext) / 2
-#define Hxmttxt		(Htext - (Hrcvtxt))
+//#define Htext		(DEFAULT_HNOM - 4 - Hwfall - Hmenu - Hstatus - Hmacros - Hqsoframe)
+//#define Hrcvtxt		(Htext) / 2
+//#define Hxmttxt		(Htext - (Hrcvtxt))
 
 #define Wmode 		80
 #define Ws2n  		100
