@@ -1,31 +1,26 @@
-//////////////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////////////
-//  Font_Browser.h      v 0.0.1                              2005-10-17 
+// ----------------------------------------------------------------------------
+//  Font_Browser.h      v 0.0.1                        2005-10-17 
 //
 //         for the Fast Light Tool Kit (FLTK) 1.1.x .
 //
 //    David Freese, w1hkj@w1hkj.com
 //    based on similar widget by Mariwan Jalal
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This file is part of fldigi.
 //
-// This library is distributed in the hope that it will be useful,
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-//
-// Please report all bugs and problems to "fltk_kurdi@yahoo.com".
-//
-//////////////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////////////
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 
 #ifndef FONTBROWSER_H
@@ -35,9 +30,9 @@
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Widget.H>
 #include <FL/Fl_Browser.H>
-#include <FL/Fl_Value_Input.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Return_Button.H>
+#include "flslider2.h"
 
 // Preview box for showing font
 class Preview_Box : public Fl_Widget
@@ -56,16 +51,20 @@ public:
 // Font browser widget
 class Font_Browser : public Fl_Window
 {
+public:
+    enum filter_t { FIXED_WIDTH, VARIABLE_WIDTH, ALL_TYPES };
+
 private:
     int		numfonts;
     Fl_Font	fontnbr;
     int		fontsize;
-    Fl_Color fontcolor;
+    Fl_Color	fontcolor;
+    filter_t	filter;
     void	*data_;
 
     Fl_Browser	*lst_Font;
     Fl_Browser	*lst_Size;
-    Fl_Value_Input *txt_Size;
+    Fl_Value_Input2 *txt_Size;
     Fl_Return_Button *btn_OK;
     Fl_Button	*btn_Cancel;
     Fl_Button	*btn_Color;
@@ -93,6 +92,9 @@ public:
     void fontName(const char* n);
 
     static bool fixed_width(Fl_Font f);
+    void fontFilter(filter_t filter);
 };
 
+extern Font_Browser* font_browser;
+
 #endif
