@@ -2,25 +2,26 @@
 // globals.h  --  constants, variables, arrays & functions that need to be
 //                  outside of any thread
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2007
 //		Dave Freese, W1HKJ
+// Copyright (C) 2007-2010
+//		Stelios Bounanos, M0GLD
 //
-// This file is part of fldigi.  Adapted in part from code contained in gmfsk 
+// This file is part of fldigi.  Adapted in part from code contained in gmfsk
 // source code distribution.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #ifndef _GLOBALS_H
@@ -50,6 +51,8 @@ enum {
 
 	MODE_CW,
 
+	MODE_CONTESTIA,
+
 	MODE_DOMINOEX4,
 	MODE_DOMINOEX5,
 	MODE_DOMINOEX8,
@@ -87,16 +90,22 @@ enum {
 	MODE_MT63_FIRST = MODE_MT63_500,
 	MODE_MT63_LAST = MODE_MT63_2000,
 
-	MODE_BPSK31,
-	MODE_QPSK31,
+	MODE_PSK31,
 	MODE_PSK63,
-	MODE_QPSK63,
+	MODE_PSK63F,
 	MODE_PSK125,
-	MODE_QPSK125,
 	MODE_PSK250,
+	MODE_PSK500,
+	MODE_QPSK31,
+	MODE_QPSK63,
+	MODE_QPSK125,
 	MODE_QPSK250,
-	MODE_PSK_FIRST = MODE_BPSK31,
-	MODE_PSK_LAST = MODE_QPSK250,
+	MODE_QPSK500,
+	MODE_PSK125R,
+	MODE_PSK250R,
+	MODE_PSK500R,
+	MODE_PSK_FIRST = MODE_PSK31,
+	MODE_PSK_LAST = MODE_PSK500R,
 
 	MODE_OLIVIA,
 
@@ -121,10 +130,12 @@ enum {
 	MODE_THROB_FIRST = MODE_THROB1,
 	MODE_THROB_LAST = MODE_THROBX4,
 
+	MODE_SSB,
 	MODE_WWV,
 	MODE_ANALYSIS,
 
-	NUM_MODES
+	NUM_MODES,
+	NUM_RXTX_MODES = NUM_MODES - 2
 };
 
 typedef intptr_t trx_mode;
@@ -136,6 +147,7 @@ struct mode_info_t {
 	const char *name;
 	const char *pskmail_name;
 	const char *adif_name;
+	const char *vid_name;
 };
 extern const struct mode_info_t mode_info[NUM_MODES];
 
@@ -164,6 +176,8 @@ public:
 std::ostream& operator<<(std::ostream& s, const qrg_mode_t& m);
 std::istream& operator>>(std::istream& s, qrg_mode_t& m);
 
+#include <bitset>
+class mode_set_t : public std::bitset<NUM_RXTX_MODES> { };
 
 enum band_t {
 	BAND_160M, BAND_80M, BAND_75M, BAND_60M, BAND_40M, BAND_30M, BAND_20M,
