@@ -1,10 +1,8 @@
 #ifndef FL_LOGBOOK_H
 #define FL_LOGBOOK_H
 
-#include "adif_io.h"
 #include "lgbook.h"
 #include "logsupport.h"
-#include "logbook.h"
 
 extern void start_logbook();
 extern void close_logbook();
