@@ -1,5 +1,27 @@
+// ----------------------------------------------------------------------------
+// mfsk.h  --  mfsk modem
 //
-//    mfsk.h  --  MFSK modem
+// Copyright (C) 2006
+//		Dave Freese, W1HKJ
+//
+// This file is part of fldigi.  Adapted from code contained in gmfsk source code 
+// distribution.
+//  gmfsk Copyright (C) 2001, 2002, 2003
+//  Tomi Manninen (oh2bns@sral.fi)
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #ifndef _MFSK_H
 #define _MFSK_H
@@ -226,6 +248,9 @@ protected:
 	void	clearbits();
 	void	sendpic(unsigned char *data, int len);
 	bool	check_picture_header(char c);
+
+	void	s2nreport(void);
+
 public:
 	mfsk (trx_mode md);
 	~mfsk ();
