@@ -60,6 +60,10 @@ protected:
 
 	int scptr;
 
+	// extended s/n reporting
+	double s2n_ncount, s2n_sum, s2n_sum2, s2n_metric;
+	bool s2n_valid;
+
 	unsigned cap;
 public:
 	modem();
@@ -91,8 +95,8 @@ public:
 //	void		set_state(state_t);
 //	void		set_state_wait(state_t);
 //	state_t		get_state();
-	void		set_freq(double);
-	int			get_freq();
+	virtual void	set_freq(double);
+	int		get_freq();
 	void		init_freqlock();
 	void		set_freqlock(bool);
 	void		set_sigsearch(int n) { sigsearch = n; freqerr = 0.0;};
@@ -119,7 +123,9 @@ public:
 
 	unsigned	get_cap(void) { return cap; }
 	enum { CAP_AFC = 1 << 0, CAP_AFC_SR = 1 << 1, CAP_REV = 1 << 2,
-	       CAP_IMG = 1 << 3, CAP_BW = 1 << 4 };
+	       CAP_IMG = 1 << 3, CAP_BW = 1 << 4, CAP_RX = 1 << 5,
+	       CAP_TX = 1 << 6
+	};
 
 // for CW modem use only
 	bool		get_cwTrack();
@@ -132,22 +138,21 @@ public:
 	virtual	void		incWPM() {};
 	virtual void		decWPM() {};
 	virtual void		toggleWPM() {};
+	virtual void		sync_parameters() {};
+	virtual void		update_Status() {};
 
 // for waterfall id transmission
 private:
 
-	static	int			wfid_mask[];
 	static  double		wfid_w[];
-	static  double		wfid_txpulse[];
 	static  double		wfid_outbuf[];
+	int		vidwidth;
 
-	void	wfid_make_pulse();
-	void	wfid_make_tones();
-	void	wfid_send(long int);
-	double	peakval(int symbol, int mask);
-	int 	findmask(int symbol);
+	void	wfid_make_tones(int numchars);
+	void	wfid_send(int numchars);
+//	double	peakval(int symbol, int mask);
+//	int 	findmask(int symbol);
 
-	void	wfid_sendchar(char c);
 	void	wfid_sendchars(std::string s);
 
 	double  PTTnco();
@@ -170,6 +175,14 @@ public:
 	void	cwid_sendtext (const std::string& s);
 	void	cwid();
 
+// for noise tests
+private:
+	void	add_noise(double *, int);
+	double	sigmaN (double es_ovr_n0);
+	double	gauss(double sigma);
+
+protected:
+	virtual void s2nreport(void);
 };
 
 extern modem *cw_modem;
@@ -199,17 +212,25 @@ extern modem *feld_CMTmodem;
 
 extern modem *psk31_modem;
 extern modem *psk63_modem;
+extern modem *psk63f_modem;
 extern modem *psk125_modem;
 extern modem *psk250_modem;
+extern modem *psk500_modem;
 
 extern modem *qpsk31_modem;
 extern modem *qpsk63_modem;
 extern modem *qpsk125_modem;
 extern modem *qpsk250_modem;
+extern modem *qpsk500_modem;
+
+extern modem *psk125r_modem;
+extern modem *psk250r_modem;
+extern modem *psk500r_modem;
 
 extern modem *rtty_modem;
 
 extern modem *olivia_modem;
+extern modem *contestia_modem;
 
 extern modem *thor4_modem;
 extern modem *thor5_modem;
@@ -235,6 +256,7 @@ extern modem *throbx4_modem;
 
 extern modem *wwv_modem;
 extern modem *anal_modem;
+extern modem *ssb_modem;
 
 
 #endif
