@@ -3,11 +3,11 @@
 
 #define	FIRLEN	64
 
-extern double fir1c[FIRLEN];
-extern double fir2c[FIRLEN];
+extern double gmfir1c[FIRLEN];
+extern double gmfir2c[FIRLEN];
 extern double syncfilt[16];
 
 extern void raisedcosfilt(double *);
-extern void wsincfilt(double *, double fc);
+extern void wsincfilt(double *, double fc, bool blackman);
 
 #endif
