@@ -1,7 +1,7 @@
 // ----------------------------------------------------------------------------
 //	qrunner.h
 //
-// Copyright (C) 2007
+// Copyright (C) 2007-2009
 //		Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.
@@ -44,10 +44,7 @@ namespace qrbind {
     using std::tr1::bind;
 };
 #else
-#   include <boost/bind.hpp>
-namespace qrbind {
-    using boost::bind;
-};
+#   error need std::bind or std::tr1::bind
 #endif
 
 #include "threads.h"
