@@ -9,19 +9,18 @@
 //  gmfsk Copyright (C) 2001, 2002, 2003
 //  Tomi Manninen (oh2bns@sral.fi)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #ifndef _RTTY_H
@@ -95,6 +94,10 @@ private:
 	Cmovavg *bitfilt;
 	fftfilt *bpfilt;
 
+	int bflen;
+	double bp_filt_lo;
+	double bp_filt_hi;
+
 	double *pipe;
 	double *dsppipe;
 	int pipeptr;
@@ -114,6 +117,7 @@ private:
 
 	double prevsymbol;
 	complex prevsmpl;
+	complex *samples;
 	
 	complex QI[1024];
 	int QIptr;
@@ -137,7 +141,7 @@ private:
 	unsigned char bitreverse(unsigned char in, int n);
 	int decode_char();
 	int rttyparity(unsigned int);
-	int rx(bool bit);
+	bool rx(bool bit);
 // transmit	
 	double nco(double freq);
 	void send_symbol(int symbol);
