@@ -1,3 +1,25 @@
+// ----------------------------------------------------------------------------
+// threads.h
+//
+// Copyright (C) 2007-2009
+//		Stelios Bounanos, M0GLD
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
+
 #ifndef THREADS_H_
 #define THREADS_H_
 
@@ -15,26 +37,36 @@ int sem_timedwait(sem_t* sem, const struct timespec* abs_timeout);
 int sem_timedwait_rel(sem_t* sem, double rel_timeout);
 int pthread_cond_timedwait_rel(pthread_cond_t* cond, pthread_mutex_t* mutex, double rel_timeout);
 
-// 3 threads use qrunner
-enum { INVALID_TID = -1, TRX_TID, QRZ_TID, RIGCTL_TID, NORIGCTL_TID,
+enum {
+	INVALID_TID = -1,
+	TRX_TID, QRZ_TID, RIGCTL_TID, NORIGCTL_TID,
 #if USE_XMLRPC
-       XMLRPC_TID,
+	XMLRPC_TID,
+#endif
+	ARQ_TID, ARQSOCKET_TID,
+	FLMAIN_TID,
+	NUM_THREADS, NUM_QRUNNER_THREADS = NUM_THREADS - 1
+};
+
+#ifdef __linux__
+void linux_log_tid(void);
+#  define LOG_THREAD_ID() linux_log_tid()
+#else
+#  define LOG_THREAD_ID()  /* nothing */
 #endif
-       ARQ_TID, ARQSOCKET_TID,
-       FLMAIN_TID, NUM_THREADS,
-       NUM_QRUNNER_THREADS = NUM_THREADS - 1 };
 
 #if USE_TLS
-#       define THREAD_ID_TYPE __thread int
-#	define CREATE_THREAD_ID() thread_id_ = INVALID_TID
-#	define SET_THREAD_ID(x)   thread_id_ = (x)
+#       define THREAD_ID_TYPE __thread intptr_t
+#       define CREATE_THREAD_ID() thread_id_ = INVALID_TID
+#	define SET_THREAD_ID(x)   do { thread_id_ = (x); LOG_THREAD_ID(); } while (0)
 #	define GET_THREAD_ID()    thread_id_
 #else
 #       define THREAD_ID_TYPE pthread_key_t
-#	define CREATE_THREAD_ID() pthread_key_create(&thread_id_, 0)
-#	define SET_THREAD_ID(x)   pthread_setspecific(thread_id_, (void *)(x))
-#	define GET_THREAD_ID()    (intptr_t)pthread_getspecific(thread_id_)
+#	define CREATE_THREAD_ID() pthread_key_create(&thread_id_, NULL)
+#	define SET_THREAD_ID(x)   do { pthread_setspecific(thread_id_, (const void *)(x + 1)); LOG_THREAD_ID(); } while (0)
+#	define GET_THREAD_ID()    ((intptr_t)pthread_getspecific(thread_id_) - 1)
 #endif // USE_TLS
+extern THREAD_ID_TYPE thread_id_;
 
 
 #ifndef NDEBUG
@@ -59,11 +91,11 @@ bool thread_in_list(int id, const int* list);
 #  define ENSURE_NOT_THREAD(...) ((void)0)
 #endif // ! NDEBUG
 
-extern THREAD_ID_TYPE thread_id_;
+
 // On POSIX systems we cancel threads by sending them SIGUSR2,
 // which will also interrupt blocking calls.  On woe32 we use
 // pthread_cancel and there is no good/sane way to interrupt.
-#if !defined(__CYGWIN__) && !defined(__MINGW32__)
+#ifndef __WOE32__
 #  define SET_THREAD_CANCEL()					\
 	do {							\
 		sigset_t usr2;					\
@@ -80,7 +112,6 @@ extern THREAD_ID_TYPE thread_id_;
 #  define CANCEL_THREAD(t__) pthread_cancel(t__);
 #endif
 
-
 #include "fl_lock.h"
 
 #endif // !THREADS_H_
