@@ -1,23 +1,23 @@
-/* 
- * Waterfall Spectrum Analyzer Widget
- * Copyright (C) 2006 Dave Freese, W1HKJ
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * Please report all bugs and problems to "w1hkj@w1hkj.com".
- */
+// ----------------------------------------------------------------------------
+// Waterfall Spectrum Analyzer Widget
+// Copyright (C) 2006-2010 Dave Freese, W1HKJ
+// Copyright (C) 2008 Stelios Bounanos, M0GLD
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #ifndef _WF_H
 #define _WF_H
@@ -32,6 +32,8 @@
 
 #include "fft.h"
 #include "fldigi-config.h"
+#include "digiscope.h"
+#include "flslider2.h"
 
 enum {
 	WF_FFT_RECTANGULAR, WF_FFT_BLACKMAN, WF_FFT_HAMMING,
@@ -48,8 +50,12 @@ enum {
 */
 
 #define FFT_LEN		4096
+//8192
+//2048
+//4096
 
 #define SC_SMPLRATE	8000
+#define WFBLOCKSIZE	512
 
 struct RGB {
 	uchar R;
@@ -88,6 +94,7 @@ public:
 	~WFdisp ();
 	int wfmag();
 	int setMag(int m);
+	void setOffset(int v);
 
 	void Mode(WFmode M) {
 		mode = M;
@@ -132,10 +139,12 @@ public:
 
 	WFspeed Speed() { return wfspeed;}
 	void Speed(WFspeed rate) { wfspeed = rate;}
-	
+
 	int Mag() { return mag;}
 	void Mag(int m) { setMag(m);}
-	
+	int Offset() { return offset;}
+	void Offset(int v) { setOffset(v);}
+
 	void initmaps();
 	void draw();
 //	void resize (int, int, int, int);
@@ -147,22 +156,23 @@ public:
 	void movetocenter();
 	void carrier(int cf);
 	int  carrier();
+	inline void makeMarker_(int width, const RGB* color, int freq, const RGB* clrMin, RGB* clrM, const RGB* clrMax);
 	void makeMarker();
 	void process_analog(double *sig, int len);
 	void processFFT();
 	void sig_data( double *sig, int len, int sr );
-	void rfcarrier(long long f) { 
+	void rfcarrier(long long f) {
 		rfc = f;
 	}
-	void USB(bool b) { 
+	void USB(bool b) {
 		usb = b;
 	}
 	bool USB() {return usb;};
 	long long rfcarrier() { return rfc;};
-	
+
 //	void useBands(bool b) { usebands = b;};
-	
-	void updateMarker() { 
+
+	void updateMarker() {
 		drawMarker();};
 	int peakFreq(int f0, int delta);
 	double powerDensity(double f0, double bw);
@@ -171,7 +181,7 @@ public:
 	double dFreq() {return dfreq;}
 	void redrawCursor();
 	void defaultColors();
-	
+
 private:
 	int disp_width;
 	int image_width;
@@ -215,7 +225,7 @@ private:
 	uchar	*fft_sig_img;
 	uchar	*sig_img;
 	uchar	*scline;
-	
+
 	short int	*fft_db;
 	int			ptrFFTbuff;
 	double	 	*circbuff;
@@ -237,7 +247,7 @@ private:
 	void drawgrayWF();
 	void drawspectrum();
 	void drawsignal();
-	
+
 
 protected:
 public:
@@ -255,8 +265,6 @@ public:
 
 class waterfall: public Fl_Group {
 	friend void x1_cb(Fl_Widget *w, void* v);
-	friend void bw_rsid_cb(Fl_Widget *w, void * v);
-	friend void bw_rsid_toggle(waterfall *);
 //	friend void slew_cb(Fl_Widget *w, void * v);
 	friend void slew_left(Fl_Widget *w, void * v);
 	friend void slew_right(Fl_Widget *w, void * v);
@@ -271,11 +279,12 @@ class waterfall: public Fl_Group {
 public:
 	waterfall(int x, int y, int w, int h, char *lbl= 0);
 	~waterfall(){};
+	void show_scope(bool on);
 	void opmode();
 	void sig_data(double *sig, int len, int sr){
 		wfdisp->sig_data(sig, len, sr);
 	}
-	void Overload(bool ovr) { 
+	void Overload(bool ovr) {
 		wfdisp->Overload(ovr);
 	}
 	int carrier() {
@@ -290,6 +299,11 @@ public:
 	bool USB();
 	void Reverse( bool v) { reverse = v;}
 	bool Reverse() { return reverse;}
+
+	void xmtrcv_selection_color(Fl_Color clr) {xmtrcv->selection_color(clr);}
+	void reverse_selection_color(Fl_Color clr) {btnRev->selection_color(clr);}
+	void xmtlock_selection_color(Fl_Color clr) {xmtlock->selection_color(clr);}
+
 	void Bandwidth(int bw)
 	{
 		wfdisp->Bandwidth(bw);
@@ -307,18 +321,20 @@ public:
 	void Speed(int rate);
 	int Mag();
 	void Mag(int m);
+	int Offset();
+	void Offset(int v);
 	int Carrier();
 	void Carrier(int f);
 
 	void movetocenter() { wfdisp->movetocenter();}
 	void redraw_marker() { wfdisp->makeMarker(); }
 	void setPrefilter(int v) {wfdisp->setPrefilter(v);}
-	
+
 	void setcolors() { wfdisp->setcolors(); }
 	void setRefLevel();
 	void setAmpSpan();
 	double dFreq() { return wfdisp->dFreq();}
-	
+
 	void setQSY(bool on) {
 		if (on)
 			qsy->activate();
@@ -329,8 +345,8 @@ public:
 	void setXMLRPC(bool on) {
 //		wfdisp->useBands(!on);
 	}
-	double Pwr(int i) { return wfdisp->Pwr(i); }	
-	
+	double Pwr(int i) { return wfdisp->Pwr(i); }
+
 	int handle(int event);
 
 	enum { WF_NOP, WF_AFC_BW, WF_SIGNAL_SEARCH, WF_SQUELCH,
@@ -339,20 +355,26 @@ public:
 	void handle_mouse_wheel(int what, int d);
 
 	Fl_Button	*btnRev;
-	Fl_Counter	*wfcarrier;
-	Fl_Counter	*wfRefLevel;
-	Fl_Counter	*wfAmpSpan;
+	Fl_Counter2	*wfcarrier;
+	Fl_Counter2	*wfRefLevel;
+	Fl_Counter2	*wfAmpSpan;
 	Fl_Light_Button	*xmtrcv;
 	Fl_Light_Button *xmtlock;
 	Fl_Button	*qsy;
 
+	void UI_select(bool);
+
+	Digiscope	*wfscope;
+
 private:
+	bool		ishidden;
+	int			wf_dim;
 	Fl_Box		*bezel;
 	WFdisp		*wfdisp;
-	Fl_Button	*bw_rsid;
+	Fl_Group	*rs1, *rs2, *hidden;
 	Fl_Button	*mode;
 	Fl_Button	*x1;
-	Fl_Button	*left; 
+	Fl_Button	*left;
 	Fl_Button	*center;
 	Fl_Button	*right;
 	Fl_Button	*wfrate;
