@@ -1,4 +1,24 @@
+// ----------------------------------------------------------------------------
 // textio.cxx
+//
+// Copyright (C) 2006-2010
+//		Dave Freese, W1HKJ
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -40,33 +60,33 @@ static char time[6];
 
 void cTextFile::writeCSVHeader(FILE *txtFile)
 {
-	if (btnSelectQSOdate->value())   fprintf (txtFile, "DATE\t");
-	if (btnSelectTimeON->value())    fprintf (txtFile, "ON\t");
-	if (btnSelectTimeOFF->value())   fprintf (txtFile, "OFF\t");
-	if (btnSelectCall->value())      fprintf (txtFile, "CALL\t");
-	if (btnSelectName->value())      fprintf (txtFile, "NAME\t");
-	if (btnSelectBand->value())      fprintf (txtFile, "BAND\t");
-	if (btnSelectFreq->value())      fprintf (txtFile, "FREQ\t");
-	if (btnSelectMode->value())      fprintf (txtFile, "MODE\t");
-	if (btnSelectTX_pwr->value())    fprintf (txtFile, "TX_PWR\t");
-	if (btnSelectRSTsent->value())   fprintf (txtFile, "RSTSENT\t");
-	if (btnSelectRSTrcvd->value())   fprintf (txtFile, "RSTRCVD\t");
-	if (btnSelectQth->value())       fprintf (txtFile, "QTH\t");
-	if (btnSelectState->value())     fprintf (txtFile, "ST\t");
-	if (btnSelectProvince->value())  fprintf (txtFile, "PR\t");
-	if (btnSelectCountry->value())   fprintf (txtFile, "CNTRY\t");
-	if (btnSelectDXCC->value())      fprintf (txtFile, "DXCC\t");
-	if (btnSelectIOTA->value())      fprintf (txtFile, "IOTA\t");
-	if (btnSelectCONT->value())      fprintf (txtFile, "CONT\t");
-	if (btnSelectITUZ->value())      fprintf (txtFile, "ITUZ\t");
-	if (btnSelectQSLrcvd->value())   fprintf (txtFile, "QSL_RCVD\t");
-	if (btnSelectQSLsent->value())   fprintf (txtFile, "QSL_SENT\t");
-	if (btnSelectNotes->value())     fprintf (txtFile, "NOTES\t");
-	if (btnSelectSerialIN->value())  fprintf (txtFile, "SERIAL RCVD\t");
-	if (btnSelectSerialOUT->value()) fprintf (txtFile, "SERIAL_SENT\t");
-	if (btnSelectXchgIn->value())    fprintf (txtFile, "XCHG1\t");
-	if (btnSelectMyXchg->value())    fprintf (txtFile, "MYXCHG");
-	fprintf (txtFile, szEOL);
+	if (btnSelectQSOdate->value())   fprintf (txtFile, "%s", "\"DATE\"");
+	if (btnSelectTimeON->value())    fprintf (txtFile, "%s", ",\"ON\"");
+	if (btnSelectTimeOFF->value())   fprintf (txtFile, "%s", ",\"OFF\"");
+	if (btnSelectCall->value())      fprintf (txtFile, "%s", ",\"CALL\"");
+	if (btnSelectName->value())      fprintf (txtFile, "%s", ",\"NAME\"");
+	if (btnSelectBand->value())      fprintf (txtFile, "%s", ",\"BAND\"");
+	if (btnSelectFreq->value())      fprintf (txtFile, "%s", ",\"FREQ\"");
+	if (btnSelectMode->value())      fprintf (txtFile, "%s", ",\"MODE\"");
+	if (btnSelectTX_pwr->value())    fprintf (txtFile, "%s", ",\"TX_PWR\"");
+	if (btnSelectRSTsent->value())   fprintf (txtFile, "%s", ",\"RSTSENT\"");
+	if (btnSelectRSTrcvd->value())   fprintf (txtFile, "%s", ",\"RSTRCVD\"");
+	if (btnSelectQth->value())       fprintf (txtFile, "%s", ",\"QTH\"");
+	if (btnSelectState->value())     fprintf (txtFile, "%s", ",\"ST\"");
+	if (btnSelectProvince->value())  fprintf (txtFile, "%s", ",\"PR\"");
+	if (btnSelectCountry->value())   fprintf (txtFile, "%s", ",\"CNTRY\"");
+	if (btnSelectDXCC->value())      fprintf (txtFile, "%s", ",\"DXCC\"");
+	if (btnSelectIOTA->value())      fprintf (txtFile, "%s", ",\"IOTA\"");
+	if (btnSelectCONT->value())      fprintf (txtFile, "%s", ",\"CONT\"");
+	if (btnSelectITUZ->value())      fprintf (txtFile, "%s", ",\"ITUZ\"");
+	if (btnSelectQSLrcvd->value())   fprintf (txtFile, "%s", ",\"QSL_RCVD\"");
+	if (btnSelectQSLsent->value())   fprintf (txtFile, "%s", ",\"QSL_SENT\"");
+	if (btnSelectNotes->value())     fprintf (txtFile, "%s", ",\"NOTES\"");
+	if (btnSelectSerialIN->value())  fprintf (txtFile, "%s", ",\"SERIAL RCVD\"");
+	if (btnSelectSerialOUT->value()) fprintf (txtFile, "%s", ",\"SERIAL_SENT\"");
+	if (btnSelectXchgIn->value())    fprintf (txtFile, "%s", ",\"XCHG1\"");
+	if (btnSelectMyXchg->value())    fprintf (txtFile, "%s", ",\"MYXCHG\"");
+	fprintf (txtFile, "%s", szEOL);
 }
 
 int cTextFile::writeCSVFile (const char *fname, cQsoDb *db) {
@@ -80,62 +100,62 @@ int cTextFile::writeCSVFile (const char *fname, cQsoDb *db) {
 			pRec = db->getRec(i);
 			if (pRec->getField(EXPORT)[0] == 'E') {
 				if (btnSelectQSOdate->value())
-					fprintf (txtFile, "%s\t", pRec->getField(QSO_DATE));
+					fprintf (txtFile, "\"%s\"", pRec->getField(QSO_DATE));
 				if (btnSelectTimeON->value())
-					fprintf (txtFile, "%s\t", pRec->getField(TIME_ON));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(TIME_ON));
 				if (btnSelectTimeOFF->value())
-					fprintf (txtFile, "%s\t", pRec->getField(TIME_OFF));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(TIME_OFF));
 				if (btnSelectCall->value())
-					fprintf (txtFile, "%s\t", pRec->getField(CALL));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(CALL));
 				if (btnSelectName->value())
-					fprintf (txtFile, "%s\t", pRec->getField(NAME));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(NAME));
 				if (btnSelectBand->value())
-					fprintf (txtFile, "%s\t", pRec->getField(BAND));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(BAND));
 				if (btnSelectFreq->value())
-					fprintf (txtFile, "%s\t", pRec->getField(FREQ));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(FREQ));
 				if (btnSelectMode->value())
-					fprintf (txtFile, "%s\t", pRec->getField(MODE));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(MODE));
 				if (btnSelectTX_pwr->value())
-					fprintf (txtFile, "%s\t", pRec->getField(TX_PWR));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(TX_PWR));
 				if (btnSelectRSTsent->value())
-					fprintf (txtFile, "%s\t", pRec->getField(RST_SENT));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(RST_SENT));
 				if (btnSelectRSTrcvd->value())
-					fprintf (txtFile, "%s\t", pRec->getField(RST_RCVD));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(RST_RCVD));
 				if (btnSelectQth->value())
-					fprintf (txtFile, "%s\t", pRec->getField(QTH));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(QTH));
 				if (btnSelectState->value())
-					fprintf (txtFile, "%s\t", pRec->getField(STATE));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(STATE));
 				if (btnSelectProvince->value())
-					fprintf (txtFile, "%s\t", pRec->getField(VE_PROV));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(VE_PROV));
 				if (btnSelectCountry->value())
-					fprintf (txtFile, "%s\t", pRec->getField(COUNTRY));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(COUNTRY));
 				if (btnSelectDXCC->value())
-					fprintf (txtFile, "%s\t", pRec->getField(DXCC));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(DXCC));
 				if (btnSelectIOTA->value())
-					fprintf (txtFile, "%s\t", pRec->getField(IOTA));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(IOTA));
 				if (btnSelectCONT->value())
-					fprintf (txtFile, "%s\t", pRec->getField(CONT));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(CONT));
 				if (btnSelectITUZ->value())
-					fprintf (txtFile, "%s\t", pRec->getField(ITUZ));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(ITUZ));
 				if (btnSelectQSLrcvd->value())
-					fprintf (txtFile, "%s\t", pRec->getField(QSL_RCVD));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(QSL_RCVD));
 				if (btnSelectQSLsent->value())
-					fprintf (txtFile, "%s\t", pRec->getField(QSL_SENT));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(QSL_SENT));
 				if (btnSelectNotes->value()) {
 					string temp = pRec->getField(NOTES);
-    				for (size_t n = 0; n < temp.length(); n++)
-	    				if (temp[n] == '\n') temp[n] = ';';
-		    			fprintf (txtFile, "%s\t", temp.c_str());
+					for (size_t n = 0; n < temp.length(); n++)
+						if (temp[n] == '\r' || temp[n] == '\n') temp[n] = '-';
+					fprintf (txtFile, ",\"%s\"", temp.c_str());
 				}
 				if (btnSelectSerialIN->value())
-					fprintf (txtFile, "%s\t", pRec->getField(SRX));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(SRX));
 				if (btnSelectSerialOUT->value())
-					fprintf (txtFile, "%s\t", pRec->getField(STX));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(STX));
 				if (btnSelectXchgIn->value())
-					fprintf (txtFile, "%s\t", pRec->getField(XCHG1));
+					fprintf (txtFile, ",\"%s\"", pRec->getField(XCHG1));
 				if (btnSelectMyXchg->value())
-					fprintf (txtFile, "%s", pRec->getField(MYXCHG));
-				fprintf (txtFile, szEOL);
+					fprintf (txtFile, ",\"%s\"", pRec->getField(MYXCHG));
+				fprintf (txtFile, "%s", szEOL);
 				pRec->putField(EXPORT,"");
 				db->qsoUpdRec(i, pRec);
 			}
@@ -176,7 +196,7 @@ void cTextFile::writeTXTHeader(FILE *txtFile)
 	if (btnSelectSerialOUT->value()) fprintf (txtFile, "%-7s", "STX");
 	if (btnSelectXchgIn->value())    fprintf (txtFile, "%-15s", "XCHG1");
 	if (btnSelectMyXchg->value())    fprintf (txtFile, "%-15s", "MYXCHG");
-	fprintf (txtFile, szEOL);
+	fprintf (txtFile, "%s", szEOL);
 }
 
 int cTextFile::writeTXTFile (const char *fname, cQsoDb *db) {
@@ -245,7 +265,7 @@ int cTextFile::writeTXTFile (const char *fname, cQsoDb *db) {
 					fprintf (txtFile, "%-15s", pRec->getField(XCHG1));
 				if (btnSelectMyXchg->value())
 					fprintf (txtFile, "%-15s", pRec->getField(MYXCHG));
-				fprintf (txtFile, szEOL);
+				fprintf (txtFile, "%s", szEOL);
 				pRec->putField(EXPORT,"");
 				db->qsoUpdRec(i, pRec);
 			}
