@@ -1,25 +1,26 @@
-//
+// ----------------------------------------------------------------------------
 // Digital Modem Program for the Fast Light Toolkit
 //
-// Copyright W1HKJ, Dave Freese 2006
+// Copyright 2006-2010, Dave Freese, W1HKJ
+// Copyright 2007-2010, Stelios Bounanos, M0GLD
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This file is part of fldigi.
 //
-// This library is distributed in the hope that it will be useful,
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-//
-// Please report all bugs and problems to "w1hkj@w1hkj.com".
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 //
+// Please report all bugs and problems to fldigi-devel@lists.berlios.de.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -106,6 +107,9 @@
 	#include "benchmark.h"
 #endif
 
+#include "icons.h"
+
+
 using namespace std;
 
 string appname;
@@ -124,6 +128,18 @@ string WrapDir;
 string TalkDir;
 string TempDir;
 string PskMailDir;
+string NBEMS_dir;
+string ARQ_dir;
+string ARQ_files_dir;
+string ARQ_recv_dir;
+string ARQ_send;
+string WRAP_dir;
+string WRAP_recv_dir;
+string WRAP_send_dir;
+string WRAP_auto_dir;
+string ICS_dir;
+string ICS_msg_dir;
+string ICS_tmp_dir;
 
 string PskMailFile;
 string ArqFilename;
@@ -140,6 +156,8 @@ cLogfile	*Maillogfile = (cLogfile *)0;
 FILE	*server;
 FILE	*client;
 bool	mailserver = false, mailclient = false, arqmode = false;
+static bool show_cpucheck = false;
+static bool iconified = false;
 
 RXMSGSTRUC rxmsgst;
 int		rxmsgid = -1;
@@ -161,6 +179,16 @@ double speed_test(int converter, unsigned repeat);
 static void setup_signal_handlers(void);
 static void checkdirectories(void);
 
+static void arg_error(const char* name, const char* arg, bool missing) noreturn__;
+
+// TODO: find out why fldigi crashes on OS X if the wizard window is
+// shown before fldigi_main.
+#ifndef __APPLE__
+#  define SHOW_WIZARD_BEFORE_MAIN_WINDOW 1
+#else
+#  define SHOW_WIZARD_BEFORE_MAIN_WINDOW 0
+#endif
+
 int main(int argc, char ** argv)
 {
 	appname = argv[0];
@@ -185,12 +213,19 @@ int main(int argc, char ** argv)
 		char dirbuf[FL_PATH_MAX + 1];
 #ifdef __WOE32__
 		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$USERPROFILE/fldigi.files/");
+		HomeDir = dirbuf;
+		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$USERPROFILE/NBEMS.files/");
+		NBEMS_dir = dirbuf;
+		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$USERPROFILE/");
+		PskMailDir = dirbuf;
 #else
 		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$HOME/.fldigi/");
-#endif
 		HomeDir = dirbuf;
+		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$HOME/.nbems/");
+		NBEMS_dir = dirbuf;
 		fl_filename_expand(dirbuf, sizeof(dirbuf) - 1, "$HOME/");
 		PskMailDir = dirbuf;
+#endif
 	}
 
 	set_platform_ui();
@@ -198,15 +233,14 @@ int main(int argc, char ** argv)
 	generate_option_help();
 	generate_version_text();
 	int arg_idx;
-	if (Fl::args(argc, argv, arg_idx, parse_args) != argc) {
-	    cerr << PACKAGE_NAME << ": bad option `" << argv[arg_idx]
-		 << "'\nTry `" << PACKAGE_NAME
-		 << " --help' for more information.\n";
-	    exit(EXIT_FAILURE);
-	}
+	if (Fl::args(argc, argv, arg_idx, parse_args) != argc)
+		arg_error(argv[0], NULL, false);
+	if (main_window_title.empty())
+		main_window_title = PACKAGE_TARNAME;
 
 	checkdirectories();
-	bool have_config = progdefaults.readDefaultsXML();
+	check_nbems_dirs();
+
 	try {
 		debug::start(string(HomeDir).append("status_log.txt").c_str());
 		time_t t = time(NULL);
@@ -218,7 +252,9 @@ int main(int argc, char ** argv)
 		debug::stop();
 	}
 
-	xmlfname = HomeDir; 
+	bool have_config = progdefaults.readDefaultsXML();
+
+	xmlfname = HomeDir;
 	xmlfname.append("rig.xml");
 
 #if !defined(__WOE32__) && !defined(__APPLE__)
@@ -232,13 +268,22 @@ int main(int argc, char ** argv)
 
 	Fl::lock();  // start the gui thread!!
 	Fl::visual(FL_RGB); // insure 24 bit color operation
-	
+
 	fl_register_images();
 	Fl::set_fonts(0);
-	
-	rigcontrol = createRigDialog();
 
-	if (!have_config) {
+	Fl::scheme(progdefaults.ui_scheme.c_str());
+	progdefaults.initFonts();
+
+	dxcc_open(string(HomeDir).append("cty.dat").c_str());
+	qsl_open(string(HomeDir).append("lotw1.txt").c_str(), QSL_LOTW);
+	if (!qsl_open(string(HomeDir).append("eqsl.txt").c_str(), QSL_EQSL))
+		qsl_open(string(HomeDir).append("AGMemberList.txt").c_str(), QSL_EQSL);
+
+	progStatus.loadLastState();
+	create_fl_digi_main(argc, argv);
+
+	if (!have_config || show_cpucheck) {
 		double speed = speed_test(SRC_SINC_FASTEST, 8);
 
 		if (speed > 150.0) {      // fast
@@ -249,7 +294,7 @@ int main(int argc, char ** argv)
 			progdefaults.slowcpu = false;
 			progdefaults.sample_converter = SRC_SINC_MEDIUM_QUALITY;
 		}
-		else if (speed > 20.0) { // slow
+		else if (speed > 15.0) { // slow
 			progdefaults.slowcpu = true;
 			progdefaults.sample_converter = SRC_SINC_FASTEST;
 		}
@@ -258,14 +303,13 @@ int main(int argc, char ** argv)
 			progdefaults.sample_converter = SRC_LINEAR;
 		}
 
-		LOG_INFO("speed factor=%f, slowcpu=%d, sample_converter=\"%s\"", speed,
-			 progdefaults.slowcpu, src_get_name(progdefaults.sample_converter));
+		LOG_INFO("CPU speed factor=%f: setting slowcpu=%s, sample_converter=\"%s\"", speed,
+			 progdefaults.slowcpu ? "true" : "false",
+			 src_get_name(progdefaults.sample_converter));
 	}
 	if (progdefaults.XmlRigFilename.empty())
 		progdefaults.XmlRigFilename = xmlfname;
 
-	progStatus.loadLastState();
-
 	if (progStatus.LOGenabled == true) {
     	Date tdy;
 	    string lfname = HomeDir;
@@ -276,22 +320,12 @@ int main(int argc, char ** argv)
 	    logfile->log_to_file_start();
 	}
 
-	dxcc_open(string(HomeDir).append("cty.dat").c_str());
-	qsl_open(string(HomeDir).append("lotw1.txt").c_str(), QSL_LOTW);
-	if (!qsl_open(string(HomeDir).append("eqsl.txt").c_str(), QSL_EQSL))
-		qsl_open(string(HomeDir).append("AGMemberList.txt").c_str(), QSL_EQSL);
-
-	Fl::scheme(progdefaults.ui_scheme.c_str());
-	progdefaults.initFonts();
-	create_fl_digi_main();
-
 #if BENCHMARK_MODE
 	return setup_benchmark();
 #endif
 
 	FSEL::create();
 
-	createConfig();
 	make_colorsfonts();
 	setTabColors();
 
@@ -304,16 +338,28 @@ int main(int argc, char ** argv)
 #if USE_HAMLIB
 	xcvr = new Rig();
 #endif
-	
+
 	push2talk = new PTT();
 
 	progdefaults.setDefaults();
 
 	atexit(sound_close);
 	sound_init();
-	trx_start();
 
 	progdefaults.initInterface();
+	trx_start();
+
+#if SHOW_WIZARD_BEFORE_MAIN_WINDOW
+	if (!have_config) {
+		show_wizard(argc, argv);
+		Fl_Window* w;
+		while ((w = Fl::first_window()) && w->visible())
+			Fl::wait();
+	}
+#endif
+
+//	progdefaults.initInterface();
+//	trx_start();
 
 // OS X will prevent the main window from being resized if we change its
 // size *after* it has been shown. With some X11 window managers, OTOH,
@@ -327,7 +373,17 @@ int main(int argc, char ** argv)
 	fl_digi_main->show(argc, argv);
 #endif
 
+	if (iconified)
+		for (Fl_Window* w = Fl::first_window(); w; w = Fl::next_window(w))
+			w->iconize();
+	update_main_title();
+
 	arq_init();
+#ifdef __WIN32__
+	open_talker();
+#else
+	grpTalker->hide();
+#endif
 
 #if USE_XMLRPC
 	XML_RPC_Server::start(progdefaults.xmlrpc_address.c_str(), progdefaults.xmlrpc_port.c_str());
@@ -338,6 +394,12 @@ int main(int argc, char ** argv)
 			LOG_ERROR("Could not start PSK reporter: %s", pskrep_error());
 
 	notify_start();
+	mode_browser = new Mode_Browser;
+
+#if !SHOW_WIZARD_BEFORE_MAIN_WINDOW
+	if (!have_config)
+		show_wizard();
+#endif
 
 	int ret = Fl::run();
 
@@ -446,6 +508,16 @@ void generate_option_help(void) {
 	     << "    Default: " << benchmark.src_type << " (" << src_get_name(benchmark.src_type) << ")\n\n"
 #endif
 
+	     << "  --cpu-speed-test\n"
+	     << "    Perform the CPU speed test, show results in the event log\n"
+	     << "    and possibly change options.\n\n"
+
+	     << "  --noise\n"
+	     << "    Unhide controls for noise tests\n\n"
+
+	     << "  --wfall-only\n"
+	     << "    Hide all controls but the waterfall\n\n"
+
 	     << "  --debug-level LEVEL\n"
 	     << "    Set the event log verbosity\n\n"
 
@@ -512,8 +584,16 @@ void exit_cb(void*) { fl_digi_main->do_callback(); }
 int parse_args(int argc, char **argv, int& idx)
 {
 	// Only handle long options
-	if ( !(strlen(argv[idx]) >= 2 && strncmp(argv[idx], "--", 2) == 0) )
+	if (!(strlen(argv[idx]) >= 2 && strncmp(argv[idx], "--", 2) == 0)) {
+		// Store the window title. We may need this early in the initialisation
+		// process, before FLTK uses it to set the main window title.
+		if (main_window_title.empty() && argc > idx &&
+		    (!strcmp(argv[idx], "-ti") || !strcmp(argv[idx], "-title")))
+			main_window_title = argv[idx + 1];
+		else if (!strcmp(argv[idx], "-i") || !strcmp(argv[idx], "-iconic"))
+			iconified = true;
 		return 0;
+	}
 
         enum { OPT_ZERO,
 #ifndef __WOE32__
@@ -521,6 +601,7 @@ int parse_args(int argc, char **argv, int& idx)
 #endif
 	       OPT_CONFIG_DIR,
 	       OPT_ARQ_ADDRESS, OPT_ARQ_PORT,
+	       OPT_SHOW_CPU_CHECK,
 #if USE_XMLRPC
 	       OPT_CONFIG_XMLRPC_ADDRESS, OPT_CONFIG_XMLRPC_PORT,
 	       OPT_CONFIG_XMLRPC_ALLOW, OPT_CONFIG_XMLRPC_DENY, OPT_CONFIG_XMLRPC_LIST,
@@ -532,30 +613,29 @@ int parse_args(int argc, char **argv, int& idx)
 	       OPT_BENCHMARK_SRC_RATIO, OPT_BENCHMARK_SRC_TYPE,
 #endif
 
-               OPT_FONT, OPT_WFALL_HEIGHT, OPT_WFALL_WIDTH, 
-               OPT_WINDOW_WIDTH, OPT_WINDOW_HEIGHT, 
-//               OPT_TOGGLE_CHECK,
+               OPT_FONT, OPT_WFALL_HEIGHT,
+               OPT_WINDOW_WIDTH, OPT_WINDOW_HEIGHT, OPT_WFALL_ONLY,
 #if USE_PORTAUDIO
                OPT_FRAMES_PER_BUFFER,
 #endif
-	       OPT_DEBUG_LEVEL,
+	       OPT_NOISE, OPT_DEBUG_LEVEL,
                OPT_EXIT_AFTER,
                OPT_DEPRECATED, OPT_HELP, OPT_VERSION, OPT_BUILD_INFO };
 
-	const char shortopts[] = "+";
+	const char shortopts[] = ":";
 	static struct option longopts[] = {
 #ifndef __WOE32__
 		{ "rx-ipc-key",	   1, 0, OPT_RX_IPC_KEY },
 		{ "tx-ipc-key",	   1, 0, OPT_TX_IPC_KEY },
 #endif
 		{ "config-dir",	   1, 0, OPT_CONFIG_DIR },
-		{ "experimental",  0, 0, OPT_DEPRECATED },
 
 		{ "arq-server-address", 1, 0, OPT_ARQ_ADDRESS },
 		{ "arq-server-port",    1, 0, OPT_ARQ_PORT },
 
+		{ "cpu-speed-test", 0, 0, OPT_SHOW_CPU_CHECK },
+
 #if USE_XMLRPC
-		{ "xmlrpc-server",         0, 0, OPT_DEPRECATED },
 		{ "xmlrpc-server-address", 1, 0, OPT_CONFIG_XMLRPC_ADDRESS },
 		{ "xmlrpc-server-port",    1, 0, OPT_CONFIG_XMLRPC_PORT },
 		{ "xmlrpc-allow",          1, 0, OPT_CONFIG_XMLRPC_ALLOW },
@@ -577,17 +657,18 @@ int parse_args(int argc, char **argv, int& idx)
 
 		{ "font",	   1, 0, OPT_FONT },
 
-		{ "wfall-width",   1, 0, OPT_WFALL_WIDTH },
 		{ "wfall-height",  1, 0, OPT_WFALL_HEIGHT },
 		{ "window-width",  1, 0, OPT_WINDOW_WIDTH },
 		{ "window-height", 1, 0, OPT_WINDOW_HEIGHT },
-		{ "twoscopes",     0, 0, OPT_DEPRECATED },
+		{ "wfall-only",    0, 0, OPT_WFALL_ONLY },
+		{ "wo",            0, 0, OPT_WFALL_ONLY },
 
 #if USE_PORTAUDIO
-		{ "frames-per-buf",1, 0, OPT_FRAMES_PER_BUFFER },
+		{ "frames-per-buffer",1, 0, OPT_FRAMES_PER_BUFFER },
 #endif
 		{ "exit-after",    1, 0, OPT_EXIT_AFTER },
 
+		{ "noise", 0, 0, OPT_NOISE },
 		{ "debug-level",   1, 0, OPT_DEBUG_LEVEL },
 
 		{ "help",	   0, 0, OPT_HELP },
@@ -598,6 +679,7 @@ int parse_args(int argc, char **argv, int& idx)
 
 	int longindex;
 	optind = idx;
+	opterr = 0;
 	int c = getopt_long(argc, argv, shortopts, longopts, &longindex);
 
 	switch (c) {
@@ -719,10 +801,6 @@ int parse_args(int argc, char **argv, int& idx)
 			Fl::set_font(FL_HELVETICA, optarg);
 			break;
 
-		case OPT_WFALL_WIDTH:
-			progdefaults.wfwidth = strtol(optarg, NULL, 10);
-			break;
-
 		case OPT_WFALL_HEIGHT:
 			progdefaults.wfheight = strtol(optarg, NULL, 10);
 			break;
@@ -745,6 +823,18 @@ int parse_args(int argc, char **argv, int& idx)
 			Fl::add_timeout(strtod(optarg, 0), exit_cb);
 			break;
 
+		case OPT_WFALL_ONLY:
+			bWF_only = true;
+			break;
+
+		case OPT_NOISE:
+			withnoise = true;
+			break;
+
+		case OPT_SHOW_CPU_CHECK:
+			show_cpucheck = true;
+			break;
+
 		case OPT_DEBUG_LEVEL:
 		{
 			int v = strtol(optarg, 0, 10);
@@ -769,9 +859,8 @@ int parse_args(int argc, char **argv, int& idx)
 			cout << build_text;
 			exit(EXIT_SUCCESS);
 
-		case '?': default:
-			cerr << "Try `" << PACKAGE_NAME << " --help' for more information.\n";
-			exit(EXIT_FAILURE);
+		case '?': case ':': default:
+			arg_error(argv[0], argv[idx], (c == ':'));
 	}
 
 	// Increment idx by the number of args we used and return that number.
@@ -790,9 +879,9 @@ int parse_args(int argc, char **argv, int& idx)
 
 void generate_version_text(void)
 {
-	version_text.assign(PACKAGE_STRING "\nCopyright (C) 2008, 2009 " PACKAGE_AUTHORS ".\n");
-	version_text.append(_("License GPLv2+: GNU GPL version 2 or later "
-			      "<http://www.gnu.org/licenses/gpl-2.0.html>\n"
+	version_text.assign(PACKAGE_STRING "\nCopyright (C) 2007-2010 " PACKAGE_AUTHORS ".\n");
+	version_text.append(_("License GPLv3+: GNU GPL version 3 or later "
+			      "<http://www.gnu.org/licenses/gpl-3.0.html>\n"
 			      "This is free software: you are free to change and redistribute it.\n"
 			      "There is NO WARRANTY, to the extent permitted by law.\n"));
 
@@ -903,6 +992,7 @@ void set_platform_ui(void)
 // Pentium 4 2.8GHz:     70
 // Pentium 3 550MHz:     13
 // UltraSparc II 270MHz: 3.5
+// Atom N280 1.66GHz:    17.7
 #define IN_RATE 48000
 #define OUT_RATE 8000
 double speed_test(int converter, unsigned repeat)
@@ -973,11 +1063,12 @@ static void setup_signal_handlers(void)
 
 static void checkdirectories(void)
 {
-	struct {
+	struct DIRS {
 		string& dir;
 		const char* suffix;
 		void (*new_dir_func)(void);
-	} dirs[] = {
+	};
+	DIRS fldigi_dirs[] = {
 		{ HomeDir, 0, 0 },
 		{ RigsDir, "rigs", 0 },
 		{ ScriptsDir, "scripts", 0 },
@@ -992,16 +1083,96 @@ static void checkdirectories(void)
 	};
 
 	int r;
-	for (size_t i = 0; i < sizeof(dirs)/sizeof(*dirs); i++) {
-		if (dirs[i].suffix)
-			dirs[i].dir.assign(HomeDir).append(dirs[i].suffix).append(PATH_SEP);
+	for (size_t i = 0; i < sizeof(fldigi_dirs)/sizeof(*fldigi_dirs); i++) {
+		if (fldigi_dirs[i].suffix)
+			fldigi_dirs[i].dir.assign(HomeDir).append(fldigi_dirs[i].suffix).append(PATH_SEP);
 
-		if ((r = mkdir(dirs[i].dir.c_str(), 0777)) == -1 && errno != EEXIST) {
-			cerr << _("Could not make directory") << ' ' << dirs[i].dir
+		if ((r = mkdir(fldigi_dirs[i].dir.c_str(), 0777)) == -1 && errno != EEXIST) {
+			cerr << _("Could not make directory") << ' ' << fldigi_dirs[i].dir
 			     << ": " << strerror(errno) << '\n';
 			exit(EXIT_FAILURE);
 		}
-		else if (r == 0 && dirs[i].new_dir_func)
-			dirs[i].new_dir_func();
+		else if (r == 0 && fldigi_dirs[i].new_dir_func)
+			fldigi_dirs[i].new_dir_func();
 	}
+
+}
+
+bool nbems_dirs_checked = false;
+
+void check_nbems_dirs(void)
+{
+	if (nbems_dirs_checked) return;
+
+	struct DIRS {
+		string& dir;
+		const char* suffix;
+		void (*new_dir_func)(void);
+	};
+	DIRS NBEMS_dirs[] = {
+		{ NBEMS_dir,     0, 0 },
+		{ ARQ_dir,       "ARQ", 0 },
+		{ ARQ_files_dir, "ARQ/files", 0 },
+		{ ARQ_recv_dir,  "ARQ/recv", 0 },
+		{ ARQ_send,      "ARQ/send", 0 },
+		{ WRAP_dir,      "WRAP", 0 },
+		{ WRAP_recv_dir, "WRAP/recv", 0 },
+		{ WRAP_send_dir, "WRAP/send", 0 },
+		{ WRAP_auto_dir, "WRAP/auto", 0 },
+		{ ICS_dir,       "ICS", 0 },
+		{ ICS_msg_dir,   "ICS/messages", 0 },
+		{ ICS_tmp_dir,   "ICS/templates", 0 },
+	};
+
+	int r;
+	for (size_t i = 0; i < sizeof(NBEMS_dirs)/sizeof(*NBEMS_dirs); i++) {
+		if (NBEMS_dirs[i].suffix)
+			NBEMS_dirs[i].dir.assign(NBEMS_dir).append(NBEMS_dirs[i].suffix).append(PATH_SEP);
+
+		if ((r = mkdir(NBEMS_dirs[i].dir.c_str(), 0777)) == -1 && errno != EEXIST) {
+			cerr << _("Could not make directory") << ' ' << NBEMS_dirs[i].dir
+			     << ": " << strerror(errno) << '\n';
+			exit(EXIT_FAILURE);
+		}
+		else if (r == 0 && NBEMS_dirs[i].new_dir_func)
+			NBEMS_dirs[i].new_dir_func();
+	}
+	nbems_dirs_checked = true;
+}
+
+// Print an error message and exit. If stderr is not a terminal
+// show an error dialog instead. On win32 we always use Fl::fatal,
+// which displays its own error window.
+static void arg_error(const char* name, const char* arg, bool missing)
+{
+	ostringstream msg;
+	msg << name << ": ";
+	if (arg && *arg) {
+		if (missing)
+			msg << "option '" << arg << "' requires an argument\n";
+		else
+			msg << "unrecognized option '" << arg << "'\n";
+	}
+	else
+		msg << "error while parsing command line\n";
+
+#ifdef __WOE32__
+	bool woe32 = true;
+#else
+	bool woe32 = false;
+#endif
+	bool tty = isatty(STDERR_FILENO);
+
+	if (tty && !woe32)
+		msg << "Try `" << name << " --help' for more information.";
+	else
+		msg << "See command line help for more information.";
+	if (tty || woe32)
+		Fl::fatal("%s", msg.str().c_str());
+	else {
+		fl_message_font(FL_HELVETICA, FL_NORMAL_SIZE);
+		fl_alert2("%s", msg.str().c_str());
+	}
+
+	exit(EXIT_FAILURE);
 }
