@@ -1,7 +1,7 @@
 // ----------------------------------------------------------------------------
 // mfsk.cxx  --  mfsk modem
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2009
 //		Dave Freese, W1HKJ
 //
 // This file is part of fldigi.  Adapted from code contained in gmfsk source code 
@@ -9,19 +9,18 @@
 //  gmfsk Copyright (C) 2001, 2002, 2003
 //  Tomi Manninen (oh2bns@sral.fi)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -84,8 +83,11 @@ void  mfsk::rx_init()
 	put_MODEstatus(mode);
 	syncfilter->reset();
 	staticburst = false;
+
+	s2n_valid = false;
 }
 
+
 void mfsk::init()
 {
 	modem::init();
@@ -127,7 +129,7 @@ mfsk::~mfsk()
 
 mfsk::mfsk(trx_mode mfsk_mode) : modem()
 {
-	cap = CAP_AFC | CAP_REV;
+	cap |= CAP_AFC | CAP_REV;
 
 	double bw, cf, flo, fhi;
 	mode = mfsk_mode;
@@ -267,7 +269,7 @@ mfsk::mfsk(trx_mode mfsk_mode) : modem()
 	afcmetric = 0.0;
 	datashreg = 1;
 
-	init();
+//	init();
 
 }
 
@@ -277,6 +279,12 @@ mfsk::mfsk(trx_mode mfsk_mode) : modem()
 //=====================================================================
 
 
+void mfsk::s2nreport(void)
+{
+	modem::s2nreport();
+	s2n_valid = false;
+}
+
 bool mfsk::check_picture_header(char c)
 {
 	char *p;
@@ -400,7 +408,20 @@ void mfsk::recvchar(int c)
 		picheader[PICHEADER -1] = 0;		
 	}
 	put_rx_char(c);
-	
+	if (progdefaults.Pskmails2nreport && (mailserver || mailclient)) {
+		if ((c == SOH) && !s2n_valid) {
+			// starts collecting s2n from first SOH in stream (since start of RX)
+			s2n_valid = true;
+			s2n_sum = s2n_sum2 = s2n_ncount = 0.0;
+		}
+		if (s2n_valid) {
+			s2n_sum += s2n_metric;
+			s2n_sum2 += (s2n_metric * s2n_metric);
+			s2n_ncount++;
+			if (c == EOT && (mailserver || mailclient))
+				s2nreport();
+		}
+	}
 }
 
 void mfsk::recvbit(int bit)
@@ -433,23 +454,29 @@ void mfsk::decodesymbol(unsigned char symbol)
 			met1 = decayavg(met1, met, 32);
 			if (met1 < met2)
 				return;
-			metric = met1 / 2.0;
+			metric = met1 / 1.5;
 		} else {
 			if ((c = dec2->decode(symbolpair, &met)) == -1)
 				return;
 			met2 = decayavg(met2, met, 32);
 			if (met2 < met1)
 				return;
-			metric = met2 / 2.0;
+			metric = met2 / 1.5;
 		}
 	} else {
 		if (symcounter) return;
 		if ((c = dec2->decode(symbolpair, &met)) == -1)
 			return;
 		met2 = decayavg(met2, met, 32);
-		metric = met2 / 2.0;
+		metric = met2 / 1.5;
 	}
-	
+
+	if (progdefaults.Pskmails2nreport && (mailserver || mailclient)) {
+		// s2n reporting: re-calibrate
+		s2n_metric = metric * 3 - 42;
+		s2n_metric = CLAMP(s2n_metric, 0.0, 100.0);
+	}
+
 	display_metric(metric);
 	
 	if (progStatus.sqlonoff && metric < progStatus.sldrSquelchValue)
@@ -650,13 +677,10 @@ void mfsk::afc()
 void mfsk::eval_s2n()
 {
 	sig = pipe[pipeptr].vector[currsymbol].mag();
-	noise = 0.0;
-	for (int i = 0; i < numtones; i++) {
-		if (i != currsymbol)
-			noise += pipe[pipeptr].vector[i].mag();
-	}	
+	noise = (numtones -1) * pipe[pipeptr].vector[prev2symbol].mag();
 	if (noise > 0)
 		s2n = decayavg ( s2n, sig / noise, 64 );
+
 }
 
 int mfsk::rx_process(const double *buf, int len)
