@@ -1,27 +1,30 @@
-// ============================================================================
+// ----------------------------------------------------------------------------
 // arq_io.cxx
 //
 // support for ARQ server/client system such as pskmail and fl_arq
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
+// Copyright (C) 2008-2010
+//		Stelios Bounanos, M0GLD
+// Copyright (C) 2009-2010
+//		John Douyere, VK2ETA
 //
 // This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-// ============================================================================
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 
 #include <config.h>
@@ -58,8 +61,58 @@
 #include <FL/Fl.H>
 #include <FL/fl_ask.H>
 
+LOG_FILE_SOURCE(debug::LOG_ARQCONTROL);
+
 using namespace std;
 
+//======================================================================
+// test code for pskmail eol issues
+
+const char *asc[256] = {
+	"<NUL>", "<SOH>", "<STX>", "<ETX>",
+	"<EOT>", "<ENQ>", "<ACK>", "<BEL>",
+	"<BS>",  "<TAB>", "<LF>",  "<VT>", 
+	"<FF>",  "<CR>",  "<SO>",  "<SI>",
+	"<DLE>", "<DC1>", "<DC2>", "<DC3>",
+	"<DC4>", "<NAK>", "<SYN>", "<ETB>",
+	"<CAN>", "<EM>",  "<SUB>", "<ESC>",
+	"<FS>",  "<GS>",  "<RS>",  "<US>",
+	" ",     "!",     "\"",    "#",    
+	"$",     "%",     "&",     "\'",   
+	"(",     ")",     "*",     "+",    
+	",",     "-",     ".",     "/",   
+	"0",     "1",     "2",     "3",    
+	"4",     "5",     "6",     "7",   
+	"8",     "9",     ":",     ";",    
+	"<",     "=",     ">",     "?",   
+	"@",     "A",     "B",     "C",    
+	"D",     "E",     "F",     "G",   
+	"H",     "I",     "J",     "K",    
+	"L",     "M",     "N",     "O",   
+	"P",     "Q",     "R",     "S",    
+	"T",     "U",     "V",     "W",   
+	"X",     "Y",     "Z",     "[",    
+	"\\",    "]",     "^",     "_",   
+	"`",     "a",     "b",     "c",    
+	"d",     "e",     "f",     "g",   
+	"h",     "i",     "j",     "k",    
+	"l",     "m",     "n",     "o",   
+	"p",     "q",     "r",     "s",    
+	"t",     "u",     "v",     "w",   
+	"x",     "y",     "z",     "{",    
+	"|",     "}",     "~",     "<DEL>"
+};
+
+string noctrl(string src)
+{
+	static string retstr;
+	retstr.clear();
+	for (size_t i = 0; i < src.length(); i++) retstr.append(asc[(int)src[i]]);
+	return retstr;
+}
+
+//======================================================================
+
 static string arqtext;
 //string::iterator pText;
 size_t pText;
@@ -82,36 +135,63 @@ void ParseMode(string src)
 		int msecs = 100;
 		if (src.length() > 7)
 			sscanf( src.substr(7, src.length() - 7).c_str(), "%d", &msecs);
-		REQ_SYNC(&PTT::set, push2talk, true);//push2talk->set(true);
+		REQ_SYNC(&PTT::set, push2talk, true);
 		MilliSleep(msecs);
-		REQ_SYNC(&PTT::set, push2talk, false);//push2talk->set(false);
-LOG_DEBUG("ARQ ptt toggled");
+		REQ_SYNC(&PTT::set, push2talk, false);
+		LOG_VERBOSE("%s", "ARQ ptt toggled");
 		return;
 	}
 	for (size_t i = 0; i < NUM_MODES; ++i) {
 		if (strlen(mode_info[i].pskmail_name) > 0) {
 			if (src == mode_info[i].pskmail_name) {
 				while (trx_state != STATE_RX) MilliSleep(50);
-				REQ_SYNC(init_modem_sync, mode_info[i].mode);
-LOG_DEBUG("ARQ new modem set to %s", mode_info[i].pskmail_name);
+				REQ_SYNC(init_modem_sync, mode_info[i].mode, 0);
+LOG_INFO("ARQ new modem set to %s", mode_info[i].pskmail_name);
 				break;
 			}
 		}
 	}
 }
 
+void ParseRSID(string src)
+{
+	if (src == "ON") {
+		LOG_VERBOSE("%s", "RsID turned ON");
+		REQ(set_button, btnRSID, 1);
+	}
+	if (src == "OFF") {
+		LOG_VERBOSE("%s", "RsID turned OFF");
+		REQ(set_button, btnRSID, 0);
+	}
+}
+
+
+void ParseTxRSID(string src)
+{
+	if (src == "ON") {
+		LOG_VERBOSE("%s", "TxRsID turned ON");
+		REQ(set_button, btnTxRSID, 1);
+	}
+	if (src == "OFF") {
+		LOG_VERBOSE("%s", "TxRsID turned OFF");
+		REQ(set_button, btnTxRSID, 0);
+	}
+}
+
 void parse_arqtext(string &toparse)
 {
 	string strCmdText;
 	string strSubCmd;
 	unsigned long int idxCmd, idxCmdEnd, idxSubCmd, idxSubCmdEnd;
 
+	LOG_DEBUG("Arq text: %s", noctrl(toparse).c_str());
+
 	idxCmd = toparse.find("<cmd>");
 	idxCmdEnd = toparse.find("</cmd>");
 
-	if ( idxCmd != string::npos && idxCmdEnd != string::npos && idxCmdEnd > idxCmd ) {
+	while ( idxCmd != string::npos && idxCmdEnd != string::npos && idxCmdEnd > idxCmd ) {
 
-LOG_DEBUG("Command string: %s", toparse.substr(idxCmd, idxCmdEnd + 6).c_str());
+		LOG_DEBUG("Cmd text: %s", toparse.substr(idxCmd, idxCmdEnd + 6).c_str());
 		strCmdText = toparse.substr(idxCmd + 5, idxCmdEnd - idxCmd - 5);
 		if (strCmdText == "server" && mailserver == false && mailclient == false) {
 			mailserver = true;
@@ -124,7 +204,7 @@ LOG_DEBUG("Command string: %s", toparse.substr(idxCmd, idxCmdEnd + 6).c_str());
 			if (progdefaults.PSKmailSweetSpot)
 				active_modem->set_freq(progdefaults.PSKsweetspot);
 			active_modem->set_freqlock(true);
-LOG_DEBUG("ARQ is set to pskmail server");
+			LOG_DEBUG("%s", "ARQ is set to pskmail server");
 		} else if (strCmdText == "client" && mailclient == false && mailserver == false) {
 			mailclient = true;
 			mailserver = false;
@@ -134,7 +214,7 @@ LOG_DEBUG("ARQ is set to pskmail server");
 			Maillogfile->log_to_file_start();
 			REQ(set_button, wf->xmtlock, 0);
 			active_modem->set_freqlock(false);
-LOG_DEBUG("ARQ is set to pskmail client");
+			LOG_DEBUG("%s", "ARQ is set to pskmail client");
 		} else if (strCmdText == "normal") {
 			mailserver = false;
 			mailclient = false;
@@ -144,20 +224,35 @@ LOG_DEBUG("ARQ is set to pskmail client");
 			}
 			REQ(set_button, wf->xmtlock, 0);
 			active_modem->set_freqlock(false);
-LOG_DEBUG("ARQ is reset to normal ops");
-		} else {
-			if ((idxSubCmd = strCmdText.find("<mode>")) != string::npos) {
-				idxSubCmdEnd = strCmdText.find("</mode>");
-				if (	idxSubCmdEnd != string::npos &&
-						idxSubCmdEnd > idxSubCmd ) {
-					strSubCmd = strCmdText.substr(idxSubCmd + 6, idxSubCmdEnd - idxSubCmd - 6);
-					ParseMode(strSubCmd);
-				}
+			LOG_DEBUG("%s", "ARQ is reset to normal ops");
+		} else if ((idxSubCmd = strCmdText.find("<mode>")) != string::npos) {
+			idxSubCmdEnd = strCmdText.find("</mode>");
+			if (	idxSubCmdEnd != string::npos &&
+					idxSubCmdEnd > idxSubCmd ) {
+				strSubCmd = strCmdText.substr(idxSubCmd + 6, idxSubCmdEnd - idxSubCmd - 6);
+				ParseMode(strSubCmd);
+			}
+		} else if ((idxSubCmd = strCmdText.find("<rsid>")) != string::npos) {
+			idxSubCmdEnd = strCmdText.find("</rsid>");
+			if (	idxSubCmdEnd != string::npos &&
+					idxSubCmdEnd > idxSubCmd ) {
+				strSubCmd = strCmdText.substr(idxSubCmd + 6, idxSubCmdEnd - idxSubCmd - 6);
+				ParseRSID(strSubCmd);
+			}
+		} else if ((idxSubCmd = strCmdText.find("<txrsid>")) != string::npos) {
+			idxSubCmdEnd = strCmdText.find("</txrsid>");
+			if (	idxSubCmdEnd != string::npos &&
+					idxSubCmdEnd > idxSubCmd ) {
+				strSubCmd = strCmdText.substr(idxSubCmd + 8, idxSubCmdEnd - idxSubCmd - 8);
+				ParseTxRSID(strSubCmd);
 			}
 		}
+
 		toparse.erase(idxCmd, idxCmdEnd - idxCmd + 6);
-		if (toparse.length() == 1 && toparse[0] == '\n')
-			toparse = "";
+		while (toparse[0] == '\n' || toparse[0] == '\r') toparse.erase(0, 1);
+
+		idxCmd = toparse.find("<cmd>");
+		idxCmdEnd = toparse.find("</cmd>");
 	}
 }
 
@@ -178,15 +273,15 @@ void process_msgque()
 		txstring.append(txmsgst.buffer);
 		parse_arqtext(txstring);
 
-		if (arqtext.empty() && !txstring.empty()) {
+		if (!bSend0x06 && arqtext.empty() && !txstring.empty()) {
 			arqtext = txstring;
 			if (mailserver && progdefaults.PSKmailSweetSpot)
 				active_modem->set_freq(progdefaults.PSKsweetspot);
 			pText = 0;
 			arq_text_available = true;
 			active_modem->set_stopflag(false);
+			LOG_DEBUG("SYSV ARQ string: %s", arqtext.c_str());
 			start_tx();
-			LOG_DEBUG("SYSV ARQ string: %s", txstring.c_str());
 			txstring.clear();
 		}
 	}
@@ -246,13 +341,13 @@ bool TLF_arqRx()
 		arqtext = "";
 		time(&start_time);
 		while (!autofile.eof()) {
-			memset(mailline,0,1000);
+			memset(mailline, 0, sizeof(mailline));
 			autofile.getline(mailline, 998); // leave space for "\n" and null byte
 			txstring.append(mailline);
 			txstring.append("\n");
 			time(&prog_time);
 			if (prog_time - start_time > TIMEOUT) {
-				LOG_ERROR("TLF file_i/o failure");
+				LOG_ERROR("TLF file I/O failure");
 				autofile.close();
 				std::remove (sAutoFile.c_str());
 				return false;
@@ -269,8 +364,8 @@ bool TLF_arqRx()
 			pText = 0;
 			arq_text_available = true;
 			active_modem->set_stopflag(false);
+			LOG_DEBUG("%s", arqtext.c_str());
 			start_tx();
-LOG_DEBUG(txstring.c_str());
 			txstring.clear();
 		}
 
@@ -278,6 +373,51 @@ LOG_DEBUG(txstring.c_str());
 	return true;
 }
 
+//-----------------------------------------------------------------------------
+// Auto transmit of file contained in WRAP_auto_dir
+//-----------------------------------------------------------------------------
+bool WRAP_auto_arqRx()
+{
+	time_t start_time, prog_time;
+	static char mailline[1000];
+	string sAutoFile = WRAP_auto_dir;
+	sAutoFile += "wrap_auto_file";
+
+	ifstream autofile(sAutoFile.c_str());
+	if(autofile) {
+		arqtext = "";
+		time(&start_time);
+		while (!autofile.eof()) {
+			memset(mailline,0,1000);
+			autofile.getline(mailline, 998); // leave space for "\n" and null byte
+			txstring.append(mailline);
+			txstring.append("\n");
+			time(&prog_time);
+			if (prog_time - start_time > TIMEOUT) {
+				LOG_ERROR("autowrap file I/O failure");
+				autofile.close();
+				std::remove (sAutoFile.c_str());
+				return false;
+			}
+		}
+		autofile.close();
+		std::remove (sAutoFile.c_str());
+
+		if (!txstring.empty()) {
+			arqtext.append("\n....start\n");
+			arqtext.append(txstring);
+			arqtext.append("\n......end\n");
+			pText = 0;
+			arq_text_available = true;
+			LOG_DEBUG("%s", arqtext.c_str());
+			start_tx();
+			txstring.clear();
+			return true;
+		}
+	}
+	return false;
+}
+
 //-----------------------------------------------------------------------------
 // Socket ARQ i/o used on all platforms
 //-----------------------------------------------------------------------------
@@ -407,14 +547,14 @@ void arq_stop()
 	arqmode = false;
 }
 
-void WriteARQsocket(unsigned int data)
+void WriteARQsocket(unsigned char* data, size_t len)
 {
 	if (!isSocketConnected) return;
-	char c = data;
 	try {
-		arqclient.send(&c, 1);
+		arqclient.send(data, len);
 	}
 	catch (const SocketException& e) {
+		LOG_ERROR("%s", e.what());
 		arq_stop();
 	}
 }
@@ -430,15 +570,7 @@ bool Socket_arqRx()
 		if ( n > 0)
 			txstring.append(instr);
 
-		if (progdefaults.rsid == true) {
-			send0x06();
-			arqtext.clear();
-			txstring.clear();
-			cmdstring.clear();
-			return true;
-		}
-
-		if (arqtext.empty() && !txstring.empty()) {
+		if (!bSend0x06 && arqtext.empty() && !txstring.empty()) {
 			arqtext = txstring;
 			parse_arqtext(arqtext);
 			if (!arqtext.empty()) {
@@ -447,13 +579,15 @@ bool Socket_arqRx()
 				pText = 0;//arqtext.begin();
 				arq_text_available = true;
 				active_modem->set_stopflag(false);
+				LOG_DEBUG("%s", arqtext.c_str());
 				start_tx();
-LOG_DEBUG(txstring.c_str());
 			}
 			txstring.clear();
+			cmdstring.clear();
+			return true;
 		}
 		cmdstring.clear();
-		return true;
+		return false;
 	}
 	catch (const SocketException& e) {
 		arq_stop();
@@ -465,7 +599,7 @@ LOG_DEBUG(txstring.c_str());
 // Send ARQ characters to ARQ client
 //-----------------------------------------------------------------------------
 #if !defined(__WOE32__) && !defined(__APPLE__)
-void WriteARQSysV(unsigned int data)
+void WriteARQSysV(unsigned char data)
 {
 	rxmsgid = msgget( (key_t) progdefaults.rx_msgid, 0666);
 	if ( rxmsgid != -1) {
@@ -476,27 +610,14 @@ void WriteARQSysV(unsigned int data)
 }
 #endif
 
-void WriteARQ( unsigned int data)
+void WriteARQ(unsigned char data)
 {
-	WriteARQsocket(data);
+	WriteARQsocket(&data, 1);
 #if !defined(__WOE32__) && !defined(__APPLE__)
 	WriteARQSysV(data);
 #endif
 }
 
-// following function used if the T/R button is pressed to stop a transmission
-// that is servicing the ARQ text buffer.  It allows the ARQ client to reset
-// itself properly
-
-void AbortARQ() {
-	if (arq_text_available) {
-		arqtext.clear();
-		pText = 0;//arqtext.begin();
-		arq_text_available = false;
-		WriteARQ(0x06);
-	}
-}
-
 //-----------------------------------------------------------------------------
 // Write End of Transmit character to ARQ client
 //-----------------------------------------------------------------------------
@@ -505,7 +626,6 @@ void send0x06()
 {
 	if (trx_state == STATE_RX) {
 		bSend0x06 = false;
-		arq_text_available = false;
 		WriteARQ(0x06);
 	}
 }
@@ -536,11 +656,14 @@ static void *arq_loop(void *args)
 			send0x06();
 
 #if !defined(__WOE32__) && !defined(__APPLE__)
-		// order of precedence; Socket, SysV, GMFSKfile
-		if (!Socket_arqRx() && !SysV_arqRx() && tlfio)
-			TLF_arqRx();
+		// order of precedence; Socket, Wrap autofile, TLF autofile
+		if (!Socket_arqRx())
+			if (!SysV_arqRx())
+				if (!WRAP_auto_arqRx())
+					TLF_arqRx();
 #else
-		Socket_arqRx();
+		if (!Socket_arqRx())
+			WRAP_auto_arqRx();
 #endif
 		pthread_mutex_unlock (&arq_mutex);
 		MilliSleep(50);
@@ -588,14 +711,62 @@ char arq_get_char()
 {
 	char c = 0x03;
 	pthread_mutex_lock (&arq_mutex);
-	if (pText != arqtext.length()-1) {
+	if (!arqtext.empty() && (pText != arqtext.length())) {
 		c = arqtext[pText++];
 	} else {
 		arqtext.clear();
 		pText = 0;
 		bSend0x06 = true;
+		arq_text_available = false;
 	}
 	pthread_mutex_unlock (&arq_mutex);
 	return c;
 }
 
+// following function used if the T/R button is pressed to stop a transmission
+// that is servicing the ARQ text buffer.  It allows the ARQ client to reset
+// itself properly
+
+void AbortARQ() {
+	pthread_mutex_lock (&arq_mutex);
+	arqtext.clear();
+	pText = 0;
+	arq_text_available = false;
+	bSend0x06 = true;
+	pthread_mutex_unlock (&arq_mutex);
+}
+
+// Special notification for PSKMAIL: new mode marked only, in following
+// format: "<DC2><Mode:newmode>", with <DC2> = '0x12'.
+void pskmail_notify_rsid(trx_mode mode)
+{
+	char buf[64];
+	int n = snprintf(buf, sizeof(buf), "%c<Mode:%s>\n", 0x12, mode_info[mode].name);
+	if (n > 0 && n < (int)sizeof(buf)) {
+		WriteARQsocket((unsigned char*)buf, n);
+#if !defined(__WOE32__) && !defined(__APPLE__)
+		for (int ii=0; ii < n; ii++)
+			WriteARQSysV((unsigned char)buf[ii]);
+#endif
+		ReceiveText->add(buf, FTextBase::CTRL);
+	}
+}
+
+// Special notification for PSKMAIL: signal to noise measured by decoder
+// format "<DC2><s2n: CC, A.a, D.d>"
+// where CC = count, A.a = average s/n, D.d = Std dev of s/n
+void pskmail_notify_s2n(double s2n_ncount, double s2n_avg, double s2n_stddev)
+{
+	char buf[64];
+	int n = snprintf(buf, sizeof(buf), "%c<s2n: %1.0f, %1.1f, %1.1f>",
+			 0x12, s2n_ncount, s2n_avg, s2n_stddev);
+	if (n > 0 && n < (int)sizeof(buf)) {
+		WriteARQsocket((unsigned char*)buf, n);
+#if !defined(__WOE32__) && !defined(__APPLE__)
+		for (int ii=0; ii < n; ii++)
+			WriteARQSysV((unsigned char)buf[ii]);
+#endif
+		ReceiveText->add(buf, FTextBase::CTRL);
+	}
+}
+
