@@ -1,27 +1,27 @@
 // ----------------------------------------------------------------------------
 // lookupcall.cxx  -- a part of fldigi
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2009
 //		Dave Freese, W1HKJ
+// Copyright (C) 2006-2007
 //		Leigh Klotz, WA5ZNU
-// Copyright (C) 2008
+// Copyright (C) 2008-2009
 //              Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -59,7 +59,7 @@
 using namespace std;
 
 
-string qrzhost = "online.qrz.com";
+string qrzhost = "xml.qrz.com"; //"online.qrz.com";
 string qrzSessionKey;
 string qrzalert;
 string qrzerror;
@@ -193,6 +193,7 @@ bool parseSessionKey(const string& sessionpage)
 
 bool parse_xml(const string& xmlpage)
 {
+//printf("%s\n", xmlpage.c_str());
 	IrrXMLReader* xml = createIrrXMLReader(new IIrrXMLStringReader(xmlpage));
 
 // If we got any result back, clear the session key so that it will be 
@@ -204,20 +205,7 @@ bool parse_xml(const string& xmlpage)
 		qrzerror.clear();
 		clear_Lookup();
 	}
-// strings for storing the data we want to get out of the file
-	string	call, 
-			fname, 
-			name, 
-			addr1, 
-			addr2, 
-			state, 
-			zip, 
-			country, 
-			latd, 
-			lond, 
-			grid, 
-			dob;
-			
+
 	TAG tag = QRZ_IGNORE;
 	
 // parse the file until end reached
@@ -230,7 +218,6 @@ bool parse_xml(const string& xmlpage)
 					case QRZ_IGNORE:
 						break;
 					case QRZ_CALL:
-						call = xml->getNodeData();
 						break;
 					case QRZ_FNAME:
 						lookup_fname =  xml->getNodeData();
@@ -239,8 +226,13 @@ bool parse_xml(const string& xmlpage)
 						lookup_name =  xml->getNodeData();
 						break;
 					case QRZ_ADDR1:
+						{
 						lookup_addr1 =  xml->getNodeData();
+						size_t apt = lookup_addr1.find("#");
+						if (apt != string::npos)
+							lookup_addr1.erase(apt, lookup_addr1.length() - apt);
 						break;
+						}
 					case QRZ_ADDR2:
 						lookup_addr2 =  xml->getNodeData();
 						break;
@@ -293,8 +285,8 @@ bool parse_xml(const string& xmlpage)
 				else if (!strcmp("state", nodeName)) 	tag = QRZ_STATE;
 				else if (!strcmp("zip", nodeName)) 		tag = QRZ_ZIP;
 				else if (!strcmp("country", nodeName))	tag = QRZ_COUNTRY;
-				else if (!strcmp("latd", nodeName)) 	tag = QRZ_LATD;
-				else if (!strcmp("lond", nodeName)) 	tag = QRZ_LOND;
+				else if (!strcmp("lat", nodeName)) 		tag = QRZ_LATD;
+				else if (!strcmp("lon", nodeName)) 		tag = QRZ_LOND;
 				else if (!strcmp("grid", nodeName)) 	tag = QRZ_GRID;
 				else if (!strcmp("dob", nodeName)) 		tag = QRZ_DOB;
 				else if (!strcmp("Alert", nodeName)) 	tag = QRZ_ALERT;
