@@ -1,7 +1,34 @@
+// ----------------------------------------------------------------------------
+// macros.cxx
+//
+// Copyright (C) 2007-2010
+//		Dave Freese, W1HKJ
+// Copyright (C) 2008-2010
+//		Stelios Bounanos, M0GLD
+// Copyright (C) 2009
+//		Chris Sylvain, KB3CS
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
+
 #include <config.h>
 
 #include "macros.h"
 
+#include "gettext.h"
 #include "main.h"
 
 #include "fl_digi.h"
@@ -14,6 +41,8 @@
 #include "status.h"
 #include "trx.h"
 #include "modem.h"
+#include "qrunner.h"
+#include "waterfall.h"
 
 #include <FL/Fl.H>
 #include <FL/filename.H>
@@ -31,8 +60,19 @@ using namespace std;
 MACROTEXT macros;
 CONTESTCNTR contest_count;
 static bool TransmitON = false;
+static bool ToggleTXRX = false;
 int mNbr;
 
+std::string qso_time = "";
+std::string qso_exchange = "";
+bool save_xchg;
+size_t  xbeg = 0, xend = 0;
+
+string text2send = "";
+string text2repeat = "";
+string text2save = "";
+size_t repeatchar = 0;
+
 struct MTAGS { const char *mTAG; void (*fp)(string &, size_t &);};
 
 void pCALL(string &, size_t &);
@@ -57,14 +97,21 @@ void pTEXT(string &, size_t &);
 void pCWID(string &, size_t &);
 void pRX(string &, size_t &);
 void pTX(string &, size_t &);
+void pTXRX(string &, size_t &);
 void pVER(string &, size_t &);
 void pCNTR(string &, size_t &);
 void pDECR(string &, size_t &);
 void pINCR(string &, size_t &);
 void pXOUT(string &, size_t &);
+void pSAVEXCHG(string &, size_t &);
+void pXBEG(string &, size_t &);
+void pXEND(string &, size_t &);
 void pLOG(string &, size_t &);
+void pLNW(string &, size_t &);
 void pTIMER(string &, size_t &);
 void pIDLE(string &, size_t &);
+void pTUNE(string &, size_t &);
+void pMODEM_compat(string &, size_t &);
 void pMODEM(string &, size_t &);
 void pEXEC(string &, size_t &);
 void pSTOP(string &, size_t &);
@@ -79,7 +126,17 @@ void pRISETIME(string &, size_t &);
 void pPRE(string &, size_t &);
 void pPOST(string &, size_t &);
 void pAFC(string &, size_t &);
-void pRSID(string &, size_t &);
+void pLOCK(string &, size_t &);
+void pRX_RSID(string &, size_t &);
+void pTX_RSID(string &, size_t &);
+void pWAIT(string&, size_t &);
+void pSRCHUP(string&, size_t&);
+void pSRCHDN(string&, size_t&);
+void pGOHOME(string&, size_t&);
+void pGOFREQ(string&, size_t&);
+void pMAPIT(string&, size_t&);
+void pREPEAT(string&, size_t&);
+
 //void pMACROS(string &, size_t &);
 
 MTAGS mtags[] = {
@@ -107,29 +164,45 @@ MTAGS mtags[] = {
 {"<CWID>",		pCWID},
 {"<RX>",		pRX},
 {"<TX>",		pTX},
+{"<TX/RX>",		pTXRX},
 {"<VER>",		pVER},
 {"<CNTR>",		pCNTR},
 {"<DECR>",		pDECR},
 {"<INCR>",		pINCR},
 {"<X1>",		pXOUT},
-{"<XOUT>",      pXOUT},
+{"<XOUT>",		pXOUT},
+{"<XBEG>",		pXBEG},
+{"<XEND>",		pXEND},
+{"<SAVEXCHG>",	pSAVEXCHG},
 {"<LOG>",		pLOG},
+{"<LNW>",		pLNW},
 {"<TIMER:",		pTIMER},
 {"<IDLE:",		pIDLE},
-{"<MODEM>",		pMODEM},
+{"<TUNE:",		pTUNE},
+{"<WAIT:",		pWAIT},
+{"<MODEM>",		pMODEM_compat},
+{"<MODEM:",		pMODEM},
 {"<EXEC>",		pEXEC},
 {"<STOP>",		pSTOP},
 {"<CONT>",		pCONT},
 {"<GET>",		pGET},
 {"<CLRRX>",		pCLRRX},
 {"<FILE:",		pFILE},
-{"<WPM:",       pWPM},
-{"<RISE:",      pRISETIME},
-{"<PRE:",       pPRE},
-{"<POST:",      pPOST},
-{"<AFC:",       pAFC},
-{"<RSID:",      pRSID},
-//{"<MACROS:",	pMACROS},
+{"<WPM:",		pWPM},
+{"<RISE:",		pRISETIME},
+{"<PRE:",		pPRE},
+{"<POST:",		pPOST},
+{"<AFC:",		pAFC},
+{"<LOCK:",		pLOCK},
+{"<RXRSID:",	pRX_RSID},
+{"<TXRSID:",	pTX_RSID},
+{"<SRCHUP>",	pSRCHUP},
+{"<SRCHDN>",	pSRCHDN},
+{"<GOHOME>",	pGOHOME},
+{"<GOFREQ:",	pGOFREQ},
+{"<MAPIT:",		pMAPIT},
+{"<MAPIT>",		pMAPIT},
+{"<REPEAT>",	pREPEAT},
 {0, 0}
 };
 
@@ -189,6 +262,16 @@ void pTIMER(string &s, size_t &i)
 	s.replace(i, endbracket - i + 1, "");
 }
 
+void pREPEAT(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	progStatus.repeatMacro = mNbr;
+	s.replace(i, endbracket - i + 1, "");
+	text2repeat = s;
+	repeatchar = 0;
+	s.insert(i, "[REPEAT]");
+}
+
 void pWPM(string &s, size_t &i)
 {
 	size_t endbracket = s.find('>',i);
@@ -249,22 +332,53 @@ void pPOST(string &s, size_t &i)
 	s.replace(i, endbracket - i + 1, "");
 }
 
-bool useIdle = false;
-int  idleTime = 0;
+bool macro_idle_on = false;
+float  idleTime = 0;
 
 void pIDLE(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	float number;
+	string sTime = s.substr(i+6, endbracket - i - 6);
+	if (sTime.length() > 0) {
+		sscanf(sTime.c_str(), "%f", &number);
+		macro_idle_on = true;
+		idleTime = number;
+	}
+	s.replace(i, endbracket - i + 1, "");
+}
+
+bool useTune = false;
+int  tuneTime = 0;
+
+void pTUNE(string &s, size_t &i)
 {
 	size_t endbracket = s.find('>',i);
 	int number;
 	string sTime = s.substr(i+6, endbracket - i - 6);
 	if (sTime.length() > 0) {
 		sscanf(sTime.c_str(), "%d", &number);
-		useIdle = true;
-		idleTime = number;
+		useTune = true;
+		tuneTime = number;
 	}
 	s.replace(i, endbracket - i + 1, "");
 }
 
+bool useWait = false;
+int  waitTime = 0;
+
+void pWAIT(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	int number;
+	string sTime = s.substr(i+6, endbracket - i - 6);
+	if (sTime.length() > 0) {
+		sscanf(sTime.c_str(), "%d", &number);
+		useWait = true;
+		waitTime = number;
+	}
+	s.replace(i, endbracket - i + 1, "");
+}
 
 void pINFO1(string &s, size_t &i)
 {
@@ -320,7 +434,9 @@ void pQTH(string &s, size_t &i)
 
 void pQSOTIME(string &s, size_t &i)
 {
-	s.replace( i, 9, inpTimeOff->value() );
+	if (qso_time.empty())
+		qso_time = inpTimeOff->value();
+	s.replace( i, 9, qso_time.c_str() );
 }
 
 void pRST(string &s, size_t &i)
@@ -426,6 +542,13 @@ void pTX(string &s, size_t &i)
 	TransmitON = true;
 }
 
+void pTXRX(string &s, size_t &i)
+{
+	s.erase(i, 6);
+	ToggleTXRX = true;
+}
+
+
 void pVER(string &s, size_t &i)
 {
 	string progname;
@@ -470,17 +593,38 @@ void pXOUT(string &s, size_t &i)
 	s.replace( i, 6, cutstring(progdefaults.myXchg.c_str()));
 }
 
-void pLOG(string &s, size_t &i)
+void pXBEG(string &s, size_t &i)
 {
-	submit_log();
-	s.replace(i, 5, "");
-	if (progdefaults.ClearOnSave)
-		clearQSO();
+	s.replace( i, 6, "");
+	xbeg = i;
 }
 
-void pMODEM(string &s, size_t &i)
+void pXEND(string &s, size_t &i)
 {
-	size_t	j, k, 
+	s.replace( i, 6, "");
+	xend = i;
+}
+
+void pSAVEXCHG(string &s, size_t &i)
+{
+	save_xchg = true;
+	s.replace( i, 10, "");
+}
+
+void pLOG(string &s, size_t &i)
+{
+	qsoSave_cb(0, 0);
+	s.replace(i, 5, "");
+}
+
+void pLNW(string &s, size_t &i)
+{
+	s.replace(i, 5, "^L");
+}
+
+void pMODEM_compat(string &s, size_t &i)
+{
+	size_t	j, k,
 			len = s.length();
 	string name;
 
@@ -502,6 +646,84 @@ void pMODEM(string &s, size_t &i)
 	s.erase(i, k-i);
 }
 
+#include <float.h>
+#include "re.h"
+
+void pMODEM(string &s, size_t &i)
+{
+	static fre_t re("<MODEM:([[:alnum:]-]+)((:[[:digit:].+-]*)*)>", REG_EXTENDED);
+
+	if (!re.match(s.c_str() + i)) {
+		size_t end = s.find('>', i);
+		if (end != string::npos)
+			s.erase(i, end - i);
+		return;
+	}
+
+	const std::vector<regmatch_t>& o = re.suboff();
+	string name = s.substr(o[1].rm_so, o[1].rm_eo - o[1].rm_so);
+	trx_mode m;
+	for (m = 0; m < NUM_MODES; m++)
+		if (name == mode_info[m].sname)
+			break;
+	// do we have arguments and a valid modem?
+	if (o.size() == 2 || m == NUM_MODES) {
+		if (m < NUM_MODES && active_modem->get_mode() != mode_info[m].mode)
+			init_modem_sync(mode_info[m].mode);
+		s.erase(i, o[0].rm_eo - i);
+		return;
+	}
+
+	// parse arguments
+	vector<double> args;
+	args.reserve(8);
+	char* end;
+	double d;
+	for (const char* p = s.c_str() + o[2].rm_so + 1; *p; p++) {
+		errno = 0;
+		d = strtod(p, &end);
+		if (!errno && p != end) {
+			args.push_back(d);
+			p = end;
+		}
+		else // push an invalid value
+			args.push_back(DBL_MIN);
+	}
+
+	try {
+		switch (m) {
+		case MODE_RTTY: // carrier shift, baud rate, bits per char
+			if (args.at(0) != DBL_MIN)
+				set_rtty_shift((int)args[0]);
+			if (args.at(1) != DBL_MIN)
+				set_rtty_baud((int)args[1]);
+			if (args.at(2) != DBL_MIN)
+				set_rtty_bits((int)args[2]);
+			break;
+		case MODE_CONTESTIA: // bandwidth, tones
+			if (args.at(0) != DBL_MIN)
+				set_contestia_bw((int)args[0]);
+			if (args.at(1) != DBL_MIN)
+				set_contestia_tones((int)args[1]);
+			break;
+		case MODE_OLIVIA: // bandwidth, tones
+			if (args.at(0) != DBL_MIN)
+				set_olivia_bw((int)args[0]);
+			if (args.at(1) != DBL_MIN)
+				set_olivia_tones((int)args[1]);
+			break;
+		default:
+			break;
+		}
+	}
+	catch (const exception& e) { }
+
+	if (active_modem->get_mode() != mode_info[m].mode)
+		init_modem_sync(mode_info[m].mode);
+
+	s.erase(i, o[0].rm_eo - i);
+}
+
 void pAFC(string &s, size_t &i)
 {
   size_t endbracket = s.find('>',i);
@@ -520,10 +742,47 @@ void pAFC(string &s, size_t &i)
   s.replace(i, endbracket - i + 1, "");
 }
 
-void pRSID(string &s, size_t &i)
+void pLOCK(string &s, size_t &i)
 {
   size_t endbracket = s.find('>',i);
   string sVal = s.substr(i+6, endbracket - i - 6);
+  if (sVal.length() > 0) {
+    // sVal = on|off|t   [ON, OFF or Toggle]
+    if (sVal.compare(0,2,"on") == 0)
+      wf->xmtlock->value(1);
+    else if (sVal.compare(0,3,"off") == 0)
+      wf->xmtlock->value(0);
+    else if (sVal.compare(0,1,"t") == 0)
+      wf->xmtlock->value(!wf->xmtlock->value());
+
+    wf->xmtlock->damage();
+    wf->xmtlock->do_callback();
+  }
+  s.replace(i, endbracket - i + 1, "");
+}
+
+void pTX_RSID(string &s, size_t &i)
+{
+  size_t endbracket = s.find('>',i);
+  string sVal = s.substr(i+8, endbracket - i - 8);
+  if (sVal.length() > 0) {
+    // sVal = on|off|t   [ON, OFF or Toggle]
+    if (sVal.compare(0,2,"on") == 0)
+      btnTxRSID->value(1);
+    else if (sVal.compare(0,3,"off") == 0)
+      btnTxRSID->value(0);
+    else if (sVal.compare(0,1,"t") == 0)
+      btnTxRSID->value(!btnTxRSID->value());
+
+    btnTxRSID->do_callback();
+  }
+  s.replace(i, endbracket - i + 1, "");
+}
+
+void pRX_RSID(string &s, size_t &i)
+{
+  size_t endbracket = s.find('>',i);
+  string sVal = s.substr(i+8, endbracket - i - 8);
   if (sVal.length() > 0) {
     // sVal = on|off|t   [ON, OFF or Toggle]
     if (sVal.compare(0,2,"on") == 0)
@@ -538,6 +797,46 @@ void pRSID(string &s, size_t &i)
   s.replace(i, endbracket - i + 1, "");
 }
 
+void pSRCHUP(string &s, size_t &i)
+{
+	s.replace( i, 8, "");
+	active_modem->searchUp();
+}
+
+void pSRCHDN(string &s, size_t &i)
+{
+	s.replace( i, 8, "");
+	active_modem->searchDown();
+}
+
+void pGOHOME(string &s, size_t &i)
+{
+	s.replace( i, 8, "");
+	if (active_modem == cw_modem)
+		active_modem->set_freq(progdefaults.CWsweetspot);
+	else if (active_modem == rtty_modem)
+		active_modem->set_freq(progdefaults.RTTYsweetspot);
+	else
+		active_modem->set_freq(progdefaults.PSKsweetspot);
+}
+
+void pGOFREQ(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	int number;
+	string sGoFreq = s.substr(i+8, endbracket - i - 8);
+	if (sGoFreq.length() > 0) {
+		sscanf(sGoFreq.c_str(), "%d", &number);
+		if (number < progdefaults.LowFreqCutoff)
+			number = progdefaults.LowFreqCutoff;
+		if (number > progdefaults.HighFreqCutoff)
+			number = progdefaults.HighFreqCutoff;
+		active_modem->set_freq(number);
+	}
+	s.replace(i, endbracket - i + 1, "");
+}
+
+
 void set_macro_env(void)
 {
 	enum {
@@ -721,6 +1020,71 @@ void pEXEC(string& s, size_t& i)
 }
 #endif // !__MINGW32__
 
+void MAPIT(int how)
+{
+	float lat = 0, lon = 0;
+	string sCALL = inpCall->value();
+	string sLOC = inpLoc->value();
+
+	string url = "http://maps.google.com/maps?q=";
+
+//	if (lookup_addr1.empty() && lookup_addr2.empty() &&
+//		lookup_state.empty() && lookup_country.empty()) {
+	if (how > 1 && !lookup_country.empty()) {
+		url.append(lookup_addr1).append(",").append(lookup_addr2).append(",");
+		url.append(lookup_state).append(",").append(lookup_country);
+	} else {
+		if (how > 0 && (!lookup_latd.empty() && !lookup_lond.empty())) {
+			url.append(lookup_latd).append(",");
+			url.append(lookup_lond);
+		} else {
+			if (sLOC.empty()) return;
+			if (sLOC.length() < 4) return;
+			if (sLOC.length() < 6) sLOC.append("aa");
+			for (size_t i = 0; i < 6; i++) sLOC[i] = toupper(sLOC[i]);
+			if (sLOC[0] -'A' > 17 || sLOC[4] - 'A' > 23 ||
+				sLOC[1] -'A' > 17 || sLOC[5] - 'A' > 23 ||
+				!isdigit(sLOC[2]) || !isdigit(sLOC[3])) return;
+			lon =	-180.0 + 
+					(sLOC[0] - 'A') * 20 +
+					(sLOC[2] - '0') * 2 +
+					(sLOC[4] - 'A' + 0.5) / 12;
+			lat = -90.0 + 
+					(sLOC[1] - 'A') * 10 +
+					(sLOC[3] - '0') +
+					(sLOC[5] - 'A' + 0.5) / 24;
+			char sdata[20];
+			snprintf(sdata, sizeof(sdata),"%10.6f", lat);
+			url.append(sdata).append(",");
+			snprintf(sdata, sizeof(sdata),"%10.6f", lon);
+			url.append(sdata);
+		}
+	}
+	if (!sCALL.empty()) url.append("(").append(sCALL).append(")");
+	else url.append("(nocall)");
+	url.append("&t=p&z=10");
+	cb_mnuVisitURL(NULL, (void*)url.c_str());
+}
+
+void pMAPIT(string &s, size_t &i)
+{
+	size_t endbracket = s.find('>',i);
+	string sVal = s.substr(i + 7, endbracket - i - 7);
+	if (sVal.length() > 0) {
+		if (sVal.compare(0,3,"adr") == 0)
+			REQ(MAPIT,2);
+		else if (sVal.compare(0,6,"latlon") == 0)
+			REQ(MAPIT,1);
+		else if (sVal.compare(0,3,"loc") == 0)
+			REQ(MAPIT,0);
+		else
+			REQ(MAPIT,2);
+	} else
+		REQ(MAPIT,2);
+	s.erase(i, s.find('>', i) + 1 - i);
+	expand = false;
+}
+
 void pSTOP(string &s, size_t &i)
 {
 	s.erase(i, s.find('>', i) + 1 - i);
@@ -743,9 +1107,9 @@ int MACROTEXT::loadMacros(const string& filename)
 	unsigned long int	   crlf; // 64 bit cpu's
 	char   szLine[4096];
 	bool   convert = false;
-	
+
 	ifstream mFile(filename.c_str());
-	
+
 	if (!mFile) {
 		create_new_macros();
 		for (int i = 0; i < 12; i++) {
@@ -754,7 +1118,7 @@ int MACROTEXT::loadMacros(const string& filename)
 		}
 		return 0;
 	}
-	
+
 	mFile.getline(szLine, 4095);
 	mLine = szLine;
 	if (mLine.find("//fldigi macro definition file") != 0) {
@@ -764,7 +1128,7 @@ int MACROTEXT::loadMacros(const string& filename)
 	if (mLine.find("extended") == string::npos) {
 		convert = true;
 		changed = true;
-	}		
+	}
 // clear all of the macros
 	for (int i = 0; i < MAXMACROS; i++) {
 		name[i] = "";
@@ -825,7 +1189,7 @@ void MACROTEXT::openMacroFile()
 {
 	string deffilename = MacrosDir;
 	deffilename.append(progStatus.LastMacroFile);
-    const char *p = FSEL::select("Open macro file", "Fldigi macro definition file\t*.mdf", deffilename.c_str());
+    const char *p = FSEL::select(_("Open macro file"), _("Fldigi macro definition file\t*.mdf"), deffilename.c_str());
     if (p) {
 		loadMacros(p);
 		progStatus.LastMacroFile = fl_filename_name(p);
@@ -837,7 +1201,7 @@ void MACROTEXT::saveMacroFile()
 {
 	string deffilename = MacrosDir;
 	deffilename.append(progStatus.LastMacroFile);
-    const char *p = FSEL::saveas("Save macro file", "Fldigi macro definition file\t*.mdf", deffilename.c_str());
+    const char *p = FSEL::saveas(_("Save macro file"), _("Fldigi macro definition file\t*.mdf"), deffilename.c_str());
     if (p) {
 		saveMacros(p);
 		progStatus.LastMacroFile = fl_filename_name(p);
@@ -861,10 +1225,19 @@ string MACROTEXT::expandMacro(int n)
 	size_t idx = 0;
 	expand = true;
 	TransmitON = false;
+	ToggleTXRX = false;
 	mNbr = n;
 	expanded = text[n];
 	MTAGS *pMtags;
 
+	xbeg = xend = -1;
+	save_xchg = false;
+	progStatus.repeatMacro = -1;
+	text2repeat.clear();
+	idleTime = 0;
+	waitTime = 0;
+	tuneTime = 0;
+
 	while ((idx = expanded.find('<', idx)) != string::npos) {
 		if (expanded.find("<MACROS:",idx) == idx) {
 			loadnewMACROS(expanded, idx);
@@ -909,34 +1282,124 @@ string MACROTEXT::expandMacro(int n)
 		GET = false;
 		return "";
 	}
-		
+
+	if (xbeg != string::npos && xend != string::npos && xend > xbeg) {
+		qso_exchange = expanded.substr(xbeg, xend - xbeg);
+	} else if (save_xchg) {
+		qso_exchange = expanded;
+		save_xchg = false;
+	}
+
+// force "^r" to be last tag in the expanded string
+	if ((idx = expanded.find("^r")) != string::npos) {
+		expanded.erase(idx, 2);
+		expanded.append("^r");
+	}
+
 	return expanded;
 }
 
-string text2send = "";
-
-void insertTextAfter(void *)
+void idleTimer(void *)
 {
-	TransmitText->add( text2send.c_str() );
+	macro_idle_on = false;
+}
+
+void continueMacro(void *)
+{
+	if ( TransmitON ) {
+		active_modem->set_stopflag(false);
+		if (macro_idle_on && idleTime > 0)
+			Fl::add_timeout(idleTime, idleTimer);
+		start_tx();
+		TransmitON = false;
+	}
 	text2send.clear();
 }
 
-void MACROTEXT::execute(int n) 
+void finishTune(void *)
 {
-	text2send = expandMacro(n);
+	trx_receive();
+// delay to allow tx/rx loop to change state
+	Fl::add_timeout(0.5, continueMacro);
+}
+
+void finishWait(void *)
+{
+	if (useTune && tuneTime > 0) {
+		trx_tune();
+		Fl::add_timeout(tuneTime, finishTune);
+		useTune = false;
+		return;
+	}
 	if ( TransmitON ) {
+		active_modem->set_stopflag(false);
+		if (macro_idle_on && idleTime > 0)
+			Fl::add_timeout(idleTime, idleTimer);
+		start_tx();
+		TransmitON = false;
+	}
+}
+
+static void set_button(Fl_Button* button, bool value)
+{
+	button->value(value);
+	button->do_callback();
+}
+
+void MACROTEXT::execute(int n)
+{
+	text2save = text2send = expandMacro(n);
+
+	if (progStatus.repeatMacro == -1)
+		TransmitText->add( text2send.c_str() );
+	else {
+		size_t p = string::npos;
+		text2send = text[n];
+		while ((p = text2send.find('<')) != string::npos)
+			text2send[p] = '[';
+		while ((p = text2send.find('>')) != string::npos)
+			text2send[p] = ']';
+		TransmitText->add( text2send.c_str() );
+	}
+	text2send.clear();
+
+	if (ToggleTXRX) {
+		text2send.clear();
+		if (!wf->xmtrcv->value()) {
+			REQ(set_button, wf->xmtrcv, true);
+			if (macro_idle_on && idleTime > 0)
+				Fl::add_timeout(idleTime, idleTimer);
+		} else
+			REQ(set_button, wf->xmtrcv, false);
+		return;
+	}
+	if (useWait && waitTime > 0) {
+		Fl::add_timeout(waitTime, finishWait);
+		useWait = false;
+		return;
+	}
+	if (useTune && tuneTime > 0) {
+		trx_tune();
+		Fl::add_timeout(tuneTime, finishTune);
+		useTune = false;
+		return;
+	}
+	if ( TransmitON ) {
+		if (macro_idle_on && idleTime > 0)
+			Fl::add_timeout(idleTime, idleTimer);
+
 		active_modem->set_stopflag(false);
 		start_tx();
 		TransmitON = false;
 	}
-	if (useIdle && idleTime > 0) {
-		Fl::add_timeout(idleTime , insertTextAfter);
-	}
-	else {
-		TransmitText->add( text2send.c_str() );
-		text2send.clear();
-	}
-	useIdle = false;
+}
+
+void MACROTEXT::repeat(int n)
+{
+	expandMacro(n);
+printf("%s\n",text2repeat.c_str());
+	macro_idle_on = false;
+	if (idleTime) progStatus.repeatIdleTime = idleTime;
 }
 
 MACROTEXT::MACROTEXT()
@@ -951,7 +1414,7 @@ MACROTEXT::MACROTEXT()
 }
 
 
-string mtext = 
+string mtext =
 "//fldigi macro definition file extended\n\
 // This file defines the macro structe(s) for the digital modem program, fldigi\n\
 // It also serves as a basis for any macros that are written by the user\n\
