@@ -5,21 +5,20 @@
  *	Copyright (C) 1999-2004 Pawel Jalocha, SP9VRC
  *	Copyright (c) 2007-2008 Dave Freese, W1HKJ
  *
- *	This file is part of fldigi.
+ *    This file is part of fldigi.
  *
- *	fldigi is free software; you can redistribute it and/or modify
- *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; either version 2 of the License, or
- *	(at your option) any later version.
+ *    Fldigi is free software: you can redistribute it and/or modify
+ *    it under the terms of the GNU General Public License as published by
+ *    the Free Software Foundation, either version 3 of the License, or
+ *    (at your option) any later version.
  *
- *	fldigi is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty oF
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	GNU General Public License for more details.
+ *    Fldigi is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU General Public License for more details.
  *
- *	You should have received a copy of the GNU General Public License
- *	along with MT63; if not, write to the Free Software
- *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *    You should have received a copy of the GNU General Public License
+ *    along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
