@@ -1,27 +1,24 @@
-//=============================================================================
-//
+// ----------------------------------------------------------------------------
 //    mt63.cxx  --  MT63 modem for fldigi
 //
-//    Copyright (C) 2007, 2008
+//    Copyright (C) 2007-2009
 //      Dave Freese, W1HKJ
 //
-//    This file is part of fldigi
+// This file is part of fldigi.
 //
-//    fldigi is free software; you can redistribute it and/or modify
-//    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2 of the License, or
-//    (at your option) any later version.
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
-//    fldigi is distributed in the hope that it will be useful,
-//    but WITHOUT ANY WARRANTY; without even the implied warranty of
-//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//    GNU General Public License for more details.
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
-//    You should have received a copy of the GNU General Public License
-//    along with gMFSK; if not, write to the Free Software
-//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-//
-//=============================================================================
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -137,17 +134,14 @@ int mt63::rx_process(const double *buf, int len)
 	static char msg1[20];
 	static char msg2[20];
 
-	if (get_freq() != (500.0 + bandwidth / 2.0))
-	    set_freq(500.0 + bandwidth / 2.0);
-
 	if (Interleave != progdefaults.mt63_interleave) {
 		Interleave = progdefaults.mt63_interleave;
 		restart();
 	}
-    if (long_integral != progdefaults.mt63_rx_integration) {
-        long_integral = progdefaults.mt63_rx_integration;
-        restart();
-    }
+	if (long_integral != progdefaults.mt63_rx_integration) {
+		long_integral = progdefaults.mt63_rx_integration;
+		restart();
+	}
 
 	if (InpBuff->EnsureSpace(len) == -1) {
 		fprintf(stderr, "mt63_rxprocess: buffer error\n");
@@ -316,7 +310,7 @@ mt63::mt63 (trx_mode mt63_mode) : modem()
 	samplerate = 8000;
 	fragmentsize = 1024;
 
-	init();
+//	init();
 }
 
 mt63::~mt63()
@@ -327,3 +321,8 @@ mt63::~mt63()
 	if (InpLevel) delete InpLevel;
 	if (InpBuff) delete InpBuff;
 }
+
+void mt63::set_freq(double)
+{
+	modem::set_freq(500.0 + bandwidth / 2.0);
+}
