@@ -1,7 +1,7 @@
+// ----------------------------------------------------------------------------
+// olivia.cxx  --  OLIVIA modem
 //
-// olivia.cc  --  OLIVIA modem
-//
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
 //
 // This file is part of fldigi.  Adapted from code contained in gmfsk source code 
@@ -9,21 +9,21 @@
 //	Copyright (C) 2005
 //	Tomi Manninen (oh2bns@sral.fi)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-//
-//
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -86,62 +86,30 @@ void olivia::tx_init(SoundBase *sc)
 	escape = 0;
 }
 
-void olivia::send_preamble()
+void olivia::send_tones()
 {
-	double freqa, freqb;
-    int i, j, sr4 = samplerate / 4;
-    double ampshape[sr4];
-    
-    for (int i = 0; i < sr4; i++) ampshape[i] = 1.0;
-    for (int i = 0; i < sr4 / 4; i++)
-        ampshape[i] = ampshape[sr4 - 1 - i] = 0.5 * (1.0 - cos(M_PI * i / (sr4/4)));
-        
-	if (reverse) { 
-		freqa = txbasefreq + (bandwidth / 2.0); 
-		freqb = txbasefreq - (bandwidth / 2.0); 
-	} else { 
-		freqa = txbasefreq - (bandwidth / 2.0); 
-		freqb = txbasefreq + (bandwidth / 2.0); 
+	if (tone_midfreq != txbasefreq || tone_bw != bandwidth) {
+		double freqa, freqb;
+		tone_bw = bandwidth;
+		tone_midfreq = txbasefreq;
+		if (reverse) { 
+			freqa = tone_midfreq + (tone_bw / 2.0); 
+			freqb = tone_midfreq - (tone_bw / 2.0); 
+		} else { 
+			freqa = tone_midfreq - (tone_bw / 2.0); 
+			freqb = tone_midfreq + (tone_bw / 2.0); 
+		}
+		preamblephase = 0;
+		for (int i = 0; i < SR4; i++)
+			tonebuff[2*SR4 + i] = tonebuff[i] = nco(freqa) * ampshape[i];
+
+		preamblephase = 0;
+		for (int i = 0; i < SR4; i++)
+			tonebuff[3*SR4 + i] = tonebuff[SR4 + i] = nco(freqb) * ampshape[i];
 	}
+	for (int j = 0; j < TONE_DURATION; j += SCBLOCKSIZE)
+		ModulateXmtr(&tonebuff[j], SCBLOCKSIZE);
 
-	for (i = 0, j = 0; i < sr4; i++, j++)
-		outbuf[i] = nco(freqa) * ampshape[j];
-	for (i = sr4, j = 0; i < 2*sr4; i++, j++)
-		outbuf[i] = nco(freqb) * ampshape[j];
-	for (i = 2*sr4, j = 0; i < 3*sr4; i++, j++)
-		outbuf[i] = nco(freqa) * ampshape[j];
-	for (i = 3*sr4, j = 0; i < samplerate; i++, j++)
-		outbuf[i] = nco(freqb) * ampshape[j];
-	ModulateXmtr(outbuf, samplerate);
-}
-
-void olivia::send_postamble()
-{
-	double freqa, freqb;
-	int i, j, sr4 = samplerate / 4;
-    double ampshape[sr4];
-    
-    for (int i = 0; i < sr4; i++) ampshape[i] = 1.0;
-    for (int i = 0; i < sr4 / 8; i++)
-        ampshape[i] = ampshape[sr4 - 1 - i] = 0.5 * (1.0 - cos(M_PI * i / (sr4/8)));
-        
-	if (reverse) { 
-		freqa = txbasefreq + (bandwidth / 2.0); 
-		freqb = txbasefreq - (bandwidth / 2.0); 
-	} else { 
-		freqa = txbasefreq - (bandwidth / 2.0); 
-		freqb = txbasefreq + (bandwidth / 2.0); 
-	}
-
-	for (i = 0, j = 0; i < sr4; i++, j++)
-		outbuf[i] = nco(freqa) * ampshape[j];
-	for (i = sr4, j = 0; i < 2*sr4; i++, j++)
-		outbuf[i] = nco(freqb) * ampshape[j];
-	for (i = 2*sr4, j = 0; i < 3*sr4; i++, j++)
-		outbuf[i] = nco(freqa) * ampshape[j];
-	for (i = 3*sr4, j = 0; i < samplerate; i++, j++)
-		outbuf[i] = nco(freqb) * ampshape[j];
-	ModulateXmtr(outbuf, samplerate);
 }
 
 void olivia::rx_init()
@@ -180,20 +148,22 @@ int olivia::tx_process()
 			restart();
 
 	if (preamblesent != 1) { 
-		send_preamble(); 
-		preamblesent = 1; 
+		send_tones(); 
+		preamblesent = 1;
+		// Olivia Transmitter class requires at least character
+		Tx->PutChar(0);
 	}
 
 // The encoder works with BitsPerSymbol length blocks. If the
 // modem already has that many characters buffered, don't try
 // to read any more. If stopflag is set, we will always read 
 // whatever there is.
-
 	if (stopflag || (Tx->GetReadReady() < Tx->BitsPerSymbol)) {
-		if ((c = get_tx_char()) == 0x03 || stopflag ) {
+		if (!stopflag && (c = get_tx_char()) == 0x03)
 			stopflag = true;
+		if (stopflag)
 			Tx->Stop();
-		} else {
+		else {
 			/* Replace un-representable characters with a dot */
 			if (c == -1)
                 c = 0;
@@ -216,12 +186,12 @@ int olivia::tx_process()
 
 	if (stopflag && Tx->DoPostambleYet() == 1 && postamblesent != 1) {
 		postamblesent = 1; 
-		preamblephase = Tx->GetSymbolPhase(); 
-		send_postamble();
+		send_tones();
 	}
 
 	if (!Tx->Running()) {
 		cwid();
+		stopflag = false;
 		return -1;
 	}
  
@@ -233,7 +203,7 @@ int olivia::rx_process(const double *buf, int len)
 {
 	int c;
 	unsigned char ch = 0;
-	double snr;
+	static double snr = 1e-3;
 	static char msg1[20];
 	static char msg2[20];
 
@@ -243,50 +213,50 @@ int olivia::rx_process(const double *buf, int len)
 		sinteg	!= progdefaults.oliviasinteg )
 			restart();
 
-	if ((lastfreq != frequency || Rx->Reverse != 0) && !reverse) {
+	if ((lastfreq != frequency || Rx->Reverse) && !reverse) {
 		Rx->FirstCarrierMultiplier = (frequency - (Rx->Bandwidth / 2)) / 500; 
 		Rx->Reverse = 0;
 		lastfreq = frequency;
 		Rx->Preset();
 	}
-	else if ((lastfreq != frequency || Rx->Reverse != 1) && reverse) {
+	else if ((lastfreq != frequency || !Rx->Reverse) && reverse) {
 		Rx->FirstCarrierMultiplier = (frequency + (Rx->Bandwidth / 2)) / 500; 
 		Rx->Reverse = 1;
 		lastfreq = frequency;
 		Rx->Preset();
 	}
 
-	Rx->SyncThreshold = progStatus.sqlonoff ? progStatus.sldrSquelchValue / 2.0 : 0.0;
+	Rx->SyncThreshold = progStatus.sqlonoff ? 
+		clamp(progStatus.sldrSquelchValue / 5.0 + 3.0, 0, 90.0) : 0.0;
 
     Rx->Process(buf, len);
+	sp = 0;
+	for (int i = frequency - Rx->Bandwidth/2; i < frequency - 1 + Rx->Bandwidth/2; i++)
+		if (wf->Pwr(i) > sp)
+			sp = wf->Pwr(i);
+	np = wf->Pwr(static_cast<int>(frequency + Rx->Bandwidth/2 + 2*Rx->Bandwidth/Rx->Tones));
+	if (np == 0) np = sp + 1e-8;
+	sigpwr = decayavg( sigpwr, sp, 10);
+	noisepwr = decayavg( noisepwr, np, 50);
+	snr = CLAMP(sigpwr / noisepwr, 0.001, 100000);
+
+	metric = clamp( 5.0 * (Rx->SignalToNoiseRatio() - 3.0), 0, 100);
+	display_metric(metric);
 
-	
 	bool gotchar = false;
 	while (Rx->GetChar(ch) > 0) {
 		if ((c = unescape(ch)) != -1 && c > 7) {
-    		put_rx_char(c);
-            gotchar = true;
-        }
+			put_rx_char(c);
+			gotchar = true;
+		}
     }
-    if (gotchar) {
-        double noisepwr1 = wf->powerDensity((1.0 * frequency - Rx->Bandwidth / 2.0 - 100.0), 50.0),
-               noisepwr2 = wf->powerDensity((1.0 * frequency + Rx->Bandwidth / 2.0 + 100.0), 50.0),
-               noisepwr = 0,
-               sigpwr   = wf->powerDensity(1.0 * frequency, 1.0 * Rx->Bandwidth);
-        noisepwr = noisepwr1 < noisepwr ? noisepwr1 : noisepwr2;
-        if (noisepwr == 0) noisepwr = 1.0e-10;
-    	metric = decayavg( metric, sigpwr / noisepwr, 8);
-    	
-	    snprintf(msg1, sizeof(msg1), "s/n %4.1f dB", 10.0 * log10(metric));
-	    put_Status1(msg1, 5, STATUS_CLEAR);
+	if (gotchar) {
+		snprintf(msg1, sizeof(msg1), "s/n: %4.1f dB", 10*log10(snr) - 20);
+		put_Status1(msg1, 5, STATUS_CLEAR);
+		snprintf(msg2, sizeof(msg2), "f/o %+4.1f Hz", Rx->FrequencyOffset());
+		put_Status2(msg2, 5, STATUS_CLEAR);
+	}
 
-	    snprintf(msg2, sizeof(msg2), "f/o %+4.1f Hz", Rx->FrequencyOffset());
-        put_Status2(msg2, 5, STATUS_CLEAR);
-
-    	snr = 3 * Rx->SignalToNoiseRatio();
-	    display_metric(snr > 100.0 ? 100.0 : snr );
-    }
-    
 	return 0;
 }
 
@@ -298,9 +268,10 @@ void olivia::restart()
 	sinteg	= progdefaults.oliviasinteg;
 	
 	samplerate = 8000;
-	
+	bandwidth = 125 * (1 << bw);
+
 	Tx->Tones = 2 * (1 << tones);
-	Tx->Bandwidth = 125 * (1 << bw);
+	Tx->Bandwidth = bandwidth;
 	Tx->SampleRate = samplerate;
 	Tx->OutputSampleRate = samplerate;
     txbasefreq = get_txfreq_woffset();
@@ -324,10 +295,11 @@ void olivia::restart()
 	txfbuffer = new double[txbufferlen];
 
 	Rx->Tones = Tx->Tones;
-	Rx->Bandwidth = Tx->Bandwidth;
+	Rx->Bandwidth = bandwidth;
 	Rx->SyncMargin = smargin;
 	Rx->SyncIntegLen = sinteg;
-	Rx->SyncThreshold = progStatus.sqlonoff ? progStatus.sldrSquelchValue : 0.0;
+	Rx->SyncThreshold = progStatus.sqlonoff ? 
+		clamp(progStatus.sldrSquelchValue / 5.0 + 3.0, 0, 90.0) : 0.0;
 
 	Rx->SampleRate = samplerate;
 	Rx->InputSampleRate = samplerate;
@@ -349,18 +321,21 @@ void olivia::restart()
 
 	put_MODEstatus("%s %" PRIuSZ "/%" PRIuSZ "", get_mode_name(), Tx->Tones, Tx->Bandwidth);
 	metric = 0;
+
+	sigpwr = 1e-10; noisepwr = 1e-8;
+	LOG_DEBUG("\nOlivia Rx parameters:\n%s", Rx->PrintParameters());
 }
 
 void olivia::init()
 {
-	modem::init();
 	restart();
+	modem::init();
 	set_scope_mode(Digiscope::BLANK);
 }
 
 olivia::olivia()
 {
-	cap = CAP_REV;
+	cap |= CAP_REV;
 
 	txfbuffer = 0;
 	samplerate = 8000;
@@ -370,7 +345,15 @@ olivia::olivia()
 
 	mode = MODE_OLIVIA;
 	lastfreq = 0;
-	init();
+
+	for (int i = 0; i < SR4; i++) ampshape[i] = 1.0;
+	for (int i = 0; i < SR4 / 8; i++)
+		ampshape[i] = ampshape[SR4 - 1 - i] = 0.5 * (1.0 - cos(M_PI * i / (SR4/8)));
+
+	for (int i = 0; i < TONE_DURATION; i++) tonebuff[i] = 0;
+
+	tone_bw = -1;
+	tone_midfreq = -1;
 }
 
 olivia::~olivia()
