@@ -1,5 +1,27 @@
-//
+// ----------------------------------------------------------------------------
 // hamlib.cxx  --  Hamlib (rig control) interface for fldigi
+//
+// Copyright (C) 2007-2009
+//		Dave Freese, W1HKJ
+// Copyright (C) 2008-2009
+//		Stelios Bounanos, M0GLD
+//
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
+
 #include <config.h>
 
 #include <cstdlib>
@@ -21,7 +43,6 @@
 #include "misc.h"
 
 #include "rigsupport.h"
-#include "rigdialog.h"
 
 #include "stacktrace.h"
 #ifdef __WOE32__
@@ -35,7 +56,7 @@ LOG_FILE_SOURCE(debug::LOG_RIGCONTROL);
 using namespace std;
 
 static pthread_mutex_t	hamlib_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_t	hamlib_thread;
+static pthread_t	*hamlib_thread = 0;
 
 static bool hamlib_exit = false;
 
@@ -44,7 +65,7 @@ static bool hamlib_qsy = false;
 static bool need_freq = false;
 static bool need_mode = false;
 static bool hamlib_bypass = false;
-static bool hamlib_closed = false;
+static bool hamlib_closed = true;//false;
 static 	int hamlib_passes = 20;
 
 static long int hamlib_freq;
@@ -69,60 +90,109 @@ void show_error(const char* msg1, const char* msg2 = 0)
 
 void hamlib_get_defaults()
 {
-    char szParam[40];
+	char szParam[40];
+	int i;
+	Rig testrig;
+	rig_model_t rigmodel;
 
-    progdefaults.HamRigModel = hamlib_get_rig_model(cboHamlibRig->index());
-    
-    xcvr->init(progdefaults.HamRigModel);
-    
-    if (xcvr->getCaps()->port_type != RIG_PORT_SERIAL) {
-        xcvr->close();
-        return;
-    }
-    
-    xcvr->getConf("serial_speed", szParam);
-    progdefaults.HamRigBaudrate = progdefaults.nBaudRate(szParam);
-    mnuBaudRate->value(progdefaults.HamRigBaudrate);
+	rigmodel = hamlib_get_rig_model(cboHamlibRig->index());
+	testrig.init(rigmodel);
 
-    xcvr->getConf("post_write_delay", szParam);
-    sscanf(szParam, "%d", &progdefaults.HamlibWait);
-    cntHamlibWait->value(progdefaults.HamlibWait);
-    
-    xcvr->getConf("write_delay", szParam);
-    sscanf(szParam, "%d", &progdefaults.HamlibWriteDelay);
-    cntHamlibWriteDelay->value(progdefaults.HamlibWriteDelay);
-    
-    xcvr->getConf("timeout", szParam);
-    sscanf(szParam, "%d", &progdefaults.HamlibTimeout);
-    cntHamlibTimeout->value(progdefaults.HamlibTimeout);
-    
-    xcvr->getConf("retry", szParam);
-    sscanf(szParam, "%d", &progdefaults.HamlibRetries);
-    cntHamlibRetries->value(progdefaults.HamlibRetries);
+	if (testrig.getCaps()->port_type != RIG_PORT_SERIAL) {
+		testrig.close();
+		return;
+	}
 
-    xcvr->getConf("rts_state", szParam);
-    if (strcmp(szParam, "ON") == 0)
-        progdefaults.HamlibRTSplus = true;
-    else
-        progdefaults.HamlibRTSplus = false;
-    chkHamlibRTSplus->value(progdefaults.HamlibRTSplus);
+	testrig.getConf("serial_speed", szParam);
+	mnuBaudRate->value(progdefaults.nBaudRate(szParam));
 
-    xcvr->getConf("dtr_state", szParam);
-    if (strcmp(szParam, "ON") == 0)
-        progdefaults.HamlibDTRplus = true;
-    else
-        progdefaults.HamlibDTRplus = false;
-    btnHamlibDTRplus->value(progdefaults.HamlibDTRplus);
-    
-    progdefaults.HamlibRTSCTSflow = false;
-    progdefaults.HamlibXONXOFFflow = false;
-    xcvr->getConf("serial_handshake", szParam);
-    if (strcmp(szParam, "Hardware") == 0) progdefaults.HamlibRTSCTSflow = true;
-    if (strcmp(szParam, "XONXOFF") == 0) progdefaults.HamlibXONXOFFflow = true;
-    chkHamlibRTSCTSflow->value(progdefaults.HamlibRTSCTSflow);
-    chkHamlibXONXOFFflow->value(progdefaults.HamlibXONXOFFflow);
+	testrig.getConf("post_write_delay", szParam);
+	sscanf(szParam, "%d", &i);
+	cntHamlibWait->value(i);
 
-    xcvr->close();
+	testrig.getConf("write_delay", szParam);
+	sscanf(szParam, "%d", &i);
+	cntHamlibWriteDelay->value(i);
+
+	testrig.getConf("timeout", szParam);
+	sscanf(szParam, "%d", &i);
+	cntHamlibTimeout->value(i);
+
+	testrig.getConf("retry", szParam);
+	sscanf(szParam, "%d", &i);
+	cntHamlibRetries->value(i);
+
+	testrig.getConf("rts_state", szParam);
+	chkHamlibRTSplus->value( strcmp(szParam, "ON") == 0 ? true : false);
+
+	testrig.getConf("dtr_state", szParam);
+	btnHamlibDTRplus->value( strcmp(szParam, "ON") == 0 ? true : false);
+
+	testrig.getConf("serial_handshake", szParam);
+	chkHamlibRTSCTSflow->value(strcmp(szParam, "Hardware") == 0 ? true : false);
+	chkHamlibXONXOFFflow->value(strcmp(szParam, "XONXOFF") == 0 ? true : false);
+
+	testrig.getConf("stop_bits", szParam);
+	valHamRigStopbits->value(strcmp(szParam, "1") == 0 ? 1 : 2);
+
+	if (!testrig.canSetPTT()) {
+		btnHamlibCMDptt->value(0);
+		btnHamlibCMDptt->deactivate();
+	} else {
+		btnHamlibCMDptt->value(1);
+		btnHamlibCMDptt->activate();
+	}
+	inpHamlibConfig->value("");
+
+	testrig.close();
+}
+
+void hamlib_restore_defaults()
+{
+	cboHamlibRig->index(hamlib_get_index(progdefaults.HamRigModel));
+	inpRIGdev->value(progdefaults.HamRigDevice.c_str());
+	cntHamlibRetries->value(progdefaults.HamlibRetries);
+	cntHamlibTimeout->value(progdefaults.HamlibTimeout);
+	cntHamlibWriteDelay->value(progdefaults.HamlibWriteDelay);
+	cntHamlibWait->value(progdefaults.HamlibWait);
+	btnHamlibCMDptt->value(progdefaults.HamlibCMDptt);
+	btnHamlibDTRplus->value(progdefaults.HamlibDTRplus);
+	chkHamlibRTSCTSflow->value(progdefaults.HamlibRTSCTSflow);
+	if (chkHamlibRTSCTSflow->value()) chkHamlibRTSplus->deactivate();
+	chkHamlibRTSplus->value(progdefaults.HamlibRTSplus);
+	chkHamlibXONXOFFflow->value(progdefaults.HamlibXONXOFFflow);
+	mnuSideband->value(progdefaults.HamlibSideband);
+	valHamRigStopbits->value(progdefaults.HamRigStopbits);
+	mnuBaudRate->value(progdefaults.HamRigBaudrate);
+	if (xcvr && xcvr->canSetPTT()) btnHamlibCMDptt->activate();
+	else btnHamlibCMDptt->deactivate();
+	btnHamlibCMDptt->value(progdefaults.HamlibCMDptt);
+	inpHamlibConfig->value(progdefaults.HamConfig.c_str());
+
+	btnInitHAMLIB->labelcolor(FL_FOREGROUND_COLOR);
+	btnInitHAMLIB->redraw_label();
+	btnRevertHAMLIB->deactivate();
+	btnRevertHAMLIB->redraw();
+}
+
+void hamlib_init_defaults()
+{
+	progdefaults.HamRigModel = hamlib_get_rig_model(cboHamlibRig->index());
+	progdefaults.HamRigDevice = inpRIGdev->value();
+	progdefaults.HamlibRetries = static_cast<int>(cntHamlibRetries->value());
+	progdefaults.HamlibTimeout = static_cast<int>(cntHamlibTimeout->value());
+	progdefaults.HamlibWriteDelay = static_cast<int>(cntHamlibWriteDelay->value());
+	progdefaults.HamlibWait = static_cast<int>(cntHamlibWait->value());
+	progdefaults.HamlibCMDptt = btnHamlibCMDptt->value();
+	progdefaults.HamlibDTRplus = btnHamlibDTRplus->value();
+	progdefaults.HamlibRTSCTSflow = chkHamlibRTSCTSflow->value();
+	progdefaults.HamlibRTSplus = chkHamlibRTSplus->value();
+	progdefaults.HamlibXONXOFFflow = chkHamlibXONXOFFflow->value();
+	progdefaults.HamlibSideband = mnuSideband->value();
+	progdefaults.HamRigStopbits = static_cast<int>(valHamRigStopbits->value());
+	progdefaults.HamRigBaudrate = mnuBaudRate->value();
+	progdefaults.HamlibCMDptt = btnHamlibCMDptt->value();
+	progdefaults.HamConfig = inpHamlibConfig->value();
 }
 
 bool hamlib_init(bool bPtt)
@@ -132,7 +202,8 @@ bool hamlib_init(bool bPtt)
 	pbwidth_t width;
 
 	hamlib_ptt = bPtt;
-	hamlib_closed = true;
+
+	hamlib_init_defaults();
 
 #ifdef __CYGWIN__
 	string port = progdefaults.HamRigDevice;
@@ -169,15 +240,10 @@ bool hamlib_init(bool bPtt)
 		if (xcvr->getCaps()->port_type == RIG_PORT_SERIAL) {
 			xcvr->setConf("serial_speed", progdefaults.strBaudRate());
 
-			if (progdefaults.HamlibRTSplus)
-				xcvr->setConf("rts_state", "ON");
-            else
-                xcvr->setConf("rts_state", "OFF");
-
 			if (progdefaults.HamlibDTRplus)
 				xcvr->setConf("dtr_state", "ON");
-            else
-                xcvr->setConf("dtr_state", "OFF");
+			else
+				xcvr->setConf("dtr_state", "OFF");
 
 			if (progdefaults.HamlibRTSCTSflow)
 				xcvr->setConf("serial_handshake", "Hardware");
@@ -185,6 +251,15 @@ bool hamlib_init(bool bPtt)
 				xcvr->setConf("serial_handshake", "XONXOFF");
 			else
 				xcvr->setConf("serial_handshake", "None");
+
+			if (!progdefaults.HamlibRTSCTSflow) {
+				if (progdefaults.HamlibRTSplus)
+					xcvr->setConf("rts_state", "ON");
+				else
+					xcvr->setConf("rts_state", "OFF");
+			}
+
+			xcvr->setConf("stop_bits", progdefaults.HamRigStopbits == 1 ? "1" : "2");
 		}
 
 		string::size_type c = progdefaults.HamConfig.find('#');
@@ -200,7 +275,6 @@ bool hamlib_init(bool bPtt)
 				conf += end;
 			}
 		}
-
 		xcvr->open();
 	}
 	catch (const RigException& Ex) {
@@ -209,15 +283,17 @@ bool hamlib_init(bool bPtt)
 		return false;
 	}
 
-	MilliSleep(500);
-
+	LOG_DEBUG("trying frequency request");
 	try {
-		need_freq = true;
-		freq = xcvr->getFreq();
-		if (freq == 0) {
-			xcvr->close();
-			show_error(__func__, "Rig not responding");
-			return false;
+		if ( !xcvr->canGetFreq() ) need_freq = true; // getFreq will return setFreq value
+		else {
+			need_freq = true;
+			freq = xcvr->getFreq();
+			if (freq == 0) {
+				xcvr->close();
+				show_error(__func__, "Rig not responding");
+				return false;
+			}
 		}
 	}
 	catch (const RigException& Ex) {
@@ -225,21 +301,26 @@ bool hamlib_init(bool bPtt)
 		need_freq = false;
 	}
 
-	LOG_INFO("trying mode query");
 	try {
-		need_mode = true;
-		mode = xcvr->getMode(width);
+		if ( !xcvr->canGetMode() ) need_mode = false;
+		else {
+			need_mode = true;
+			mode = xcvr->getMode(width);
+		}
 	}
 	catch (const RigException& Ex) {
 		show_error("Get Mode", Ex.what());
 		need_mode = false;
 	}
-	
+
 	try {
 		if (hamlib_ptt == true) {
-        	LOG_INFO("trying PTT");
-		    xcvr->setPTT(RIG_PTT_OFF);
-        }
+			LOG_VERBOSE("trying PTT");
+			if (!xcvr->canSetPTT())
+				hamlib_ptt = false;
+			else
+				xcvr->setPTT(RIG_PTT_OFF);
+		}
 	}
 	catch (const RigException& Ex) {
 		show_error("Set Ptt", Ex.what());
@@ -248,24 +329,27 @@ bool hamlib_init(bool bPtt)
 
 	if (need_freq == false && need_mode == false && hamlib_ptt == false ) {
 		xcvr->close();
-		LOG_INFO("Failed freq/mode/ptt");
+		LOG_VERBOSE("Failed freq/mode/ptt");
 		return false;
 	}
 
 	hamlib_freq = 0;
-	hamlib_rmode = RIG_MODE_NONE;//RIG_MODE_USB;
+	hamlib_rmode = RIG_MODE_NONE;
 
 	hamlib_exit = false;
 	hamlib_bypass = false;
-	
-	if (pthread_create(&hamlib_thread, NULL, hamlib_loop, NULL) < 0) {
+
+	hamlib_thread = new pthread_t;
+
+	if (pthread_create(hamlib_thread, NULL, hamlib_loop, NULL) < 0) {
 		show_error(__func__, "pthread_create failed");
 		xcvr->close();
+		hamlib_thread = 0;
 		return false;
-	} 
+	}
 
 	init_Hamlib_RigDialog();
-	
+
 	hamlib_closed = false;
 	return true;
 }
@@ -274,25 +358,20 @@ void hamlib_close(void)
 {
 	ENSURE_THREAD(FLMAIN_TID);
 
-	if (hamlib_closed || !xcvr->isOnLine())
+	if (hamlib_closed)
 		return;
 
-	hamlib_exit = true;
-	int count = 20;
-	while (!hamlib_closed) {
-		MilliSleep(50);
-		if (!count--) {
-			show_error(__func__, "Hamlib thread not responding, forcing shutdown");
-			xcvr->close();
-			diediedie();
-		}
-	}
+	pthread_mutex_lock(&hamlib_mutex);
+		hamlib_exit = true;
+	pthread_mutex_unlock(&hamlib_mutex);
 
-	xcvr->close();
-	if (rigcontrol)
-		rigcontrol->hide();
+	pthread_join(*hamlib_thread, NULL);
+	delete hamlib_thread;
+	hamlib_thread = 0;
+
+	if (xcvr->isOnLine()) xcvr->close();
 	wf->USB(true);
-//	wf->setQSY(0);
+
 }
 
 bool hamlib_active(void)
@@ -302,7 +381,7 @@ bool hamlib_active(void)
 
 void hamlib_set_ptt(int ptt)
 {
-	if (xcvr->isOnLine() == false) 
+	if (xcvr->isOnLine() == false)
 		return;
 	if (!hamlib_ptt)
 		return;
@@ -318,21 +397,15 @@ void hamlib_set_ptt(int ptt)
 	pthread_mutex_unlock(&hamlib_mutex);
 }
 
-void hamlib_set_qsy(long long f, long long fmid)
+void hamlib_set_qsy(long long f)
 {
-	if (xcvr->isOnLine() == false) 
+	if (xcvr->isOnLine() == false)
 		return;
 	pthread_mutex_lock(&hamlib_mutex);
 	double fdbl = f;
 	hamlib_qsy = false;
 	try {
 		xcvr->setFreq(fdbl);
-		if (active_modem->freqlocked() == true) {
-			active_modem->set_freqlock(false);
-			active_modem->set_freq((int)fmid);
-			active_modem->set_freqlock(true);
-		} else
-			active_modem->set_freq((int)fmid);
 		wf->rfcarrier(f);
 		wf->movetocenter();
 	}
@@ -351,7 +424,6 @@ int hamlib_setfreq(long f)
 		try {
 			LOG_DEBUG("%ld", f);
 			xcvr->setFreq(f);
-//			wf->rfcarrier(f);//(hamlib_freq);
 		}
 		catch (const RigException& Ex) {
 			show_error("SetFreq", Ex.what());
@@ -416,7 +488,7 @@ static void *hamlib_loop(void *args)
 	long int freq = 0L;
 	rmode_t  numode = RIG_MODE_NONE;
 	bool freqok = false, modeok = false;
-	
+
 	for (;;) {
 		MilliSleep(100);
 		if (hamlib_exit)
@@ -425,7 +497,7 @@ static void *hamlib_loop(void *args)
 			continue;
 // hamlib locked while accessing hamlib serial i/o
 		pthread_mutex_lock(&hamlib_mutex);
-		
+
 		if (need_freq) {
 			freq_t f;
 			try {
@@ -444,7 +516,7 @@ static void *hamlib_loop(void *args)
 		}
 		if (hamlib_exit)
 			break;
-			
+
 		if (need_mode && hamlib_rmode == numode) {
 			try {
 				numode = xcvr->getMode(hamlib_pbwidth);
@@ -467,7 +539,7 @@ static void *hamlib_loop(void *args)
 			show_frequency(hamlib_freq);
 			wf->rfcarrier(hamlib_freq);
 		}
-		
+
 		if (modeok && (hamlib_rmode != numode)) {
 			hamlib_rmode = numode;
 			show_mode(modeString(hamlib_rmode));
@@ -480,7 +552,7 @@ static void *hamlib_loop(void *args)
 					  hamlib_rmode == RIG_MODE_ECSSLSB ||
 					  hamlib_rmode == RIG_MODE_RTTY));
 		}
-		
+
 		if (hamlib_exit)
 			break;
 	}
