@@ -2,10 +2,12 @@
 //
 //    ptt.cxx --  PTT control
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2009
 //		Dave Freese, W1HKJ
-// Copyright (C) 2008
+// Copyright (C) 2008-2009
 //		Stelios Bounanos, M0GLD
+// Copyright (C) 2009
+//		Diane Bruce, VA3DB
 //
 // This file is part of fldigi.  Adapted from code contained in gmfsk source code 
 // distribution.
@@ -14,19 +16,18 @@
 //  Copyright (C) 2004
 //  Lawrence Glaister (ve7it@shaw.ca)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -79,7 +80,7 @@ void PTT::reset(ptt_t dev)
 {
 	close_all();
 
-	LOG_INFO("Setting PTT to %d", dev);
+//	LOG_VERBOSE("Setting PTT to %d", dev);
 
 	switch (pttdev = dev) {
 #if HAVE_UHROUTER
@@ -310,7 +311,7 @@ void PTT::open_parport(void)
 #endif
 
 	if (!isparport) {
-		LOG_INFO("%s: not a supported parallel port device", progdefaults.PTTdev.c_str());
+		LOG_VERBOSE("%s: not a supported parallel port device", progdefaults.PTTdev.c_str());
 		close_parport();
 		pttfd = -1;
 	}
@@ -506,7 +507,7 @@ void PTT::open_uhrouter(void)
 			}
 		}
 	}
-	LOG_INFO("Will try %s", (start == end ? keyers[start].name : "all keyers"));
+	LOG_VERBOSE("Will try %s", (start == end ? keyers[start].name : "all keyers"));
 
 	int uhrfd[2];
 	uhrfd[0] = uhrfd[1] = uhkfd[0] = uhkfd[1] = uhfd[0] = uhfd[1] = -1;
@@ -527,7 +528,7 @@ void PTT::open_uhrouter(void)
 	for (size_t i = start; i < end; i++) {
 		// open keyer
 		if (!get_fifos(uhrfd, &keyers[i].keyer, 1, fifo_name, len) || *fifo_name == '\0') {
-			LOG_INFO("Keyer \"%s\" not found", keyers[i].name);
+			LOG_VERBOSE("Keyer \"%s\" not found", keyers[i].name);
 			continue;
 		}
 
@@ -536,7 +537,7 @@ void PTT::open_uhrouter(void)
 			LOG_ERROR("Could not open keyer %s", keyers[i].name);
 			continue;
 		}
-		LOG_INFO("Opened keyer %s", keyers[i].name);
+		LOG_VERBOSE("Opened keyer %s", keyers[i].name);
 
 		unsigned char port = OPENPTT;
 		if (!get_fifos(uhkfd, &port, 1, fifo_name, len)) {
@@ -548,7 +549,7 @@ void PTT::open_uhrouter(void)
 			continue;
 		}
 
-		LOG_INFO("Successfully opened PTT port of keyer %s", keyers[i].name);
+		LOG_VERBOSE("Successfully opened PTT port of keyer %s", keyers[i].name);
 		break;
 	}
 
@@ -579,7 +580,7 @@ void PTT::set_uhrouter(bool ptt)
 
 	// send command
 	*buf = '0' + ptt;
-	LOG_INFO("Sending PTT=%uc", *buf);
+	LOG_VERBOSE("Sending PTT=%uc", *buf);
 	struct timeval t = { 2, 0 };
 	if (tm_write(uhfd[1], buf, 1, &t) != 1) {
 		LOG_ERROR("Could not set PTT: %s", strerror(errno));
@@ -596,7 +597,7 @@ void PTT::set_uhrouter(bool ptt)
 		LOG_ERROR("No reply to PTT command within %jd seconds", (intmax_t)t.tv_sec);
 		break;
 	default:
-		LOG_INFO("Received \"%s\"", str2hex(buf, n));
+		LOG_VERBOSE("Received \"%s\"", str2hex(buf, n));
 		// last received char should be '1'(?)
 		break;
 	}
