@@ -1,9 +1,24 @@
-// ====================================================================
+// ----------------------------------------------------------------------------
+// rigio.cxx
 //
-// rigio.cxx  -- thread for reading/writing to serial port
-//               to which the transceiver is connected
+// Copyright (C) 2007-2009
+//		Dave Freese, W1HKJ
 //
-// ====================================================================
+// This file is part of fldigi.
+//
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -23,7 +38,6 @@
 #endif
 
 #include "rigsupport.h"
-#include "rigdialog.h"
 #include "rigxml.h"
 #include "trx.h"
 #include "serial.h"
@@ -32,73 +46,67 @@
 #include "threads.h"
 #include "qrunner.h"
 #include "confdialog.h"
+#include "status.h"
 
 LOG_FILE_SOURCE(debug::LOG_RIGCONTROL);
 
 using namespace std;
 
 Cserial rigio;
-static pthread_mutex_t	rigCAT_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t  rigCAT_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t		*rigCAT_thread = 0;
 
-static bool				rigCAT_exit = false;
-static bool				rigCAT_open = false;
-static bool				rigCAT_bypass = false;
-static unsigned char	replybuff[200];
-static unsigned char	sendbuff[200];
+static bool			 rigCAT_exit = false;
+static bool			 rigCAT_open = false;
+static bool			 rigCAT_bypass = false;
 
-static string 			sRigWidth = "";
-static string 			sRigMode = "";
+static string		   sRigWidth = "";
+static string		   sRigMode = "";
 static long long		llFreq = 0;
 
 static bool nonCATrig = false;
-static bool noXMLfile = false;
-long long  noCATfreq = 3580000L;
-string noCATmode = "USB";
-string noCATwidth = "";
 
 static void *rigCAT_loop(void *args);
 
-int  readtimeout, readwait;
+#define RXBUFFSIZE 2000
+static unsigned char replybuff[RXBUFFSIZE+1];
+static unsigned char retbuf[3];
 
-bool hexout(const string& s, int retnbr)
+bool sendCommand (string s, int retnbr)
 {
-	LOG_DEBUG("cmd %" PRIuSZ " bytes = %s", s.length(), str2hex(s.data(), s.length()));
-	
-	for (int n = 0; n < progdefaults.RigCatRetries; n++) {	
-		int num = 0;
-		memset(sendbuff,0, 200);
-		for (unsigned int i = 0; i < s.length(); i++)
-			sendbuff[i] = s[i];
-// write the command string
-		rigio.FlushBuffer();
-		rigio.WriteBuffer(sendbuff, s.size());
-		if (progdefaults.RigCatECHO == true) {
-//			MilliSleep(50);
-			num = rigio.ReadBuffer (replybuff, s.size());
-			LOG_DEBUG("echoed %d bytes = %s", num, str2hex(replybuff, num));
-		}
-// wait before trying to read response
-		if ((readwait = progdefaults.RigCatWait) > 0)
-			while (readwait--)
-				MilliSleep(1);
-// read the rig response if one is expected
-		memset (replybuff, 0, 200);
-		if (retnbr > 0) {
-			num = rigio.ReadBuffer (replybuff, retnbr > 200 ? 200 : retnbr);
-			if (num)
-				LOG_DEBUG("read %d bytes: %s", num, str2hex(replybuff, num));
-			else
-				LOG_DEBUG("no reply ... retry # %d", n);
-            if (num == retnbr)
-                return true; // response OK
-		} else
-		    return true; // no response needed
-		if ((readtimeout = progdefaults.RigCatTimeout) > 0)
-			while (readtimeout--)
-				MilliSleep(1);
+	int numwrite = (int)s.length();
+	int readafter = progdefaults.RigCatWait;
+	int numread;
+	int retval;
+	numread = retnbr;
+	if (progdefaults.RigCatECHO) numread += numwrite;
+	readafter =
+		progdefaults.RigCatWait + (int) ceilf (
+			numread * (9 + progdefaults.RigCatStopbits) *
+			1000.0 / rigio.Baud() );
+
+	LOG_DEBUG("%s", str2hex(s.data(), s.length()));
+
+	retval = rigio.WriteBuffer((unsigned char *)s.c_str(), numwrite);
+	if (retval <= 0)
+		LOG_ERROR("Write error %d", retval);
+
+	memset(replybuff, 0, RXBUFFSIZE + 1);
+	numread = 0;
+	MilliSleep( readafter );
+	while (numread < RXBUFFSIZE) {
+		memset(retbuf, 0, 2);
+		if (rigio.ReadBuffer(retbuf, 1) == 0) break;
+		replybuff[numread] = retbuf[0];
+		numread++;
 	}
-	return false; // timed out
+	LOG_DEBUG("reply %s", str2hex(replybuff, numread));
+	if (numread > retnbr) {
+		memmove(replybuff, replybuff + numread - retnbr, retnbr);
+		numread = retnbr;
+	}
+
+	return (numread == retnbr);
 }
 
 string to_bcd_be(long long freq, int len)
@@ -136,7 +144,7 @@ long long fm_bcd (size_t p, int len)
 	if (len & 1) numchars ++;
 	for (i = 0; i < numchars; i++) {
 		f *=10;
-		f += replybuff[p + i] >> 4;
+		f += (replybuff[p + i] >> 4) & 0x0F;
 		f *= 10;
 		f += replybuff[p + i] & 0x0F;
 	}
@@ -274,26 +282,24 @@ long long fm_freqdata(DATA d, size_t p)
 	int num, den;
 	num = (int)(d.resolution * 100);
 	den = 100;
+	long long fret = 0;
 	if (d.dtype == "BCD") {
-		if (d.reverse == true) {
-			// LOG_INFO("BCD reverse");
-			return (long long int)(fm_bcd_be(p, d.size) * num / den);
-		} else {
-			// LOG_INFO("BCD normal");
-			return (long long int)(fm_bcd(p, d.size)  * num / den);
-		}
+		if (d.reverse == true)
+			fret = (long long int)(fm_bcd_be(p, d.size) * num / den);
+		else
+			fret = (long long int)(fm_bcd(p, d.size)  * num / den);
 	} else if (d.dtype == "BINARY") {
 		if (d.reverse == true)
-			return (long long int)(fm_binary_be(p, d.size)  * num / den);
+			fret = (long long int)(fm_binary_be(p, d.size)  * num / den);
 		else
-			return (long long int)(fm_binary(p, d.size)  * num / den);
+			fret = (long long int)(fm_binary(p, d.size)  * num / den);
 	} else if (d.dtype == "DECIMAL") {
 		if (d.reverse == true)
-			return (long long int)(fm_decimal_be(p, d.size)  * num / den);
+			fret = (long long int)(fm_decimal_be(p, d.size)  * num / den);
 		else
-			return (long long int)(fm_decimal(p, d.size)  * num / den);
+			fret = (long long int)(fm_decimal(p, d.size)  * num / den);
 	}
-	return 0;
+	return fret;
 }
 
 long long rigCAT_getfreq()
@@ -301,12 +307,12 @@ long long rigCAT_getfreq()
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
+	size_t p = 0, len1 = 0, len2 = 0, pData = 0;
+	long long f = 0;
 
-	if (nonCATrig == true || noXMLfile) {
-		return noCATfreq;
+	if (nonCATrig) {
+		return progStatus.noCATfreq;
 	}
-	
-	LOG_DEBUG("get frequency");
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -314,10 +320,9 @@ long long rigCAT_getfreq()
 			break;
 		++itrCmd;
 	}
-	
+
 	if (itrCmd == commands.end()) {
-		LOG_DEBUG("Cmd not defined");
-		return -2; // get_freq command is not defined!
+		return progStatus.noCATfreq; // get_freq command is not defined!
 	}
 
 	modeCmd = *itrCmd;
@@ -328,62 +333,62 @@ long long rigCAT_getfreq()
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
 
-	if (modeCmd.info.size() == 0)
-		return 0;
+	if ( !modeCmd.info.size() ) return 0;
 
 	for (list<XMLIOS>::iterator preply = reply.begin(); preply != reply.end(); ++preply) {
 		if (preply->SYMBOL != modeCmd.info)
 			continue;
 
 		XMLIOS  rTemp = *preply;
-		size_t p = 0, pData = 0;
-		size_t len = rTemp.str1.size();
+		len1 = rTemp.str1.size();
+		len2 = rTemp.str2.size();
 
-		// send the command
-		if (hexout(strCmd, rTemp.size ) == false) {
-			LOG_ERROR("Hexout failed");
-			return -1;
-		}
-
-		// check the pre data string
-		if (len) {
-			for (size_t i = 0; i < len; i++) {
-				if ((char)rTemp.str1[i] != (char)replybuff[i]) {
-					LOG_WARN("failed pre data string test @ %" PRIuSZ, i);
-					return 0;
-				}
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			if (n && progdefaults.RigCatTimeout > 0)
+				MilliSleep(progdefaults.RigCatTimeout);
+// send the command
+			if ( !sendCommand(strCmd, rTemp.size) ) {
+				LOG_VERBOSE("sendCommand failed");
+				goto retry_get_freq;
 			}
-			p = len;
-		}
-		if (rTemp.fill1) p += rTemp.fill1;
-		pData = p;
-		if (rTemp.data.dtype == "BCD") {
-			p += rTemp.data.size / 2;
-			if (rTemp.data.size & 1) p++;
-		} else
-			p += rTemp.data.size;
-		// check the post data string
-		len = rTemp.str2.size();
-		if (rTemp.fill2) p += rTemp.fill2;
-		if (len) {
-			for (size_t i = 0; i < len; i++)
-				if ((char)rTemp.str2[i] != (char)replybuff[p + i]) {
-					LOG_WARN("failed post data string test @ %" PRIuSZ, i);
-					return 0;
+// check the pre data string
+			p = 0;
+			pData = 0;
+			if (len1) {
+				for (size_t i = 0; i < len1; i++) {
+					if ((char)rTemp.str1[i] != (char)replybuff[i]) {
+						LOG_VERBOSE("failed pre data string test @ %" PRIuSZ, i);
+						goto retry_get_freq;
+					}
 				}
-		}
-		// convert the data field
-		long long f = fm_freqdata(rTemp.data, pData);
-		if ( f >= rTemp.data.min && f <= rTemp.data.max) {
-			LOG_INFO("freq value returned %ld", (long)f);
-			return f;
-		}
-		else {
-			LOG_WARN("freq value failed %lld", f);
-			return 0;
+				p = len1;
+			}
+			if (rTemp.fill1) p += rTemp.fill1;
+			pData = p;
+			if (rTemp.data.dtype == "BCD") {
+				p += rTemp.data.size / 2;
+				if (rTemp.data.size & 1) p++;
+			} else
+				p += rTemp.data.size;
+// check the post data string
+			if (rTemp.fill2) p += rTemp.fill2;
+
+			if (len2) {
+				for (size_t i = 0; i < len2; i++)
+					if ((char)rTemp.str2[i] != (char)replybuff[p + i]) {
+						LOG_VERBOSE("failed post data string test @ %" PRIuSZ, i);
+						goto retry_get_freq;
+					}
+			}
+// convert the data field
+			f = fm_freqdata(rTemp.data, pData);
+			if ( f >= rTemp.data.min && f <= rTemp.data.max)
+				return f;
+			LOG_VERBOSE("freq: %lld", f);
+retry_get_freq: ;
 		}
 	}
-
+	LOG_WARN("Retries failed");
 	return 0;
 }
 
@@ -393,12 +398,13 @@ void rigCAT_setfreq(long long f)
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
 
-	if (noXMLfile || nonCATrig) {
-		noCATfreq = f;
+	progStatus.noCATfreq = f;
+
+	if (nonCATrig) {
 		return;
 	}
-	
-	LOG_DEBUG("set frequency %lld", f);
+
+//	LOG_DEBUG("set frequency %lld", f);
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -407,12 +413,12 @@ void rigCAT_setfreq(long long f)
 		++itrCmd;
 	}
 	if (itrCmd == commands.end()) {
-		noCATfreq = f;
+		LOG_WARN("SET_FREQ not defined");
 		return;
 	}
-	
+
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
 
@@ -427,30 +433,40 @@ void rigCAT_setfreq(long long f)
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				pthread_mutex_lock(&rigCAT_mutex);
-					hexout(strCmd, rTemp.size);
-				pthread_mutex_unlock(&rigCAT_mutex);
+				bool ok = false;
+				for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+					pthread_mutex_lock(&rigCAT_mutex);
+					ok = sendCommand(strCmd, rTemp.size);
+					pthread_mutex_unlock(&rigCAT_mutex);
+					if (ok) return;
+				}
 				return;
 			}
 			preply++;
 		}
 	} else {
-		pthread_mutex_lock(&rigCAT_mutex);
-			hexout(strCmd, 0);
-		pthread_mutex_unlock(&rigCAT_mutex);
+		bool ok = false;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			pthread_mutex_lock(&rigCAT_mutex);
+			ok = sendCommand(strCmd, 0);
+			pthread_mutex_unlock(&rigCAT_mutex);
+			if (ok) return;
+		}
 	}
+	LOG_WARN("Retries failed");
 }
 
 string rigCAT_getmode()
 {
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
-	string strCmd;
-	
-	if (nonCATrig == true || noXMLfile) 
-		return noCATmode;
+	list<MODE>::iterator mode;
+	list<MODE> *pmode;
+	string strCmd, mData;
+	size_t len;
 
-	LOG_DEBUG("get mode");
+	if (nonCATrig)
+		return progStatus.noCATmode;
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -459,80 +475,81 @@ string rigCAT_getmode()
 		++itrCmd;
 	}
 	if (itrCmd == commands.end())
-		return "";
+		return progStatus.noCATmode;
+
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
 
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
 
-	if (modeCmd.info.size()) {
-		list<XMLIOS>::iterator preply = reply.begin();
-		while (preply != reply.end()) {
-			if (preply->SYMBOL == modeCmd.info) {
-				XMLIOS  rTemp = *preply;
-				size_t p = 0, pData = 0;
-// send the command
-				if (hexout(strCmd, rTemp.size) == false) {
-					return "";
-				}
+	if (!modeCmd.info.size()) return "";
 
-// check the pre data string				
-				size_t len = rTemp.str1.size();
-				if (len) {
-					for (size_t i = 0; i < len; i++)
-						if ((char)rTemp.str1[i] != (char)replybuff[i])
-							return "";
-					p = len;
-				}
-				if (rTemp.fill1) p += rTemp.fill1;
-				pData = p;
-// check the post data string
-				p += rTemp.data.size;
-				len = rTemp.str2.size();
-				if (rTemp.fill2) p += rTemp.fill2;
-				if (len) {
-					for (size_t i = 0; i < len; i++)
-						if ((char)rTemp.str2[i] != (char)replybuff[p + i])
-							return "";
-				}
-// convert the data field
-				string mData = "";
-				for (int i = 0; i < rTemp.data.size; i++)
-					mData += (char)replybuff[pData + i];
-// new for FT100 and the ilk that use bit fields
-				if (rTemp.data.size == 1) {
-					unsigned char d = mData[0];
-					if (rTemp.data.shiftbits)
-						d >>= rTemp.data.shiftbits;
-					d &= rTemp.data.andmask;
-					mData[0] = d;
-				}
-				list<MODE>::iterator mode;
-				list<MODE> *pmode;
-				if (lmodes.empty() == false)
-					pmode = &lmodes;
-				else if (lmodeREPLY.empty() == false)
-					pmode = &lmodeREPLY;
-				else
-					return "";
-				mode = pmode->begin();
-				while (mode != pmode->end()) {
-					if ((*mode).BYTES == mData)
-						break;
-					mode++;
-				}
-				if (mode != pmode->end())
-					return ((*mode).SYMBOL);
-				else
-					return "";
+	for (list<XMLIOS>::iterator preply = reply.begin(); preply != reply.end(); ++preply) {
+		if (preply->SYMBOL != modeCmd.info)
+			continue;
+
+		XMLIOS  rTemp = *preply;
+
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			if (n && progdefaults.RigCatTimeout > 0)
+				MilliSleep(progdefaults.RigCatTimeout);
+			size_t p = 0, pData = 0;
+// send the command
+			if (!sendCommand(strCmd, rTemp.size)) goto retry_get_mode;
+// check the pre data string
+			len = rTemp.str1.size();
+			if (len) {
+				for (size_t i = 0; i < len; i++)
+					if ((char)rTemp.str1[i] != (char)replybuff[i]) {
+						LOG_VERBOSE("failed pre data string test @ %" PRIuSZ, i);
+						goto retry_get_mode;
+					}
+				p = len;
 			}
-			preply++;
+			if (rTemp.fill1) p += rTemp.fill1;
+			pData = p;
+// check the post data string
+			p += rTemp.data.size;
+			len = rTemp.str2.size();
+			if (rTemp.fill2) p += rTemp.fill2;
+			if (len) {
+				for (size_t i = 0; i < len; i++)
+					if ((char)rTemp.str2[i] != (char)replybuff[p + i])
+						goto retry_get_mode;
+			}
+// convert the data field
+			mData = "";
+			for (int i = 0; i < rTemp.data.size; i++)
+				mData += (char)replybuff[pData + i];
+// for FT100 and the ilk that use bit fields
+			if (rTemp.data.size == 1) {
+				unsigned char d = mData[0];
+				if (rTemp.data.shiftbits)
+					d >>= rTemp.data.shiftbits;
+				d &= rTemp.data.andmask;
+				mData[0] = d;
+			}
+			if (lmodes.empty() == false)
+					pmode = &lmodes;
+			else if (lmodeREPLY.empty() == false)
+				pmode = &lmodeREPLY;
+			else
+				goto retry_get_mode;
+			mode = pmode->begin();
+			while (mode != pmode->end()) {
+				if ((*mode).BYTES == mData)
+					break;
+				mode++;
+			}
+			if (mode != pmode->end())
+				return ((*mode).SYMBOL);
+retry_get_mode: ;
 		}
 	}
-
+	LOG_WARN("Retries failed");
 	return "";
 }
 
@@ -541,14 +558,13 @@ void rigCAT_setmode(const string& md)
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
-	
-	if (nonCATrig == true || noXMLfile) {
-		noCATmode = md;
+
+	progStatus.noCATmode = md;
+
+	if (nonCATrig) {
 		return;
 	}
 
-	LOG_DEBUG("set mode %s", md.c_str());
-
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
 		if ((*itrCmd).SYMBOL == "SETMODE")
@@ -556,10 +572,10 @@ void rigCAT_setmode(const string& md)
 		++itrCmd;
 	}
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
-	
+
 	if ( modeCmd.data.size > 0 ) {
 		list<MODE>::iterator mode;
 		list<MODE> *pmode;
@@ -587,30 +603,40 @@ void rigCAT_setmode(const string& md)
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				pthread_mutex_lock(&rigCAT_mutex);
-					hexout(strCmd, rTemp.size);
-				pthread_mutex_unlock(&rigCAT_mutex);
+				bool ok = false;
+				for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+					pthread_mutex_lock(&rigCAT_mutex);
+					ok = sendCommand(strCmd, rTemp.size);
+					pthread_mutex_unlock(&rigCAT_mutex);
+					if (ok) return;
+				}
 				return;
 			}
 			preply++;
 		}
 	} else {
-		pthread_mutex_lock(&rigCAT_mutex);
-			hexout(strCmd, 0);
-		pthread_mutex_unlock(&rigCAT_mutex);
+		bool ok = false;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			pthread_mutex_lock(&rigCAT_mutex);
+			ok = sendCommand(strCmd, 0);
+			pthread_mutex_unlock(&rigCAT_mutex);
+			if (ok) return;
+		}
 	}
+	LOG_WARN("Retries failed");
 }
 
 string rigCAT_getwidth()
 {
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
-	string strCmd;
+	list<BW>::iterator bw;
+	list<BW> *pbw;
+	string strCmd, mData;
+	size_t len = 0, p = 0, pData = 0;
 
-	if (nonCATrig == true || noXMLfile) 
-		return noCATwidth;
-	
-	LOG_DEBUG("get width");
+	if (nonCATrig)
+		return progStatus.noCATwidth;
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -621,78 +647,80 @@ string rigCAT_getwidth()
 	if (itrCmd == commands.end())
 		return "";
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
 
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
 
-	if (modeCmd.info.size()) {
-		list<XMLIOS>::iterator preply = reply.begin();
-		while (preply != reply.end()) {
-			if (preply->SYMBOL == modeCmd.info) {
-				XMLIOS  rTemp = *preply;
-				size_t p = 0, pData = 0;
+	if (!modeCmd.info.size()) return "";
+
+	for (list<XMLIOS>::iterator preply = reply.begin(); preply != reply.end(); ++preply) {
+		if (preply->SYMBOL != modeCmd.info)
+			continue;
+
+		XMLIOS  rTemp = *preply;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			if (n && progdefaults.RigCatTimeout > 0)
+				MilliSleep(progdefaults.RigCatTimeout);
+
+			p = 0;
+			pData = 0;
 // send the command
-				if (hexout(strCmd, rTemp.size) == false) {
-					return "";
-				}
-				
-				
-// check the pre data string				
-				size_t len = rTemp.str1.size();
-				if (len) {
-					for (size_t i = 0; i < len; i++)
-						if ((char)rTemp.str1[i] != (char)replybuff[i])
-							return "";
-					p = pData = len;
-				}
-				if (rTemp.fill1) p += rTemp.fill1;
-				pData = p;
-				p += rTemp.data.size;
-// check the post data string
-				if (rTemp.fill2) p += rTemp.fill2;
-				len = rTemp.str2.size();
-				if (len) {
-					for (size_t i = 0; i < len; i++)
-						if ((char)rTemp.str2[i] != (char)replybuff[p + i])
-							return "";
-				}
-// convert the data field
-				string mData = "";
-				for (int i = 0; i < rTemp.data.size; i++)
-					mData += (char)replybuff[pData + i];
-// new for FT100 and the ilk that use bit fields
-				if (rTemp.data.size == 1) {
-					unsigned char d = mData[0];
-					if (rTemp.data.shiftbits)
-						d >>= rTemp.data.shiftbits;
-					d &= rTemp.data.andmask;
-					mData[0] = d;
-				}
-				list<BW>::iterator bw;
-				list<BW> *pbw;
-				if (lbws.empty() == false)
-					pbw = &lbws;
-				else if (lbwREPLY.empty() == false)
-					pbw = &lbwREPLY;
-				else
-					return "";
-				bw = pbw->begin();
-				while (bw != pbw->end()) {
-					if ((*bw).BYTES == mData)
-						break;
-					bw++;
-				}
-				if (bw != pbw->end())
-					return ((*bw).SYMBOL);
-				else
-					return "";
+			if ( !sendCommand(strCmd, rTemp.size) ) goto retry_get_width;
+// check the pre data string
+			len = rTemp.str1.size();
+			if (len) {
+				for (size_t i = 0; i < len; i++)
+					if ((char)rTemp.str1[i] != (char)replybuff[i]) {
+						LOG_VERBOSE("failed pre data string test @ %" PRIuSZ, i);
+						goto retry_get_width;
+					}
+				p = pData = len;
 			}
-			preply++;
+			if (rTemp.fill1) p += rTemp.fill1;
+			pData = p;
+			p += rTemp.data.size;
+// check the post data string
+			if (rTemp.fill2) p += rTemp.fill2;
+			len = rTemp.str2.size();
+			if (len) {
+				for (size_t i = 0; i < len; i++)
+					if ((char)rTemp.str2[i] != (char)replybuff[p + i])
+						goto retry_get_width;
+			}
+// convert the data field
+			mData = "";
+			for (int i = 0; i < rTemp.data.size; i++)
+				mData += (char)replybuff[pData + i];
+// new for FT100 and the ilk that use bit fields
+			if (rTemp.data.size == 1) {
+				unsigned char d = mData[0];
+				if (rTemp.data.shiftbits)
+					d >>= rTemp.data.shiftbits;
+				d &= rTemp.data.andmask;
+				mData[0] = d;
+			}
+			if (lbws.empty() == false)
+				pbw = &lbws;
+			else if (lbwREPLY.empty() == false)
+				pbw = &lbwREPLY;
+			else
+				goto retry_get_width;
+			bw = pbw->begin();
+			while (bw != pbw->end()) {
+				if ((*bw).BYTES == mData)
+					break;
+				bw++;
+			}
+			if (bw != pbw->end())
+				return ((*bw).SYMBOL);
+retry_get_width: ;
 		}
+		preply++;
 	}
+	LOG_WARN("Retries failed");
 	return "";
 }
 
@@ -701,14 +729,12 @@ void rigCAT_setwidth(const string& w)
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
-	
-	LOG_DEBUG("set width");
 
-	if (noXMLfile) {
-		noCATwidth = w;
+	if (nonCATrig) {
+		progStatus.noCATwidth = w;
 		return;
 	}
-	
+
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
 		if ((*itrCmd).SYMBOL == "SETBW")
@@ -716,14 +742,14 @@ void rigCAT_setwidth(const string& w)
 		++itrCmd;
 	}
 	if (itrCmd == commands.end()) {
-		noCATwidth = w;
+		progStatus.noCATwidth = w;
 		return;
 	}
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
-	
+
 	if ( modeCmd.data.size > 0 ) {
 
 		list<BW>::iterator bw;
@@ -745,25 +771,33 @@ void rigCAT_setwidth(const string& w)
 	}
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
-		
+
 	if (modeCmd.ok.size()) {
 		list<XMLIOS>::iterator preply = reply.begin();
 		while (preply != reply.end()) {
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				pthread_mutex_lock(&rigCAT_mutex);
-					hexout(strCmd, rTemp.size);
-				pthread_mutex_unlock(&rigCAT_mutex);
-				return;
+				bool ok = false;
+				for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+					pthread_mutex_lock(&rigCAT_mutex);
+					ok = sendCommand(strCmd, rTemp.size);
+					pthread_mutex_unlock(&rigCAT_mutex);
+					if (ok) return;
+				}
 			}
 			preply++;
 		}
 	} else {
-		pthread_mutex_lock(&rigCAT_mutex);
-			hexout(strCmd, 0);
-		pthread_mutex_unlock(&rigCAT_mutex);
+		bool ok = false;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			pthread_mutex_lock(&rigCAT_mutex);
+			ok = sendCommand(strCmd, 0);
+			pthread_mutex_unlock(&rigCAT_mutex);
+			if (ok) return;
+		}
 	}
+	LOG_WARN("Retries failed");
 }
 
 void rigCAT_pttON()
@@ -771,11 +805,11 @@ void rigCAT_pttON()
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
-	
-	LOG_INFO("ptt ON");
 
 	rigio.SetPTT(1); // always execute the h/w ptt if enabled
 
+	if (nonCATrig) return;
+
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
 		if ((*itrCmd).SYMBOL == "PTTON")
@@ -785,7 +819,7 @@ void rigCAT_pttON()
 	if (itrCmd == commands.end())
 		return;
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
 	if (modeCmd.str2.empty() == false)
@@ -797,15 +831,27 @@ void rigCAT_pttON()
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				hexout(strCmd, rTemp.size);
+				bool ok = false;
+				for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+					pthread_mutex_lock(&rigCAT_mutex);
+					ok = sendCommand(strCmd, rTemp.size);
+					pthread_mutex_unlock(&rigCAT_mutex);
+					if (ok) return;
+				}
 				return;
 			}
 			preply++;
 		}
 	} else {
-		hexout(strCmd, 0);
+		bool ok = false;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			pthread_mutex_lock(&rigCAT_mutex);
+			ok = sendCommand(strCmd, 0);
+			pthread_mutex_unlock(&rigCAT_mutex);
+			if (ok) return;
+		}
 	}
-	
+	LOG_WARN("Retries failed");
 }
 
 void rigCAT_pttOFF()
@@ -813,10 +859,9 @@ void rigCAT_pttOFF()
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
-	
-	LOG_INFO("ptt OFF");
 
 	rigio.SetPTT(0); // always execute the h/w ptt if enabled
+	if (nonCATrig) return;
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -827,7 +872,7 @@ void rigCAT_pttOFF()
 	if (itrCmd == commands.end())
 		return;
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
 	if (modeCmd.str2.empty() == false)
@@ -839,14 +884,27 @@ void rigCAT_pttOFF()
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				hexout(strCmd, rTemp.size);
+				bool ok = false;
+				for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+					pthread_mutex_lock(&rigCAT_mutex);
+					ok = sendCommand(strCmd, rTemp.size);
+					pthread_mutex_unlock(&rigCAT_mutex);
+					if (ok) return;
+				}
 				return;
 			}
 			preply++;
 		}
 	} else {
-		hexout(strCmd, 0);
+		bool ok = false;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			pthread_mutex_lock(&rigCAT_mutex);
+			ok = sendCommand(strCmd, 0);
+			pthread_mutex_unlock(&rigCAT_mutex);
+			if (ok) return;
+		}
 	}
+	LOG_WARN("Retries failed");
 }
 
 void rigCAT_sendINIT()
@@ -854,11 +912,9 @@ void rigCAT_sendINIT()
 	XMLIOS modeCmd;
 	list<XMLIOS>::iterator itrCmd;
 	string strCmd;
-	
-	if (noXMLfile)
+
+	if (nonCATrig)
 		return;
-		
-	LOG_DEBUG("INIT rig");
 
 	itrCmd = commands.begin();
 	while (itrCmd != commands.end()) {
@@ -869,176 +925,204 @@ void rigCAT_sendINIT()
 	if (itrCmd == commands.end())
 		return;
 	modeCmd = *itrCmd;
-	
+
 	if ( modeCmd.str1.empty() == false)
 		strCmd.append(modeCmd.str1);
 	if (modeCmd.str2.empty() == false)
 		strCmd.append(modeCmd.str2);
-	
+
 	if (modeCmd.ok.size()) {
 		list<XMLIOS>::iterator preply = reply.begin();
 		while (preply != reply.end()) {
 			if (preply->SYMBOL == modeCmd.ok) {
 				XMLIOS  rTemp = *preply;
 // send the command
-				pthread_mutex_lock(&rigCAT_mutex);
-					hexout(strCmd, rTemp.size);
-				pthread_mutex_unlock(&rigCAT_mutex);
+				bool ok = false;
+				for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+					pthread_mutex_lock(&rigCAT_mutex);
+					ok = sendCommand(strCmd, rTemp.size);
+					pthread_mutex_unlock(&rigCAT_mutex);
+					if (ok) return;
+				}
 				return;
 			}
 			preply++;
 		}
 	} else {
-		pthread_mutex_lock(&rigCAT_mutex);
-			hexout(strCmd, 0);
-		pthread_mutex_unlock(&rigCAT_mutex);
+		bool ok = false;
+		for (int n = 0; n < progdefaults.RigCatRetries; n++) {
+			pthread_mutex_lock(&rigCAT_mutex);
+			ok = sendCommand(strCmd, 0);
+			pthread_mutex_unlock(&rigCAT_mutex);
+			if (ok) return;
+		}
 	}
+	LOG_WARN("Retries failed");
 }
 
 void rigCAT_defaults()
 {
-    progdefaults.XmlRigBaudrate = xmlrig.baud;
-    mnuXmlRigBaudrate->value(xmlrig.baud);
-    
-    progdefaults.RigCatRTSplus = xmlrig.rts;
-    btnRigCatRTSplus->value(xmlrig.rts);
-    
-	progdefaults.RigCatDTRplus = xmlrig.dtr;
+	mnuXmlRigBaudrate->value(xmlrig.baud);
+	valRigCatStopbits->value(xmlrig.stopbits);
+	btnRigCatRTSplus->value(xmlrig.rts);
 	btnRigCatDTRplus->value(xmlrig.dtr);
-	
-	progdefaults.RigCatRTSptt = xmlrig.rtsptt;
 	btnRigCatRTSptt->value(xmlrig.rtsptt);
-	
-	progdefaults.RigCatDTRptt = xmlrig.dtrptt;
 	btnRigCatDTRptt->value(xmlrig.dtrptt);
-	
-	progdefaults.RigCatRTSCTSflow = xmlrig.rtscts;
 	chkRigCatRTSCTSflow->value(xmlrig.rtscts);
+	cntRigCatRetries->value(xmlrig.retries);
+	cntRigCatTimeout->value(xmlrig.timeout);
+	cntRigCatWait->value(xmlrig.write_delay);
+	btnRigCatEcho->value(xmlrig.echo);
+	btnRigCatCMDptt->value(xmlrig.cmdptt);
+}
 
-    progdefaults.RigCatRetries = xmlrig.retries;
-    cntRigCatRetries->value(xmlrig.retries);
-    
-    progdefaults.RigCatTimeout = xmlrig.timeout;
-    cntRigCatTimeout->value(xmlrig.timeout);
-    
-    progdefaults.RigCatWait = xmlrig.write_delay;
-    cntRigCatWait->value(xmlrig.write_delay);
-    
-    progdefaults.RigCatECHO = xmlrig.echo;
-    btnRigCatEcho->value(xmlrig.echo);
-    
-    progdefaults.RigCatCMDptt = xmlrig.cmdptt;
-    btnRigCatCMDptt->value(xmlrig.cmdptt);
+void rigCAT_restore_defaults()
+{
+	inpXmlRigDevice->value(progdefaults.XmlRigDevice.c_str());
+	mnuXmlRigBaudrate->value(progdefaults.XmlRigBaudrate);
+	valRigCatStopbits->value(progdefaults.RigCatStopbits);
+	btnRigCatRTSplus->value(progdefaults.RigCatRTSplus);
+	btnRigCatDTRplus->value(progdefaults.RigCatDTRplus);
+	btnRigCatRTSptt->value(progdefaults.RigCatRTSptt);
+	btnRigCatDTRptt->value(progdefaults.RigCatDTRptt);
+	chkRigCatRTSCTSflow->value(progdefaults.RigCatRTSCTSflow);
+	cntRigCatRetries->value(progdefaults.RigCatRetries);
+	cntRigCatTimeout->value(progdefaults.RigCatTimeout);
+	cntRigCatWait->value(progdefaults.RigCatWait);
+	btnRigCatEcho->value(progdefaults.RigCatECHO);
+	btnRigCatCMDptt->value(progdefaults.RigCatCMDptt);
 
+	btnInitRIGCAT->labelcolor(FL_FOREGROUND_COLOR);
+	btnRevertRIGCAT->deactivate();
+	dlgConfig->redraw();
+}
+
+void rigCAT_init_defaults()
+{
+	progdefaults.XmlRigDevice = inpXmlRigDevice->value();
+	progdefaults.XmlRigBaudrate = mnuXmlRigBaudrate->value();
+	progdefaults.RigCatStopbits = static_cast<int>(valRigCatStopbits->value());
+	progdefaults.RigCatRTSplus = btnRigCatRTSplus->value();
+	progdefaults.RigCatDTRplus = btnRigCatDTRplus->value();
+	progdefaults.RigCatRTSptt = btnRigCatRTSptt->value();
+	progdefaults.RigCatDTRptt = btnRigCatDTRptt->value();
+	progdefaults.RigCatRTSCTSflow = chkRigCatRTSCTSflow->value();
+	progdefaults.RigCatRetries = static_cast<int>(cntRigCatRetries->value());
+	progdefaults.RigCatTimeout = static_cast<int>(cntRigCatTimeout->value());
+	progdefaults.RigCatWait = static_cast<int>(cntRigCatWait->value());
+	progdefaults.RigCatECHO = btnRigCatEcho->value();
+	progdefaults.RigCatCMDptt = btnRigCatCMDptt->value();
 }
 
 bool rigCAT_init(bool useXML)
 {
-	
+
 	if (rigCAT_open == true) {
 		LOG_ERROR("RigCAT already open");
 		return false;
 	}
-	
-	noXMLfile = true;
+
 	sRigMode = "";
 	sRigWidth = "";
-	nonCATrig = true;
 
 	if (useXML == true) {
-		if (readRigXML() == false)
-			LOG_ERROR("No rig.xml file present");
-		else {	
-			noXMLfile = false;
-			rigio.Device(progdefaults.XmlRigDevice);
-			rigio.Baud(progdefaults.BaudRate(progdefaults.XmlRigBaudrate));
-			rigio.RTS(progdefaults.RigCatRTSplus);
-			rigio.DTR(progdefaults.RigCatDTRplus);
-			rigio.RTSptt(progdefaults.RigCatRTSptt);
-			rigio.DTRptt(progdefaults.RigCatDTRptt);
-			rigio.RTSCTS(progdefaults.RigCatRTSCTSflow);
-			
-			LOG_INFO("\n\
+		rigCAT_init_defaults();
+		rigio.Device(progdefaults.XmlRigDevice);
+		rigio.Baud(progdefaults.BaudRate(progdefaults.XmlRigBaudrate));
+		rigio.RTS(progdefaults.RigCatRTSplus);
+		rigio.DTR(progdefaults.RigCatDTRplus);
+		rigio.RTSptt(progdefaults.RigCatRTSptt);
+		rigio.DTRptt(progdefaults.RigCatDTRptt);
+		rigio.RTSCTS(progdefaults.RigCatRTSCTSflow);
+		rigio.Stopbits(progdefaults.RigCatStopbits);
+
+		LOG_VERBOSE("\n\
 Serial port parameters:\n\
-device     : %s\n\
+device	 : %s\n\
 baudrate   : %d\n\
-retries    : %d\n\
-timeout    : %d\n\
-wait       : %d\n\
+stopbits   : %d\n\
+retries	: %d\n\
+timeout	: %d\n\
+wait	   : %d\n\
 initial rts: %+d\n\
 use rts ptt: %c\n\
 initial dtr: %+d\n\
 use dtr ptt: %c\n\
 flowcontrol: %c\n\
-echo       : %c\n",
-	        	rigio.Device().c_str(),
-				rigio.Baud(),
-				progdefaults.RigCatRetries, 
-				progdefaults.RigCatTimeout, 
-				progdefaults.RigCatWait,
-				(rigio.RTS() ? +12 : -12), (rigio.RTSptt() ? 'T' : 'F'), 
-				(rigio.DTR() ? +12 : -12), (rigio.DTRptt() ? 'T' : 'F'),
-				(rigio.RTSCTS() ? 'T' : 'F'),
-				progdefaults.RigCatECHO ? 'T' : 'F');
+echo	   : %c\n",
+		rigio.Device().c_str(),
+		rigio.Baud(),
+		rigio.Stopbits(),
+		progdefaults.RigCatRetries,
+		progdefaults.RigCatTimeout,
+		progdefaults.RigCatWait,
+			(rigio.RTS() ? +12 : -12), (rigio.RTSptt() ? 'T' : 'F'),
+			(rigio.DTR() ? +12 : -12), (rigio.DTRptt() ? 'T' : 'F'),
+			(rigio.RTSCTS() ? 'T' : 'F'),
+			progdefaults.RigCatECHO ? 'T' : 'F');
 
-			if (rigio.OpenPort() == false) {
-				LOG_ERROR("Cannot open serial port %s", rigio.Device().c_str());
-				return false;
-			}
-			sRigMode = "";
-			sRigWidth = "";
-	
-			nonCATrig = false;
-	
-			if (rigCAT_getfreq() == -1) { // will set nonCATrig to true if getfreq not spec'd
-				LOG_ERROR("Xcvr Freq request not answered");
-				rigio.ClosePort();
-				return false;
-			} else {
-				rigCAT_sendINIT();
-				init_Xml_RigDialog();
-			}
+		if (rigio.OpenPort() == false) {
+			LOG_ERROR("Cannot open serial port %s", rigio.Device().c_str());
+			nonCATrig = true;
+			init_NoRig_RigDialog();
+			return false;
 		}
-	} else
+		sRigMode = "";
+		sRigWidth = "";
+
+		nonCATrig = false;
+
+		if (rigCAT_getfreq() == -1) { // will set nonCATrig to true if getfreq not spec'd
+			LOG_ERROR("Xcvr Freq request not answered");
+			rigio.ClosePort();
+			nonCATrig = true;
+			init_NoRig_RigDialog();
+			return false;
+		} else {
+			nonCATrig = false;
+			rigCAT_sendINIT();
+			init_Xml_RigDialog();
+		}
+	} else {
+		nonCATrig = true;
 		init_NoRig_RigDialog();
-	
+	}
+
 	llFreq = 0;
 	rigCAT_bypass = false;
 	rigCAT_exit = false;
-	
+
 	rigCAT_thread = new pthread_t;
 
 	if (pthread_create(rigCAT_thread, NULL, rigCAT_loop, NULL) < 0) {
 		LOG_ERROR("pthread_create failed");
 		rigio.ClosePort();
 		return false;
-	} 
+	}
+	LOG_VERBOSE("New thread %p", rigCAT_thread);
 
 	rigCAT_open = true;
+
 	return true;
 }
 
 void rigCAT_close(void)
 {
-	int count = 200;
 	if (rigCAT_open == false)
 		return;
-	rigCAT_exit = true;
-	
-	while (rigCAT_open == true) {
-		MilliSleep(50);
-		count--;
-		if (!count) {
-			LOG_ERROR("RigCAT stuck");
-			pthread_mutex_lock(&rigCAT_mutex);
-			rigio.ClosePort();
-			pthread_mutex_unlock(&rigCAT_mutex);
-			exit(0);
-		}
-	}
+
+	pthread_mutex_lock(&rigCAT_mutex);
+		rigCAT_exit = true;
+	pthread_mutex_unlock(&rigCAT_mutex);
+
+	pthread_join(*rigCAT_thread, NULL);
+
+	LOG_VERBOSE("Deleting thread %p", rigCAT_thread);
 	delete rigCAT_thread;
 	rigCAT_thread = 0;
-	LOG_DEBUG("RigCAT closed, count = %d", count);
+
+	rigio.ClosePort();
+	wf->USB(true);
 }
 
 bool rigCAT_active(void)
@@ -1050,32 +1134,23 @@ void rigCAT_set_ptt(int ptt)
 {
 	if (rigCAT_open == false)
 		return;
-	pthread_mutex_lock(&rigCAT_mutex);
-		if (ptt) {
-			rigCAT_pttON();
-			rigCAT_bypass = true;
-		} else{
-			rigCAT_pttOFF();
-			rigCAT_bypass = false;
-		}
-	pthread_mutex_unlock(&rigCAT_mutex);
+	if (ptt) {
+		rigCAT_pttON();
+		rigCAT_bypass = true;
+	} else{
+		rigCAT_pttOFF();
+		rigCAT_bypass = false;
+	}
 }
 
 #ifndef RIGCATTEST
-void rigCAT_set_qsy(long long f, long long fmid)
+void rigCAT_set_qsy(long long f)
 {
 	if (rigCAT_open == false)
 		return;
 
 	// send new freq to rig
 	rigCAT_setfreq(f);
-
-	if (active_modem->freqlocked() == true) {
-		active_modem->set_freqlock(false);
-		active_modem->set_freq((int)fmid);
-		active_modem->set_freqlock(true);
-	} else
-		active_modem->set_freq((int)fmid);
 	wf->rfcarrier(f);
 	wf->movetocenter();
 }
@@ -1083,8 +1158,8 @@ void rigCAT_set_qsy(long long f, long long fmid)
 
 bool ModeIsLSB(const string& s)
 {
-	if (noXMLfile) {
-		if (s == "LSB")
+	if (nonCATrig) {
+		if (s == "LSB" || s == "PKTLSB" || s == "CW" || s == "RTTY")
 			return true;
 		return false;
 	}
@@ -1105,8 +1180,8 @@ static void *rigCAT_loop(void *args)
 	string sWidth, sMode;
 
 	for (;;) {
-		MilliSleep(100);
-		
+		MilliSleep(200);
+
 		if (rigCAT_bypass == true)
 			continue;
 		if (rigCAT_exit == true)
@@ -1124,12 +1199,12 @@ static void *rigCAT_loop(void *args)
 			sMode = rigCAT_getmode();
 		pthread_mutex_unlock(&rigCAT_mutex);
 
-		if ((freq > 0) && (freq != llFreq)) {
+		if ((freq >= 0) && (freq != llFreq)) {
 			llFreq = freq;
 			show_frequency(freq);
 			wf->rfcarrier(freq);
 		}
-	
+
 		if (sWidth.size() && sWidth != sRigWidth) {
 			sRigWidth = sWidth;
 			show_bw(sWidth);
@@ -1145,10 +1220,7 @@ static void *rigCAT_loop(void *args)
 		}
 	}
 
-	if (rigcontrol)
-		rigcontrol->hide();
 	wf->USB(true);
-//	wf->setQSY(0);
 
 	pthread_mutex_lock(&rigCAT_mutex);
 		LOG_DEBUG("Exiting rigCAT loop, closing serial port");
