@@ -1,12 +1,26 @@
-//########################################################################
+// ----------------------------------------------------------------------------
+// rigsupport.cxx - support functions file
 //
-//  rigsupport.cxx
+// Copyright (C) 2007-2009
+//		Dave Freese, W1HKJ
+// Copyright (C) 2008-2009
+//		Stelios Bounanos, M0GLD
 //
-//  rig control - support functions file
+// This file is part of fldigi.
 //
-//  copywrite David Freese, w1hkj@w1hkj.com
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
-//########################################################################
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -19,7 +33,6 @@
 #include <iterator>
 #include <cstring>
 
-#include "rigdialog.h"
 #include "rigsupport.h"
 #include "rigxml.h"
 #include "rigMEM.h"
@@ -42,7 +55,6 @@ LOG_FILE_SOURCE(debug::LOG_RIGCONTROL);
 
 using namespace std;
 
-Fl_Double_Window *rigcontrol = (Fl_Double_Window *)0;
 string windowTitle;
 
 vector<qrg_mode_t> freqlist;
@@ -76,7 +88,7 @@ typedef enum {
 	RIG_MODE_SAH =          (1<<18),/*!< \c SAH -- Synchronous AM upper (higher) sideband */
 	RIG_MODE_DSB =			(1<<19), /*!< \c DSB -- Double sideband suppressed carrier */
 } rmode_t;
-    
+
 #endif
 
 struct rmode_name_t {
@@ -113,11 +125,6 @@ struct rmode_name_t {
 map<string, rmode_t> mode_nums;
 map<rmode_t, string> mode_names;
 
-void selMode(rmode_t m)
-{
-	opMODE->value(mode_names[m].c_str());
-}
-
 void qso_selMode(rmode_t m)
 {
 	qso_opMODE->value(mode_names[m].c_str());
@@ -128,17 +135,9 @@ string modeString(rmode_t m)
 	return mode_names[m].c_str();
 }
 
-void selFreq(long int f)
-{
-	if (FreqDisp)
-		FreqDisp->value(f);
-}
-
 void initOptionMenus()
 {
-	opMODE->clear();
-	if (progdefaults.docked_rig_control)
-		qso_opMODE->clear();
+	qso_opMODE->clear();
 	list<MODE>::iterator MD;
 	list<MODE> *pMD = 0;
 	if (lmodes.empty() == false)
@@ -149,27 +148,17 @@ void initOptionMenus()
 	if (pMD) {
 		MD = pMD->begin();
 		while (MD != pMD->end()) {
-			opMODE->add( (*MD).SYMBOL.c_str());
-			if (progdefaults.docked_rig_control)
-				qso_opMODE->add( (*MD).SYMBOL.c_str());
+			qso_opMODE->add( (*MD).SYMBOL.c_str());
 			MD++;
 		}
-		opMODE->activate();
-		opMODE->index(0);
-		if (progdefaults.docked_rig_control) {
-			qso_opMODE->activate();
-			qso_opMODE->index(0);
-		}
+		qso_opMODE->activate();
+		qso_opMODE->index(0);
 	}
 	else {
-		opMODE->deactivate();
-		if (progdefaults.docked_rig_control)
-			qso_opMODE->deactivate();
+		qso_opMODE->deactivate();
 	}
-	
-	opBW->clear();
-	if (progdefaults.docked_rig_control)
-		qso_opBW->clear();
+
+	qso_opBW->clear();
 	list<BW>::iterator bw;
 	list<BW> *pBW = 0;
 	if (lbws.empty() == false)
@@ -180,42 +169,29 @@ void initOptionMenus()
 	if (pBW) {
 		bw = pBW->begin();
 		while (bw != pBW->end()) {
-			opBW->add( (*bw).SYMBOL.c_str());
-			if (progdefaults.docked_rig_control)
-				qso_opBW->add( (*bw).SYMBOL.c_str());
+			qso_opBW->add( (*bw).SYMBOL.c_str());
 			bw++;
 		}
-		opBW->activate();
-		opBW->index(0);
-		if (progdefaults.docked_rig_control) {
-			qso_opBW->activate();
-			qso_opBW->index(0);
-		}
+		qso_opBW->activate();
+		qso_opBW->index(0);
 	}
 	else {
-		opBW->deactivate();
-		if (progdefaults.docked_rig_control)
-			qso_opBW->deactivate();
+		qso_opBW->deactivate();
 	}
 }
 
 void clearList()
 {
 	freqlist.clear();
-	FreqSelect->clear();
-	if (progdefaults.docked_rig_control)
-		qso_opBrowser->clear();
+	qso_opBrowser->clear();
 }
 
 void updateSelect()
 {
-	FreqSelect->clear();
 	if (freqlist.empty())
 		return;
 	for (size_t i = 0; i < freqlist.size(); i++) {
-		FreqSelect->add(freqlist[i].str().c_str());
-		if (progdefaults.docked_rig_control)
-			qso_opBrowser->add(freqlist[i].str().c_str());
+		qso_opBrowser->add(freqlist[i].str().c_str());
 	}
 }
 
@@ -235,7 +211,7 @@ size_t updateList(long rf, int freq, string rmd, trx_mode md)
 		return pos - freqlist.begin();
 	else
 		return 0;
-	
+
 }
 
 size_t addtoList(long val)
@@ -244,19 +220,14 @@ size_t addtoList(long val)
 
 	m.rfcarrier = val;
 
-	if (progdefaults.docked_rig_control) {
 	if (strlen(qso_opMODE->value()))
 		m.rmode = qso_opMODE->value();
-	} else {
-	if (strlen(opMODE->value()))
-		m.rmode = opMODE->value();
-	}
-	
+
 	if (active_modem) {
 		m.carrier = active_modem->get_freq();
 		m.mode = active_modem->get_mode();
 	}
-	return updateList(val, m.carrier, m.rmode, m.mode);	 
+	return updateList(val, m.carrier, m.rmode, m.mode);
 }
 
 bool readFreqList()
@@ -276,7 +247,7 @@ bool readFreqList()
 	}
 	sort(freqlist.begin(), freqlist.end());
 	updateSelect();
-	
+
 	freqfile.close();
 
 	return freqlist.size();
@@ -306,11 +277,10 @@ void buildlist()
 	memset(fwidths, 0, sizeof(fwidths));
 	// these need to be a little wider than fl_width thinks
 	fwidths[max_rmode] = fwidths[max_mode] = 10;
-	fl_font(FreqSelect->textfont(), FreqSelect->textsize());
 
 	fwidths[max_rfcarrier] += (int)ceil(fl_width("999999.999"));
 
-    fwidths[max_rmode] += (int)ceil(fl_width("XXXX"));
+	fwidths[max_rmode] += (int)ceil(fl_width("XXXX"));
 
 	// find mode with longest shortname
 	size_t s, smax = 0, mmax = 0;
@@ -323,83 +293,52 @@ void buildlist()
 	}
 	fwidths[max_mode] += (int)ceil(fl_width(mode_info[mmax].sname));
 
-	FreqSelect->column_widths(fwidths);
-
-
 	if (readFreqList() == true)
 		return;
 	Fl::lock();
-	updateList (1807000L, 1000, "USB", MODE_BPSK31 );
+	updateList (1807000L, 1000, "USB", MODE_PSK31 );
 	updateList (3505000L, 800, "USB", MODE_CW);
-	updateList (3580000L, 1000, "USB", MODE_BPSK31 );
+	updateList (3580000L, 1000, "USB", MODE_PSK31 );
 	updateList (1000500L, 800, "USB", MODE_CW);
-	updateList (10135000L, 1000, "USB", MODE_BPSK31 );
+	updateList (10135000L, 1000, "USB", MODE_PSK31 );
 	updateList (7005000L, 800, "USB", MODE_CW);
-	updateList (7030000L, 1000, "USB", MODE_BPSK31 );
-	updateList (7070000L, 1000, "USB", MODE_BPSK31 );
+	updateList (7030000L, 1000, "USB", MODE_PSK31 );
+	updateList (7070000L, 1000, "USB", MODE_PSK31 );
 	updateList (14005000L, 800, "USB", MODE_CW);
-	updateList (14070000L, 1000, "USB", MODE_BPSK31 );
-	updateList (18100000L, 1000, "USB", MODE_BPSK31 );
+	updateList (14070000L, 1000, "USB", MODE_PSK31 );
+	updateList (18100000L, 1000, "USB", MODE_PSK31 );
 	updateList (21005000L, 800, "USB", MODE_CW);
-	updateList (21070000L, 1000, "USB", MODE_BPSK31 );
-	updateList (24920000L, 1000, "USB", MODE_BPSK31 );
+	updateList (21070000L, 1000, "USB", MODE_PSK31 );
+	updateList (24920000L, 1000, "USB", MODE_PSK31 );
 	updateList (28005000L, 800, "USB", MODE_CW);
-	updateList (28120000, 1000, "USB", MODE_BPSK31 );
+	updateList (28120000, 1000, "USB", MODE_PSK31 );
 	updateSelect();
-	FreqDisp->value(freqlist[0].rfcarrier);
 	Fl::unlock();
 }
 
-void setMode()
-{
-#if USE_HAMLIB
-	if (progdefaults.chkUSEHAMLIBis)
-		hamlib_setmode(mode_nums[opMODE->value()]);
-	else
-#endif
-	if (progdefaults.chkUSERIGCATis)
-		rigCAT_setmode(opMODE->value());
-	else if (progdefaults.chkUSEMEMMAPis)
-		rigMEM_setmode(opMODE->value());
-	else
-		rigCAT_setmode(opMODE->value());
-}
-
 int cb_qso_opMODE()
 {
-	if (!progdefaults.docked_rig_control) return 0;
-	
+//	if (!progdefaults.docked_rig_control) return 0;
+
 #if USE_HAMLIB
 	if (progdefaults.chkUSEHAMLIBis)
 		hamlib_setmode(mode_nums[qso_opMODE->value()]);
 	else
 #endif
-	if (progdefaults.chkUSERIGCATis)
-		rigCAT_setmode(qso_opMODE->value());
-	else if (progdefaults.chkUSEMEMMAPis)
+	if (progdefaults.chkUSEMEMMAPis)
 		rigMEM_setmode(qso_opMODE->value());
-	else if (!progdefaults.chkUSEXMLRPCis)
+	else
 		rigCAT_setmode(qso_opMODE->value());
 	return 0;
-}
-
-void setBW()
-{
-	rigCAT_setwidth (opBW->value());
+//	if (progdefaults.chkUSERIGCATis || progdefaults.chkUSEXMLRPCis)
+//		rigCAT_setmode(qso_opMODE->value());
+//	else if (progdefaults.chkUSEMEMMAPis)
+//		rigMEM_setmode(qso_opMODE->value());
+//	return 0;
 }
 
 int cb_qso_opBW()
 {
-	if (!progdefaults.docked_rig_control) return 0;
-// hamlib width not yet implemented in fldigi
-//#if USE_HAMLIB
-//	if (progdefaults.chkUSEHAMLIBis)
-//		hamlib_setwidth(qso_opBW->value());
-//	else
-//#endif
-//	if (progdefaults.chkUSERIGCATis || progdefaults.chkUSEMEMMAPis)
-//		rigCAT_setwidth(qso_opBW->value());
-//	else
     if (!progdefaults.chkUSEXMLRPCis)
 		rigCAT_setwidth(qso_opBW->value());
 	return 0;
@@ -420,52 +359,31 @@ void sendFreq(long int f)
 		rigCAT_setfreq(f);
 }
 
-int movFreq()
+void qso_movFreq(Fl_Widget* w, void*)
 {
+	cFreqControl *fc = (cFreqControl *)w;
 	long int f;
-	f = FreqDisp->value();
+	f = fc->value();
+	if (fc == qsoFreqDisp1) {
+		qsoFreqDisp2->value(f);
+		qsoFreqDisp3->value(f);
+	} else if (fc == qsoFreqDisp2) {
+		qsoFreqDisp1->value(f);
+		qsoFreqDisp3->value(f);
+	} else {
+		qsoFreqDisp1->value(f);
+		qsoFreqDisp2->value(f);
+	}
+
 	sendFreq(f);
-	return 0;
-}
-
-int qso_movFreq()
-{
-	if (!progdefaults.docked_rig_control) return 0;
-	long int f;
-	f = qsoFreqDisp->value();
-	sendFreq(f);
-	return 0;
-}
-
-void selectFreq()
-{
-	int n = FreqSelect->value() - 1;
-
-	if (freqlist[n].rmode != "NONE") {
-		opMODE->value(freqlist[n].rmode.c_str());
-		setMode();
-	}
-
-	if (freqlist[n].rfcarrier > 0) {
-		FreqDisp->value(freqlist[n].rfcarrier);
-		sendFreq(freqlist[n].rfcarrier);
-	}
-
-	if (freqlist[n].mode != NUM_MODES) {
-		if (freqlist[n].mode != active_modem->get_mode())
-			init_modem_sync(freqlist[n].mode);
-		if (freqlist[n].carrier > 0)
-			active_modem->set_freq(freqlist[n].carrier);
-	}
+	return;
 }
 
 void qso_selectFreq()
 {
-	if (!progdefaults.docked_rig_control) return;
-	
 	int n = qso_opBrowser->value();
 	if (!n) return;
-	
+
 	n -= 1;
 // transceiver mode
 	if (freqlist[n].rmode != "NONE") {
@@ -474,7 +392,9 @@ void qso_selectFreq()
 	}
 // transceiver frequency
 	if (freqlist[n].rfcarrier > 0) {
-		qsoFreqDisp->value(freqlist[n].rfcarrier);
+		qsoFreqDisp1->value(freqlist[n].rfcarrier);
+		qsoFreqDisp2->value(freqlist[n].rfcarrier);
+		qsoFreqDisp3->value(freqlist[n].rfcarrier);
 		sendFreq(freqlist[n].rfcarrier);
 	}
 // modem type & audio sub carrier
@@ -488,11 +408,9 @@ void qso_selectFreq()
 
 void qso_setFreq()
 {
-	if (!progdefaults.docked_rig_control) return;
-
 	int n = qso_opBrowser->value();
 	if (!n) return;
-	
+
 	n -= 1;
 // transceiver frequency
 	if (freqlist[n].rfcarrier > 0) {
@@ -501,50 +419,21 @@ void qso_setFreq()
 	}
 }
 
-void delFreq()
-{
-	int v = FreqSelect->value() - 1;
-
-	if (v >= 0) {
-		freqlist.erase(freqlist.begin() + v);
-		FreqSelect->remove(v + 1);
-		if (qso_opBrowser)
-			qso_opBrowser->remove(v + 1);
-	}
-}
-
 void qso_delFreq()
 {
-	if (!progdefaults.docked_rig_control) return;
-	
 	int v = qso_opBrowser->value() - 1;
 
 	if (v >= 0) {
 		freqlist.erase(freqlist.begin() + v);
-		FreqSelect->remove(v + 1);
 		qso_opBrowser->remove(v + 1);
 	}
 }
 
-void addFreq()
-{
-	long freq = FreqDisp->value();
-	if (freq) {
-		size_t pos = addtoList(freq);
-		FreqSelect->insert(pos+1, freqlist[pos].str().c_str());
-		if (qso_opBrowser)
-				qso_opBrowser->insert(pos+1, freqlist[pos].str().c_str());
-	}
-}
-
 void qso_addFreq()
 {
-	if (!progdefaults.docked_rig_control) return;
-
 	long freq = qsoFreqDisp->value();
 	if (freq) {
 		size_t pos = addtoList(freq);
-		FreqSelect->insert(pos+1, freqlist[pos].str().c_str());
 		qso_opBrowser->insert(pos+1, freqlist[pos].str().c_str());
 	}
 }
@@ -552,50 +441,33 @@ void qso_addFreq()
 void setTitle()
 {
 	if (windowTitle.length() > 0) {
-//std::cout << windowTitle.c_str() << std::endl;
-		if (rigcontrol) {
-			rigcontrol->label(windowTitle.c_str());
-		}
-		if (progdefaults.docked_rig_control) {
-			txtRigName->label(windowTitle.c_str());
-			txtRigName->redraw_label();
-		}
+		txtRigName->label(windowTitle.c_str());
+		txtRigName->redraw_label();
 	} else {
-		if (rigcontrol) {
-			rigcontrol->label("");
-		}
-		if (progdefaults.docked_rig_control) {
-			txtRigName->label(_("Non CAT mode"));
-			txtRigName->redraw_label();
-		}
+		txtRigName->label();
+		txtRigName->redraw_label();
 	}
 }
 
 bool init_Xml_RigDialog()
 {
-LOG_DEBUG("xml rig");
+	LOG_DEBUG("xml rig");
 	initOptionMenus();
 	clearList();
 	buildlist();
+	windowTitle = xmlrig.rigTitle;
 	setTitle();
 	return true;
 }
 
 bool init_NoRig_RigDialog()
 {
-LOG_DEBUG("no rig");
-	opBW->deactivate();
-	opMODE->clear();
-
-	if (progdefaults.docked_rig_control) {
-		qso_opBW->deactivate();
-		qso_opMODE->clear();
-	}		
+	LOG_DEBUG("no rig");
+	qso_opBW->deactivate();
+	qso_opMODE->clear();
 
 	for (size_t i = 0; i < sizeof(modes)/sizeof(modes[0]); i++) {
-		opMODE->add(modes[i].name);
-		if (progdefaults.docked_rig_control)
-			qso_opMODE->add(modes[i].name);
+		qso_opMODE->add(modes[i].name);
 	}
 	LSBmodes.clear();
 	LSBmodes.push_back("LSB");
@@ -603,100 +475,62 @@ LOG_DEBUG("no rig");
 	LSBmodes.push_back("RTTY");
 	LSBmodes.push_back("PKTLSB");
 
-	opMODE->index(0);
-	opMODE->activate();
-
-	if (progdefaults.docked_rig_control) {
-		qso_opMODE->index(0);
-		qso_opMODE->activate();
-	}
+	qso_opMODE->index(0);
+	qso_opMODE->activate();
 
 	clearList();
 	buildlist();
 
-	windowTitle = _("Rig Not Specified");
+	windowTitle = _("Enter Xcvr Freq");
 	setTitle();
-		
+
 	return true;
 }
 
 bool init_rigMEM_RigDialog()
 {
-LOG_DEBUG("Mem Mapped rig");
-	opBW->deactivate();
-	opMODE->clear();
+	LOG_DEBUG("Mem Mapped rig");
 
-	if (progdefaults.docked_rig_control) {
-		qso_opBW->deactivate();
-		qso_opMODE->clear();
-	}		
+	qso_opBW->deactivate();
+	qso_opMODE->clear();
 
-	opMODE->add("LSB");
-	opMODE->add("USB");
-	opMODE->index(0);
-	opMODE->activate();
-
-	if (progdefaults.docked_rig_control) {
-		qso_opBW->deactivate();
-		qso_opMODE->clear();
-		qso_opMODE->add("LSB");
-		qso_opMODE->add("USB");
-		qso_opMODE->index(0);
-		qso_opMODE->activate();
-	}
+	qso_opBW->deactivate();
+	qso_opMODE->clear();
+	qso_opMODE->add("LSB");
+	qso_opMODE->add("USB");
+	qso_opMODE->index(0);
+	qso_opMODE->activate();
 
 	clearList();
 	buildlist();
 
 	windowTitle = _("Memory Mapped Rig");
 	setTitle();
-		
+
 	return true;
 }
 
 #if USE_HAMLIB
 bool init_Hamlib_RigDialog()
 {
-LOG_DEBUG("hamlib");
-	opBW->deactivate();
-	opMODE->clear();
+	LOG_DEBUG("hamlib");
+
+	qso_opBW->deactivate();
+	qso_opMODE->clear();
 
-	if (progdefaults.docked_rig_control) {
-		qso_opBW->deactivate();
-		qso_opMODE->clear();
-	}		
-	
 	for (size_t i = 0; i < sizeof(modes)/sizeof(modes[0]); i++) {
 		mode_nums[modes[i].name] = modes[i].mode;
 		mode_names[modes[i].mode] = modes[i].name;
-		opMODE->add(modes[i].name);
-		if (progdefaults.docked_rig_control)
-			qso_opMODE->add(modes[i].name);
+		qso_opMODE->add(modes[i].name);
 	}
 	clearList();
 	buildlist();
 
 	windowTitle = "Hamlib ";
 	windowTitle.append(xcvr->getName());
-	
+
 	setTitle();
 
 	return true;
 }
 #endif
-
-Fl_Double_Window *createRigDialog()
-{
-	Fl_Double_Window *w;
-	w = rig_dialog();
-	if (!w) return NULL;
-	FreqDisp->SetONOFFCOLOR(
-		fl_rgb_color(	progdefaults.FDforeground.R, 
-						progdefaults.FDforeground.G, 
-						progdefaults.FDforeground.B),
-		fl_rgb_color(	progdefaults.FDbackground.R, 
-						progdefaults.FDbackground.G, 
-						progdefaults.FDbackground.B));	
-	w->xclass(PACKAGE_NAME);
-	return w;
-}
