@@ -1,12 +1,24 @@
-//########################################################################
+// ----------------------------------------------------------------------------
+// rigxml.cxx - parse a rig control xml file
 //
-//  rigxml.cxx
+// Copyright (C) 2007-2009
+//		Dave Freese, W1HKJ
 //
-//  parse a rig control xml file
+// This file is part of fldigi.
 //
-//  copywrite David Freese, w1hkj@w1hkj.com
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
-//########################################################################
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -16,6 +28,7 @@
 
 #include <FL/filename.H>
 
+#include "gettext.h"
 #include "rigio.h"
 #include "rigxml.h"
 #include "rigsupport.h"
@@ -54,6 +67,7 @@ void parsePOST_WRITE_DELAY(size_t &);
 void parseRETRIES(size_t &);
 void parseTIMEOUT(size_t &);
 void parseBAUDRATE(size_t &);
+void parseSTOPBITS(size_t &);
 void parseRTSCTS(size_t &);
 void parseCMDPTT(size_t &);
 void parseRTSPLUS(size_t &);
@@ -114,7 +128,7 @@ TAGS rigdeftags[] = {
 	{"<MODE-REPLY", parseMODEREPLY},
 	{"<TITLE",		parseTITLE},
 	{"<LSBMODES",	parseLSBMODES},
-	{"<!--",        parseCOMMENTS},
+	{"<!--",		parseCOMMENTS},
 	{"<PROGRAMMER", parseDISCARD},
 	{"<STATUS",		parseDISCARD},
 	{"<WRITE_DELAY", parseWRITE_DELAY},
@@ -129,6 +143,7 @@ TAGS rigdeftags[] = {
 	{"<DTRPTT", parseDTRPTT},
 	{"<ECHO", parseECHO},
 	{"<CMDPTT", parseCMDPTT},
+	{"<STOPBITS", parseSTOPBITS},
 	{0, 0} 
 };
 
@@ -178,7 +193,7 @@ void print(size_t &p0, int indent)
 #ifdef DEBUGXML
 	std::string istr(indent, '\t');
 	size_t tend = strXML.find(">", p0);
-	LOG_INFO("%s%s", istr.c_str(), strXML.substr(p0, tend - p0 + 1).c_str());
+	LOG_VERBOSE("%s%s", istr.c_str(), strXML.substr(p0, tend - p0 + 1).c_str());
 #endif
 }
 
@@ -471,7 +486,7 @@ void parseBWREPLY(size_t &p0)
 void parseTITLE(size_t &p0)
 {
 	size_t pend = tagEnd(p0);
-	windowTitle = getElement(p0);
+	xmlrig.rigTitle = getElement(p0);
 	p0 = pend;
 }
 
@@ -492,43 +507,51 @@ void parseRIG(size_t &p0)
 
 void parseBAUDRATE(size_t &p0)
 {
-    string sVal = getElement(p0);
-    xmlrig.baud = progdefaults.nBaudRate(sVal.c_str());
+	string sVal = getElement(p0);
+	xmlrig.baud = progdefaults.nBaudRate(sVal.c_str());
+	size_t pend = tagEnd(p0);
+	p0 = pend;
+}
+
+void parseSTOPBITS(size_t &p0){
+	int val = getInt(p0);
+	if (val < 0 || val > 2) val = 2;
+	xmlrig.stopbits = val;
 	size_t pend = tagEnd(p0);
 	p0 = pend;
 }
 
 void parseWRITE_DELAY(size_t &p0){
-    int val = getInt(p0);
-    xmlrig.write_delay = val;
-    size_t pend = tagEnd(p0);
+	int val = getInt(p0);
+	xmlrig.write_delay = val;
+	size_t pend = tagEnd(p0);
 	p0 = pend;
 }
 
 void parsePOST_WRITE_DELAY(size_t &p0){
-    int val = getInt(p0);
-    xmlrig.post_write_delay = val;
+	int val = getInt(p0);
+	xmlrig.post_write_delay = val;
 	size_t pend = tagEnd(p0);
 	p0 = pend;
 }
 
 void parseRETRIES(size_t &p0){
-    int val = getInt(p0);
-    xmlrig.retries = val;
+	int val = getInt(p0);
+	xmlrig.retries = val;
 	size_t pend = tagEnd(p0);
 	p0 = pend;
 }
 
 void parseTIMEOUT(size_t &p0){
-    int val = getInt(p0);
-    xmlrig.timeout = val;
+	int val = getInt(p0);
+	xmlrig.timeout = val;
 	size_t pend = tagEnd(p0);
 	p0 = pend;
 }
 
 void parseRTSCTS(size_t &p0){
-    bool val = getBool(p0);
-    xmlrig.rtscts = val;
+	bool val = getBool(p0);
+	xmlrig.rtscts = val;
 	size_t pend = tagEnd(p0);
 	p0 = pend;
 }
@@ -566,17 +589,17 @@ void parseDTRPTT(size_t &p0)
 }
 
 void parseCMDPTT(size_t &p0) {
-    bool val = getBool(p0);
-    xmlrig.cmdptt = val;
-    size_t pend = tagEnd(p0);
-    p0 = pend;
+	bool val = getBool(p0);
+	xmlrig.cmdptt = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
 }
 
 void parseECHO(size_t &p0) {
-    bool val = getBool(p0);
-    xmlrig.echo = val;
-    size_t pend = tagEnd(p0);
-    p0 = pend;
+	bool val = getBool(p0);
+	xmlrig.echo = val;
+	size_t pend = tagEnd(p0);
+	p0 = pend;
 }
 
 //---------------------------------------------------------------------
@@ -843,11 +866,13 @@ void selectRigXmlFilename()
 {
 	string deffilename;
 	deffilename = progdefaults.XmlRigFilename;
-    const char *p = FSEL::select("Open rig xml file", "Fldigi rig xml definition file\t*.xml", deffilename.c_str());
-    if (p) {
-   		progdefaults.XmlRigFilename = p;
-   		txtXmlRigFilename->value(fl_filename_name(p));
-   		readRigXML();
-   		rigCAT_defaults();
+	const char *p = FSEL::select(_("Open rig xml file"), _("Fldigi rig xml definition file\t*.xml"), deffilename.c_str());
+	if (p) {
+		progdefaults.XmlRigFilename = p;
+		txtXmlRigFilename->value(fl_filename_name(p));
+		rigCAT_close();
+		readRigXML();
+		rigCAT_defaults();
 	}
 }
+
