@@ -1,10 +1,24 @@
-//-----------------------------------------------------------------------------
+// ----------------------------------------------------------------------------
+// serial.cxx - Serial I/O class
 //
-// Serial i/o class
+// Copyright (C) 2007-2010
+//		Dave Freese, W1HKJ
 //
-// copyright Dave Freese 2006, w1hkj@w1hkj.com
+// This file is part of fldigi.
 //
-//-----------------------------------------------------------------------------
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Fldigi is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 #include <config.h>
 
@@ -38,21 +52,22 @@ Cserial::Cserial() {
 	rtsptt = dtrptt = false;
 	rtscts = false;
 	status = 0;
+	stopbits = 2;
 	fd = -1;
 }
 
 Cserial::~Cserial() {
-	ClosePort(); 
+	ClosePort();
 }
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::OpenPort
-// Description	    : Opens the port specified by strPortName
-// Return type		: BOOL 
-// Argument         : c_string strPortName
+// Description		: Opens the port specified by strPortName
+// Return type		: BOOL
+// Argument		 : c_string strPortName
 ///////////////////////////////////////////////////////
 bool Cserial::OpenPort()  {
-	
+
 #ifdef __CYGWIN__
 	com_to_tty(device);
 #endif
@@ -63,55 +78,66 @@ bool Cserial::OpenPort()  {
 
 	tcgetattr (fd, &oldtio);
 	newtio = oldtio;
-	
+
+	// 8 data bits
 	newtio.c_cflag &= ~CSIZE;
-	newtio.c_cflag |= CS8 | CLOCAL | CREAD;
+	newtio.c_cflag |= CS8;
+	// enable receiver, set local mode
+	newtio.c_cflag |= (CLOCAL | CREAD);
+	// no parity
+	newtio.c_cflag &= ~PARENB;
 
-    newtio.c_cflag &= ~PARENB;
-
-	if (0)
-		newtio.c_cflag &= ~CSTOPB; // 1 stop bit
+	if (stopbits == 1)
+		// 1 stop bit
+		newtio.c_cflag &= ~CSTOPB;
 	else
-		newtio.c_cflag |= CSTOPB; // 2 stop bit
+		// 2 stop bit
+		newtio.c_cflag |= CSTOPB;
 
 	if (rtscts)
+		// h/w handshake
 		newtio.c_cflag |= CRTSCTS;
 	else
+		// no h/w handshake
 		newtio.c_cflag &= ~CRTSCTS;
 
+	// raw input
 	newtio.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
+	// raw output
 	newtio.c_oflag &= ~OPOST;
+	// software flow control disabled
+	newtio.c_iflag &= ~IXON;
+	// do not translate CR to NL
+	newtio.c_iflag &= ~ICRNL;
 
-    newtio.c_iflag &= ~IXON;
-	
  	switch(baud) {
  		case 300:
 			speed = B300;
-    		break;
+			break;
   		case 1200:
 			speed = B1200;
-    		break;
+			break;
   		case 2400:
 			speed = B2400;
-    		break;
+			break;
   		case 4800:
 			speed = B4800;
-    		break;
+			break;
   		case 9600:
 			speed = B9600;
-    		break;
+			break;
   		case 19200:
 			speed = B19200;
-    		break;
+			break;
   		case 38400:
 			speed = B38400;
-    		break;
+			break;
 		case 57600:
 			speed = B57600;
-    		break;
+			break;
   		case 115200:
 			speed = B115200;
-    		break;
+			break;
   		default:
   			speed = B1200;
   	}
@@ -119,7 +145,7 @@ bool Cserial::OpenPort()  {
 	cfsetospeed(&newtio, speed);
 
 	tcsetattr (fd, TCSANOW, &newtio);
-	
+
 	ioctl(fd, TIOCMGET, &status);
 	origstatus = status;
 
@@ -135,53 +161,53 @@ bool Cserial::OpenPort()  {
 			status &= ~TIOCM_RTS;		// clear the RTS bit
 	}
 	ioctl(fd, TIOCMSET, &status);
-	
+
 	return true;
 }
 
 ///////////////////////////////////////////////////////
-// Function name    : Cserial::setPTT
-// Return type      : void
+// Function name	: Cserial::setPTT
+// Return type	  : void
 ///////////////////////////////////////////////////////
 void Cserial::SetPTT(bool b)
 {
 	if (fd < 0) {
-		LOG_DEBUG("ptt fd < 0");
+		LOG_DEBUG("PTT fd < 0");
 		return;
 	}
 	if (dtrptt || rtsptt) {
-        ioctl(fd, TIOCMGET, &status);
-	    LOG_INFO("h/w ptt %d, Status %X", b, status);
-	    if (b == true) {                                  // ptt enabled
-		    if (dtrptt && dtr)  status &= ~TIOCM_DTR;     // toggle low
-		    if (dtrptt && !dtr) status |= TIOCM_DTR;      // toggle high
-            if (rtscts == false) {
-			    if (rtsptt && rts)  status &= ~TIOCM_RTS; // toggle low
-			    if (rtsptt && !rts) status |= TIOCM_RTS;  // toggle high
-		    }
-	    } else {                                          // ptt disabled
-		    if (dtrptt && dtr)  status |= TIOCM_DTR;      // toggle high
-		    if (dtrptt && !dtr) status &= ~TIOCM_DTR;     // toggle low
-		    if (rtscts == false) {
-			    if (rtsptt && rts)  status |= TIOCM_RTS;  // toggle high
-			    if (rtsptt && !rts) status &= ~TIOCM_RTS; // toggle low
-		    }
-	    }
-	    LOG_INFO("Status %02X, %s", status & 0xFF, uint2bin(status, 8));
-	    ioctl(fd, TIOCMSET, &status);
-    }
-    LOG_DEBUG("No ptt specified");
+		ioctl(fd, TIOCMGET, &status);
+		LOG_DEBUG("H/W PTT %d, status %X", b, status);
+		if (b == true) {					  // ptt enabled
+			if (dtrptt && dtr)  status &= ~TIOCM_DTR;	 // toggle low
+			if (dtrptt && !dtr) status |= TIOCM_DTR;	  // toggle high
+			if (rtscts == false) {
+				if (rtsptt && rts)  status &= ~TIOCM_RTS; // toggle low
+				if (rtsptt && !rts) status |= TIOCM_RTS;  // toggle high
+			}
+		} else {						  // ptt disabled
+			if (dtrptt && dtr)  status |= TIOCM_DTR;	  // toggle high
+			if (dtrptt && !dtr) status &= ~TIOCM_DTR;	 // toggle low
+			if (rtscts == false) {
+				if (rtsptt && rts)  status |= TIOCM_RTS;  // toggle high
+				if (rtsptt && !rts) status &= ~TIOCM_RTS; // toggle low
+			}
+		}
+		LOG_DEBUG("Status %02X, %s", status & 0xFF, uint2bin(status, 8));
+		ioctl(fd, TIOCMSET, &status);
+	}
+	LOG_DEBUG("No PTT specified");
 }
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::ClosePort
-// Description	    : Closes the Port
-// Return type		: void 
+// Description		: Closes the Port
+// Return type		: void
 ///////////////////////////////////////////////////////
 void Cserial::ClosePort()
 {
 	if (fd < 0) return;
-	LOG_INFO("Serial port closed, fd = %d", fd);
+	LOG_DEBUG("Serial port closed, fd = %d", fd);
 	ioctl(fd, TIOCMSET, &origstatus);
 	tcsetattr (fd, TCSANOW, &oldtio);
 	close(fd);
@@ -208,9 +234,9 @@ bool  Cserial::IOselect ()
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::ReadBuffer
-// Description	    : Reads upto nchars from the selected port
-// Return type		: # characters received 
-// Argument         : pointer to buffer; # chars to read
+// Description		: Reads upto nchars from the selected port
+// Return type		: # characters received
+// Argument		 : pointer to buffer; # chars to read
 ///////////////////////////////////////////////////////
 int  Cserial::ReadBuffer (unsigned char *buf, int nchars)
 {
@@ -227,15 +253,15 @@ int  Cserial::ReadBuffer (unsigned char *buf, int nchars)
 			return nread;
 		nread += retnum;
 		nchars -= retnum;
-	}	
+	}
 	return nread;
 }
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::WriteBuffer
-// Description	    : Writes a string to the selected port
-// Return type		: BOOL 
-// Argument         : BYTE by
+// Description		: Writes a string to the selected port
+// Return type		: BOOL
+// Argument		 : BYTE by
 ///////////////////////////////////////////////////////
 int Cserial::WriteBuffer(unsigned char *buff, int n)
 {
@@ -251,7 +277,7 @@ int Cserial::WriteBuffer(unsigned char *buff, int n)
 ///////////////////////////////////////////////////////
 void Cserial::FlushBuffer()
 {
-	if (fd < 0) 
+	if (fd < 0)
 		return;
 	tcflush (fd, TCIFLUSH);
 }
@@ -262,9 +288,9 @@ using namespace std;
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::OpenPort
-// Description	    : Opens the port specified by strPortName
-// Return type		: BOOL 
-// Argument         : CString strPortName
+// Description		: Opens the port specified by strPortName
+// Return type		: BOOL
+// Argument		 : CString strPortName
 ///////////////////////////////////////////////////////
 BOOL Cserial::OpenPort()
 {
@@ -274,17 +300,26 @@ BOOL Cserial::OpenPort()
 	COMportname += device;
 
 	hComm = CreateFile(COMportname.c_str(),
-              GENERIC_READ | GENERIC_WRITE,
-              0,
-              0,
-              OPEN_EXISTING,
-              0,
-              0);
+			  GENERIC_READ | GENERIC_WRITE,
+			  0,
+			  0,
+			  OPEN_EXISTING,
+			  0,
+			  0);
 
-	if(hComm == INVALID_HANDLE_VALUE)
+	if(hComm == INVALID_HANDLE_VALUE) {
+		LOG_ERROR("Invalid handle");
 		return FALSE;
+	}
 
-	ConfigurePort( baud, 8, FALSE, NOPARITY, ONESTOPBIT);
+	if (!ConfigurePort( baud, 8, FALSE, NOPARITY, stopbits)) {
+		CloseHandle(hComm);
+		hComm = INVALID_HANDLE_VALUE;
+		LOG_ERROR("Could not configure port");
+		return FALSE;
+	}
+
+	FlushBuffer();
 
 	return TRUE;
 }
@@ -292,76 +327,39 @@ BOOL Cserial::OpenPort()
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::ClosePort
-// Description	    : Closes the Port
-// Return type		: void 
+// Description		: Closes the Port
+// Return type		: void
 ///////////////////////////////////////////////////////
 void Cserial::ClosePort()
 {
 	if (hComm) {
 		bPortReady = SetCommTimeouts (hComm, &CommTimeoutsSaved);
 		CloseHandle(hComm);
+		hComm = INVALID_HANDLE_VALUE;
 	}
 	return;
 }
 
-
-///////////////////////////////////////////////////////
-// Function name	: Cserial::GetBytesRead
-// Description	    : 
-// Return type		: DWORD 
-///////////////////////////////////////////////////////
-DWORD Cserial::GetBytesRead()
-{
-	return nBytesRead;
-}
-
-///////////////////////////////////////////////////////
-// Function name	: Cserial::GetBytesWritten
-// Description	    : returns total number of bytes written to port
-// Return type		: DWORD 
-///////////////////////////////////////////////////////
-DWORD Cserial::GetBytesWritten()
-{
-	return nBytesWritten;
-}
-
-
-///////////////////////////////////////////////////////
-// Function name	: Cserial::ReadByte
-// Description	    : Reads a byte from the selected port
-// Return type		: BOOL 
-// Argument         : BYTE& by
-///////////////////////////////////////////////////////
-BOOL Cserial::ReadByte(UCHAR& by)
-{
-static	BYTE byResByte[1024];
-static	DWORD dwBytesTxD=0;
-
-	if (!hComm) return FALSE;
-	if (ReadFile (hComm, &byResByte[0], 1, &dwBytesTxD, 0))	{
-		if (dwBytesTxD == 1) {
-			by = (UCHAR)byResByte[0];
-			return TRUE;
-		}
-	}
-	by = 0;
-	return FALSE;
-}
-
 int  Cserial::ReadData (unsigned char *buf, int nchars)
 {
+	if (!hComm)
+	return FALSE;
+
 	DWORD dwRead = 0;
-	if (!ReadFile(hComm, buf, nchars, &dwRead, NULL))
-		return 0;
+	ReadFile(hComm, buf, nchars, &dwRead, NULL);
 	return (int) dwRead;
 }
 
-int Cserial::ReadChars (unsigned char *buf, int nchars, int msec)
+BOOL Cserial::ReadByte(unsigned char & by)
 {
-	if (msec) Sleep(msec);
-	return ReadData (buf, nchars);
+static	BYTE byResByte[2];
+	if (ReadData(byResByte, 1) == 1) {
+		by = byResByte[0];
+		return true;
+	}
+	return false;
 }
-	
+
 void Cserial::FlushBuffer()
 {
 	unsigned char c;
@@ -370,13 +368,15 @@ void Cserial::FlushBuffer()
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::WriteByte
-// Description	    : Writes a Byte to teh selected port
-// Return type		: BOOL 
-// Argument         : BYTE by
+// Description		: Writes a Byte to teh selected port
+// Return type		: BOOL
+// Argument		 : BYTE by
 ///////////////////////////////////////////////////////
 BOOL Cserial::WriteByte(UCHAR by)
 {
-	if (!hComm) return FALSE;
+	if (!hComm)
+		return FALSE;
+
 	nBytesWritten = 0;
 	if (WriteFile(hComm,&by,1,&nBytesWritten,NULL)==0)
 		return FALSE;
@@ -385,30 +385,36 @@ BOOL Cserial::WriteByte(UCHAR by)
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::WriteBuffer
-// Description	    : Writes a string to the selected port
-// Return type		: BOOL 
-// Argument         : BYTE by
+// Description		: Writes a string to the selected port
+// Return type		: BOOL
+// Argument		 : BYTE by
 ///////////////////////////////////////////////////////
 int Cserial::WriteBuffer(unsigned char *buff, int n)
 {
-	if (!hComm) 
-		return 0;
-//	busyflag = true;
+	if (!hComm)
+		return -1;
+
 	WriteFile (hComm, buff, n, &nBytesWritten, NULL);
-//	busyflag = false;
+	if (!nBytesWritten) {
+		LOG_DEBUG("Reopening comm port");
+		ClosePort();
+		OpenPort();
+		WriteFile (hComm, buff, n, &nBytesWritten, NULL);
+	}
+
 	return nBytesWritten;
 }
 
 
 ///////////////////////////////////////////////////////
 // Function name	: Cserial::SetCommunicationTimeouts
-// Description	    : Sets the timeout for the selected port
-// Return type		: BOOL 
-// Argument         : DWORD ReadIntervalTimeout
-// Argument         : DWORD ReadTotalTimeoutMultiplier
-// Argument         : DWORD ReadTotalTimeoutConstant
-// Argument         : DWORD WriteTotalTimeoutMultiplier
-// Argument         : DWORD WriteTotalTimeoutConstant
+// Description		: Sets the timeout for the selected port
+// Return type		: BOOL
+// Argument		 : DWORD ReadIntervalTimeout
+// Argument		 : DWORD ReadTotalTimeoutMultiplier
+// Argument		 : DWORD ReadTotalTimeoutConstant
+// Argument		 : DWORD WriteTotalTimeoutMultiplier
+// Argument		 : DWORD WriteTotalTimeoutConstant
 ///////////////////////////////////////////////////////
 BOOL Cserial::SetCommunicationTimeouts(
 	DWORD ReadIntervalTimeout, // msec
@@ -418,148 +424,150 @@ BOOL Cserial::SetCommunicationTimeouts(
 	DWORD WriteTotalTimeoutConstant
 )
 {
-	if((bPortReady = GetCommTimeouts (hComm, &CommTimeoutsSaved))==0)	{
+	CommTimeouts.ReadIntervalTimeout = ReadIntervalTimeout;
+	CommTimeouts.ReadTotalTimeoutMultiplier = ReadTotalTimeoutMultiplier;
+	CommTimeouts.ReadTotalTimeoutConstant = ReadTotalTimeoutConstant;
+	CommTimeouts.WriteTotalTimeoutConstant = WriteTotalTimeoutConstant;
+	CommTimeouts.WriteTotalTimeoutMultiplier = WriteTotalTimeoutMultiplier;
+
+	LOG_DEBUG("\n\
+Read Interval Timeout............... %8ld %8ld\n\
+Read Total Timeout Multiplier....... %8ld %8ld\n\
+Read Total Timeout Constant Timeout. %8ld %8ld\n\
+Write Total Timeout Constant........ %8ld %8ld\n\
+Write Total Timeout Multiplier...... %8ld %8ld",
+		 CommTimeoutsSaved.ReadIntervalTimeout,
+		 CommTimeouts.ReadIntervalTimeout,
+		 CommTimeoutsSaved.ReadTotalTimeoutMultiplier,
+		 CommTimeouts.ReadTotalTimeoutMultiplier,
+		 CommTimeoutsSaved.ReadTotalTimeoutConstant,
+		 CommTimeouts.ReadTotalTimeoutConstant,
+		 CommTimeoutsSaved.WriteTotalTimeoutConstant,
+		 CommTimeouts.WriteTotalTimeoutConstant,
+		 CommTimeoutsSaved.WriteTotalTimeoutMultiplier,
+		 CommTimeouts.WriteTotalTimeoutMultiplier);
+
+	bPortReady = SetCommTimeouts (hComm, &CommTimeouts);
+
+	if(bPortReady ==0) {
+		CloseHandle(hComm);
+		hComm = INVALID_HANDLE_VALUE;
 		return FALSE;
 	}
-    LOG_INFO("\n\
-COMMTIMOUT Original values:\n\
-    Read Interval Timeout............... %ld\n\
-    Read Total Timeout Multiplier....... %ld\n\
-    Read Total Timeout Constant Timeout. %ld\n\
-    Write Total Timeout Constant........ %ld\n\
-    Write Total Timeout Multiplier...... %ld",
-	     CommTimeoutsSaved.ReadIntervalTimeout,
-	     CommTimeoutsSaved.ReadTotalTimeoutMultiplier,
-	     CommTimeoutsSaved.ReadTotalTimeoutConstant,
-	     CommTimeoutsSaved.WriteTotalTimeoutConstant,
-	     CommTimeoutsSaved.WriteTotalTimeoutMultiplier);
 
-    CommTimeouts.ReadIntervalTimeout = ReadIntervalTimeout;
-    CommTimeouts.ReadTotalTimeoutMultiplier = ReadTotalTimeoutMultiplier;
-    CommTimeouts.ReadTotalTimeoutConstant = ReadTotalTimeoutConstant;
-    CommTimeouts.WriteTotalTimeoutConstant = WriteTotalTimeoutConstant;
-    CommTimeouts.WriteTotalTimeoutMultiplier = WriteTotalTimeoutMultiplier;
-
-    bPortReady = SetCommTimeouts (hComm, &CommTimeouts);
-
-    if(bPortReady ==0) {
-	    CloseHandle(hComm);
-	    return FALSE;
-    }
-
-    return TRUE;
+	return TRUE;
 }
 
-BOOL Cserial::SetCommTimeout() {
 /*
  * ReadIntervalTimeout
- * 
- * The maximum time allowed to elapse between the arrival of two bytes on the 
- * communications line, in milliseconds. During a ReadFile operation, the time 
- * period begins when the first byte is received. If the interval between the 
- * arrival of any two bytes exceeds this amount, the ReadFile operation is 
- * completed and any buffered data is returned. A value of zero indicates that 
+ *
+ * The maximum time allowed to elapse between the arrival of two bytes on the
+ * communications line, in milliseconds. During a ReadFile operation, the time
+ * period begins when the first byte is received. If the interval between the
+ * arrival of any two bytes exceeds this amount, the ReadFile operation is
+ * completed and any buffered data is returned. A value of zero indicates that
  * interval time-outs are not used.
- * 
- * A value of MAXDWORD, combined with zero values for both the 
- * ReadTotalTimeoutConstant and ReadTotalTimeoutMultiplier members, specifies 
- * that the read operation is to return immediately with the bytes that have 
+ *
+ * A value of MAXDWORD, combined with zero values for both the
+ * ReadTotalTimeoutConstant and ReadTotalTimeoutMultiplier members, specifies
+ * that the read operation is to return immediately with the bytes that have
  * already been received, even if no bytes have been received.
- * 
+ *
  * ReadTotalTimeoutMultiplier
- * 
- * The multiplier used to calculate the total time-out period for read 
- * operations, in milliseconds. For each read operation, this value is 
+ *
+ * The multiplier used to calculate the total time-out period for read
+ * operations, in milliseconds. For each read operation, this value is
  * multiplied by the requested number of bytes to be read.
- * 
+ *
  * ReadTotalTimeoutConstant
- * 
+ *
  * A constant used to calculate the total time-out period for read operations,
- * in milliseconds. For each read operation, this value is added to the product 
+ * in milliseconds. For each read operation, this value is added to the product
  * of the ReadTotalTimeoutMultiplier member and the requested number of bytes.
- * 
- * A value of zero for both the ReadTotalTimeoutMultiplier and 
- * ReadTotalTimeoutConstant members indicates that total time-outs are not 
+ *
+ * A value of zero for both the ReadTotalTimeoutMultiplier and
+ * ReadTotalTimeoutConstant members indicates that total time-outs are not
  * used for read operations.
- * 
+ *
  * WriteTotalTimeoutMultiplier
- * 
- * The multiplier used to calculate the total time-out period for write 
- * operations, in milliseconds. For each write operation, this value is 
+ *
+ * The multiplier used to calculate the total time-out period for write
+ * operations, in milliseconds. For each write operation, this value is
  * multiplied by the number of bytes to be written.
- * 
+ *
  * WriteTotalTimeoutConstant
- * 
- * A constant used to calculate the total time-out period for write operations, 
- * in milliseconds. For each write operation, this value is added to the product 
- * of the WriteTotalTimeoutMultiplier member and the number of bytes to be 
+ *
+ * A constant used to calculate the total time-out period for write operations,
+ * in milliseconds. For each write operation, this value is added to the product
+ * of the WriteTotalTimeoutMultiplier member and the number of bytes to be
  * written.
- * 
- * A value of zero for both the WriteTotalTimeoutMultiplier and 
- * WriteTotalTimeoutConstant members indicates that total time-outs are not 
+ *
+ * A value of zero for both the WriteTotalTimeoutMultiplier and
+ * WriteTotalTimeoutConstant members indicates that total time-outs are not
  * used for write operations.
- * 
+ *
  * Remarks
- * 
- * If an application sets ReadIntervalTimeout and ReadTotalTimeoutMultiplier to 
- * MAXDWORD and sets ReadTotalTimeoutConstant to a value greater than zero and 
- * less than MAXDWORD, one of the following occurs when the ReadFile function 
+ *
+ * If an application sets ReadIntervalTimeout and ReadTotalTimeoutMultiplier to
+ * MAXDWORD and sets ReadTotalTimeoutConstant to a value greater than zero and
+ * less than MAXDWORD, one of the following occurs when the ReadFile function
  * is called:
- * 
- * If there are any bytes in the input buffer, ReadFile returns immediately 
+ *
+ * If there are any bytes in the input buffer, ReadFile returns immediately
  * with the bytes in the buffer.
- * 
- * If there are no bytes in the input buffer, ReadFile waits until a byte 
+ *
+ * If there are no bytes in the input buffer, ReadFile waits until a byte
  * arrives and then returns immediately.
- * 
- * If no bytes arrive within the time specified by ReadTotalTimeoutConstant, 
+ *
+ * If no bytes arrive within the time specified by ReadTotalTimeoutConstant,
  * ReadFile times out.
 */
-//  return SetCommunicationTimeouts (0, 500L, 0L, 100L, 0L );
-  return SetCommunicationTimeouts (
-    0L,    // Read Interval Timeout
-    50l,   // Read Total Timeout Multiplier
-    0l,    // Read Total Timeout Constant
-    50L,   // Write Total Timeout Constant
-    0L     // Write Total Timeout Multiplier
-    );
+BOOL Cserial::SetCommTimeout() {
+	return SetCommunicationTimeouts (
+		MAXDWORD, // Read Interval Timeout
+		MAXDWORD, // Read Total Timeout Multiplier
+		10,       // Read Total Timeout Constant
+		50,       // Write Total Timeout Constant
+		5         // Write Total Timeout Multiplier
+	);
   }
 
 ///////////////////////////////////////////////////////
 // Function name	: ConfigurePort
-// Description	    : Configures the Port
-// Return type		: BOOL 
-// Argument         : DWORD BaudRate
-// Argument         : BYTE ByteSize
-// Argument         : DWORD fParity
-// Argument         : BYTE  Parity
-// Argument         : BYTE StopBits
+// Description		: Configures the Port
+// Return type		: BOOL
+// Argument		 : DWORD BaudRate
+// Argument		 : BYTE ByteSize
+// Argument		 : DWORD fParity
+// Argument		 : BYTE  Parity
+// Argument		 : BYTE StopBits
 ///////////////////////////////////////////////////////
 BOOL Cserial::ConfigurePort(DWORD	BaudRate,
-			    BYTE	ByteSize,
-			    DWORD	dwParity,
-			    BYTE	Parity,
-			    BYTE	StopBits)
+				BYTE	ByteSize,
+				DWORD	dwParity,
+				BYTE	Parity,
+				BYTE	StopBits)
 {
 	if((bPortReady = GetCommState(hComm, &dcb))==0) {
-		LOG_ERROR("GetCommState Error on %s", device.c_str());
+		LOG_ERROR("GetCommState error on %s", device.c_str());
 		CloseHandle(hComm);
+		hComm = INVALID_HANDLE_VALUE;
 		return FALSE;
 	}
 
-	dcb.BaudRate            = BaudRate;
-	dcb.ByteSize            = ByteSize;
-	dcb.Parity              = Parity ;
-	dcb.StopBits            = StopBits;
-	dcb.fBinary             = TRUE;
-	dcb.fDsrSensitivity     = FALSE;
-	dcb.fParity             = dwParity;
-	dcb.fOutX               = FALSE;
-	dcb.fInX                = FALSE;
-	dcb.fNull               = FALSE;
-	dcb.fAbortOnError       = TRUE;
-	dcb.fOutxCtsFlow        = FALSE;
-	dcb.fOutxDsrFlow        = FALSE;
+	dcb.BaudRate			= BaudRate;
+	dcb.ByteSize			= ByteSize;
+	dcb.Parity				= Parity ;
+	dcb.StopBits			= (StopBits == 1 ? ONESTOPBIT : TWOSTOPBITS);
+	dcb.fBinary				= TRUE;
+	dcb.fDsrSensitivity		= FALSE;
+	dcb.fParity				= dwParity;
+	dcb.fOutX				= FALSE;
+	dcb.fInX				= FALSE;
+	dcb.fNull				= FALSE;
+	dcb.fAbortOnError		= TRUE;
+	dcb.fOutxCtsFlow		= FALSE;
+	dcb.fOutxDsrFlow		= FALSE;
 
 	if (dtr)
 		dcb.fDtrControl = DTR_CONTROL_ENABLE;
@@ -580,25 +588,31 @@ BOOL Cserial::ConfigurePort(DWORD	BaudRate,
 	bPortReady = SetCommState(hComm, &dcb);
 	if(bPortReady == 0) {
 		CloseHandle(hComm);
+		hComm = INVALID_HANDLE_VALUE;
+		LOG_ERROR("Port not available");
 		return FALSE;
 	}
+
+	if ( (bPortReady = GetCommTimeouts (hComm, &CommTimeoutsSaved) ) == 0)
+		return FALSE;
+
 	return SetCommTimeout();
 }
 
 ///////////////////////////////////////////////////////
-// Function name    : Cserial::setPTT
-// Return type      : void
+// Function name	: Cserial::setPTT
+// Return type	  : void
 ///////////////////////////////////////////////////////
 void Cserial::SetPTT(bool b)
 {
+	if ( !(dtrptt || rtsptt) )
+		return;
 	if(hComm == INVALID_HANDLE_VALUE) {
 		LOG_ERROR("Invalid handle");
 		return;
 	}
-	if ( !(dtrptt || rtsptt) )
-		return;
-	LOG_INFO("PTT = %d, DTRptt = %d, DTR = %d, RTSptt = %d, RTS = %d", 
-		 b, dtrptt, dtr, rtsptt, rts);
+	LOG_DEBUG("PTT = %d, DTRptt = %d, DTR = %d, RTSptt = %d, RTS = %d",
+		  b, dtrptt, dtr, rtsptt, rts);
 
 	if (b == true) {				// ptt enabled
 		if (dtrptt && dtr)
