@@ -2,27 +2,26 @@
 //
 //      sound.cxx
 //
-// Copyright (C) 2006-2007
+// Copyright (C) 2006-2010
 //              Dave Freese, W1HKJ
 //
-// Copyright (C) 2007-2008
+// Copyright (C) 2007-2010
 //              Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -55,6 +54,7 @@
 #  include <dlfcn.h>
 #endif
 
+#include "gettext.h"
 #include "sound.h"
 #include "configuration.h"
 #include "status.h"
@@ -66,62 +66,59 @@
 #include "ringbuffer.h"
 #include "debug.h"
 
+#define	SND_BUF_LEN		65536
+// #define	SRC_BUF_LEN		(8*SND_BUF_LEN)
+
+// We never write duplicate/QSK/PseudoFSK data to the sound files
+#define SNDFILE_CHANNELS 1
+
 using namespace std;
 
 LOG_FILE_SOURCE(debug::LOG_AUDIO);
 
-// Define these constants here to avoid littering the code with magic numbers.
-// Audio input is mono in SoundPort and SoundPulse, stereo in SoundOSS
-#define INPUT_CHANNELS 1
-// Audio output is always stereo: we copy the left channel to the right channel,
-// unless we are in CW or RTTY mode with QSK or PseudoFSK enabled -- this data
-// then goes into the right channel.
-#define OUTPUT_CHANNELS 2
-// We never write duplicate/QSK/PseudoFSK data to the sound files
-#define SNDFILE_CHANNELS 1
 
 SoundBase::SoundBase()
         : sample_frequency(0),
 	  txppm(progdefaults.TX_corr), rxppm(progdefaults.RX_corr),
-          tx_src_state(0), tx_src_data(0), rx_src_state(0), rx_src_data(0),
-          snd_buffer(0), src_buffer(0), wrt_buffer(0),
+          tx_src_state(0), rx_src_state(0),
+          wrt_buffer(new double[SND_BUF_LEN]),
 #if USE_SNDFILE
           ofCapture(0), ifPlayback(0), ofGenerate(0),
 #endif
 	  capture(false), playback(false), generate(false)
-{ }
+{
+	memset(wrt_buffer, 0, SND_BUF_LEN * sizeof(*wrt_buffer));
+}
 
 SoundBase::~SoundBase()
 {
-	if (snd_buffer) delete [] snd_buffer;
-	if (src_buffer) delete [] src_buffer;
-	if (wrt_buffer) delete [] wrt_buffer;
-	if (tx_src_data) delete tx_src_data;
-	if (rx_src_data) delete rx_src_data;
-	if (rx_src_state) src_delete (rx_src_state);
-	if (tx_src_state) src_delete (tx_src_state);
+	delete [] wrt_buffer;
+
 #if USE_SNDFILE
-	if (ofGenerate) sf_close(ofGenerate);
-	if (ofCapture) sf_close(ofCapture);
-	if (ifPlayback) sf_close(ifPlayback);
+	if (ofGenerate)
+		sf_close(ofGenerate);
+	if (ofCapture)
+		sf_close(ofCapture);
+	if (ifPlayback)
+		sf_close(ifPlayback);
 #endif
 }
 
 #if USE_SNDFILE
 void SoundBase::get_file_params(const char* def_fname, const char** fname, int* format)
 {
-	std::string filters = "Waveform Audio Format\t*.wav\n" "AU\t*.{au,snd}\n";
+	std::string filters = _("Waveform Audio Format\t*.wav\n" "AU\t*.{au,snd}\n");
 	int nfilt = 2;
 	if (format_supported(SF_FORMAT_FLAC | SF_FORMAT_PCM_16)) {
-		filters += "Free Lossless Audio Codec\t*.flac";
+		filters += _("Free Lossless Audio Codec\t*.flac");
 		nfilt++;
 	}
 
 	int fsel;
 	if (strstr(def_fname, "playback"))
-		*fname = FSEL::select("Audio file", filters.c_str(), def_fname, &fsel);
+		*fname = FSEL::select(_("Audio file"), filters.c_str(), def_fname, &fsel);
 	else
-		*fname = FSEL::saveas("Audio file", filters.c_str(), def_fname, &fsel);
+		*fname = FSEL::saveas(_("Audio file"), filters.c_str(), def_fname, &fsel);
 	if (!*fname)
 		return;
 
@@ -132,7 +129,7 @@ void SoundBase::get_file_params(const char* def_fname, const char** fname, int*
 		*format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
 		break;
 	case 1:
-		*format = SF_FORMAT_AU | SF_FORMAT_DOUBLE | SF_ENDIAN_CPU;
+		*format = SF_FORMAT_AU | SF_FORMAT_FLOAT | SF_ENDIAN_CPU;
 		break;
 	case 2:
 		*format = SF_FORMAT_FLAC | SF_FORMAT_PCM_16;
@@ -234,13 +231,13 @@ int SoundBase::Generate(bool val)
 	return 1;
 }
 
-sf_count_t SoundBase::read_file(SNDFILE* file, double* buf, size_t count)
+sf_count_t SoundBase::read_file(SNDFILE* file, float* buf, size_t count)
 {
-	sf_count_t r = sf_readf_double(file, buf, count);
+	sf_count_t r = sf_readf_float(file, buf, count);
 
 	while (r < (sf_count_t)count) {
 		sf_seek(file, 0, SEEK_SET);
-		r += sf_readf_double(file, buf + r, count - r);
+		r += sf_readf_float(file, buf + r, count - r);
                 if (r == 0)
                         break;
         }
@@ -248,17 +245,34 @@ sf_count_t SoundBase::read_file(SNDFILE* file, double* buf, size_t count)
 	return r;
 }
 
+#define WRITE_FILE(type_, file_, buf_, count_)					\
+	if (capture || !progdefaults.EnableMixer)				\
+		return sf_writef_ ## type_(file, buf, count);			\
+	/* generated audio; apply transmit mixer level */			\
+	size_t nw = count;							\
+	type_* wrtbuf = (type_*)wrt_buffer;					\
+	for (size_t n = MIN(SND_BUF_LEN, count); count; count -= n) {		\
+		for (size_t i = 0; i < n; i++)					\
+			wrtbuf[i] = buf[i] * progStatus.XmtMixer;		\
+		if (sf_writef_ ## type_(file, wrtbuf, n) != (sf_count_t)n) {	\
+			LOG_ERROR("sf_write error: %s", sf_strerror(file));	\
+			break;							\
+		}								\
+	}									\
+	return nw - count
+
+
+sf_count_t SoundBase::write_file(SNDFILE* file, float* buf, size_t count)
+{
+	WRITE_FILE(float, file, buf, count);
+}
+
 sf_count_t SoundBase::write_file(SNDFILE* file, double* buf, size_t count)
 {
-	if (capture || !progdefaults.EnableMixer)
-		return sf_writef_double(file, buf, count);
-	else {
-		for (size_t n = 0; n < count; n++)
-			wrt_buffer[n] = buf[n] * progStatus.XmtMixer;
-		return sf_write_double(file, wrt_buffer, count);
-	}
+	WRITE_FILE(double, file, buf, count);
 }
 
+
 bool SoundBase::format_supported(int format)
 {
 
@@ -277,7 +291,7 @@ void SoundBase::tag_file(SNDFILE *sndfile, const char *title)
 {
 	int err;
 	if ((err = sf_set_string(sndfile, SF_STR_TITLE, title)) != 0) {
-		LOG_ERROR("sf_set_string STR_TITLE: %s", sf_error_number(err));
+		LOG_INFO("sf_set_string STR_TITLE: %s", sf_error_number(err));
 		return;
 	}
 
@@ -300,6 +314,10 @@ void SoundBase::tag_file(SNDFILE *sndfile, const char *title)
 
 
 #if USE_OSS
+
+#define MAXSC 32767.0f
+#define maxsc 32000.0
+
 SoundOSS::SoundOSS(const char *dev ) {
 	device			= dev;
 	cbuff			= 0;
@@ -314,31 +332,18 @@ SoundOSS::SoundOSS(const char *dev ) {
 		LOG_ERROR("device %s error: %s", device.c_str(), e.what());
 	}
 
-	int err;
-	try {
-		snd_buffer	= new float [2*SND_BUF_LEN];
-		src_buffer	= new float [2*SND_BUF_LEN];
-		wrt_buffer	= new double [SND_BUF_LEN];
-		cbuff		= new unsigned char [4 * SND_BUF_LEN];
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot allocate libsamplerate buffers");
-		throw;
-	}
+	snd_buffer = new float [2 * SND_BUF_LEN];
+	src_buffer = new float [2 * SND_BUF_LEN];
+	cbuff      = new unsigned char [4 * SND_BUF_LEN];
+
 	memset(snd_buffer, 0, 2 * SND_BUF_LEN * sizeof(*snd_buffer));
-	memset(src_buffer, 0, 2 * SND_BUF_LEN * sizeof(*snd_buffer));
-	memset(wrt_buffer, 0, SND_BUF_LEN * sizeof(*snd_buffer));
+	memset(src_buffer, 0, 2 * SND_BUF_LEN * sizeof(*src_buffer));
 	memset(cbuff,      0, 4 * SND_BUF_LEN * sizeof(*cbuff));
 
-	try {
-		tx_src_data = new SRC_DATA;
-		rx_src_data = new SRC_DATA;
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot create libsamplerate data structures");
-		throw;
-	}
+	tx_src_data = new SRC_DATA;
+	rx_src_data = new SRC_DATA;
 
+	int err;
 	rx_src_state = src_new(progdefaults.sample_converter, 2, &err);
 	if (rx_src_state == 0)
 		throw SndException(src_strerror(err));
@@ -357,7 +362,18 @@ SoundOSS::SoundOSS(const char *dev ) {
 SoundOSS::~SoundOSS()
 {
 	Close();
-	if (cbuff) delete [] cbuff;
+
+	delete tx_src_data;
+	delete rx_src_data;
+
+	if (rx_src_state)
+		src_delete(rx_src_state);
+	if (tx_src_state)
+		src_delete(tx_src_state);
+
+	delete [] snd_buffer;
+	delete [] src_buffer;
+	delete [] cbuff;
 }
 
 void SoundOSS::setfragsize()
@@ -490,9 +506,9 @@ int SoundOSS::BufferSize( int seconds )
 
 bool SoundOSS::wait_till_finished()
 {
-	if (ioctl(device_fd, SNDCTL_DSP_POST, 1) == -1 )
+	if (ioctl(device_fd, SNDCTL_DSP_POST, (void*)1) == -1 )
 		return false;
-	if (ioctl(device_fd, SNDCTL_DSP_SYNC, 0) == -1)
+	if (ioctl(device_fd, SNDCTL_DSP_SYNC, (void*)0) == -1)
 		return false; /* format (or ioctl()) not supported by device */
 	return true; /* all sound has been played */
 }
@@ -506,7 +522,7 @@ bool SoundOSS::reset_device()
 	return 1; /* sounddevice has been reset */
 }
 
-size_t SoundOSS::Read(double *buffer, size_t buffersize)
+size_t SoundOSS::Read(float *buffer, size_t buffersize)
 {
 	short int *ibuff = (short int *)cbuff;
 	int numread;
@@ -718,7 +734,7 @@ void SoundPort::initialize(void)
         if ((ndev = Pa_GetDeviceCount()) < 0)
                 throw SndPortException(ndev);
         if (ndev == 0)
-                throw SndPortException("No available audio devices");
+                throw SndException(ENODEV, "No available audio devices");
 
         devs.reserve(ndev);
         for (PaDeviceIndex i = 0; i < ndev; i++)
@@ -750,10 +766,12 @@ const vector<double>& SoundPort::get_supported_rates(const string& name, unsigne
 
 
 SoundPort::SoundPort(const char *in_dev, const char *out_dev)
-        : req_sample_rate(0), fbuf(0)
+        : req_sample_rate(0)
 {
         sd[0].device = in_dev;
         sd[1].device = out_dev;
+	sd[0].params.channelCount = progdefaults.in_channels;
+	sd[1].params.channelCount = progdefaults.out_channels;
         sd[0].stream = sd[1].stream = 0;
         sd[0].frames_per_buffer = sd[1].frames_per_buffer = paFramesPerBufferUnspecified;
         sd[0].dev_sample_rate = sd[1].dev_sample_rate = 0;
@@ -788,31 +806,12 @@ SoundPort::SoundPort(const char *in_dev, const char *out_dev)
 		pthread_cond_init(sd[i].ccond, NULL);
 	}
 
-	try {
-		tx_src_data = new SRC_DATA;
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot create libsamplerate data structures");
-		throw;
-	}
-	try {
-		src_buffer = new float[OUTPUT_CHANNELS * SND_BUF_LEN];
-		fbuf = new float[OUTPUT_CHANNELS * SND_BUF_LEN];
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot allocate libsamplerate buffers");
-		throw;
-	}
-	try {
-		wrt_buffer  = new double[SND_BUF_LEN];
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot allocate write buffer");
-		throw;
-	}
-	memset(src_buffer, 0, OUTPUT_CHANNELS * SND_BUF_LEN * sizeof(*src_buffer));
-	memset(fbuf, 0, OUTPUT_CHANNELS * SND_BUF_LEN * sizeof(*fbuf));
-	memset(wrt_buffer, 0, SND_BUF_LEN * sizeof(*wrt_buffer));
+	tx_src_data = new SRC_DATA;
+	src_buffer = new float[sd[1].params.channelCount * SND_BUF_LEN];
+	fbuf = new float[MAX(sd[0].params.channelCount, sd[1].params.channelCount) * SND_BUF_LEN];
+
+	memset(src_buffer, 0, sd[1].params.channelCount * SND_BUF_LEN * sizeof(*src_buffer));
+	memset(fbuf, 0, MAX(sd[0].params.channelCount, sd[1].params.channelCount) * SND_BUF_LEN * sizeof(*fbuf));
 }
 
 SoundPort::~SoundPort()
@@ -843,6 +842,13 @@ SoundPort::~SoundPort()
 	delete sd[0].rb;
 	delete sd[1].rb;
 
+	if (rx_src_state)
+		src_delete(rx_src_state);
+	if (tx_src_state)
+		src_delete(tx_src_state);
+
+	delete tx_src_data;
+	delete [] src_buffer;
         delete [] fbuf;
 }
 
@@ -854,7 +860,19 @@ int SoundPort::Open(int mode, int freq)
 	// do we need to (re)initialise the streams?
 	int ret = 0;
 	int sr[2] = { progdefaults.in_sample_rate, progdefaults.out_sample_rate };
-	for (size_t i = 0; i < 2; i++) {
+
+	// initialize stream if it is a JACK device, regardless of mode
+	device_iterator idev;
+	if (mode == O_WRONLY && (idev = name_to_device(sd[0].device, 0)) != devs.end() &&
+	    Pa_GetHostApiInfo((*idev)->hostApi)->type == paJACK)
+		mode = O_RDWR;
+	if (mode == O_RDONLY && (idev = name_to_device(sd[1].device, 1)) != devs.end() &&
+	    Pa_GetHostApiInfo((*idev)->hostApi)->type == paJACK)
+		mode = O_RDWR;
+
+	size_t start = (mode == O_RDONLY || mode == O_RDWR) ? 0 : 1,
+		end = (mode == O_WRONLY || mode == O_RDWR) ? 1 : 0;
+	for (size_t i = start; i <= end; i++) {
 		if ( !(stream_active(i) && (Pa_GetHostApiInfo((*sd[i].idev)->hostApi)->type == paJACK ||
 					    old_sample_rate == freq ||
 					    sr[i] != SAMPLE_RATE_AUTO)) ) {
@@ -963,7 +981,7 @@ void SoundPort::Abort(unsigned dir)
         } while (0)
 
 
-size_t SoundPort::Read(double *buf, size_t count)
+size_t SoundPort::Read(float *buf, size_t count)
 {
 #if USE_SNDFILE
         if (playback) {
@@ -984,12 +1002,12 @@ size_t SoundPort::Read(double *buf, size_t count)
 		src_set_ratio(rx_src_state, sd[0].src_ratio);
 	}
 
-	size_t maxframes = (size_t)floor(sd[0].rb->length() * sd[0].src_ratio);
+	size_t maxframes = (size_t)floor(sd[0].rb->length() * sd[0].src_ratio / sd[0].params.channelCount);
         if (unlikely(count > maxframes)) {
                 size_t n = 0;
                 while (count > maxframes) {
                         n += Read(buf, maxframes);
-                        buf += maxframes;
+                        buf += maxframes * sd[0].params.channelCount;
                         count -= maxframes;
                 }
                 if (count > 0)
@@ -1003,34 +1021,39 @@ size_t SoundPort::Read(double *buf, size_t count)
 		size_t n = 0;
 		sd[0].blocksize = SCBLOCKSIZE;
 		while (n < count) {
-			if  ((r = src_callback_read(rx_src_state, sd[0].src_ratio, count - n, rbuf + n)) == 0)
+			if  ((r = src_callback_read(rx_src_state, sd[0].src_ratio,
+						    count - n, rbuf + n * sd[0].params.channelCount)) == 0)
 				throw SndException(ETIMEDOUT);
 			n += r;
 		}
 	}
 	else {
 		bool timeout = false;
-		WAIT_FOR_COND( (sd[0].rb->read_space() >= count / sd[0].src_ratio), sd[0].rwsem,
-				(MAX(1.0, 2 * count / sd->dev_sample_rate)) );
+		WAIT_FOR_COND( (sd[0].rb->read_space() >= count * sd[0].params.channelCount / sd[0].src_ratio), sd[0].rwsem,
+			       (MAX(1.0, 2 * count * sd[0].params.channelCount / sd->dev_sample_rate)) );
 		if (timeout)
 			throw SndException(ETIMEDOUT);
 		ringbuffer<float>::vector_type vec[2];
 		sd[0].rb->get_rv(vec);
-		if (vec[0].len >= count) {
+		if (vec[0].len >= count * sd[0].params.channelCount) {
 			rbuf = vec[0].buf;
 			sd[0].advance = vec[0].len;
 		}
 		else
-			sd[0].rb->read(fbuf, count);
+			sd[0].rb->read(fbuf, count * sd[0].params.channelCount);
 	}
 	if (sd[0].advance) {
 		sd[0].rb->read_advance(sd[0].advance);
 		sd[0].advance = 0;
 	}
 
-        // convert to double
-        for (size_t i = 0; i < count; i++)
-                buf[i] = rbuf[i];
+	if (sd[0].params.channelCount == 1)
+		memcpy(buf, rbuf, count * sizeof(float));
+	else {
+		// write first channel
+		for (size_t i = 0; i < count; i++)
+			buf[i] = rbuf[sd[0].params.channelCount * i];
+	}
 
 #if USE_SNDFILE
 	if (capture)
@@ -1049,13 +1072,16 @@ size_t SoundPort::Write(double *buf, size_t count)
 
         // copy input to both channels
         for (size_t i = 0; i < count; i++)
-                fbuf[OUTPUT_CHANNELS * i] = fbuf[OUTPUT_CHANNELS * i + 1] = buf[i];
+                fbuf[sd[1].params.channelCount * i] = fbuf[sd[1].params.channelCount * i + 1] = buf[i];
 
         return resample_write(fbuf, count);
 }
 
 size_t SoundPort::Write_stereo(double *bufleft, double *bufright, size_t count)
 {
+	if (sd[1].params.channelCount != 2)
+		return Write(bufleft, count);
+
 #if USE_SNDFILE
 	if (generate)
 		write_file(ofCapture, bufleft, count);
@@ -1063,8 +1089,8 @@ size_t SoundPort::Write_stereo(double *bufleft, double *bufright, size_t count)
 
         // interleave into fbuf
         for (size_t i = 0; i < count; i++) {
-                fbuf[OUTPUT_CHANNELS * i] = bufleft[i];
-                fbuf[OUTPUT_CHANNELS * i + 1] = bufright[i];
+                fbuf[sd[1].params.channelCount * i] = bufleft[i];
+                fbuf[sd[1].params.channelCount * i + 1] = bufright[i];
         }
 
         return resample_write(fbuf, count);
@@ -1073,14 +1099,14 @@ size_t SoundPort::Write_stereo(double *bufleft, double *bufright, size_t count)
 
 size_t SoundPort::resample_write(float* buf, size_t count)
 {
-        size_t maxframes = (size_t)floor((sd[1].rb->length() / OUTPUT_CHANNELS) / tx_src_data->src_ratio);
+        size_t maxframes = (size_t)floor((sd[1].rb->length() / sd[1].params.channelCount) / tx_src_data->src_ratio);
         maxframes /= 2; // don't fill the buffer
 
         if (unlikely(count > maxframes)) {
                 size_t n = 0;
                 while (count > maxframes) {
                         n += resample_write(buf, maxframes);
-                        buf += OUTPUT_CHANNELS * maxframes;
+                        buf += sd[1].params.channelCount * maxframes;
                         count -= maxframes;
                 }
                 if (count > 0)
@@ -1088,13 +1114,13 @@ size_t SoundPort::resample_write(float* buf, size_t count)
                 return n;
         }
 
-        assert(count * OUTPUT_CHANNELS * tx_src_data->src_ratio <= sd[1].rb->length());
+        assert(count * sd[1].params.channelCount * tx_src_data->src_ratio <= sd[1].rb->length());
 
         ringbuffer<float>::vector_type vec[2];
         sd[1].rb->get_wv(vec);
         float* wbuf = buf;
         if (req_sample_rate != sd[1].dev_sample_rate || progdefaults.TX_corr != 0) {
-                if (vec[0].len >= OUTPUT_CHANNELS * (size_t)ceil(count * tx_src_data->src_ratio))
+                if (vec[0].len >= sd[1].params.channelCount * (size_t)ceil(count * tx_src_data->src_ratio))
                         wbuf = vec[0].buf; // direct write in the rb
                 else
                         wbuf = src_buffer;
@@ -1112,10 +1138,12 @@ size_t SoundPort::resample_write(float* buf, size_t count)
 		int r;
                 if ((r = src_process(tx_src_state, tx_src_data)) != 0)
                        throw SndException(src_strerror(r));
+		if (tx_src_data->output_frames_gen == 0) // input was too small
+			return count;
 
                 count = tx_src_data->output_frames_gen;
                 if (wbuf == vec[0].buf) { // advance write pointer and return
-                        sd[1].rb->write_advance(OUTPUT_CHANNELS * count);
+                        sd[1].rb->write_advance(sd[1].params.channelCount * count);
                         sem_trywait(sd[1].rwsem);
                         return count;
                 }
@@ -1124,11 +1152,11 @@ size_t SoundPort::resample_write(float* buf, size_t count)
         // if we didn't do a direct resample into the rb, or didn't resample at all,
         // we must now copy buf into the ringbuffer, possibly waiting for space first
         bool timeout = false;
-        WAIT_FOR_COND( (sd[1].rb->write_space() >= OUTPUT_CHANNELS * count), sd[1].rwsem,
-                       (MAX(1.0, 2 * OUTPUT_CHANNELS * count / sd[1].dev_sample_rate)) );
+        WAIT_FOR_COND( (sd[1].rb->write_space() >= sd[1].params.channelCount * count), sd[1].rwsem,
+                       (MAX(1.0, 2 * sd[1].params.channelCount * count / sd[1].dev_sample_rate)) );
         if (timeout)
                 throw SndException(ETIMEDOUT);
-        sd[1].rb->write(wbuf, OUTPUT_CHANNELS * count);
+        sd[1].rb->write(wbuf, sd[1].params.channelCount * count);
 
         return count;
 }
@@ -1166,7 +1194,7 @@ void SoundPort::src_data_reset(unsigned dir)
                 if (rx_src_state)
                         src_delete(rx_src_state);
                 rx_src_state = src_callback_new(src_read_cb, progdefaults.sample_converter,
-                                                INPUT_CHANNELS, &err, &sd[0]);
+                                                sd[0].params.channelCount, &err, &sd[0]);
                 if (!rx_src_state)
                         throw SndException(src_strerror(err));
                 sd[0].src_ratio = req_sample_rate / (sd[0].dev_sample_rate * (1.0 + rxppm / 1e6));
@@ -1174,14 +1202,13 @@ void SoundPort::src_data_reset(unsigned dir)
         else if (dir == 1) {
                 if (tx_src_state)
                         src_delete(tx_src_state);
-                tx_src_state = src_new(progdefaults.sample_converter, OUTPUT_CHANNELS, &err);
+                tx_src_state = src_new(progdefaults.sample_converter, sd[1].params.channelCount, &err);
                 if (!tx_src_state)
                         throw SndException(src_strerror(err));
                 tx_src_data->src_ratio = sd[1].dev_sample_rate * (1.0 + txppm / 1e6) / req_sample_rate;
         }
 
-        int channels[2] = { INPUT_CHANNELS, OUTPUT_CHANNELS };
-        rbsize = ceil2((unsigned)(2 * channels[dir] * SCBLOCKSIZE *
+        rbsize = ceil2((unsigned)(2 * sd[dir].params.channelCount * SCBLOCKSIZE *
                                   MAX(req_sample_rate, sd[dir].dev_sample_rate) /
                                   MIN(req_sample_rate, sd[dir].dev_sample_rate)));
         if (dir == 0) {
@@ -1212,8 +1239,8 @@ long SoundPort::src_read_cb(void* arg, float** data)
 
         // wait for data
         bool timeout = false;
-        WAIT_FOR_COND( (sd->rb->read_space() >= SCBLOCKSIZE), sd->rwsem,
-                       (MAX(1.0, 2 * SCBLOCKSIZE / sd->dev_sample_rate)) );
+        WAIT_FOR_COND( (sd->rb->read_space() >= (size_t)sd[0].params.channelCount * SCBLOCKSIZE), sd->rwsem,
+                       (MAX(1.0, 2 * sd[0].params.channelCount * SCBLOCKSIZE / sd->dev_sample_rate)) );
         if (timeout) {
                 *data = 0;
                 return 0;
@@ -1225,24 +1252,27 @@ long SoundPort::src_read_cb(void* arg, float** data)
         *data = vec[0].buf;
         sd->advance = vec[0].len;
 
-        return vec[0].len;
+        return vec[0].len / sd[0].params.channelCount;
+}
+
+SoundPort::device_iterator SoundPort::name_to_device(const string& name, unsigned dir)
+{
+	device_iterator i;
+	for (i = devs.begin(); i != devs.end(); ++i)
+		if (name == (*i)->name && (dir ? (*i)->maxOutputChannels : (*i)->maxInputChannels))
+			break;
+	return i;
 }
 
 void SoundPort::init_stream(unsigned dir)
 {
 	const char* dir_str[2] = { "input", "output" };
-	PaDeviceIndex conf_idx[2] = { progdefaults.PortInIndex, progdefaults.PortOutIndex };
 	PaDeviceIndex idx = paNoDevice;
 
 	LOG_DEBUG("looking for device \"%s\"", sd[dir].device.c_str());
 
-        for (sd[dir].idev = devs.begin(); sd[dir].idev != devs.end(); ++sd[dir].idev) {
-                if (sd[dir].device == (*sd[dir].idev)->name) {
-			idx = sd[dir].idev - devs.begin(); // save this device index
-			if (idx == conf_idx[dir]) // found it
-				break;
-		}
-	}
+	if ((sd[dir].idev = name_to_device(sd[dir].device, dir)) != devs.end())
+		idx = sd[dir].idev - devs.begin();
         if (idx == paNoDevice) { // no match
 		LOG_ERROR("Could not find device \"%s\", using default device", sd[dir].device.c_str());
 		PaDeviceIndex def = (dir == 0 ? Pa_GetDefaultInputDevice() : Pa_GetDefaultOutputDevice());
@@ -1262,14 +1292,12 @@ void SoundPort::init_stream(unsigned dir)
 
 	if (dir == 0) {
 		sd[0].params.device = idx;
-		sd[0].params.channelCount = INPUT_CHANNELS;
 		sd[0].params.sampleFormat = paFloat32;
 		sd[0].params.suggestedLatency = (*sd[dir].idev)->defaultHighInputLatency;
 		sd[0].params.hostApiSpecificStreamInfo = NULL;
 	}
 	else {
 		sd[1].params.device = idx;
-		sd[1].params.channelCount = OUTPUT_CHANNELS;
 		sd[1].params.sampleFormat = paFloat32;
 		if (host_api->type == paMME)
                         sd[1].params.suggestedLatency = (*sd[dir].idev)->defaultLowOutputLatency;
@@ -1310,7 +1338,8 @@ void SoundPort::init_stream(unsigned dir)
 		<< "\ndefault high output latency: " << (*sd[dir].idev)->defaultHighOutputLatency
 		<< "\n";
 
-	LOG_DEBUG("using %s device \"%s\":\n%s", dir_str[dir], sd[dir].device.c_str(), device_text[dir].str().c_str());
+	LOG_DEBUG("using %s (%d ch) device \"%s\":\n%s", dir_str[dir], sd[dir].params.channelCount,
+		  sd[dir].device.c_str(), device_text[dir].str().c_str());
 
 
         sd[dir].dev_sample_rate = find_srate(dir);
@@ -1318,9 +1347,10 @@ void SoundPort::init_stream(unsigned dir)
 		LOG_DEBUG("%s: resampling %f <=> %f", dir_str[dir],
 			  sd[dir].dev_sample_rate, req_sample_rate);
 
-        if (progdefaults.PortFramesPerBuffer > 0)
+        if (progdefaults.PortFramesPerBuffer > 0) {
                 sd[dir].frames_per_buffer = progdefaults.PortFramesPerBuffer;
-	LOG_DEBUG("%s: frames_per_buffer=%u", dir_str[dir], sd[dir].frames_per_buffer);
+		LOG_DEBUG("%s: frames_per_buffer=%u", dir_str[dir], sd[dir].frames_per_buffer);
+	}
 }
 
 void SoundPort::start_stream(unsigned dir)
@@ -1374,7 +1404,7 @@ int SoundPort::stream_process(const void* in, void* out, unsigned long nframes,
         if (in) {
                 switch (sd->state) {
                 case spa_continue: // write into the rb, post rwsem if we wrote anything
-                        if (sd->rb->write(reinterpret_cast<const float*>(in), nframes))
+                        if (sd->rb->write(reinterpret_cast<const float*>(in), sd->params.channelCount * nframes))
                                 sem_post(sd->rwsem);
                         break;
                 case spa_drain: case spa_pause: // signal the cv
@@ -1386,8 +1416,8 @@ int SoundPort::stream_process(const void* in, void* out, unsigned long nframes,
         else if (out) {
                 float* outf = reinterpret_cast<float*>(out);
                 // if we are paused just pretend that the rb was empty
-                size_t nread = (sd->state == spa_pause) ? 0 : sd->rb->read(outf, OUTPUT_CHANNELS * nframes);
-                memset(outf + nread, 0, (OUTPUT_CHANNELS * nframes - nread) * sizeof(float)); // fill rest with zeroes
+                size_t nread = (sd->state == spa_pause) ? 0 : sd->rb->read(outf, sd->params.channelCount * nframes);
+                memset(outf + nread, 0, (sd->params.channelCount * nframes - nread) * sizeof(float)); // fill rest with 0
 
                 switch (sd->state) {
                 case spa_continue: // post rwsem if we read anything
@@ -1437,9 +1467,9 @@ bool SoundPort::full_duplex_device(const PaDeviceInfo* dev)
         return dev->maxInputChannels > 0 && dev->maxOutputChannels > 0;
 }
 
-bool SoundPort::must_close(void)
+bool SoundPort::must_close(int dir)
 {
-        return false;
+	return Pa_GetHostApiInfo((*sd[dir].idev)->hostApi)->type != paJACK;
 }
 
 // Determine the sample rate that we will use. We try the modem's rate
@@ -1469,8 +1499,8 @@ void SoundPort::probe_supported_rates(const device_iterator& idev)
 {
 	PaStreamParameters params[2];
 	params[0].device = params[1].device = idev - devs.begin();
-	params[0].channelCount = INPUT_CHANNELS;
-	params[1].channelCount = OUTPUT_CHANNELS;
+	params[0].channelCount = progdefaults.in_channels;
+	params[1].channelCount = progdefaults.out_channels;
 	params[0].sampleFormat = params[1].sampleFormat = paFloat32;
 	params[0].suggestedLatency = (*idev)->defaultHighInputLatency;
 	params[1].suggestedLatency = (*idev)->defaultHighOutputLatency;
@@ -1522,10 +1552,10 @@ void SoundPort::init_hostapi_ext(void)
 	const char* err = dlerror();
 	set_jack_client_name = (PaError (*)(const char*))dlsym(handle, "PaJack_SetClientName");
 	if (!(err = dlerror()))
-		set_jack_client_name(PACKAGE_TARNAME);
+		set_jack_client_name(main_window_title.c_str());
 #  ifndef NDEBUG
         else
-		LOG_INFO("dlsym(PaJack_SetClientName) error: %s", err);
+		LOG_VERBOSE("dlsym(PaJack_SetClientName) error: %s", err);
 #  endif
 #endif
 }
@@ -1536,46 +1566,43 @@ void SoundPort::init_hostapi_ext(void)
 #if USE_PULSEAUDIO
 
 SoundPulse::SoundPulse(const char *dev)
-	: fbuf(0)
 {
 	sd[0].stream = sd[1].stream = 0;
 	sd[0].dir = PA_STREAM_RECORD; sd[1].dir = PA_STREAM_PLAYBACK;
 	sd[0].stream_params.format = sd[1].stream_params.format = PA_SAMPLE_FLOAT32LE;
-	sd[0].stream_params.channels = INPUT_CHANNELS;
-	sd[1].stream_params.channels = OUTPUT_CHANNELS;
+	sd[0].stream_params.channels = progdefaults.in_channels;
+	sd[1].stream_params.channels = progdefaults.out_channels;
 
-	try {
-		tx_src_data = new SRC_DATA;
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot create libsamplerate data structures");
-		throw;
-	}
-	try {
-		snd_buffer = new float[INPUT_CHANNELS * SND_BUF_LEN];
-		src_buffer = new float[OUTPUT_CHANNELS * SND_BUF_LEN];
-		fbuf = new float[MAX(INPUT_CHANNELS, OUTPUT_CHANNELS) * SND_BUF_LEN];
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot allocate libsamplerate buffers");
-		throw;
-	}
-	try {
-		wrt_buffer  = new double[SND_BUF_LEN];
-	}
-	catch (const std::bad_alloc& e) {
-		LOG_ERROR("Cannot allocate write buffer");
-		throw;
-	}
-	memset(snd_buffer, 0, INPUT_CHANNELS * SND_BUF_LEN * sizeof(*snd_buffer));
-	memset(src_buffer, 0, OUTPUT_CHANNELS * SND_BUF_LEN * sizeof(*src_buffer));
-	memset(fbuf, 0, MAX(INPUT_CHANNELS, OUTPUT_CHANNELS) * SND_BUF_LEN * sizeof(*fbuf));
-	memset(wrt_buffer, 0, SND_BUF_LEN * sizeof(*wrt_buffer));
+	sd[0].buffer_attrs.maxlength = sd[0].buffer_attrs.minreq = sd[0].buffer_attrs.prebuf =
+		sd[0].buffer_attrs.tlength = (uint32_t)-1;
+	sd[1].buffer_attrs.fragsize = sd[1].buffer_attrs.maxlength = sd[1].buffer_attrs.minreq =
+		sd[1].buffer_attrs.prebuf = (uint32_t)-1;
+	sd[0].buffer_attrs.fragsize = SCBLOCKSIZE * sizeof(float);
+	sd[1].buffer_attrs.tlength = SCBLOCKSIZE * sizeof(float);
+
+	tx_src_data = new SRC_DATA;
+
+	snd_buffer = new float[sd[0].stream_params.channels * SND_BUF_LEN];
+	src_buffer = new float[sd[1].stream_params.channels * SND_BUF_LEN];
+	fbuf = new float[MAX(sd[0].stream_params.channels, sd[1].stream_params.channels) * SND_BUF_LEN];
+
+	memset(snd_buffer, 0, sd[0].stream_params.channels * SND_BUF_LEN * sizeof(*snd_buffer));
+	memset(src_buffer, 0, sd[1].stream_params.channels * SND_BUF_LEN * sizeof(*src_buffer));
+	memset(fbuf, 0, MAX(sd[0].stream_params.channels, sd[1].stream_params.channels) * SND_BUF_LEN * sizeof(*fbuf));
 }
 
 SoundPulse::~SoundPulse()
 {
 	Close();
+
+	delete tx_src_data;
+	if (rx_src_state)
+		src_delete(rx_src_state);
+	if (tx_src_state)
+		src_delete(tx_src_state);
+
+	delete [] snd_buffer;
+	delete [] src_buffer;
 	delete [] fbuf;
 }
 
@@ -1597,8 +1624,10 @@ int SoundPulse::Open(int mode, int freq)
 
 		sd[i].stream_params.rate = freq;
 		snprintf(sname, sizeof(sname), "%s (%u)", (i ? "playback" : "capture"), getpid());
-		sd[i].stream = pa_simple_new(server, PACKAGE_TARNAME, sd[i].dir, NULL,
-					     sname, &sd[i].stream_params, NULL, NULL, &err);
+		setenv("PULSE_PROP_application.icon_name", PACKAGE_TARNAME, 1);
+		sd[i].stream = pa_simple_new(server, main_window_title.c_str(), sd[i].dir, NULL,
+					     sname, &sd[i].stream_params, NULL,
+					     &sd[i].buffer_attrs, &err);
 		if (!sd[i].stream)
 			throw SndPulseException(err);
 	}
@@ -1608,20 +1637,9 @@ int SoundPulse::Open(int mode, int freq)
 
 void SoundPulse::Close(unsigned dir)
 {
-        unsigned start, end;
-        if (dir == UINT_MAX) {
-                start = 0;
-                end = 1;
-        }
-        else
-                start = end = dir;
-
-        for (unsigned i = start; i <= end; i++) {
-                if (sd[i].stream) {
-                        flush(i);
-                        Abort(i);
-                }
-	}
+	if (dir == 1 || dir == UINT_MAX)
+		flush(1);
+	Abort(dir);
 }
 
 void SoundPulse::Abort(unsigned dir)
@@ -1645,22 +1663,34 @@ void SoundPulse::Abort(unsigned dir)
 
 void SoundPulse::flush(unsigned dir)
 {
-        unsigned start, end;
-        if (dir == UINT_MAX) {
-                start = 0;
-                end = 1;
-        }
-        else
-                start = end = dir;
-
 	int err = PA_OK;
-        for (unsigned i = start; i <= end; i++) {
-                if (!sd[i].stream)
-                        continue;
-                pa_simple_drain(sd[i].stream, &err);
-		if (err != PA_OK)
-			LOG_ERROR("%s", pa_strerror(err));
-        }
+	if ((dir == 1 || dir == UINT_MAX) && sd[1].stream) {
+		// wait for 2 SCBLOCKSIZE intervals and then flush the pa_write
+		// stream
+		MilliSleep(SCBLOCKSIZE * 1000 / sd[1].stream_params.rate);
+		pa_simple_flush(sd[1].stream, &err);
+	}
+
+	if ((dir == 0 || dir == UINT_MAX) && sd[0].stream) {
+		// We need to flush the captured audio that PA has been
+		// buffering for us while we were transmitting.  We will use
+		// pa_simple_get_latency() which, contrary to the docs, also
+		// works for capture streams.  It tells us how much earlier the
+		// data that would be returned by pa_simple_read() was actually
+		// captured, and we read and discard all that data.
+		pa_usec_t t = pa_simple_get_latency(sd[0].stream, &err);
+		if (t && err == PA_OK) {
+			size_t bytes = pa_usec_to_bytes(t, &sd[0].stream_params);
+			while (bytes > SND_BUF_LEN) {
+				pa_simple_read(sd[0].stream, snd_buffer, SND_BUF_LEN, &err);
+				if (err != PA_OK)
+					break;
+				bytes -= SND_BUF_LEN;
+			}
+			if (bytes)
+				pa_simple_read(sd[0].stream, snd_buffer, bytes, &err);
+		}
+	}
 }
 
 size_t SoundPulse::Write(double* buf, size_t count)
@@ -1671,21 +1701,24 @@ size_t SoundPulse::Write(double* buf, size_t count)
 #endif
 
         for (size_t i = 0; i < count; i++)
-                fbuf[OUTPUT_CHANNELS * i] = fbuf[OUTPUT_CHANNELS * i + 1] = buf[i];
+                fbuf[sd[1].stream_params.channels * i] = fbuf[sd[1].stream_params.channels * i + 1] = buf[i];
 
 	return resample_write(fbuf, count);
 }
 
 size_t SoundPulse::Write_stereo(double* bufleft, double* bufright, size_t count)
 {
+	if (sd[1].stream_params.channels != 2)
+		return Write(bufleft, count);
+
 #if USE_SNDFILE
 	if (generate)
 		write_file(ofGenerate, bufleft, count);
 #endif
 
 	for (size_t i = 0; i < count; i++) {
-		fbuf[OUTPUT_CHANNELS * i] = bufleft[i];
-		fbuf[OUTPUT_CHANNELS * i + 1] = bufright[i];
+		fbuf[sd[1].stream_params.channels * i] = bufleft[i];
+		fbuf[sd[1].stream_params.channels * i + 1] = bufright[i];
 	}
 
 	return resample_write(fbuf, count);
@@ -1709,12 +1742,14 @@ size_t SoundPulse::resample_write(float* buf, size_t count)
                 tx_src_data->end_of_input = 0;
                 if ((err = src_process(tx_src_state, tx_src_data)) != 0)
 			throw SndException(src_strerror(err));
+		if (tx_src_data->output_frames_gen == 0) // input was too small
+			return count;
 
                 wbuf = tx_src_data->data_out;
                 count = tx_src_data->output_frames_gen;
         }
 
-	if (pa_simple_write(sd[1].stream, wbuf, count * OUTPUT_CHANNELS * sizeof(float), &err) == -1)
+	if (pa_simple_write(sd[1].stream, wbuf, count * sd[1].stream_params.channels * sizeof(float), &err) == -1)
 		throw SndPulseException(err);
 
 	return count;
@@ -1735,7 +1770,7 @@ long SoundPulse::src_read_cb(void* arg, float** data)
 	return p->sd[0].blocksize;
 }
 
-size_t SoundPulse::Read(double *buf, size_t count)
+size_t SoundPulse::Read(float *buf, size_t count)
 {
 #if USE_SNDFILE
 	if (playback) {
@@ -1744,6 +1779,7 @@ size_t SoundPulse::Read(double *buf, size_t count)
 			for (size_t i = 0; i < count; i++)
 				buf[i] *= progStatus.RcvMixer;
 		if (!capture) {
+			flush(0);
 			MilliSleep((long)ceil((1e3 * count) / sample_frequency));
 			return count;
 		}
@@ -1760,20 +1796,17 @@ size_t SoundPulse::Read(double *buf, size_t count)
 		size_t n = 0;
 		sd[0].blocksize = SCBLOCKSIZE;
 		while (n < count) {
-			if ((r = src_callback_read(rx_src_state, sd[0].src_ratio, count - n, fbuf + n)) == 0)
+			if ((r = src_callback_read(rx_src_state, sd[0].src_ratio, count - n, buf + n)) == 0)
 				return n;
 			n += r;
 		}
         }
 	else {
 		int err;
-		if (pa_simple_read(sd[0].stream, fbuf, sizeof(float) * count, &err) == -1)
+		if (pa_simple_read(sd[0].stream, buf, sizeof(float) * count, &err) == -1)
 			throw SndPulseException(err);
 	}
 
-	for (size_t i = 0; i < count; i++)
-                buf[i] = fbuf[i];
-
 #if USE_SNDFILE
 	if (capture)
                 write_file(ofCapture, buf, count);
@@ -1831,7 +1864,7 @@ size_t SoundNull::Write_stereo(double* bufleft, double* bufright, size_t count)
 	return count;
 }
 
-size_t SoundNull::Read(double *buf, size_t count)
+size_t SoundNull::Read(float *buf, size_t count)
 {
 #if USE_SNDFILE
 	if (playback) {
@@ -1851,4 +1884,5 @@ size_t SoundNull::Read(double *buf, size_t count)
 	MilliSleep((long)ceil((1e3 * count) / sample_frequency));
 
 	return count;
+
 }
