@@ -1,7 +1,7 @@
 // ----------------------------------------------------------------------------
 //      pskrep.cxx
 //
-// Copyright (C) 2008
+// Copyright (C) 2008-2009
 //              Stelios Bounanos, M0GLD
 //
 // This is a client for N1DQ's PSK Automatic Propagation Reporter
@@ -404,7 +404,7 @@ void pskrep::append(string call, const char* loc, long long freq, trx_mode mode,
 	band_map_t& bandq = queue[call][band(freq)];
 	if (bandq.empty() || rtime - bandq.back().rtime >= DUP_INTERVAL) { // add new
 		bandq.push_back(rcpt_report_t(mode, freq, rtime, rtype, loc));
-		LOG_INFO("Added (call=\"%s\", loc=\"%s\", mode=\"%s\", freq=%lld, time=%jd, type=%u)",
+		LOG_VERBOSE("Added (call=\"%s\", loc=\"%s\", mode=\"%s\", freq=%lld, time=%jd, type=%u)",
 			 call.c_str(), loc, mode_info[mode].adif_name, freq, (intmax_t)rtime, rtype);
 		new_count++;
 		save_queue();
@@ -414,7 +414,7 @@ void pskrep::append(string call, const char* loc, long long freq, trx_mode mode,
 		if (r.status != PSKR_STATUS_SENT && *loc && r.locator != loc) { // update last
 			r.locator = loc;
 			r.rtype = rtype;
-			LOG_INFO("Updated (call=\"%s\", loc=\"%s\", mode=\"%s\", freq=%lld, time=%jd, type=%u)",
+			LOG_VERBOSE("Updated (call=\"%s\", loc=\"%s\", mode=\"%s\", freq=%lld, time=%jd, type=%u)",
 				 call.c_str(), loc, mode_info[r.mode].adif_name, r.freq, (intmax_t)r.rtime, rtype);
 			save_queue();
 		}
@@ -449,7 +449,7 @@ bool pskrep::progress(void)
 	}
 
 send_reports:
-	LOG_INFO("Found %u new report(s)", nrep);
+	LOG_VERBOSE("Found %u new report(s)", nrep);
 	if (nrep) {
 		if (!sender.send()) {
 			LOG_ERROR("Sender failed, disabling pskreporter");
