@@ -1,25 +1,26 @@
 // ----------------------------------------------------------------------------
 // trx.cxx  --  Main transmit/receive control loop / thread
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2010
 //		Dave Freese, W1HKJ
+// Copyright (C) 2007-2010
+//		Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.  Adapted in part from code contained in gmfsk 
 // source code distribution.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -73,21 +74,113 @@ static int	_trx_tune;
 // Ringbuffer for the audio "history". A pointer into this buffer
 // is also passed to the waterfall signal drawing routines.
 #define NUMMEMBUFS 1024
-ringbuffer<double> trxrb(ceil2(NUMMEMBUFS * SCBLOCKSIZE));
-// Vector used for direct access to the ringbuffer
-ringbuffer<double>::vector_type rbvec[2];
+static ringbuffer<double> trxrb(ceil2(NUMMEMBUFS * SCBLOCKSIZE));
+static float fbuf[SCBLOCKSIZE];
 bool    bHistory = false;
 
 static bool trxrunning = false;
 
-static bool rsid_detecting = false;
-
 #include "tune.cxx"
 
 //=============================================================================
+
+// Draws the xmit data one WFBLOCKSIZE-sized block at a time
+static void trx_xmit_wfall_draw(int samplerate)
+{
+	ENSURE_THREAD(FLMAIN_TID);
+
+	ringbuffer<double>::vector_type rv[2];
+
+#define block_read_(vec_)						\
+	while (vec_.len >= WFBLOCKSIZE) {				\
+		wf->sig_data(vec_.buf, WFBLOCKSIZE, samplerate);	\
+		vec_.len -= WFBLOCKSIZE;				\
+		vec_.buf += WFBLOCKSIZE;				\
+		trxrb.read_advance(WFBLOCKSIZE);				\
+	}
+
+	trxrb.get_rv(rv);
+	block_read_(rv[0]); // read blocks from the first vector
+
+	if (rv[0].len + rv[1].len < WFBLOCKSIZE)
+		return;
+	if (rv[0].len == 0)
+		block_read_(rv[1]);
+#undef block_read_
+
+	// read non-contiguous data into tmp buffer so that we can
+	// still draw it one block at a time
+	if (unlikely(trxrb.read_space() >= WFBLOCKSIZE)) {
+		double buf[WFBLOCKSIZE];
+		do {
+			trxrb.read(buf, WFBLOCKSIZE);
+			wf->sig_data(buf, WFBLOCKSIZE, samplerate);
+		} while (trxrb.read_space() >= WFBLOCKSIZE);
+	}
+}
+
+// Called by trx_trx_transmit_loop() to handle data that may be left in the
+// ringbuffer when we stop transmitting. Will pad with zeroes to a multiple of
+// WFBLOCKSIZE.
+static void trx_xmit_wfall_end(int samplerate)
+{
+	ENSURE_THREAD(TRX_TID);
+
+	size_t pad = WFBLOCKSIZE - trxrb.read_space() % WFBLOCKSIZE;
+	if (pad == WFBLOCKSIZE) // rb empty or multiple of WFBLOCKSIZE
+		return;
+
+	ringbuffer<double>::vector_type wv[2];
+	trxrb.get_wv(wv, pad);
+	assert(wv[0].len + wv[1].len == pad);
+
+	if (likely(wv[0].len)) { // fill first vector, write rest to second vector
+		memset(wv[0].buf, 0, wv[0].len * sizeof(*wv[0].buf));
+		if (pad > wv[0].len)
+			memset(wv[1].buf, 0, (pad - wv[0].len) * sizeof(*wv[1].buf));
+	}
+	else // all write space is in the second write vector
+		memset(wv[1].buf, 0, pad * sizeof(*wv[1].buf));
+
+	trxrb.write_advance(pad);
+
+	REQ(trx_xmit_wfall_draw, samplerate);
+}
+
+// Copy buf to the ringbuffer if it has enough space. Queue a waterfall
+// request whenever there are at least WFBLOCKSIZE samples to draw.
+void trx_xmit_wfall_queue(int samplerate, const double* buf, size_t len)
+{
+	ENSURE_THREAD(TRX_TID);
+
+	ringbuffer<double>::vector_type wv[2];
+	trxrb.get_wv(wv, len);
+	if (unlikely(wv[0].len + wv[1].len < len)) // not enough space
+		return;
+
+#define write_(vec_, len_)					\
+	for (size_t i = 0; i < len_; i++)			\
+		vec_[i] = buf[i] * progdefaults.TxMonitorLevel;
+
+	size_t n = MIN(wv[0].len, len);
+	write_(wv[0].buf, n);
+	if (len > n) { // write the rest to the second vector
+		buf += n;
+		n = len - n;
+		write_(wv[1].buf, n);
+	}
+#undef write_
+
+	trxrb.write_advance(len);
+	if (trxrb.read_space() >= WFBLOCKSIZE)
+		REQ(trx_xmit_wfall_draw, samplerate);
+}
+
+//=============================================================================
+
 void trx_trx_receive_loop()
 {
-	int  numread;
+	size_t  numread;
 	int  current_samplerate;
 	assert(powerof2(SCBLOCKSIZE));
 
@@ -125,57 +218,19 @@ void trx_trx_receive_loop()
 	}
 	active_modem->rx_init();
 
-	while (1) {
-		if (progdefaults.rsid == true && rsid_detecting == false) {
-			rsid_detecting = true;
-			try {
-				current_samplerate = ReedSolomon->samplerate();
-				scard->Open(O_RDONLY, current_samplerate);
-			}
-			catch (const SndException& e) {
-				LOG_ERROR("%s", e.what());
-				put_status(e.what(), 5);
-				scard->Close();
-				if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
-					sound_close();
-					sound_init();
-				}
-				MilliSleep(1000);
-				return;
-			}
-		}
-		if (progdefaults.rsid == false && rsid_detecting == true) {
-			rsid_detecting = false;
-			active_modem->rx_init();
-			try {
-				current_samplerate = active_modem->get_samplerate();
-				scard->Open(O_RDONLY, current_samplerate);
-			}
-			catch (const SndException& e) {
-				LOG_ERROR("%s", e.what());
-				put_status(e.what(), 5);
-				scard->Close();
-				if (e.error() == EBUSY && progdefaults.btnAudioIOis == SND_IDX_PORT) {
-					sound_close();
-					sound_init();
-				}
-				MilliSleep(1000);
-				return;
-			}
-		}
-		// If we change to an 8000Hz modem while RSID is on we'll never detect anything.
-		// Toggle rsid_detecting so that the audio device is reopened with the ReedSolomon
-		// samplerate in the next loop iteration.
-		if (progdefaults.rsid && rsid_detecting && current_samplerate != ReedSolomon->samplerate())
-			rsid_detecting = false;
+	ringbuffer<double>::vector_type rbvec[2];
 
+	while (1) {
 		try {
+			numread = 0;
+			while (numread < SCBLOCKSIZE && trx_state == STATE_RX)
+				numread += scard->Read(fbuf + numread, SCBLOCKSIZE - numread);
 			if (trxrb.write_space() == 0) // discard some old data
 				trxrb.read_advance(SCBLOCKSIZE);
 			trxrb.get_wv(rbvec);
-			numread = 0;
-			while (numread < SCBLOCKSIZE && trx_state == STATE_RX)
-				numread += scard->Read(rbvec[0].buf + numread, SCBLOCKSIZE - numread);
+			// convert to double and write to rb
+			for (size_t i = 0; i < numread; i++)
+				rbvec[0].buf[i] = fbuf[i];
 		}
 		catch (const SndException& e) {
 			scard->Close();
@@ -191,10 +246,9 @@ void trx_trx_receive_loop()
 		REQ(&waterfall::sig_data, wf, rbvec[0].buf, numread, current_samplerate);
 
 		if (!bHistory) {
-			if (rsid_detecting == true)
-				ReedSolomon->search(rbvec[0].buf, numread);
-			else
-				active_modem->rx_process(rbvec[0].buf, numread);
+			active_modem->rx_process(rbvec[0].buf, numread);
+			if (progdefaults.rsid)
+				ReedSolomon->receive(fbuf, numread);
 		}
 		else {
 			bool afc = progStatus.afconoff;
@@ -212,6 +266,8 @@ void trx_trx_receive_loop()
 			active_modem->HistoryON(false);
 		}
 	}
+	if (scard->must_close(O_RDONLY))
+		scard->Close(O_RDONLY);
 }
 
 
@@ -239,9 +295,9 @@ void trx_trx_transmit_loop()
 		push2talk->set(true);
 		active_modem->tx_init(scard);
 
-		if (progdefaults.TransmitRSid == true)
+		if (progdefaults.TransmitRSid)
 			ReedSolomon->send(true);
-		
+
 		while (trx_state == STATE_TX) {
 			try {
 				if (active_modem->tx_process() < 0)
@@ -256,12 +312,15 @@ void trx_trx_transmit_loop()
 			}
 		}
 
-		if (progdefaults.TransmitRSid == true)
-			ReedSolomon->send(false);
-		
-        scard->flush();
-		if (scard->must_close())
-			scard->Close();
+		trx_xmit_wfall_end(current_samplerate);
+
+//		if (progdefaults.TransmitRSid)
+//			ReedSolomon->send(false);
+
+		scard->flush();
+		if (scard->must_close(O_WRONLY))
+			scard->Close(O_WRONLY);
+
 	} else
 		MilliSleep(10);
 
@@ -311,8 +370,9 @@ void trx_tune_loop()
 			return;
 		}
 		scard->flush();
-		if (scard->must_close())
-			scard->Close();
+		if (scard->must_close(O_WRONLY))
+			scard->Close(O_WRONLY);
+
 		_trx_tune = 0;
 	} else
 		MilliSleep(10);
@@ -331,6 +391,8 @@ void *trx_loop(void *args)
 	for (;;) {
 		if (unlikely(old_state != trx_state)) {
 			old_state = trx_state;
+			if (trx_state == STATE_TX || trx_state == STATE_TUNE)
+				trxrb.reset();
 			trx_signal_state();
 		}
 		switch (trx_state) {
@@ -366,11 +428,14 @@ void *trx_loop(void *args)
 }
 
 //=============================================================================
-modem* new_modem;
+static modem* new_modem;
+static int new_freq;
 
 void trx_start_modem_loop()
 {
 	if (new_modem == active_modem) {
+		if (new_freq > 0)
+			active_modem->set_freq(new_freq);
 		active_modem->restart();
 		trx_state = STATE_RX;
 		return;
@@ -380,6 +445,8 @@ void trx_start_modem_loop()
 
 	new_modem->init();
 	active_modem = new_modem;
+	if (new_freq > 0)
+		active_modem->set_freq(new_freq);
 	trx_state = STATE_RX;
 	REQ(&waterfall::opmode, wf);
 
@@ -390,9 +457,10 @@ void trx_start_modem_loop()
 }
 
 //=============================================================================
-void trx_start_modem(modem *m)
+void trx_start_modem(modem* m, int f)
 {
 	new_modem = m;
+	new_freq = f;
 	trx_state = STATE_NEW_MODEM;
 }
 
