@@ -1,26 +1,26 @@
+// ----------------------------------------------------------------------------
+// waterfall.cxx - Waterfall Spectrum Analyzer Widget
 //
-//  Waterfall Spectrum Analyzer Widget
+// Copyright (C) 2006-2010
+//		Dave Freese, W1HKJ
+// Copyright (C) 2007-2010
+//		Stelios Bounanos, M0GLD
 //
-// Copyright W1HKJ, Dave Freese 2006
+// This file is part of fldigi.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// Fldigi is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
-//
-// Please report all bugs and problems to "w1hkj@w1hkj.com".
-//
-
+// You should have received a copy of the GNU General Public License
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
+// ----------------------------------------------------------------------------
 
 //#define USE_BLACKMAN
 //#define USE_HAMMING
@@ -65,7 +65,11 @@
 #include "macros.h"
 #include "arq_io.h"
 #include "confdialog.h"
+#include "flmisc.h"
 #include "gettext.h"
+#include "rtty.h"
+#include "flslider2.h"
+#include "debug.h"
 
 using namespace std;
 
@@ -111,16 +115,16 @@ short int *tmp_fft_db;
 WFdisp::WFdisp (int x0, int y0, int w0, int h0, char *lbl) :
 			  Fl_Widget(x0,y0,w0,h0,"") {
 	disp_width = w();
-	if (disp_width > IMAGE_WIDTH/4)
-		disp_width = IMAGE_WIDTH/4;
-	scale_width = IMAGE_WIDTH + 1000;
+	if (disp_width > progdefaults.HighFreqCutoff/4)
+		disp_width = progdefaults.HighFreqCutoff/4;
+	scale_width = IMAGE_WIDTH * 2;
 	image_height = h() - WFTEXT - WFSCALE - WFMARKER;
-    image_area      = IMAGE_WIDTH * image_height;
-    sig_image_area  = IMAGE_WIDTH * h();
+	image_area      = IMAGE_WIDTH * image_height;
+	sig_image_area  = IMAGE_WIDTH * h();
 	RGBsize			= sizeof(RGB);
-	RGBwidth		= RGBsize * IMAGE_WIDTH;
+	RGBwidth		= RGBsize * scale_width;
 	fft_img			= new RGBI[image_area];
-	markerimage		= new RGB[IMAGE_WIDTH * WFMARKER];
+	markerimage		= new RGB[scale_width * WFMARKER];
 	scaleimage		= new uchar[scale_width * WFSCALE];
 	scline			= new uchar[scale_width];
 	fft_sig_img 	= new uchar[image_area];
@@ -131,16 +135,16 @@ WFdisp::WFdisp (int x0, int y0, int w0, int h0, char *lbl) :
 	circbuff		= new double[FFT_LEN * 2];
 	fftout			= new double[FFT_LEN * 2];
 	wfft			= new Cfft(FFT_LEN);
-    fftwindow       = new double[FFT_LEN * 2];
+	fftwindow       = new double[FFT_LEN * 2];
 	setPrefilter(progdefaults.wfPreFilter);
-	
+
 	for (int i = 0; i < FFT_LEN*2; i++)
 		circbuff[i] = fftout[i] = 0.0;
 
 	mag = 1;
 	step = 4;
 	dispcolor = true;
-	offset = 0;	
+	offset = 0;
 	sigoffset = 0;
 	ampspan = 75;
 	reflevel = -10;
@@ -151,7 +155,7 @@ WFdisp::WFdisp (int x0, int y0, int w0, int h0, char *lbl) :
 	mode = WATERFALL;
 	centercarrier = false;
 	overload = false;
-    peakaudio = 0.0;
+	peakaudio = 0.0;
 	rfc = 0L;
 	usb = true;
 	wfspeed = NORMAL;
@@ -195,76 +199,129 @@ void WFdisp::initMarkers() {
 	memset(c2, 196, RGBwidth);
 }
 
-void WFdisp::makeMarker() {
-	RGB *clrMin, *clrMax, *clrM, *clrPos;
-	clrMin = markerimage + IMAGE_WIDTH;
-	clrMax = clrMin + (WFMARKER - 2) * IMAGE_WIDTH;
+// draw a marker of specified width and colour centred at freq and clrM
+inline void WFdisp::makeMarker_(int width, const RGB* color, int freq, const RGB* clrMin, RGB* clrM, const RGB* clrMax)
+{
+	if (active_modem->get_mode() == MODE_RTTY) {
+	// rtty has two bandwidth indicators on the waterfall
+	// upper and lower frequency
+		int shift = static_cast<int>(progdefaults.rtty_shift >= 0 ?
+			     _SHIFT[progdefaults.rtty_shift] : progdefaults.rtty_custom_shift);
+		int bw_limit_hi = (int)(shift / 2 + progdefaults.RTTY_BW / 2.0);
+		int bw_limit_lo = (int)(shift / 2 - progdefaults.RTTY_BW / 2.0);
+		int bw_freq = static_cast<int>(freq + 0.5);
+		int bw_lower1 = -bw_limit_hi;
+		int bw_upper1 = -bw_limit_lo;
+		int bw_lower2 = bw_limit_lo;
+		int bw_upper2 = bw_limit_hi;
+		if (bw_lower1 + bw_freq < 0) 
+			bw_lower1 -= bw_lower1 + bw_freq;
+		if (bw_upper1 + bw_freq < 0)
+			bw_lower2 -= bw_lower2 + bw_freq;
+		if (bw_upper2 + bw_freq > scale_width)
+			bw_upper2 -= bw_upper2 + bw_freq - scale_width;
+		if (bw_lower2 + bw_freq > scale_width)
+			bw_lower2 -= bw_lower2 + bw_freq - scale_width;
+	// draw it
+		RGB* clrPos;
+		for (int y = 0; y < WFMARKER - 2; y++) {
+			for (int x = bw_lower1; x < bw_upper1; x++) {
+				clrPos = clrM + x + y * scale_width;
+				if (clrPos > clrMin && clrPos < clrMax)
+					*clrPos = *color;
+			}
+			for (int x = bw_lower2; x < bw_upper2; x++) {
+				clrPos = clrM + x + y * scale_width;
+				if (clrPos > clrMin && clrPos < clrMax)
+					*clrPos = *color;
+			}
+		}
+		return;
+	}
+
+	int bw_lower = -width, bw_upper = width;
+	if (bw_lower + static_cast<int>(freq+0.5) < 0)
+		bw_lower -= bw_lower + static_cast<int>(freq+0.5);
+	if (bw_upper + static_cast<int>(freq+0.5) > scale_width)
+		bw_upper -= bw_upper + static_cast<int>(freq+0.5) - scale_width;
+	// draw it
+	RGB* clrPos;
+	for (int y = 0; y < WFMARKER - 2; y++) {
+		for (int x = bw_lower; x < bw_upper; x++) {
+			clrPos = clrM + x + y * scale_width;
+			if (clrPos > clrMin && clrPos < clrMax)
+				*clrPos = *color;
+		}
+	}
+}
+
+void WFdisp::makeMarker()
+{
+	if (unlikely(!active_modem))
+		return;
+
+	RGB *clrMin, *clrMax, *clrM;
+	clrMin = markerimage + scale_width;
+	clrMax = clrMin + (WFMARKER - 2) * scale_width;
 	memset(clrMin, 0, RGBwidth * (WFMARKER - 2));
 	clrM = clrMin + (int)((double)carrierfreq + 0.5);
 
-	int bw, marker_width = bandwidth;
-	if (active_modem) {
-		int mode = active_modem->get_mode();
-		if (mode >= MODE_BPSK31 && mode <= MODE_QPSK250)
-			marker_width += mailserver ? progdefaults.ServerOffset :
-				progdefaults.SearchRange;
-		if (mode >= MODE_FELDHELL && mode <= MODE_HELL80)
-			marker_width = (int)progdefaults.HELL_BW;
-        if (mode == MODE_RTTY)
-            marker_width = (int)progdefaults.RTTY_BW;
-	}
+	int marker_width = bandwidth;
+	int mode = active_modem->get_mode();
+	if (mode >= MODE_PSK_FIRST && mode <= MODE_PSK_LAST)
+		marker_width += mailserver ? progdefaults.ServerOffset :
+			progdefaults.SearchRange;
+	else if (mode >= MODE_FELDHELL && mode <= MODE_HELL80)
+		marker_width = (int)progdefaults.HELL_BW;
+	else if (mode == MODE_RTTY)
+		marker_width = (int)_SHIFT[progdefaults.rtty_shift];
 	marker_width = (int)(marker_width / 2.0 + 1);
 
 	RGBmarker.R = progdefaults.bwTrackRGBI.R;
 	RGBmarker.G = progdefaults.bwTrackRGBI.G;
 	RGBmarker.B = progdefaults.bwTrackRGBI.B;
 
-	// clamp marker to image width
-	bw = marker_width;
-	int bw_lower = -bw, bw_upper = +bw;
-	if (bw_lower + static_cast<int>(carrierfreq+0.5) < 0)
-            bw_lower -= bw_lower + static_cast<int>(carrierfreq+0.5);
-	if (bw_upper + static_cast<int>(carrierfreq+0.5) > IMAGE_WIDTH)
-            bw_upper -= bw_upper + static_cast<int>(carrierfreq+0.5) - IMAGE_WIDTH;
-	for (int y = 0; y < WFMARKER - 2; y++) {
-		for (int i = bw_lower; i < bw_upper; i++) {
-			clrPos = clrM + i + y * IMAGE_WIDTH;
-			if (clrPos > clrMin && clrPos < clrMax)
-				*clrPos = RGBmarker;
-		}
+	makeMarker_(marker_width, &RGBmarker, carrierfreq, clrMin, clrM, clrMax);
+
+	if (unlikely(active_modem->freqlocked())) {
+		int txfreq = static_cast<int>(active_modem->get_txfreq());
+		adjust_color_inv(RGBmarker.R, RGBmarker.G, RGBmarker.B, FL_BLACK, FL_RED);
+		makeMarker_( static_cast<int>(bandwidth / 2.0 + 1),
+					 &RGBmarker, txfreq,
+					 clrMin, clrMin + (int)((double)txfreq + 0.5), clrMax);
 	}
 
 	if (!wantcursor) return;
-	
-	if (cursorpos > disp_width - bandwidth / 2 / step)
-		cursorpos = disp_width - bandwidth / 2 / step;
-	if (cursorpos >= (IMAGE_WIDTH - offset - bandwidth/2)/step)
-		cursorpos = (IMAGE_WIDTH - offset - bandwidth/2)/step - 1;
-	if (cursorpos < bandwidth / 2 / step)
-		cursorpos = bandwidth / 2 / step + 1;
-	
+
+	if (cursorpos > progdefaults.HighFreqCutoff - bandwidth / 2 / step)
+		cursorpos = progdefaults.HighFreqCutoff - bandwidth / 2 / step;
+	if (cursorpos >= (progdefaults.HighFreqCutoff - offset - bandwidth/2)/step)
+		cursorpos = (progdefaults.HighFreqCutoff - offset - bandwidth/2)/step;
+	if (cursorpos < (progdefaults.LowFreqCutoff + bandwidth / 2) / step)
+		cursorpos = (progdefaults.LowFreqCutoff + bandwidth / 2) / step;
+
 // Create the cursor marker
 	double xp = offset + step * cursorpos;
-	if (xp < bandwidth / 2.0 || xp > (IMAGE_WIDTH - bandwidth / 2.0))
+	if (xp < bandwidth / 2.0 || xp > (progdefaults.HighFreqCutoff - bandwidth / 2.0))
 		return;
-	clrM = markerimage + IMAGE_WIDTH + (int)(xp + 0.5);
+	clrM = markerimage + scale_width + (int)(xp + 0.5);
 	RGBcursor.R = progdefaults.cursorLineRGBI.R;
 	RGBcursor.G = progdefaults.cursorLineRGBI.G;
 	RGBcursor.B = progdefaults.cursorLineRGBI.B;
 
-	bw = marker_width;
+	int bw = marker_width;
 	for (int y = 0; y < WFMARKER - 2; y++) {
-		int incr = y * IMAGE_WIDTH;
+		int incr = y * scale_width;
 		int msize = (WFMARKER - 2 - y)*RGBsize*step/4;
-		*(clrM + incr - 1)	= 
-		*(clrM + incr)		= 
+		*(clrM + incr - 1)	=
+		*(clrM + incr)		=
 		*(clrM + incr + 1)	= RGBcursor;
 
 		if (xp - (bw + msize) > 0)
 			for (int i = bw - msize; i <= bw + msize; i++)
 				*(clrM - i + incr) = RGBcursor;
 
-		if (xp + (bw + msize) < IMAGE_WIDTH)
+		if (xp + (bw + msize) < scale_width)
 			for (int i = bw - msize; i <= bw + msize; i++)
 				*(clrM + i + incr) = RGBcursor;
 	}
@@ -313,8 +370,8 @@ void WFdisp::setcolors() {
 			r = palette[n].R + (int)(1.0 * i * (palette[n+1].R - palette[n].R) / 32.0);
 			g = palette[n].G + (int)(1.0 * i * (palette[n+1].G - palette[n].G) / 32.0);
 			b = palette[n].B + (int)(1.0 * i * (palette[n+1].B - palette[n].B) / 32.0);
-			mag2RGBI[i + 32*n].R = r; 
-			mag2RGBI[i + 32*n].G = g; 
+			mag2RGBI[i + 32*n].R = r;
+			mag2RGBI[i + 32*n].G = g;
 			mag2RGBI[i + 32*n].B = b;
 		}
 	}
@@ -329,7 +386,7 @@ void WFdisp::initmaps() {
 	memset (markerimage, 0, IMAGE_WIDTH * WFMARKER);
 	memset (fft_sig_img, 0, image_area);
 	memset (sig_img, 0, sig_image_area);
-	
+
 	memset (mag2RGBI, 0, sizeof(mag2RGBI));
 	initMarkers();
 	makeScale();
@@ -339,7 +396,7 @@ void WFdisp::initmaps() {
 int WFdisp::peakFreq(int f0, int delta)
 {
 	double threshold = 0.0;
-	int f1, fmin =	(int)((f0 - delta)), 
+	int f1, fmin =	(int)((f0 - delta)),
 		f2, fmax =	(int)((f0 + delta));
 	f1 = fmin; f2 = fmax;
 	if (fmin < 0 || fmax > IMAGE_WIDTH) return f0;
@@ -362,7 +419,7 @@ double WFdisp::powerDensity(double f0, double bw)
 	double pwrdensity = 0.0;
 	int flower = (int)((f0 - bw/2)),
 		fupper = (int)((f0 + bw/2));
-	if (flower < 0 || fupper > IMAGE_WIDTH) 
+	if (flower < 0 || fupper > IMAGE_WIDTH)
 		return 0.0;
 	for (int i = flower; i <= fupper; i++)
 		pwrdensity += pwr[i];
@@ -395,20 +452,20 @@ void WFdisp::processFFT() {
 	int    ptrSample;
 	if (prefilter != progdefaults.wfPreFilter)
 	    setPrefilter(progdefaults.wfPreFilter);
-	    
+
 	scale = (double)SC_SMPLRATE / srate;
-    scale *= FFT_LEN / 2000.0;
+	scale *= FFT_LEN / 2000.0;
 
 	if (dispcnt == 0) {
 		memset (fftout, 0, FFT_LEN*2*sizeof(double));
 		ptrSample = ptrCB;
-        for (int i = 0; i < FFT_LEN * 2 * progdefaults.latency / 8; i++) {
-            fftout[i] = fftwindow[i * 8 / progdefaults.latency] * circbuff[ptrSample];
-            ptrSample = (ptrSample + 1) % (FFT_LEN *2);
-        }
-        
+		for (int i = 0; i < FFT_LEN * 2 * progdefaults.latency / 8; i++) {
+			fftout[i] = fftwindow[i * 8 / progdefaults.latency] * circbuff[ptrSample];
+			ptrSample = (ptrSample + 1) % (FFT_LEN *2);
+		}
+
 		wfft->rdft(fftout);
-FL_LOCK_D();		
+FL_LOCK_D();
 		double pw;
 		int ffth;
 		for (int i = 0; i < IMAGE_WIDTH; i++) {
@@ -488,53 +545,49 @@ void WFdisp::redrawCursor()
 //	cursormoved = true;
 }
 
-void WFdisp::sig_data( double *sig, int len, int sr ) {
+void WFdisp::sig_data( double *sig, int len, int sr )
+{
 	if (wfspeed == PAUSE)
-		return;
+		goto update_freq;
 
-//if sound card sampling rate changed reset the waterfall buffer
+	// if sound card sampling rate changed reset the waterfall buffer
 	if (srate != sr) {
 		srate = sr;
 		memset (circbuff, 0, FFT_LEN * 2 * sizeof(double));
 		ptrCB = 0;
 	}
 
-	overload = false;
-    double overval, peak = 0.0;
-	for (int i = 0; i < len; i++) {
-		overval = fabs(circbuff[ptrCB] = sig[i]);
-		ptrCB = (ptrCB + 1) % (FFT_LEN *2);	
-		if (overval > peak) peak = overval;
+	{
+		overload = false;
+		double overval, peak = 0.0;
+		for (int i = 0; i < len; i++) {
+			overval = fabs(circbuff[ptrCB] = sig[i]);
+			ptrCB = (ptrCB + 1) % (FFT_LEN *2);
+			if (overval > peak) peak = overval;
+		}
+		peakaudio = 0.1 * peak + 0.9 * peakaudio;
 	}
 
-    peakaudio = 0.1 * peak + 0.9 * peakaudio;
-
 	if (mode == SCOPE)
 		process_analog(circbuff, FFT_LEN * 2);
 	else
 		processFFT();
-		
+
 	put_WARNstatus(peakaudio);
 
+update_freq:
 	static char szFrequency[14];
-	
-//	if (usebands)
-//		rfc = (long long)(atof(cboBand->value()) * 1000.0);
 	if (rfc != 0) { // use a boolean for the waterfall
 		if (usb)
 			dfreq = rfc + active_modem->get_txfreq();
-		else	
+		else
 			dfreq = rfc - active_modem->get_txfreq();
 		snprintf(szFrequency, sizeof(szFrequency), "%-.3f", dfreq / 1000.0);
 	} else {
 		dfreq = active_modem->get_txfreq();
 		snprintf(szFrequency, sizeof(szFrequency), "%-.0f", dfreq);
 	}
-	FL_LOCK_D();
 	inpFreq->value(szFrequency);
-	FL_UNLOCK_D();
-
-	return;
 }
 
 
@@ -547,13 +600,18 @@ void WFdisp::checkoffset() {
 		if (sigoffset > (IMAGE_WIDTH - disp_width))
 			sigoffset = IMAGE_WIDTH - disp_width;
 	} else {
+		if (offset > (int)(progdefaults.HighFreqCutoff - step * disp_width))
+			offset = (int)(progdefaults.HighFreqCutoff - step * disp_width);
 		if (offset < 0)
 			offset = 0;
-		if (offset > (int)(IMAGE_WIDTH - step * disp_width))
-			offset = (int)(IMAGE_WIDTH - step * disp_width);
 	}
 }
 
+void WFdisp::setOffset(int v) {
+	offset = v;
+	checkoffset();
+}
+
 void WFdisp::slew(int dir) {
 	if (mode == SCOPE)
 		sigoffset += dir;
@@ -586,11 +644,11 @@ void WFdisp::checkWidth()
 {
 	disp_width = w();
 	if (mag == MAG_1) step = 4;
-	if (mag == MAG_1 && disp_width > IMAGE_WIDTH/4)
-		disp_width = IMAGE_WIDTH/4;
+	if (mag == MAG_1 && disp_width > progdefaults.HighFreqCutoff/4)
+		disp_width = progdefaults.HighFreqCutoff/4;
 	if (mag == MAG_2) step = 2;
-	if (mag == MAG_2 && disp_width > IMAGE_WIDTH/2)
-		disp_width = IMAGE_WIDTH/2;
+	if (mag == MAG_2 && disp_width > progdefaults.HighFreqCutoff/2)
+		disp_width = progdefaults.HighFreqCutoff/2;
 	if (mag == MAG_4) step = 1;
 }
 
@@ -632,30 +690,27 @@ int WFdisp::wfmag() {
 
 
 void WFdisp::drawScale() {
-	int fw = 60, xchar;
+	int fw = 60, xoff;
 	static char szFreq[20];
 	double fr;
 	uchar *pixmap;
-	
+
 	if (progdefaults.wf_audioscale)
 		pixmap = (scaleimage + (int)offset);
 	else if (usb || !rfc)
 		pixmap = (scaleimage +  (int)((rfc % 1000 + offset)) );
 	else
 		pixmap = (scaleimage + (int)((1000 - rfc % 1000 + offset)));
-	
+
 	fl_draw_image_mono(
-		pixmap, 
-		x(), y() + WFTEXT, 
-		disp_width, WFSCALE, 
+		pixmap,
+		x(), y() + WFTEXT,
+		w(), WFSCALE,
 		step, scale_width);
-		
-	fl_color(FL_BLACK);
-	fl_rectf(x(), y(), disp_width, WFTEXT);
-	
+
 	fl_color(fl_rgb_color(228));
 	fl_font(progdefaults.WaterfallFontnbr, progdefaults.WaterfallFontsize);
-	for (int i = 1; i < 10; i++) {
+	for (int i = 1; ; i++) {
 		if (progdefaults.wf_audioscale)
 			fr = 500.0 * i;
 		else {
@@ -670,15 +725,16 @@ void WFdisp::drawScale() {
 			snprintf(szFreq, sizeof(szFreq), "%7.1f", fr);
 		fw = (int)fl_width(szFreq);
 		if (progdefaults.wf_audioscale)
-			xchar = (int) (( (1000.0/step) * i - fw) / 2.0 - offset /step );
+			xoff = (int) (( (1000.0/step) * i - fw) / 2.0 - offset /step );
 		else if (usb)
-			xchar = (int) ( ( (1000.0/step) * i - fw) / 2.0 - 
+			xoff = (int) ( ( (1000.0/step) * i - fw) / 2.0 -
 							(offset + rfc % 500) /step );
 		else
-			xchar = (int) ( ( (1000.0/step) * i - fw) / 2.0 - 
+			xoff = (int) ( ( (1000.0/step) * i - fw) / 2.0 -
 							(offset + 500 - rfc % 500) /step );
-		if (xchar > 0 && (xchar + fw) < disp_width)
-			fl_draw(szFreq, x() + xchar, y() + 10 );
+		if (xoff > 0 && xoff < w() - fw)
+			fl_draw(szFreq, x() + xoff, y() + 10 );
+		if (xoff > w() - fw) break;
 	}
 }
 
@@ -686,9 +742,9 @@ void WFdisp::drawMarker() {
 	if (mode == SCOPE) return;
 	uchar *pixmap = (uchar *)(markerimage + (int)(offset));
 	fl_draw_image(
-		pixmap, 
-		x(), y() + WFSCALE + WFTEXT, 
-		disp_width, WFMARKER, 
+		pixmap,
+		x(), y() + WFSCALE + WFTEXT,
+		w(), WFMARKER,
 		step * RGBsize, RGBwidth);
 }
 
@@ -701,7 +757,7 @@ void WFdisp::update_waterfall() {
 	p2 = p1;
 	p3 = fft_img;
 	p4 = p3;
-	
+
 	short* limit = tmp_fft_db + image_area - step + 1;
 
 	for (int row = 0; row < image_height; row++) {
@@ -720,7 +776,7 @@ void WFdisp::update_waterfall() {
 					p2 += step;
 					if (p2 > limit) break;
 				}
-			else 
+			else
 				for (int col = 0; col < disp_width; col++) {
 					*(p4++) = mag2RGBI[ *p2 ];
 					p2 += step;
@@ -739,7 +795,7 @@ void WFdisp::update_waterfall() {
 					p2 += step;
 					if (p2 > limit) break;
 				}
-			else 
+			else
 				for (int col = 0; col < disp_width; col++) {
 					*(p4++) = mag2RGBI[ *p2 ];
 					p2 += step;
@@ -753,19 +809,21 @@ void WFdisp::update_waterfall() {
 	if (progdefaults.UseBWTracks) {
 		RGBI  *pos1 = fft_img + (carrierfreq - offset - bandwidth/2) / step;
 		RGBI  *pos2 = fft_img + (carrierfreq - offset + bandwidth/2) / step;
-		if (pos1 >= fft_img && pos2 < fft_img + disp_width)
+		if (unlikely(pos2 == fft_img + disp_width))
+			pos2--;
+		if (likely(pos1 >= fft_img && pos2 < fft_img + disp_width)) {
 			for (int y = 0; y < image_height; y ++) {
 				*pos1 = *pos2 = progdefaults.bwTrackRGBI;
 				pos1 += disp_width;
 				pos2 += disp_width;
 			}
+		}
 	}
 }
 
 void WFdisp::drawcolorWF() {
 	uchar *pixmap = (uchar *)fft_img;
-	fl_color(FL_BLACK);
-	fl_rectf(x() + disp_width, y(), w() - disp_width, h());
+
 	update_waterfall();
 
 	if (wantcursor && (progdefaults.UseCursorLines || progdefaults.UseCursorCenterLine) ) {
@@ -784,17 +842,21 @@ void WFdisp::drawcolorWF() {
 			}
 	}
 
+	fl_color(FL_BLACK);
+	fl_rectf(x(), y(), w(), WFSCALE + WFMARKER + WFTEXT);
+	fl_color(fl_rgb_color(palette[0].R, palette[0].G, palette[0].B));
+	fl_rectf(x(), y() + WFSCALE + WFMARKER + WFTEXT, w(), image_height);
 	fl_draw_image(
-		pixmap, x(), y() + WFSCALE + WFMARKER + WFTEXT, 
-		disp_width, image_height, 
+		pixmap, x(), y() + WFSCALE + WFMARKER + WFTEXT,
+		disp_width, image_height,
 		sizeof(RGBI), disp_width * sizeof(RGBI) );
 	drawScale();
 }
 
+// following method is not used in versions > 3.12
 void WFdisp::drawgrayWF() {
 	uchar *pixmap = (uchar*)fft_img;
-	fl_color(FL_BLACK);
-	fl_rectf(x() + disp_width, y(), w() - disp_width, h());
+
 	update_waterfall();
 
 	if (wantcursor && (progdefaults.UseCursorLines || progdefaults.UseCursorCenterLine) ) {
@@ -813,10 +875,13 @@ void WFdisp::drawgrayWF() {
 			}
 	}
 
+	fl_color(FL_BLACK);
+	fl_rectf(x(), y(), w(), WFSCALE + WFMARKER + WFTEXT + image_height);
+
 	fl_draw_image_mono(
-		pixmap + 3, 
-		x(), y() + WFSCALE + WFMARKER + WFTEXT, 
-		disp_width, image_height, 
+		pixmap + 3,
+		x(), y() + WFSCALE + WFMARKER + WFTEXT,
+		disp_width, image_height,
 		sizeof(RGBI), disp_width * sizeof(RGBI));
 	drawScale();
 }
@@ -871,14 +936,14 @@ void WFdisp::drawspectrum() {
 			pos2 += IMAGE_WIDTH/step;
 		}
 	}
-	
+
 	fl_color(FL_BLACK);
-	fl_rectf(x() + disp_width, y(), w() - disp_width, h());
-	
+	fl_rectf(x(), y(), w(), WFSCALE + WFMARKER + WFTEXT + image_height);
+
 	fl_draw_image_mono(
-		pixmap, 
-		x(), y() + WFSCALE + WFMARKER + WFTEXT, 
-		disp_width, image_height, 
+		pixmap,
+		x(), y() + WFSCALE + WFMARKER + WFTEXT,
+		disp_width, image_height,
 		1, IMAGE_WIDTH / step);
 	drawScale();
 }
@@ -930,13 +995,13 @@ void slew_left(Fl_Widget *w, void * v) {
 	waterfall *wf = (waterfall *)w->parent();
 	wf->wfdisp->slew(-100);
 	restoreFocus();
-} 
+}
 
 void slew_right(Fl_Widget *w, void * v) {
 	waterfall *wf = (waterfall *)w->parent();
 	wf->wfdisp->slew(100);
 	restoreFocus();
-} 
+}
 
 
 void center_cb(Fl_Widget *w, void *v) {
@@ -949,6 +1014,7 @@ void carrier_cb(Fl_Widget *w, void *v) {
 	Fl_Counter *cntr = (Fl_Counter *)w;
 	waterfall *wf = (waterfall *)w->parent();
 	int selfreq = (int) cntr->value();
+	if (selfreq > progdefaults.HighFreqCutoff) selfreq = progdefaults.HighFreqCutoff - wf->wfdisp->Bandwidth() / 2;
 	stopMacroTimer();
 	active_modem->set_freq(selfreq);
 	wf->wfdisp->carrier(selfreq);
@@ -957,40 +1023,43 @@ void carrier_cb(Fl_Widget *w, void *v) {
 
 void qsy_cb(Fl_Widget *w, void *v)
 {
-        static vector<qrg_mode_t> qsy_stack;
-        qrg_mode_t m;
+	static vector<qrg_mode_t> qsy_stack;
+	qrg_mode_t m;
 
-        if (Fl::event_button() != FL_RIGHT_MOUSE) {
-                // store
-                m.rfcarrier = wf->rfcarrier();
-                m.carrier = active_modem->get_freq();
-                qsy_stack.push_back(m);
+	wf->xmtlock->value(0);
+	wf->xmtlock->do_callback();
 
-                // qsy to the sweet spot frequency that is the center of the PBF in the rig
-                switch (active_modem->get_mode()) {
-                case MODE_CW:
-                        m.carrier = (long long)progdefaults.CWsweetspot;
-                        break;
-                case MODE_RTTY:
-                        m.carrier = (long long)progdefaults.RTTYsweetspot;
-                        break;
-                default:
-                        m.carrier = (long long)progdefaults.PSKsweetspot;
-                        break;
-                }
+	if (Fl::event_button() != FL_RIGHT_MOUSE) {
+// store
+		m.rfcarrier = wf->rfcarrier();
+		m.carrier = active_modem->get_freq();
+		qsy_stack.push_back(m);
+
+// qsy to the sweet spot frequency that is the center of the PBF in the rig
+		switch (active_modem->get_mode()) {
+			case MODE_CW:
+				m.carrier = (long long)progdefaults.CWsweetspot;
+				break;
+			case MODE_RTTY:
+				m.carrier = (long long)progdefaults.RTTYsweetspot;
+				break;
+			default:
+				m.carrier = (long long)progdefaults.PSKsweetspot;
+				break;
+		}
 		if (wf->USB())
 			m.rfcarrier += (wf->carrier() - m.carrier);
 		else
 			m.rfcarrier -= (wf->carrier() - m.carrier);
-        }
-        else { // qsy to top of stack
-                if (qsy_stack.size()) {
-                        m = qsy_stack.back();
-                        qsy_stack.pop_back();
-                }
-        }
+	}
+	else { // qsy to top of stack
+		if (qsy_stack.size()) {
+			m = qsy_stack.back();
+			qsy_stack.pop_back();
+		}
+	}
 
-        if (m.carrier > 0)
+	if (m.carrier > 0)
 		qsy(m.rfcarrier, m.carrier);
 	restoreFocus();
 }
@@ -1024,6 +1093,11 @@ void xmtrcv_cb(Fl_Widget *w, void *vi)
 	Fl_Light_Button *b = (Fl_Light_Button *)w;
 	int v = b->value();
 	FL_UNLOCK_D();
+	if (!(active_modem->get_cap() & modem::CAP_TX)) {
+		b->value(0);
+		restoreFocus();
+		return;
+	}
 	if (v == 1) {
 		stopMacroTimer();
 		active_modem->set_stopflag(false);
@@ -1114,107 +1188,108 @@ void btnRev_cb(Fl_Widget *w, void *v) {
 void btnMem_cb(Fl_Widget *, void *menu_event)
 {
 	static std::vector<qrg_mode_t> qrg_list;
-        enum { SELECT, APPEND, REPLACE, REMOVE, CLEAR };
-        int op = SELECT, elem = 0;
+	enum { SELECT, APPEND, REPLACE, REMOVE, CLEAR };
+	int op = SELECT, elem = 0;
 
-        if (menu_event) { // event on popup menu
-                elem = wf->mbtnMem->value();
+	if (menu_event) { // event on popup menu
+		elem = wf->mbtnMem->value();
 
-                switch (Fl::event_button()) {
-                case FL_MIDDLE_MOUSE:
-                        op = REPLACE;
-                        break;
-                case FL_LEFT_MOUSE: case FL_RIGHT_MOUSE: default:
-                        op = (Fl::event_state() & FL_SHIFT) ? REMOVE : SELECT;
-                        break;
-                }
-        }
-        else { // button press
-                switch (Fl::event_button()) {
-                case FL_RIGHT_MOUSE:
-                        return;
-                case FL_MIDDLE_MOUSE: // select last
-                        if ((elem = qrg_list.size() - 1) < 0)
-                                return;
-                        op = SELECT;
-                        break;
-                case FL_LEFT_MOUSE: default:
-                        op = (Fl::event_state() & FL_SHIFT) ? CLEAR : APPEND;
-                        break;
-                }
-        }
-
-        qrg_mode_t m;
-        switch (op) {
-        case SELECT:
-                m = qrg_list[elem];
-                if (active_modem != *mode_info[m.mode].modem)
-                        init_modem_sync(m.mode);
-                if (m.rfcarrier && m.rfcarrier != wf->rfcarrier())
-			qsy(m.rfcarrier, m.carrier);
-                else
-                        active_modem->set_freq(m.carrier);
-                break;
-        case REMOVE:
-                wf->mbtnMem->remove(elem);
-                qrg_list.erase(qrg_list.begin() + elem);
-                break;
-        case CLEAR:
-                wf->mbtnMem->clear();
-                qrg_list.clear();
-                break;
-        case APPEND: case REPLACE:
-                m.rfcarrier = wf->rfcarrier();
-                m.carrier = active_modem->get_freq();
-                m.mode = active_modem->get_mode();
-                if (op == APPEND) {
-			if (find(qrg_list.begin(), qrg_list.end(), m) == qrg_list.end())
-				qrg_list.push_back(m);
-			else
+		switch (Fl::event_button()) {
+			case FL_MIDDLE_MOUSE:
+				op = REPLACE;
+				break;
+			case FL_LEFT_MOUSE: case FL_RIGHT_MOUSE: default:
+				op = (Fl::event_state() & FL_SHIFT) ? REMOVE : SELECT;
 				break;
 		}
-                else
-                        qrg_list[elem] = m;
+	}
+	else { // button press
+		switch (Fl::event_button()) {
+			case FL_RIGHT_MOUSE:
+				return;
+			case FL_MIDDLE_MOUSE: // select last
+				if ((elem = qrg_list.size() - 1) < 0)
+					return;
+				op = SELECT;
+				break;
+			case FL_LEFT_MOUSE: default:
+				op = (Fl::event_state() & FL_SHIFT) ? CLEAR : APPEND;
+				break;
+			}
+	}
 
-        // write the menu item text
-        {
-                ostringstream o;
-                o << mode_info[m.mode].sname << " @@ ";
-                if (m.rfcarrier > 0) { // write 1000s separators
-                        char s[20], *p = s + sizeof(s) - 1;
-                        int i = 0;
+	qrg_mode_t m;
+	switch (op) {
+		case SELECT:
+			m = qrg_list[elem];
+			if (active_modem != *mode_info[m.mode].modem)
+				init_modem_sync(m.mode);
+			if (m.rfcarrier && m.rfcarrier != wf->rfcarrier())
+				qsy(m.rfcarrier, m.carrier);
+			else
+				active_modem->set_freq(m.carrier);
+			break;
+		case REMOVE:
+			wf->mbtnMem->remove(elem);
+			qrg_list.erase(qrg_list.begin() + elem);
+			break;
+		case CLEAR:
+			wf->mbtnMem->clear();
+			qrg_list.clear();
+			break;
+		case APPEND: case REPLACE:
+			m.rfcarrier = wf->rfcarrier();
+			m.carrier = active_modem->get_freq();
+			m.mode = active_modem->get_mode();
+			if (op == APPEND) {
+				if (find(qrg_list.begin(), qrg_list.end(), m) == qrg_list.end())
+					qrg_list.push_back(m);
+			else
+				break;
+			}
+			else
+				qrg_list[elem] = m;
+// write the menu item text
+			{
+				ostringstream o;
+				o << mode_info[m.mode].sname << " @@ ";
+				if (m.rfcarrier > 0) { // write 1000s separators
+					char s[20], *p = s + sizeof(s) - 1;
+					int i = 0;
 
-                        *p = '\0';
-                        do {
-                                if (i % 3 == 0 && i)
-                                        *--p = '.';
-                                *--p = '0' + m.rfcarrier % 10;
-                                ++i;
-                        } while ((m.rfcarrier /= 10) && p > s);
+					*p = '\0';
+					do {
+						if (i % 3 == 0 && i)
+							*--p = '.';
+							*--p = '0' + m.rfcarrier % 10;
+							++i;
+					} while ((m.rfcarrier /= 10) && p > s);
 
-                        o << p << (wf->USB() ? " + " : " - ");
-                }
-                o << m.carrier;
-                if (op == APPEND)
-                        wf->mbtnMem->add(o.str().c_str());
-                else
-                        wf->mbtnMem->replace(elem, o.str().c_str());
-        }
-                break;
-        }
+					o << p << (wf->USB() ? " + " : " - ");
+				}
+				o << m.carrier;
+				if (op == APPEND)
+					wf->mbtnMem->add(o.str().c_str());
+				else
+					wf->mbtnMem->replace(elem, o.str().c_str());
+			}
+			break;
+	}
 
-        restoreFocus();
+	restoreFocus();
 }
 
 void waterfall::opmode() {
 	int val = (int)active_modem->get_bandwidth();
-	if (wfdisp->carrier() < val/2)
-		wfdisp->carrier( val/2 );
-	if (wfdisp->carrier() > IMAGE_WIDTH - val/2)
-		wfdisp->carrier( IMAGE_WIDTH - val/2);
+
+	wfdisp->carrier((int)CLAMP(
+		wfdisp->carrier(), 
+		progdefaults.LowFreqCutoff + val / 2,
+		progdefaults.HighFreqCutoff - val / 2));
+
 	wfdisp->Bandwidth( val );
-	FL_LOCK_D();	
-	wfcarrier->range(val/2, IMAGE_WIDTH - val/2-1);
+	FL_LOCK_D();
+	wfcarrier->range(progdefaults.LowFreqCutoff + val/2, progdefaults.HighFreqCutoff - val/2);
 	FL_UNLOCK_D();
 }
 
@@ -1226,7 +1301,7 @@ void waterfall::carrier(int f) {
 	FL_UNLOCK_D();
 }
 
-int waterfall::Speed() { 
+int waterfall::Speed() {
 	return (int)wfdisp->Speed();
 }
 
@@ -1259,7 +1334,7 @@ int waterfall::Mag() {
 	return wfdisp->Mag();
 }
 
-void waterfall::Mag(int m) { 
+void waterfall::Mag(int m) {
 	FL_LOCK_D();
 	wfdisp->Mag(m);
 	if (m == MAG_1) x1->label("x1");
@@ -1269,6 +1344,16 @@ void waterfall::Mag(int m) {
 	FL_UNLOCK_D();
 }
 
+int waterfall::Offset() {
+	return wfdisp->Offset();
+}
+
+void waterfall::Offset(int v) {
+	FL_LOCK_D();
+	wfdisp->Offset(v);
+	FL_UNLOCK_D();
+}
+
 int waterfall::Carrier()
 {
 	return wfdisp->carrier();
@@ -1284,7 +1369,7 @@ extern void viewer_redraw();
 	wfdisp->rfcarrier(cf);
 	viewer_redraw();
 }
-	
+
 long long waterfall::rfcarrier() {
 	return wfdisp->rfcarrier();
 }
@@ -1311,29 +1396,48 @@ void waterfall::USB(bool b) {
 extern void viewer_redraw();
 	viewer_redraw();
 }
-	
+
 bool waterfall::USB() {
 	return wfdisp->USB();
 }
 
+void waterfall::show_scope(bool on)
+{
+	if (on) {
+		wfscope->show();
+		wfscope->position(wf->x() + wf->w() - wf_dim - BEZEL, wf->y());
+		wfdisp->size( wf->w() - 2 * BEZEL - wf_dim, wf_dim - 2 * BEZEL);
+		rs1->init_sizes();
+	} else {
+		wfscope->hide();
+		wfscope->position(wf->x() + wf->w(), wf->y());
+		wfdisp->size( wf->w() - 2 * BEZEL, wf_dim - 2 * BEZEL);
+		rs1->init_sizes();
+	}
+}
+
 waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	Fl_Group(x0,y0,w0,h0,lbl) {
 	int xpos;
 	float ratio;// = w0 < 600 ? w0 / 600.0 : 1.0;
 	ratio = w0 * 1.0 / bwdths;
 
-	buttonrow = h() + y() - BTN_HEIGHT - BEZEL;
-	bezel = new Fl_Box(
-				FL_DOWN_BOX, 
-				x(), 
-				y(), 
-				w(), 
-				h() - BTN_HEIGHT - 2 * BEZEL, 0);
-	wfdisp = new WFdisp(x() + BEZEL, 
-			y() + BEZEL, 
+	wf_dim = h() - BTN_HEIGHT - 4;
+
+	buttonrow = h() + y() - BTN_HEIGHT - 1;
+
+	rs1 = new Fl_Group(x(), y(), w(), wf_dim);
+		rs1->box(FL_DOWN_BOX);
+		wfdisp = new WFdisp(
+			x() + BEZEL,
+			y() + BEZEL,
 			w() - 2 * BEZEL,
-			h() - BTN_HEIGHT - 4 * BEZEL);
-	
+			wf_dim - 2 * BEZEL);
+		wfscope = new Digiscope (x() + w(), y(), wf_dim, wf_dim);
+		rs1->resizable(wfdisp);
+	rs1->end();
+	wfscope->hide();
+
 	xpos = x() + wSpace;
 
 	mode = new Fl_Button(xpos, buttonrow, (int)(bwFFT*ratio), BTN_HEIGHT, "WF");
@@ -1341,7 +1445,7 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	mode->tooltip(_("Waterfall / FFT / Scope"));
 
 	xpos = xpos + (int)(bwFFT*ratio) + wSpace;
-	wfRefLevel = new Fl_Counter(xpos, buttonrow, (int)(cwRef*ratio), BTN_HEIGHT );
+	wfRefLevel = new Fl_Counter2(xpos, buttonrow, (int)(cwRef*ratio), BTN_HEIGHT );
 	wfRefLevel->callback(reflevel_cb, 0);
 	wfRefLevel->step(1.0);
 	wfRefLevel->precision(0);
@@ -1352,7 +1456,7 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	wfRefLevel->type(FL_SIMPLE_COUNTER);
 
 	xpos = xpos + (int)(cwRef*ratio) + wSpace;
-	wfAmpSpan = new Fl_Counter(xpos, buttonrow, (int)(cwRef*ratio), BTN_HEIGHT );
+	wfAmpSpan = new Fl_Counter2(xpos, buttonrow, (int)(cwRef*ratio), BTN_HEIGHT );
 	wfAmpSpan->callback(ampspan_cb, 0);
 	wfAmpSpan->step(1.0);
 	wfAmpSpan->precision(0);
@@ -1388,12 +1492,12 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	wfrate->tooltip(_("Waterfall drop speed"));
 
 	xpos = xpos + (int)(bwRate*ratio) + wSpace;
-	wfcarrier = new Fl_Counter(xpos, buttonrow, (int)(cwCnt*ratio), BTN_HEIGHT );
+	wfcarrier = new Fl_Counter2(xpos, buttonrow, (int)(cwCnt*ratio), BTN_HEIGHT );
 	wfcarrier->callback(carrier_cb, 0);
 	wfcarrier->step(1.0);
 	wfcarrier->lstep(10.0);
 	wfcarrier->precision(0);
-	wfcarrier->range(16.0, IMAGE_WIDTH - 16.0);
+	wfcarrier->range(16.0, progdefaults.HighFreqCutoff - 16.0);
 	wfcarrier->value(wfdisp->carrier());
 	wfcarrier->tooltip(_("Adjust cursor frequency"));
 
@@ -1415,27 +1519,95 @@ waterfall::waterfall(int x0, int y0, int w0, int h0, char *lbl) :
 	xmtlock = new Fl_Light_Button(xpos, buttonrow, (int)(bwXmtLock*ratio), BTN_HEIGHT, "Lk");
 	xmtlock->callback(xmtlock_cb, 0);
 	xmtlock->value(0);
-	xmtlock->selection_color(FL_RED);
+	xmtlock->selection_color(progdefaults.LkColor);
 	xmtlock->tooltip(_("Lock transmit frequency"));
 
 	xpos = xpos + (int)(bwXmtLock*ratio) + wSpace;
 	btnRev = new Fl_Light_Button(xpos, buttonrow, (int)(bwRev*ratio), BTN_HEIGHT, "Rv");
 	btnRev->callback(btnRev_cb, 0);
 	btnRev->value(0);
-	btnRev->selection_color(FL_GREEN);
+	btnRev->selection_color(progdefaults.RevColor);
 	btnRev->tooltip(_("Reverse"));
 	reverse = false;
 
 	xpos = w() - (int)(bwXmtRcv*ratio) - wSpace;
 	xmtrcv = new Fl_Light_Button(xpos, buttonrow, (int)(bwXmtRcv*ratio) - BEZEL, BTN_HEIGHT, "T/R");
 	xmtrcv->callback(xmtrcv_cb, 0);
-	xmtrcv->selection_color(FL_RED);
+	xmtrcv->selection_color(progdefaults.XmtColor);
 	xmtrcv->value(0);
 	xmtrcv->tooltip(_("Transmit/Receive"));
-	
 	end();
 }
 
+void waterfall::UI_select(bool on) {
+	if (on) {
+		if (!progdefaults.WF_UIrev)
+			btnRev->hide(); else btnRev->show();
+		if (!progdefaults.WF_UIwfcarrier)
+			wfcarrier->hide(); else wfcarrier->show();
+		if (!progdefaults.WF_UIwfreflevel)
+			wfRefLevel->hide(); else wfRefLevel->show();
+		if (!progdefaults.WF_UIwfampspan)
+			wfAmpSpan->hide(); else wfAmpSpan->show();
+		if (!progdefaults.WF_UIxmtlock)
+			xmtlock->hide(); else xmtlock->show();
+		if (!progdefaults.WF_UIqsy)
+			qsy->hide(); else qsy->show();
+		if (!progdefaults.WF_UIwfmode)
+			mode->hide(); else mode->show();
+		if (!progdefaults.WF_UIx1)
+			x1->hide(); else x1->show();
+		if (!progdefaults.WF_UIwfshift) {
+			left->hide();
+			center->hide();
+			right->hide();
+		} else {
+			left->show();
+			center->show();
+			right->show();
+		}
+		if (!progdefaults.WF_UIwfdrop)
+			wfrate->hide(); else wfrate->show();
+		if (!progdefaults.WF_UIwfstore) {
+			btnMem->hide();
+			mbtnMem->hide();
+		} else {
+			btnMem->show();
+			mbtnMem->show();
+		}
+//if (noUI) xmtrcv->hide();
+	} else {
+		btnRev->show();
+		wfcarrier->show();
+		wfRefLevel->show();
+		wfAmpSpan->show();
+		xmtlock->show();
+		qsy->show();
+		mode->show();
+		x1->show();
+		left->show();
+		center->show();
+		right->show();
+		wfrate->show();
+		btnMem->show();
+		mbtnMem->show();
+	}
+	btnRev->redraw();
+	wfcarrier->redraw();
+	wfRefLevel->redraw();
+	wfAmpSpan->redraw();
+	xmtlock->redraw();
+	qsy->redraw();
+	mode->redraw();
+	x1->redraw();
+	left->redraw();
+	center->redraw();
+	right->redraw();
+	wfrate->redraw();
+	btnMem->redraw();
+	mbtnMem->redraw();
+}
+
 int waterfall::handle(int event)
 {
 	if (event != FL_MOUSEWHEEL || Fl::event_inside(wfdisp))
@@ -1445,22 +1617,17 @@ int waterfall::handle(int event)
 	if ( !((d = Fl::event_dy()) || (d = Fl::event_dx())) )
 		return 1;
 
- 	Fl_Valuator* v[] = { sldrSquelch, wfcarrier, wfRefLevel, wfAmpSpan, valRcvMixer, valXmtMixer };
-  	for (size_t i = 0; i < sizeof(v)/sizeof(v[0]); i++) {
-  		if (Fl::event_inside(v[i])) {
- 			if ((v[i] == sldrSquelch && !progdefaults.docked_scope) ||
-			    v[i] == valRcvMixer || v[i] == valXmtMixer)
-				d = -d;
-			v[i]->value(v[i]->clamp(v[i]->increment(v[i]->value(), -d)));
-			v[i]->do_callback();
-			return 1;
-		}
-	}
-
 	// this does not belong here, but we don't have access to this widget's
 	// handle method (or its parent's)
 	if (Fl::event_inside(MODEstatus)) {
-		init_modem(d > 0 ? MODE_NEXT : MODE_PREV);
+		trx_mode mode = active_modem->get_mode();
+		for (;;) {
+			mode = WCLAMP(mode + d, 0, NUM_MODES - 1);
+			if ((mode >= NUM_RXTX_MODES && mode < NUM_MODES) ||
+			    progdefaults.visible_modes.test(mode))
+				break;
+		}
+		init_modem(mode);
 		return 1;
 	}
 
@@ -1477,7 +1644,7 @@ int waterfall::handle(int event)
 		return 1;
 	}
 
-	return 0;
+	return Fl_Group::handle(event);
 }
 
 static Fl_Cursor cursor = FL_CURSOR_DEFAULT;
@@ -1574,7 +1741,7 @@ int WFdisp::handle(int event)
 		break;
 	case FL_DRAG: case FL_PUSH:
 		stopMacroTimer();
-	
+
 		switch (eb = Fl::event_button()) {
 		case FL_RIGHT_MOUSE:
 			wantcursor = false;
@@ -1615,7 +1782,10 @@ int WFdisp::handle(int event)
 			if (progdefaults.WaterfallHistoryDefault)
 				bHistory = true;
 			newcarrier = cursorFreq(xpos);
-			newcarrier = (int)CLAMP(newcarrier, wf->wfcarrier->minimum(), wf->wfcarrier->maximum());
+			newcarrier = (int)CLAMP(
+				newcarrier, 
+				progdefaults.LowFreqCutoff + active_modem->get_bandwidth() / 2, 
+				progdefaults.HighFreqCutoff - active_modem->get_bandwidth() / 2);
 			active_modem->set_freq(newcarrier);
 			if (!(Fl::event_state() & FL_SHIFT))
 				active_modem->set_sigsearch(SIGSEARCH);
@@ -1662,7 +1832,7 @@ int WFdisp::handle(int event)
 	case FL_MOUSEWHEEL:
 	{
 		stopMacroTimer();
-	
+
 		int d;
 		if ( !((d = Fl::event_dy()) || (d = Fl::event_dx())) )
 			break;
@@ -1688,20 +1858,26 @@ int WFdisp::handle(int event)
 	case FL_KEYBOARD:
 	{
 		stopMacroTimer();
-	
+
 		int d = (Fl::event_state() & FL_CTRL) ? 10 : 1;
 		int k = Fl::event_key();
 		switch (k) {
 		case FL_Left: case FL_Right:
 			if (k == FL_Left)
 				d = -d;
-			oldcarrier = newcarrier = (int)CLAMP(carrier() + d, wf->wfcarrier->minimum(), wf->wfcarrier->maximum());
+			oldcarrier = newcarrier = (int)CLAMP(
+				carrier() + d, 
+				progdefaults.LowFreqCutoff + active_modem->get_bandwidth() / 2, 
+				progdefaults.HighFreqCutoff - active_modem->get_bandwidth() / 2);
 			active_modem->set_freq(newcarrier);
 			redrawCursor();
 			break;
 		case FL_Tab:
 			restoreFocus();
 			break;
+		default:
+			restoreFocus();
+			return TransmitText->handle(event);
 		}
 		break;
 	}
@@ -1767,8 +1943,7 @@ void waterfall::handle_mouse_wheel(int what, int d)
 		return;
 	case WF_SQUELCH:
 		val = sldrSquelch;
-		if (!progdefaults.docked_scope)
-			d = -d;
+		d = -d;
 		msg_fmt = "%s = %2.0f %%";
 		msg_label = "Squelch";
 		break;
