@@ -1,10 +1,10 @@
 // ----------------------------------------------------------------------------
 //      FTextRXTX.cxx
 //
-// Copyright (C) 2007-2008
+// Copyright (C) 2007-2010
 //              Stelios Bounanos, M0GLD
 //
-// Copyright (C) 2008
+// Copyright (C) 2008-2010
 //              Dave Freese, W1HKJ
 //
 // This file is part of fldigi.
@@ -25,6 +25,7 @@
 
 #include <config.h>
 
+#include <string>
 #include <cstring>
 #include <cstdlib>
 #include <cstdio>
@@ -69,20 +70,46 @@
 
 using namespace std;
 
+
+// Fl_Scrollbar wrapper to draw marks on the slider background.
+// Currently only implemented for a vertical scrollbar.
+class MVScrollbar : public Fl_Scrollbar
+{
+	struct mark_t {
+		double pos;
+		Fl_Color color;
+		mark_t(double pos_, Fl_Color color_) : pos(pos_), color(color_) { }
+	};
+
+public:
+	MVScrollbar(int X, int Y, int W, int H, const char* l = 0)
+		: Fl_Scrollbar(X, Y, W, H, l), draw_marks(false) { }
+
+	void draw(void);
+	void mark(Fl_Color c) { marks.push_back(mark_t(maximum() - 1.0, c)); redraw(); }
+	bool has_marks(void) { return !marks.empty(); }
+	void show_marks(bool b) { draw_marks = b; redraw(); }
+	void clear(void) { marks.clear(); redraw(); }
+
+private:
+	vector<mark_t> marks;
+	bool draw_marks;
+};
+
 static void show_font_warning(FTextBase* w);
 
 Fl_Menu_Item FTextRX::menu[] = {
-	{ make_icon_label(_("&Look up call"), net_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Call"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Name"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("QT&H"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&State"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Province"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("Countr&y"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Locator"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&RST(r)"), enter_key_icon), 0, 0, 0,  FL_MENU_DIVIDER, _FL_MULTI_LABEL },
-	{ make_icon_label(_("Serial number"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("Exchange In"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Look up call"), net_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Call"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Name"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("QTH"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("State"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Province"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Country"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Locator"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("RST(r)"), enter_key_icon), 0, 0, 0,  FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Exchange In"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Serial number"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 	{ make_icon_label(_("Insert marker"), insert_link_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 
 	{ 0 }, // VIEW_MENU_COPY
@@ -92,6 +119,7 @@ Fl_Menu_Item FTextRX::menu[] = {
 	{ 0 }, // VIEW_MENU_WRAP
 
 	{ _("Quick entry"),      0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL },
+	{ _("Scroll hints"),      0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL },
 	{ 0 }
 };
 
@@ -105,12 +133,22 @@ Fl_Menu_Item FTextRX::menu[] = {
 /// @param h
 /// @param l
 FTextRX::FTextRX(int x, int y, int w, int h, const char *l)
-        : FTextView(x, y, w, h, l), quick_entry(false)
+        : FTextView(x, y, w, h, l)
 {
 	memcpy(menu + RX_MENU_COPY, FTextView::menu, (FTextView::menu->size() - 1) * sizeof(*FTextView::menu));
 	context_menu = menu;
 	init_context_menu();
 	menu[RX_MENU_QUICK_ENTRY].clear();
+	menu[RX_MENU_SCROLL_HINTS].clear();
+
+	// Replace the scrollbar widget
+	MVScrollbar* mvsb = new MVScrollbar(mVScrollBar->x(), mVScrollBar->y(),
+					    mVScrollBar->w(), mVScrollBar->h(), NULL);
+	mvsb->show_marks(false);
+	mvsb->callback(mVScrollBar->callback(), mVScrollBar->user_data());
+	remove(mVScrollBar);
+	delete mVScrollBar;
+	Fl_Group::add(mVScrollBar = mvsb);
 }
 
 FTextRX::~FTextRX()
@@ -202,14 +240,83 @@ out:
 	return FTextView::handle(event);
 }
 
+/// Adds a char to the buffer
+///
+/// @param c The character
+/// @param attr The attribute (@see enum text_attr_e); RECV if omitted.
+///
+void FTextRX::add(unsigned char c, int attr)
+{
+	if (c == '\r')
+		return;
+
+	// The user may have moved the cursor by selecting text or
+	// scrolling. Place it at the end of the buffer.
+	if (mCursorPos != tbuf->length())
+		insert_position(tbuf->length());
+
+	switch (c) {
+	case '\b':
+		// we don't call kf_backspace because it kills selected text
+		tbuf->remove(tbuf->length() - 1, tbuf->length());
+		sbuf->remove(sbuf->length() - 1, sbuf->length());
+		break;
+	case '\n':
+		// maintain the scrollback limit, if we have one
+		if (max_lines > 0 && tbuf->count_lines(0, tbuf->length()) >= max_lines) {
+			int le = tbuf->line_end(0) + 1; // plus 1 for the newline
+			tbuf->remove(0, le);
+			sbuf->remove(0, le);
+		}
+		// fall-through
+	default:
+		char s[] = { '\0', '\0', FTEXT_DEF + attr, '\0' };
+		const char *cp;
+
+		if ((c < ' ' || c == 127) && attr != CTRL) // look it up
+			cp = ascii[(unsigned char)c];
+		else { // insert verbatim
+			s[0] = c;
+			cp = &s[0];
+		}
+
+		for (int i = 0; cp[i]; ++i)
+			sbuf->append(s + 2);
+		insert(cp);
+		break;
+	}
+}
+
 void FTextRX::set_quick_entry(bool b)
 {
-	if ((quick_entry = b))
+	if (b)
 		menu[RX_MENU_QUICK_ENTRY].set();
 	else
 		menu[RX_MENU_QUICK_ENTRY].clear();
 }
 
+void FTextRX::set_scroll_hints(bool b)
+{
+	if (b)
+		menu[RX_MENU_SCROLL_HINTS].set();
+	else
+		menu[RX_MENU_SCROLL_HINTS].clear();
+	static_cast<MVScrollbar*>(mVScrollBar)->show_marks(b);
+}
+
+void FTextRX::mark(FTextBase::TEXT_ATTR attr)
+{
+	if (attr == NATTR)
+		attr = CLICK_START;
+	static_cast<MVScrollbar*>(mVScrollBar)->mark(styles[attr].color);
+}
+
+void FTextRX::clear(void)
+{
+	FTextBase::clear();
+	static_cast<MVScrollbar*>(mVScrollBar)->clear();
+}
+
 void FTextRX::setFont(Fl_Font f, int attr)
 {
 	FTextBase::setFont(f, attr);
@@ -267,7 +374,7 @@ static fre_t call("([[:alnum:]]?[[:alpha:]/]+[[:digit:]]+[[:alnum:]/]+)", REG_EX
 
 void FTextRX::handle_qso_data(int start, int end)
 {
-	char* s = get_word(start, end);
+	char* s = get_word(start, end, progdefaults.nonwordchars.c_str());
 	if (!s)
 		return;
 	char* p = s;
@@ -292,43 +399,70 @@ void FTextRX::handle_qso_data(int start, int end)
 	target->value(p);
 	target->do_callback();
 	free(s);
+	restoreFocus(NULL);
 }
 
 void FTextRX::handle_context_menu(void)
 {
-	// toggle visibility of quick entry items
+	bool contest_ui = progStatus.Rig_Contest_UI ||
+		(progStatus.contest && !progStatus.NO_RIGLOG && !progStatus.Rig_Log_UI);
+
+	unsigned shown = 0;
+
+#define show_item(x_) (shown |=  (1 << x_))
+#define hide_item(x_) (shown &= ~(1 << x_))
+#define test_item(x_) (shown &   (1 << x_))
+
+	show_item(RX_MENU_CALL);
+
+	if (contest_ui || (progStatus.contest && !progStatus.NO_RIGLOG && !progStatus.Rig_Log_UI)) {
+		show_item(RX_MENU_SERIAL);
+		show_item(RX_MENU_XCHG);
+	}
+	else if (progdefaults.QRZ != QRZNONE) // "Look up call" shown only in non-contest mode
+		show_item(RX_MENU_QRZ_THIS);
+
 	if (menu[RX_MENU_QUICK_ENTRY].value()) {
-		menu[RX_MENU_CALL].flags &= ~FL_MENU_DIVIDER;
 		for (size_t i = RX_MENU_NAME; i <= RX_MENU_RST_IN; i++)
-			menu[i].show();
+			show_item(i);
+		menu[RX_MENU_CALL].flags &= ~FL_MENU_DIVIDER;
 	}
 	else {
+		if (!contest_ui && !progStatus.contest) {
+			show_item(RX_MENU_NAME);
+			show_item(RX_MENU_QTH);
+			show_item(RX_MENU_RST_IN);
+		}
 		menu[RX_MENU_CALL].flags |= FL_MENU_DIVIDER;
-		for (size_t i = RX_MENU_NAME; i <= (progStatus.contest ? RX_MENU_LOC : RX_MENU_RST_IN); i++)
+	}
+
+	if (static_cast<MVScrollbar*>(mVScrollBar)->has_marks())
+		menu[RX_MENU_SCROLL_HINTS].show();
+	else
+		menu[RX_MENU_SCROLL_HINTS].hide();
+
+	for (size_t i = RX_MENU_QRZ_THIS; i <= RX_MENU_XCHG; i++) {
+		if (test_item(i))
+			menu[i].show();
+		else
 			menu[i].hide();
 	}
 
+#undef show_item
+#undef hide_item
+#undef test_item
+
+	// availability of editing items depend on buffer state
 	set_active(&menu[RX_MENU_COPY], tbuf->selected());
 	set_active(&menu[RX_MENU_CLEAR], tbuf->length());
 	set_active(&menu[RX_MENU_SELECT_ALL], tbuf->length());
 	set_active(&menu[RX_MENU_SAVE], tbuf->length());
+
 	if (wrap)
 		menu[RX_MENU_WRAP].set();
 	else
 		menu[RX_MENU_WRAP].clear();
 
-	if (progStatus.contest) {
-			menu[RX_MENU_SERIAL].show();
-			menu[RX_MENU_XCHG].show();
-	} else {
-			menu[RX_MENU_SERIAL].hide();
-			menu[RX_MENU_XCHG].hide();
-	}
-
-	if (progdefaults.QRZ != QRZNONE)
-		menu[RX_MENU_QRZ_THIS].show();
-	else
-		menu[RX_MENU_QRZ_THIS].hide();
 	show_context_menu();
 }
 
@@ -396,23 +530,33 @@ void FTextRX::menu_cb(size_t item)
 
 	case RX_MENU_QUICK_ENTRY:
 		menu[item].flags ^= FL_MENU_VALUE;
-		quick_entry = menu[item].value();
+		if (menu[item].value())
+			handle_context_menu();
 		break;
 
 	case RX_MENU_WRAP:
 		set_word_wrap(!wrap);
+		restore_wrap = wrap;
+		break;
+
+	case RX_MENU_SCROLL_HINTS:
+		menu[item].flags ^= FL_MENU_VALUE;
+		static_cast<MVScrollbar*>(mVScrollBar)->show_marks(menu[item].value());
 		break;
 	}
 
+	restoreFocus(NULL);
+
 	if (!input)
 		return;
-	char* s = get_word(popx, popy);
+	char* s = get_word(popx, popy, progdefaults.nonwordchars.c_str());
 	if (!s)
 		return;
 
 	if (item == RX_MENU_XCHG) { // append
 		input->position(input->size());
-		input->insert(" ", 1);
+		if (input->size())
+			input->insert(" ", 1);
 		input->insert(s);
 	}
 	else
@@ -423,7 +567,7 @@ void FTextRX::menu_cb(size_t item)
 
 const char* FTextRX::dxcc_lookup_call(int x, int y)
 {
-	char* s = get_word(x - this->x(), y - this->y());
+	char* s = get_word(x - this->x(), y - this->y(), progdefaults.nonwordchars.c_str());
 	char* mem = s;
 	if (!(s && *s && call.match(s))) {
 		free(s);
@@ -527,10 +671,10 @@ void FTextRX::dxcc_tooltip(void* obj)
 
 
 Fl_Menu_Item FTextTX::menu[] = {
-	{ make_icon_label(_("&Transmit"), tx_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Receive"), rx_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Abort"), process_stop_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("Send &image..."), image_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Transmit"), tx_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Receive"), rx_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Abort"), process_stop_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Send image..."), image_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 
 	{ 0 }, // EDIT_MENU_CUT
 	{ 0 }, // EDIT_MENU_COPY
@@ -605,6 +749,17 @@ void FTextTX::clear_sent(void)
 	txpos = 0;
 	bkspaces = 0;
 	PauseBreak = false;
+	set_word_wrap(restore_wrap);
+}
+
+/// Returns boolean <eot> end of text
+///
+/// true if empty buffer
+/// false if characters remain
+///
+bool FTextTX::eot(void)
+{
+	return (insert_position() == txpos);
 }
 
 /// Returns the next character to be transmitted.
@@ -809,8 +964,9 @@ int FTextTX::handle_key_macro(int key)
 
 int FTextTX::handle_dnd_drag(int pos)
 {
-	if (pos >= txpos)
+	if (pos >= txpos) {
 		return FTextEdit::handle_dnd_drag(pos);
+	}
 	else // refuse drop inside transmitted text
 		return 0;
 }
@@ -897,11 +1053,15 @@ void FTextTX::menu_cb(size_t item)
 	case TX_MENU_PASTE:
 		kf_paste(0, this);
 		break;
-	case TX_MENU_READ:
+	case TX_MENU_READ: {
+		restore_wrap = wrap;
+		set_word_wrap(false);
 		readFile();
 		break;
+	}
 	case TX_MENU_WRAP:
 		set_word_wrap(!wrap);
+		restore_wrap = wrap;
 		break;
 	}
 }
@@ -1013,3 +1173,45 @@ static void show_font_warning(FTextBase* w)
 		*warn = false;
 	}
 }
+
+
+// ----------------------------------------------------------------------------
+
+
+void MVScrollbar::draw(void)
+{
+	Fl_Scrollbar::draw();
+
+	if (marks.empty() || !draw_marks)
+		return;
+
+	assert((type() & FL_HOR_SLIDER) == 0);
+
+	// Calculate the slider knob position and height.  For a vertical scrollbar,
+	// the scroll buttons' height is the scrollbar width and the minimum knob
+	// height is half that.
+	int H = h() - Fl::box_dh(box()) - 2 * w(); // internal height (minus buttons)
+	int slider_h = (int)(slider_size() * H + 0.5);
+	int min_h = (w() - Fl::box_dw(box())) / 2 + 1;
+	if (slider_h < min_h)
+		slider_h = min_h;
+	double val = (Fl_Slider::value() - minimum()) / (maximum() - minimum());
+	int slider_y = (int)(val * (H - slider_h) + 0.5) + w(); // relative to y()
+	// This would draw a green rectangle around the slider knob:
+	// fl_color(FL_GREEN);
+	// fl_rect(x(), y() + slider_y, w() - Fl::box_dw(box()), slider_h);
+
+	int x1 = x() + Fl::box_dx(box()), x2 = x1 + w() - Fl::box_dw(box()) - 1, ypos;
+	// Convert stored scrollbar values to vertical positions and draw
+	// lines inside the widget if they don't overlap with the knob area.
+	for (vector<mark_t>::const_iterator i = marks.begin(); i != marks.end(); ++i) {
+		ypos = static_cast<int>(w() + H * i->pos / maximum());
+		// Don't draw over slider knob
+		if ((ypos > slider_y && ypos < slider_y + slider_h) ||
+		    (ypos < slider_y + slider_h && ypos > slider_y))
+			continue;
+		ypos += y();
+		fl_color(i->color);
+		fl_line(x1, ypos, x2, ypos);
+	}
+}
