@@ -4,7 +4,7 @@
 // Copyright (C) 2007-2009
 //              Stelios Bounanos, M0GLD
 //
-// Copyright (C) 2008
+// Copyright (C) 2008-2009
 //              Dave Freese, W1HKJ
 //
 // This file is part of fldigi.
@@ -67,14 +67,16 @@ FTextBase::FTextBase(int x, int y, int w, int h, const char *l)
 	textsize(FL_NORMAL_SIZE);
 	textcolor(FL_FOREGROUND_COLOR);
 
-	tbuf = new Fl_Text_Buffer;
-	sbuf = new Fl_Text_Buffer;
+	tbuf = new Fl_Text_Buffer_mod;
+	sbuf = new Fl_Text_Buffer_mod;
 
-	cursor_style(Fl_Text_Editor_mod::NORMAL_CURSOR);
 	buffer(tbuf);
 	highlight_data(sbuf, styles, NATTR, FTEXT_DEF, 0, 0);
+	cursor_style(Fl_Text_Editor_mod::NORMAL_CURSOR);
 
 	wrap_mode(wrap, wrap_col);
+	restore_wrap = wrap;
+//	wrap_restore = true;
 
 	// Do we want narrower scrollbars? The default width is 16.
 	// scrollbar_width((int)floor(scrollbar_width() * 3.0/4.0));
@@ -82,6 +84,13 @@ FTextBase::FTextBase(int x, int y, int w, int h, const char *l)
 	reset_styles(SET_FONT | SET_SIZE | SET_COLOR);
 }
 
+void FTextBase::clear()
+{
+	tbuf->text("");
+	sbuf->text("");
+	set_word_wrap(restore_wrap);
+}
+
 int FTextBase::handle(int event)
 {
         if (event == FL_MOUSEWHEEL && !Fl::event_inside(this))
@@ -97,6 +106,37 @@ int FTextBase::handle(int event)
 		return Fl_Text_Editor_mod::handle(event);
 }
 
+/// @see FTextRX::add
+///
+/// @param s 
+/// @param attr 
+///
+void FTextBase::add(const char *s, int attr)
+{
+	// handle the text attribute first
+	int n = strlen(s);
+	char a[n + 1];
+	memset(a, FTEXT_DEF + attr, n);
+	a[n] = '\0';
+	sbuf->replace(insert_position(), insert_position() + n, a);
+
+	insert(s);
+}
+
+/// @see FTextBase::add
+///
+/// @param s 
+/// @param attr 
+///
+void FTextBase::add(unsigned char c, int attr)
+{
+	char s[] = { FTEXT_DEF + attr, '\0' };
+	sbuf->replace(insert_position(), insert_position() + 1, s);
+
+	s[0] = c;
+	insert(s);
+}
+
 void FTextBase::set_word_wrap(bool b)
 {
 	wrap_mode((wrap = b), wrap_col);
@@ -330,7 +370,7 @@ void FTextBase::saveFile(void)
 ///
 /// @return The selection, or the word text at (x,y). <b>Must be freed by the caller</b>.
 ///
-char* FTextBase::get_word(int x, int y, bool ontext)
+char* FTextBase::get_word(int x, int y, const char* nwchars, bool ontext)
 {
 	int p = xy_to_position(x + this->x(), y + this->y(), Fl_Text_Display_mod::CURSOR_POS);
 	int start, end;
@@ -342,11 +382,13 @@ char* FTextBase::get_word(int x, int y, bool ontext)
 			return tbuf->selection_text();
 	}
 
-	if (!tbuf->findchars_backward(p, " \t\n", &start))
+	string nonword = nwchars;
+	nonword.append(" \t\n");
+	if (!tbuf->findchars_backward(p, nonword.c_str(), &start))
 		start = 0;
 	else
 		start++;
-	if (!tbuf->findchars_forward(p, " .,;|\t\n", &end))
+	if (!tbuf->findchars_forward(p, nonword.c_str(), &end))
 		end = tbuf->length();
 
 	if (ontext && (p < start || p >= end))
@@ -419,11 +461,11 @@ void FTextBase::reset_styles(int set)
 // ----------------------------------------------------------------------------
 
 Fl_Menu_Item FTextView::menu[] = {
-	{ make_icon_label(_("&Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("C&lear"), edit_clear_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Clear"), edit_clear_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ make_icon_label(_("Select All"), edit_select_all_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
-	{ make_icon_label(_("Save &as..."), save_as_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
-	{ _("Word &wrap"),       0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL },
+	{ make_icon_label(_("Save as..."), save_as_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ _("Word wrap"),       0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL },
 	{ 0 }
 };
 
@@ -450,53 +492,6 @@ FTextView::FTextView(int x, int y, int w, int h, const char *l)
 	init_context_menu();
 }
 
-/// Adds a char to the buffer
-///
-/// @param c The character
-/// @param attr The attribute (@see enum text_attr_e); RECV if omitted.
-///
-void FTextView::add(unsigned char c, int attr)
-{
-	if (c == '\r')
-		return;
-
-	// The user may have moved the cursor by selecting text or
-	// scrolling. Place it at the end of the buffer.
-	if (mCursorPos != tbuf->length())
-		insert_position(tbuf->length());
-
-	switch (c) {
-	case '\b':
-		// we don't call kf_backspace because it kills selected text
-		tbuf->remove(tbuf->length() - 1, tbuf->length());
-		sbuf->remove(sbuf->length() - 1, sbuf->length());
-		break;
-	case '\n':
-		// maintain the scrollback limit, if we have one
-		if (max_lines > 0 && tbuf->count_lines(0, tbuf->length()) >= max_lines) {
-			int le = tbuf->line_end(0) + 1; // plus 1 for the newline
-			tbuf->remove(0, le);
-			sbuf->remove(0, le);
-		}
-		// fall-through
-	default:
-		char s[] = { '\0', '\0', FTEXT_DEF + attr, '\0' };
-		const char *cp;
-
-		if ((c < ' ' || c == 127) && attr != CTRL) // look it up
-			cp = ascii[(unsigned char)c];
-		else { // insert verbatim
-			s[0] = c;
-			cp = &s[0];
-		}
-
-		for (int i = 0; cp[i]; ++i)
-			sbuf->append(s + 2);
-		insert(cp);
-		break;
-	}
-}
-
 /// Handles fltk events for this widget.
 
 /// We only care about mouse presses (to display the popup menu and prevent
@@ -572,6 +567,7 @@ void FTextView::menu_cb(size_t item)
 		break;
 	case VIEW_MENU_WRAP:
 		set_word_wrap(!wrap);
+		restore_wrap = wrap;
 		break;
 	}
 }
@@ -629,12 +625,12 @@ loop:
 
 
 Fl_Menu_Item FTextEdit::menu[] = {
-	{ make_icon_label(_("Cu&t"), edit_cut_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("&Paste"), edit_paste_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ make_icon_label(_("C&lear"), edit_clear_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
-	{ make_icon_label(_("Insert &file..."), file_open_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
-	{ _("Word &wrap"), 0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL } ,
+	{ make_icon_label(_("Cut"), edit_cut_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Copy"), edit_copy_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Paste"), edit_paste_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Clear"), edit_clear_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ make_icon_label(_("Insert file..."), file_open_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ _("Word wrap"), 0, 0, 0, FL_MENU_TOGGLE, FL_NORMAL_LABEL } ,
 	{ 0 }
 };
 
@@ -698,37 +694,6 @@ int FTextEdit::handle(int event)
 	return FTextBase::handle(event);
 }
 
-/// @see FTextView::add
-///
-/// @param s 
-/// @param attr 
-///
-void FTextEdit::add(const char *s, int attr)
-{
-	// handle the text attribute first
-	int n = strlen(s);
-	char a[n + 1];
-	memset(a, FTEXT_DEF + attr, n);
-	a[n] = '\0';
-	sbuf->replace(insert_position(), insert_position() + n, a);
-
-	insert(s);
-}
-
-/// @see FTextEdit::add
-///
-/// @param s 
-/// @param attr 
-///
-void FTextEdit::add(unsigned char c, int attr)
-{
-	char s[] = { FTEXT_DEF + attr, '\0' };
-	sbuf->replace(insert_position(), insert_position() + 1, s);
-
-	s[0] = c;
-	insert(s);
-}
-
 /// Handles keyboard events to override Fl_Text_Editor_mod's handling of some
 /// keystrokes.
 ///
@@ -799,6 +764,9 @@ int FTextEdit::handle_dnd_drag(int pos)
 /// @return 1 or FTextBase::handle(FL_PASTE)
 int FTextEdit::handle_dnd_drop(void)
 {
+	restore_wrap = wrap;
+	set_word_wrap(false);
+
 	if (Fl::event_shift())
 		return FTextBase::handle(FL_PASTE);
 
@@ -876,6 +844,7 @@ void FTextEdit::menu_cb(size_t item)
 		break;
 	case EDIT_MENU_WRAP:
 		set_word_wrap(!wrap);
+		restore_wrap = wrap;
 		break;
 	}
 }
