@@ -1,24 +1,23 @@
 // ----------------------------------------------------------------------------
 // wwv.cxx  -- wwv monitoring modem
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2009
 //		Dave Freese, W1HKJ
 //
-// This file is part of fldigi. 
+// This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 //
 // This modem is only used for reception of WWV "tick" signals to determine
@@ -89,6 +88,7 @@ wwv::wwv() : modem()
 	
 	vidfilter = new Cmovavg(16);
 	
+	cap &= ~CAP_TX;
 }
 
 
